<?php
    /**
     * @package     Application
     * @author      Quentin Zervaas
     */
    class Application
    {
        const VERSION        = '2.80.22';
        const COMPAT_VERSION = '2.80.16';
        const NAME           = 'Recite CMS';

        const SITE_CP     = 'cp';
        const SITE_CLIENT = 'client';
        const SITE_CLI    = 'cli';
        const SITE_DAV    = 'dav';
        const SITE_ADMIN  = 'admin';

        protected static $_config;
        protected static $_db;
        protected static $_clientId;
        protected static $_domainId;
        protected static $_domain;

        protected static $_logger = array(
            'logger'      => null,
            'emailWriter' => null
        );

        protected static $_locale;
        protected static $_localeName;

        protected static $_error = false;

        protected static $_site = null;
        public static $sites = array(
            self::SITE_CP     => 'Control Panel',
            self::SITE_ADMIN  => 'Administration',
            self::SITE_CLIENT => 'Client Site',
            self::SITE_CLI    => 'Command Line'
        );

        protected static $_profiling = false;

        private static $_licenses = array();

        /**
         * version
         *
         * Return the version of this codebase
         *
         * @return  string
         */
        public static function version()
        {
            return self::VERSION;
        }


        public static function startup($site = null)
        {
            umask(0006);

            if ($site == self::SITE_CLIENT) {
                try {
                    Application_Cache::HandleRequest();
                }
                catch (Exception $ex) {
                }
            }

            @date_default_timezone_set('Australia/Adelaide');

            try {
                $options = new Application_Cache_Options();
                $options->setMaxLifetime(604800)
                        ->setForceCaching(true);

                Zend_Locale_Data::SetCache(Application_Cache::GetCache('_locale', null, $options));
            }
            catch (Exception $ex) { }

            self::GetLocale();

            if (array_key_exists($site, self::$sites)) {
                self::$_site = $site;
            }

            self::_loadConfig();
            self::_loadLogger(); // call again to use enact config settings
            self::_loadDatabase();

            if (coalesce($_SERVER['RECITE_PROFILING'], false)) {
                self::EnableProfiling();
            }

            switch ($site) {
                case self::SITE_DAV:
                    try {
                        $ids = Application_Client_Domain::GetClientIdAndDomainIdForCurrentDomain(true);
                        self::SetClientId($ids['client']);
                        self::SetDomainId($ids['domain']);
                        $client = self::GetClient();
                    }
                    catch (Exception $ex) {
                        Application_Client_Site_Maintenance::SendNoClientErrorMessage();
                        exit;
                    }

                    if (!$client->isActive()) {
                        Application_Client_Site_Maintenance::SendInactiveClientErrorMessage();
                    }

                    $user = new Application_User();
                    Zend_Registry::set('user', $user);

                    break;

                case self::SITE_CLIENT:
                    try {
                        $ids = Application_Client_Domain::GetClientIdAndDomainIdForCurrentDomain(false);

                        self::SetClientId($ids['client']);
                        self::SetDomainId($ids['domain']);
                        $client = self::GetClient();
                    }
                    catch (Exception $ex) {
                        Application_Client_Site_Maintenance::SendNoClientErrorMessage();
                        exit;
                    }

                    if ($client->status == Application_Client::STATUS_INACTIVE || $client->status == Application_Client::STATUS_DISABLED) {
                        Application_Client_Site_Maintenance::SendInactiveClientErrorMessage();
                    }

                    $user = new Application_User();
                    Zend_Registry::set('user', $user);

                    // check license keys
                    try {
                        $type = self::GetLicenseType();
                    }
                    catch (Application_License_Exception_NoLicense $ex) {
                        Application_Client_Site_Maintenance::SendNoLicenseErrorMessage();
                    }
                    catch (Application_License_Exception_Expired $ex) {
                        Application_Client_Site_Maintenance::SendLicenseExpiredMessage();
                    }
                    catch (Application_License_Exception_Invalid $ex) {
                        Application_Client_Site_Maintenance::SendInvalidLicenseErrorMessage();
                    }
                    catch (Exception $ex) {
                        Application_Client_Site_Maintenance::SendNoLicenseErrorMessage();
                    }

                    break;
            }
        }

        public static function getSite()
        {
            return self::$_site;
        }

        public static function getSiteLabel()
        {
            return self::$sites[self::$_site];
        }

        public static function getBaseDir()
        {
            return realpath(dirname(__FILE__) . '/../..');
        }

        public static function getDataDir()
        {
            return self::getBaseDir() . DIRECTORY_SEPARATOR . 'data';
        }

        public static function getTmpDir()
        {
            return self::getDataDir() . DIRECTORY_SEPARATOR . 'tmp';
        }

        public static function getApplicationDir()
        {
            return self::getBaseDir() . DIRECTORY_SEPARATOR . 'application';
        }

        public static function setDb(Db_Adapter_Abstract $db)
        {
            self::$_db = $db;
            Zend_Registry::set('db', $db);
        }

        public static function getDb()
        {
            return self::$_db;
        }

        public static function getConfig()
        {
            return self::$_config;
        }

        public static function getConfigFilename()
        {
            $filename = trim(coalesce($_SERVER['RECITE_CONFIG_FILE']));
            if (strlen($filename) == 0)
                $filename = 'recite.ini';

            return self::getDataDir() . DIRECTORY_SEPARATOR . basename($filename);
        }

        public static function getConfigSection()
        {
            $section = trim(coalesce($_SERVER['RECITE_CONFIG_SECTION']));
            if (strlen($section) == 0)
                $section = 'production';

            return $section;
        }


        public static function getLogger()
        {
            return self::$_logger['logger'];
        }


        protected static function _loadLogger()
        {
            if (!self::$_logger['logger'] instanceof Zend_Log) {
                self::$_logger['logger'] = new Zend_Log(new Zend_Log_Writer_Null());
                Zend_Registry::set('logger', self::$_logger['logger']);
            }
/*
            if (!self::$_logger['emailWriter'] instanceof Zend_Log_Writer_Abstract) {
                self::$_logger['emailWriter'] = new Application_Log_Emailer();
                self::$_logger['emailWriter']->addFilter(new Zend_Log_Filter_Priority(Zend_Log::CRIT));
                self::$_logger['logger']->addWriter(self::$_logger['emailWriter']);
            }
*/
            $config = self::getConfig();

            //$email = '';
            if ($config instanceof Zend_Config) {
                try {
                    $path = dirname($config->debug->debug_log_file);
                    if (!file_exists($path)) {
                        @mkdir($path, 0777);
                    }

                    $writer = new Zend_Log_Writer_Stream($config->debug->debug_log_file);
                    $writer->setFormatter(new Application_Log_Formatter());
                    self::$_logger['logger']->addWriter($writer);
                }
                catch (Exception $ex) { }

                //$email = trim($config->log->email);
            }
/*
            $validator = new Zend_Validate_EmailAddress();
            if (strlen($email) == 0 || !$validator->isValid($email))
                $email = coalesce($_SERVER['SERVER_ADMIN']);

            try {
                self::$_logger['emailWriter']->setEmail($email);
            }
            catch (Exception $ex) { }
*/
            Application_Auditor::Enable($config->debug->system_audit);
        }

        protected static function _loadConfig()
        {
            self::$_config = Application_Config::GetConfig(self::getConfigFilename(), self::getConfigSection());
            Zend_Registry::set('config', self::$_config);
        }

        protected static function _loadDatabase()
        {
            $config = self::getConfig();

            $db = Db_Adapter_Abstract::factory($config->database->dsn, array('persistent' => isset($config->database->persistent) && $config->database->persistent));
            $db->getConnection();

            self::setDb($db);
        }

        public static function SetLocale($name)
        {
            if (self::$_locale instanceof Zend_Locale) {
                try {
                    self::$_locale->setLocale($name);
                    self::$_localeName = (string) self::$_locale;
                }
                catch (Exception $ex) {
                }
            }
            else {
                self::$_localeName = $name;
            }
        }

        public static function GetLocale()
        {
            if (!self::$_locale instanceof Zend_Locale) {
                try {
                    if (strlen(self::$_localeName) > 0) {
                        $locale = new Zend_Locale(self::$_localeName);
                    }
                    else {
                        $locale = new Zend_Locale(Zend_Locale::ENVIRONMENT);
                    }
                }
                catch (Exception $ex) {
                    $locale = new Zend_Locale('en_AU');
                }

                self::$_locale = $locale;
            }

            return self::$_locale;
        }

        public static function EnableProfiling($flag = true)
        {
            self::$_profiling = (bool) $flag;

            if (self::$_profiling) {
                $profiler = new Db_Profiler('Database Queries');
                self::$_db->setProfiler($profiler);

                $profiler->setEnabled(true);
            }
            else {
                $profiler = self::$_db->getProfiler();
                if ($profiler instanceof Zend_Db_Profiler) {
                    $profiler->setEnabled(false);
                }
            }
        }

        public static function IsProfiling()
        {
            return self::$_profiling;
        }

        public static function SetClient(Application_Client $client)
        {
            Zend_Registry::set('client', $client);
            self::$_clientId = $client->getId();
        }

        public static function SetClientId($clientId)
        {
            if ($clientId <= 0) {
                throw new Exception('Invalid client ID');
            }

            self::$_clientId = $clientId;
            Zend_Registry::set('client', null);
        }

        public static function GetClient()
        {
            if (Zend_Registry::IsRegistered('client')) {
                $client = Zend_Registry::get('client');

                if ($client instanceof Application_Client && $client->saved()) {
                    return $client;
                }
            }

            $clientId = self::GetClientId();
            $client = new Application_Client();
            $client->load($clientId);

            self::SetClient($client);

            return $client;
        }

        public static function GetClientId()
        {
            if (self::$_clientId > 0) {
                return self::$_clientId;
            }
            else {
                throw new Exception('Client not set');
            }
        }

        public static function SetDomainId($domainId)
        {
            if ($domainId <= 0) {
                throw new Exception('Invalid domain ID');
            }

            self::$_domainId = $domainId;
        }

        public static function GetDomainId()
        {
            $id = (int) self::$_domainId;

            if ($id <= 0) {
                throw new Exception('Domain ID not set');
            }

            return $id;
        }

        public static function GetCurrentDomain()
        {
            if (strlen(self::$_domain) == 0) {
                try {
                    $domainId = self::GetDomainId();
                    $domain = new Application_Client_Domain();
                    $domain->load($domainId);

                    if (!$domain->saved() || $domain->is_dav) {
                        throw new Exception('Invalid domain');
                    }

                    $ret = $domain->url;
                }
                catch (Exception $ex) {
                    $ret = self::GetClient()->getPrimaryDomain();
                }

                self::$_domain = $ret;
            }

            return self::$_domain;
        }

        public static function GetBrand()
        {
            if (!Zend_Registry::IsRegistered('brand')) {
                $client = self::GetClient();
                Zend_Registry::Set('brand', $client->getBrand());
            }

            return Zend_Registry::Get('brand');
        }

        public static function GetUser()
        {
            if (Zend_Registry::IsRegistered('user')) {
                $user = Zend_Registry::get('user');

                if ($user instanceof Application_User && $user->saved()) {
                    return $user;
                }
            }

            throw new Application_Exception('User not set');
        }

        public static function GetUserHash()
        {
            return Zend_Registry::Get('user_hash');
        }


        public static function GetCpUser()
        {
            if (self::GetSite() == self::SITE_CP) {
                return self::GetUser();
            }
            else if (self::GetSite() == self::SITE_CLIENT) {
                if (Zend_Registry::IsRegistered('user_cp')) {
                    $user = Zend_Registry::get('user_cp');

                    if ($user instanceof Application_User && $user->saved()) {
                        return $user;
                    }
                }
            }

            throw new Exception('User not set');
        }

        public static function ClearClient()
        {
            Zend_Registry::set('client', null);
        }

        public static function GetAclManager()
        {
            return Zend_Registry::get('acl');
        }

        final public static function GetLicenseType($product = null)
        {
            if (!function_exists('ioncube_file_is_encoded') || !ioncube_file_is_encoded()) {
                return Application_License_Manager::TYPE_UNRESTRICTED;
            }

            //Check for ini setting to override the need for licenses
            $config = self::getConfig();
            if ($config instanceof Zend_Config) {
                try {
                    if(isset($config->settings->override_license)){
                        if($config->settings->override_license === md5(sprintf('%s-%s-%s', php_uname('s'), php_uname('n'), 'salt-mie8Phei'))){
                            return Application_License_Manager::TYPE_UNRESTRICTED;
                        }
                    }
                }
                catch(Exception $e){}
            }

            $mainProduct = 'recite-cms-2';
            $product     = trim($product);

            if (strlen($product) == 0) {
                $product = $mainProduct;
            }

            if (!array_key_exists($product, self::$_licenses)) {
                try {
                    if (IN_DEVELOPMENT) {
                        self::$_licenses[$product] = Application_License_Manager::TYPE_DEVELOPER;
                        throw new Exception('No license required');
                    }

                    $paths = Application_License_Manager::GetLicensePathsForClientAndProduct(self::GetClient(), $product);

                    foreach ($paths as $fullpath) {
                        $wasEncrypted = false;
                        $data = ioncube_read_file($fullpath, $wasEncrypted);

                        if (!$wasEncrypted) {
                            continue;
                        }

                        $license = array();
                        foreach (explode("\n", $data) as $line) {
                            $pair = explode('=', $line, 2);
                            if (!is_array($pair) || count($pair) != 2) {
                                continue;
                            }

                            $k = trim($pair[0]);
                            $v = trim($pair[1]);

                            if (strlen($k) > 0 && strlen($v) > 0) {
                                $license[$k] = $v;
                            }
                        }

                        if (coalesce($license['product']) != $product) {
                            continue;
                        }

                        $ts = (int) coalesce($license['expiryts']);

                        if ($ts > 0 && $ts < time()) {
                            self::$_licenses[$product] = new Application_License_Exception_Expired($product, sprintf('License expired %s', date('r', $ts)));
                            throw self::$_licenses[$product];
                        }

                        $type = coalesce($license['type']);

                        if (!Application_License_Manager::IsValidLicenseType($type)) {
                            self::$_licenses[$product] = new Application_License_Exception_Invalid($product, 'Invalid license type');
                            throw self::$_licenses[$product];
                        }

                        self::$_licenses[$product] = $type;
                        throw new Exception('License found');
                    }

                    self::$_licenses[$product] = new Application_License_Exception_NoLicense($product, 'No license found');
                    throw self::$_licenses[$product];
                }
                catch (Exception $ex) {
                }
            }

            $ret = self::$_licenses[$product];

            if ($ret instanceof Exception) {
                throw $ret;
            }

            return $ret;
        }
    }
?>
