<?php
    class Application_Auditor_Entry extends DatabaseObject
    {
        public static $table = 'application_audit_log';

        protected $_user;
        protected $_client;

        public function __construct()
        {
            parent::__construct(Application::GetDb(), self::$table, 'entry_id');

            $this->add('user_id');
            $this->add('user_hash');
            $this->add('role_id');
            $this->add('client_id');
            $this->add('site');
            $this->add('module');
            $this->add('http_user_agent');
            $this->add('remote_addr');
            $this->add('message');
            $this->add('memo', array(), self::TYPE_ARRAY);
            $this->add('ts_created', time(), self::TYPE_TIMESTAMP);
        }

        public function __destruct()
        {
            parent::__destruct();
            unset($this->_user, $this->_client);
        }

        /**
         * Misc methods
         */

        public function __toString()
        {
            return $this->message;
        }

        public function getUser()
        {
            if (!$this->_user instanceof Application_User) {
                $this->_user = new Application_User();
                $this->_user->load($this->user_id);
            }

            return $this->_user;
        }

        public function getClient()
        {
            if (!$this->_client instanceof Application_Client) {
                $this->_client = new Application_Client();
                $this->_client->load($this->client_id);
            }

            return $this->_client;
        }

        public function hasSite()
        {
            return strlen($this->site) > 0;
        }

        public function getSiteLabel()
        {
            if (array_key_exists($this->site, Application::$sites))
                return Application::$sites[$this->site];

            return null;
        }


        public function hasModule()
        {
            return strlen($this->module) > 0;
        }

        public function getModule()
        {
            return Application_Module_Manager::Factory($this->module);
        }

        public function getModuleLabel()
        {
            return (string) $this->getModule();
        }

        /**
         * Static methods
         */

        public function GetEntriesCount($options)
        {
            $select = self::_GetBaseQuery($options);
            $select->from(null, 'count(*)');

            return (int) Application::GetDb()->fetchOne($select);
        }

        public function GetEntries($options)
        {
            $defaults = array(
                'order'  => array('l.entry_id desc'),
                'offset' => 0,
                'limit'  => 0
            );

            $options = ArrayTools::buildOptions($options, $defaults);

            $select = self::_GetBaseQuery($options);
            $select->from(null, 'l.*')
                   ->order($options['order']);

            $options['offset'] = max(0, (int) $options['offset']);
            if ($options['limit'] > 0)
                $select->limit($options['limit'], $options['offset']);

            $db = Application::GetDb();

            $result = $db->query($select);
            $data = array();
            while ($row = $result->fetch())
                $data[] = $row;

            $entries = parent::BuildMultiple($db, __CLASS__, $data);

            self::_LoadUsers($entries);
            self::_LoadClients($entries);

            return $entries;
        }

        private function _LoadUsers($entries)
        {
            $user_ids = array();

            foreach ($entries as $entry)
                $user_ids[$entry->user_id] = $entry->user_id;

            $user_ids = ArrayTools::clean($user_ids, true);

            if (count($user_ids) == 0)
                return;

            $options = array(
                'user_ids' => $user_ids
            );

            $users = Application_User::GetUsers($options);

            foreach ($entries as $entry_id => $entry) {
                if (array_key_exists($entry->user_id, $users))
                    $entry->_user = $users[$entry->user_id];
                else
                    $entry->_user = new Application_User();
            }
        }

        private function _LoadClients($entries)
        {
            $client_ids = array();

            foreach ($entries as $entry)
                $client_ids[$entry->client_id] = $entry->client_id;

            $client_ids = ArrayTools::clean($client_ids, true);

            if (count($client_ids) == 0)
                return;

            $options = array(
                'client_ids' => $client_ids
            );

            $clients = Application_Client::GetClients($options);

            foreach ($entries as $entry_id => $entry) {
                if (array_key_exists($entry->client_id, $clients))
                    $entry->_client = $clients[$entry->client_id];
                else
                    $entry->_client = new Application_Client();
            }
        }


        public function _GetBaseQuery($options)
        {
            $defaults = array(
                'entry_ids'  => array(),
                'user_ids'   => array(),
                'client_ids' => array(),
                'modules'    => array(),
                'sites'      => array()
            );

            $options = ArrayTools::buildOptions($options, $defaults);

            $options['entry_ids']  = ArrayTools::clean($options['entry_ids'], true);
            $options['user_ids']   = ArrayTools::clean($options['user_ids'], true);
            $options['client_ids'] = ArrayTools::clean($options['client_ids'], true);
            $options['modules']    = ArrayTools::clean($options['modules']);
            $options['sites']      = ArrayTools::clean($options['sites']);

            $db = Application::GetDb();
            $select = $db->select();
            $select->from(array('l' => self::$table), array());

            if (count($options['entry_ids']) > 0)
                $select->where('l.entry_id in (?)', $options['entry_ids']);

            if (count($options['user_ids']) > 0)
                $select->where('l.user_id in (?)', $options['user_ids']);

            if (count($options['client_ids']) > 0)
                $select->where('l.client_id in (?)', $options['client_ids']);

            if (count($options['modules']) > 0)
                $select->where('l.module in (?)', $options['modules']);

            if (count($options['sites']) > 0)
                $select->where('l.site in (?)', $options['sites']);

            return $select;
        }

        public function __set($k, $v)
        {
            if ($k == 'client_id') {
                $v = (int) $v;

                if ($v <= 0) {
                    $v = null;
                }
            }

            return parent::__set($k, $v);
        }
    }
?>
