<?php
    /**
     * Application_Auth_UserTracker
     *
     * Used for keeping track of the users that access client web sites (in addition to
     * the regular authentication methods. This class serves two primary purposes:
     * 1) to be aware of a logged-in control panel user accessing the client web site in
     *    order to be able to manage containers. This security should be treated carefully
     *    and shouldn't be relied upon for "important" things (such as actually editing
     *    content or settings)
     * 2) to track anonymous users between sessions. This was originally intended with
     *    shopping cart functionality in mind
     *
     * @author  Quentin Zervaas
     */
    class Application_Auth_UserTracker
    {
        // store the singleton instance
        protected static $_instance = null;
        protected static $_db = null;

        protected static $_session_namespace = 'user_tracker';
        protected static $table = 'users_cp_login_tracking';

        const SESSION_TIMEOUT_MINUTES = 30;

        protected $_unique_user_hash = null;

        protected static $_admin_cookie_name = '__admin';

        private function __construct()
        { }

        public static function getInstance()
        {
            if (null === self::$_instance) {
                self::$_instance = new self();
                self::$_db = Zend_Registry::get('db');
            }

            return self::$_instance;
        }


        /**
         * registerSessionActivity
         *
         * Update the tracker table to indicate a user is active in the control panel
         */
        public function registerSessionActivity($user_id, $client_id)
        {
            // first clean up stale entries
            $this->deleteStaleEntries();

            $remote_addr = HTTP::GetRemoteAddr();
            $remote_user_agent = coalesce($_SERVER['HTTP_USER_AGENT']);

            // now check if there's already a tracked session this user_id/client_id combination
            $login_id = $this->sessionExists($user_id, $client_id, $remote_addr, $remote_user_agent);
            $exists = $login_id > 0;

            // finally, update the existing session or create the new one
            if ($exists) {
                $query = sprintf('update %s set ts_updated = now() where login_id = %d',
                                 self::$table,
                                 $login_id);
            }
            else {
                $query = sprintf("insert into %s (user_id, client_id, remote_addr, remote_user_agent, ts_updated) values (%d, %d, %s, %s, now())",
                                 self::$table,
                                 $user_id,
                                 $client_id,
                                 self::$_db->quote($remote_addr),
                                 self::$_db->quote($remote_user_agent));
            }
            self::$_db->query($query);
        }


        /**
         * deleteEntries
         *
         * Delete all entries (including alive) tracker entries for the given
         * user and client
         */
        public function deleteEntries($user_id = null, $client_id = null)
        {
            $where = array();

            if (!is_null($user_id) && $user_id > 0)
                $where[] = sprintf('user_id = %d', $user_id);

            if (!is_null($client_id) && $client_id > 0)
                $where[] = sprintf('client_id = %d', $client_id);

            if (count($where) > 0) {
                $query = sprintf('delete from %s where %s',
                                 self::$table,
                                 join(' and ', $where));
                self::$_db->query($query);
            }
        }

        public function deleteForUsers($userIds)
        {
            $userIds = ArrayTools::Clean($userIds, true);

            if (count($userIds) == 0) {
                return;
            }

            $query = sprintf(
                'delete from %s where user_id in (%s)',
                self::$table,
                self::$_db->quote($userIds)
            );

            self::$_db->query($query);
        }

        public function deleteForDirectory($directoryId)
        {
            $query = sprintf(
                'delete from %s where user_id in (select user_id from %s where directory_id = %d)',
                self::$table,
                Application_User::$table,
                $directoryId
            );

            self::$_db->query($query);
        }


        /**
         * deleteStaleEntries
         *
         * Delete timed out tracker entries
         */
        public function deleteStaleEntries()
        {
            $helper = self::$_db->getStatementHelper();

            $where = array(
                $helper->getOlderThanExpression('ts_updated', self::SESSION_TIMEOUT_MINUTES, Zend_Date::MINUTE)
            );

            self::$_db->delete(self::$table, $where);
        }


        /**
         * sessionExists
         *
         * Check if a session exists for the given user/client
         */
        public function sessionExists($user_id = null, $client_id = null, $remote_addr = null, $remote_user_agent = null)
        {
            $session = $this->getSession($user_id, $client_id, $remote_addr, $remote_user_agent);

            return $session->login_id > 0;
        }

        public function getSession($user_id = null, $client_id = null, $remote_addr = null, $remote_user_agent = null)
        {
            $db      = Application::GetDb();
            $session = new Zend_Session_Namespace('cp_auth');
            $loginId = (int) $session->login_id;

            if ($loginId > 0) {
                $select = self::$_db->select();
                $select->from(self::$table, '*')
                       ->where('client_id = ?', $client_id)
                       ->where('login_id = ?', $loginId);

                $result = $db->query($select);

                if ($row = $result->fetch()) {
                    return $row;
                }
            }

            $select = self::$_db->select();
            $select->from(self::$table, '*');

            if (!is_null($user_id)) {
                $select->where('user_id = ?', $user_id);
            }

            if (!is_null($client_id)) {
                $select->where('client_id = ?', $client_id);
            }

            if (!is_null($remote_addr)) {
                $select->where('remote_addr = ?', $remote_addr);
            }

            if (!is_null($remote_user_agent)) {
                $select->where('remote_user_agent = ?', $remote_user_agent);
            }

            $result = $db->query($select);

            if ($row = $result->fetch()) {
                $session->login_id = $row->login_id;
                return $row;
            }

            $ret = new stdClass;
            $ret->login_id = 0;

            return $ret;
        }


        /**
         * getUserHash
         *
         * Get a unique user hash for the current user. If one doesn't exist, generate it
         * and write it to the current session (best reliability) and to cookies (second
         * best, used for returning users)
         */
        public function getUserHash(Zend_Controller_Request_Abstract $request)
        {
            if (is_null($this->_unique_user_hash) || strlen($this->_unique_user_hash) == 0) {
                $hash = $this->_getHash($request);

                if (strlen($hash) == 0) {
                    $hash = $this->_generateHash($request);
                    $this->_storeHash($request, $hash);
                }

                $this->_unique_user_hash = md5($hash . $request->getServer('HTTP_USER_AGENT'));
            }
            return $this->_unique_user_hash;
        }

        protected function _getHash($request)
        {
            $session = new Zend_Session_Namespace(self::$_session_namespace);

            if (isset($session->hash) && strlen($session->hash) > 0)
                return $session->hash;

            return $request->getCookie('uid');
        }

        protected function _generateHash($request)
        {
            return uniqid(rand(), true);
        }

        protected function _storeHash($request, $hash)
        {
            $session = new Zend_Session_Namespace(self::$_session_namespace);
            $session->hash = $hash;

            setcookie('uid', $hash, time() + 31536000, '/');
        }

        public static function TryForAdmin()
        {
            if (isset($_GET['admin']) && $_GET['admin']) {
                return true;
            }

            if (isset($_COOKIE[self::$_admin_cookie_name]) && $_COOKIE[self::$_admin_cookie_name]) {
                return true;
            }

            if (isset($_POST[self::$_admin_cookie_name]) && $_POST[self::$_admin_cookie_name]) {
                return true;
            }

            return false;
        }


        /**
         * This method returns an array of roles that the currently logged-in user has.
         * The array has two elements: one called user which holds the user ID, and one
         * called roles, of which each element is a role ID that the user has for the
         * current client
         *
         * @return  array
         */
        public function getControlPanelRoles($client_id, Application_Client_Site_Request $request)
        {
            $tryForAdmin = self::TryForAdmin();

            $close = (bool) $request->getQuery('__close');

            if ($close || !$tryForAdmin) {
                self::ClearAdminCookie();
                return array();
            }


            $session = $this->getSession(null, $client_id, $request->remote_addr, $request->http_user_agent);

            if ($session->login_id > 0) {
                // now find roles for current user / client combination

                $user_id = (int) $session->user_id;

                if ($user_id > 0 && $client_id == $session->client_id) {
                    $options = array(
                        'directory_ids' => null,
                        'user_ids'      => $user_id,
                        'client_ids'    => $client_id
                    );

                    $roles = Application_User_Role::GetRoles($options);

                    $ret = array(
                        'user'  => $user_id,
                        'roles' => ArrayTools::clean(array_keys($roles), true)
                    );

                    if (count($ret['roles']) > 0) {
                        self::SetAdminCookie();
                        return $ret;
                    }
                }
            }

            self::ClearAdminCookie();
            return array();
        }

        public static function SetAdminCookie()
        {
            setcookie(self::$_admin_cookie_name, 1, time() + 3600, '/');
        }

        public static function ClearAdminCookie()
        {
            setcookie(self::$_admin_cookie_name, 0, time() + 3600, '/');
        }
    }
?>