<?php
    class Application_Brand extends DatabaseObject
    {
        static $table = 'brands';
        public $profile;
        protected $_domains;

        protected static $_staticSettings = array();

        public function __construct()
        {
            parent::__construct(Application::GetDb(), self::$table, 'brand_id');

            $this->add('brand_name');
            $this->add('brand_name_url');

            $this->profile = new Application_Brand_Profile($this->db);
        }

        public function __destruct()
        {
            parent::__destruct();
            unset($this->profile);
        }

        /**
         * Callbacks
         */

        protected function postLoad()
        {
            $this->profile->setBrandId($this->getId());
            $this->profile->load();
        }

        protected function preDelete()
        {
            Application_Cp_Tab_Manager::DeleteDefaultTabs($this);
            Application_Module_Manager::ClearModules($this);
            Application_Driver_Manager::ClearDrivers($this);

            $queries = array(
                sprintf('delete from %s_domains where brand_id = %d', $this->getTable(), $this->getId()),
                sprintf('update clients set brand_id = null where brand_id = %d', $this->getId())
            );

            foreach ($queries as $query) {
                $this->db->query($query);
            }

            $event = new Application_Event('/core/brand/predelete');
            $event->set('brand', $this);
            $event->trigger();

            $this->profile->delete();

            return true;
        }

        protected function postInsert()
        {
            Application_Settings_Manager::DeployBrandDefaults($this);

            $this->profile->setBrandId($this->getId());
            $this->profile->save();

            $message = new Application_Auditor_Message('Brands: Brand created');
            $message->addMemo('brand_id', $this->getId())->record();

            $event = new Application_Event('/core/brand/postinsert');
            $event->set('brand', $this);
            $event->trigger();

            return true;
        }

        protected function preUpdate()
        {
            $event = new Application_Event('/core/brand/preupdate');
            $event->set('brand', $this);
            $event->trigger();

            return true;
        }

        protected function postUpdate()
        {
            $this->profile->save();

            $message = new Application_Auditor_Message('Brands: Brand updated');
            $message->addMemo('brand_id', $this->getId())->record();

            $event = new Application_Event('/core/brand/postupdate');
            $event->set('brand', $this);
            $event->trigger();

            return true;
        }

        protected function postDelete()
        {
            $message = new Application_Auditor_Message('Brands: Brand deleted');
            $message->addMemo('brand_id', $this->getDeletedId())->record();

            $event = new Application_Event('/core/brand/postdelete');
            $event->set('id', $this->getDeletedId());
            $event->trigger();

            Application_HttpdConfig::RemoveBrand($this->getDeletedId());

            return true;
        }

        /**
         * Loaders
         */

        public function loadRecordForDomain($domain = '')
        {
            if (strlen($domain) == 0)
                $domain = HTTP_URL::GenerateCurrentURL(true);

            $fields = $this->getSelectFields();

            $select = $this->db->select();
            $select->from(array('b'  => $this->getTable()), $fields)
                   ->joinNatural(array('bd' => 'brands_domains'), array())
                   ->where('lower(url) = lower(?)', $domain);

            if ($this->_load($select))
                return true;

            $domain2 = HTTP_URL::GenerateCurrentURL(true, true);
            if ($domain == $domain2)
                return false;

            $select = $this->db->select();
            $select->from(array('b'  => $this->getTable()), $fields)
                   ->joinNatural(array('bd' => 'brands_domains'), array())
                   ->where('lower(url) = lower(?)', $domain2);

            return $this->_load($select);
        }

        function loadRecordByUrl($url)
        {
            $fields = array_keys($this->properties);
            $query = sprintf("select %s, %s from %s where lower(brand_name_url) = lower(%s)",
                             $this->getIdField(),
                             join(',', $fields),
                             $this->getTable(),
                             $this->db->quote($url));
            return $this->_load($query);
        }

        /**
         * Misc functions
         */

        public function __toString()
        {
            return $this->brand_name;
        }

        public function getClientTemplateData()
        {
            $ret = array(
                'title' => $this->brand_name,
                'url'   => $this->getPrimaryDomain()
            );

            return $ret;
        }

        public function setSetting($name, $val)
        {
            $this->profile->set(Profile_Field::getFormKey($name), $val);
            return $this;
        }

        public function getSetting($name)
        {
            return $this->profile->get(Profile_Field::getFormKey($name));
        }

        public static function GetStaticSetting($brandId, $name)
        {
            if (!array_key_exists($brandId, self::$_staticSettings)) {
                $db = Application::GetDb();
                $select = $db->select();
                $select->from(self::$table . '_profile', array('profile_key', 'profile_value'))
                       ->where('brand_id = ?', $brandId);

                $profile = array();

                foreach ($db->fetchPairs($select) as $k => $v) {
                    $key = Profile_Field::getFormKey($k);

                    $profile[$key] = $v;
                }
                self::$_staticSettings[$brandId] = $profile;
            }

            $key = Profile_Field::getFormKey($name);

            if (array_key_exists($key, self::$_staticSettings[$brandId])) {
                return self::$_staticSettings[$brandId][$key];
            }
            return null;
        }

        /**
         * Domain functions
         */

        public function domainAdd($url)
        {
            $domain = new Application_Brand_Domain();

            if (!$domain->set('url', $url)) {
                return false;
            }

            if ($domain->domainExists($domain->url)) {
                return false;
            }

            $domain->brand_id = $this->getId();

            $ret = $domain->save();

            $this->_domains = null;

            return $ret;
        }

        public function getDomains()
        {
            if (!is_array($this->_domains)) {
                $this->_domains = Application_Brand_Domain::GetDomains(array('brand_ids' => $this->getId()));
            }

            return $this->_domains;
        }

        public function getPrimaryDomain()
        {
            foreach ($this->getDomains() as $domain) {
                return $domain->url;
            }

            return null;
        }

        /**
         * Static methods
         */

        public function GetBrands()
        {
            $db = Application::GetDb();

            $query  = sprintf('select * from %s order by lower(brand_name)', self::$table);
            $result = $db->query($query);
            $data   = array();
            while ($row = $result->fetch()) {
                $data[] = $row;
            }

            $ret = parent::BuildMultiple($db, __CLASS__, $data);
            self::_LoadProfile($ret);

            return $ret;
        }

        private function _LoadProfile(&$arr)
        {
            if (count($arr) == 0) {
                return;
            }

            $db    = Application::GetDb();
            $ids   = array_keys($arr);
            $class = __CLASS__ . '_Profile';

            try {
                $profiles = Profile::BuildMultiple($db, $class, array('brand_id' => $ids));
            }
            catch (Exception $ex) {
                $profiles = array();
            }

            foreach ($arr as $id => $row) {
                if (array_key_exists($id, $profiles) && $profiles[$id] instanceof $class) {
                    $arr[$id]->profile = $profiles[$id];
                }
                else {
                    $arr[$id]->profile = new $class($db, $id);
                }
            }
        }

        /**
         * Getters / setters / checkers
         */

        public function check($k, $v)
        {
            if (!parent::check($k, $v))
                return false;

            if ($k == 'brand_name')
                return strlen(trim($v)) > 0;

            return true;
        }

        public function __set($k, $v)
        {
            switch ($k) {
                case 'brand_name':
                    $this->brand_name_url = $v;
                    break;
                case 'brand_name_url':
                    $v = Text::urlize($v, '-.');
                    break;
            }

            parent::__set($k, $v);
        }
    }
?>
