<?php
    class Application_Brand_Domain extends DatabaseObject
    {
        static $table = 'brands_domains';

        public function __construct()
        {
            parent::__construct(Application::GetDb(), self::$table, 'domain_id');

            $this->add('brand_id');
            $this->add('url');
            $this->add('ranking');
        }

        public function __destruct()
        {
            parent::__destruct();
        }

        public function preInsert()
        {
            $select = $this->db->select();
            $select->from(self::$table, new Zend_Db_Expr('max(coalesce(ranking, 0)) + 1'))
                   ->where('brand_id = ?', $this->brand_id);

            $this->ranking = (int) $this->db->fetchOne($select);

            return true;
        }

        public function postInsert()
        {
            $message = new Application_Auditor_Message('Brands: Domain added to brand');
            $message->addMemo('brand_id', $this->brand_id)
                    ->addMemo('url', $this->url)
                    ->record();
            return true;
        }

        public function postUpdate()
        {
            $message = new Application_Auditor_Message('Brands: Domain updated for brand');
            $message->addMemo('brand_id', $this->brand_id)
                    ->addMemo('url', $this->url)
                    ->record();
            return true;
        }

        public function postDelete()
        {
            $message = new Application_Auditor_Message('Brands: Domain deleted from brand');
            $message->addMemo('brand_id', $this->brand_id)
                    ->addMemo('url', $this->url)
                    ->record();
            return true;
        }

        public function domainExists($url)
        {
            $query = sprintf("select count(*) as num_hits from %s where url = %s",
                             $this->getTable(),
                             $this->db->quote($url));

            if ($this->saved())
                $query .= sprintf('and %s != %d', $this->getIdField(), $this->getId());

            return (int) $this->db->fetchOne($query);
        }

        public function loadRecordForBrand($brand_id, $id)
        {
            if ($brand_id == 0 || $id == 0)
                return false;

            $brand_id = (int) $brand_id;
            $id = (int) $id;

            $query = sprintf('select %s from %s where brand_id = %d and %s = %d',
                             join(', ', $this->getSelectFields()),
                             $this->getTable(),
                             $brand_id,
                             $this->getIdField(),
                             $id);

            return $this->_load($query);
        }

        /**
         * Static methods
         */

        public static function GetDomains($options)
        {
            $select = self::_GetBaseQuery($options);
            $select->from(null, 'd.*')
                   ->order('d.ranking');

            $db = Application::GetDb();
            $result = $db->query($select);

            $data = array();
            while ($row = $result->fetch()) {
                $data[] = $row;
            }

            $domains = parent::BuildMultiple($db, __CLASS__, $data);

            return $domains;
        }

        public static function GetDomainsCount($options)
        {
            $select = self::_GetBaseQuery($options);
            $select->from(null, 'count(*)');

            return $db->fetchOne($select);
        }

        protected static function _GetBaseQuery($options)
        {
            $defaults = array(
                'domain_ids' => array(),
                'brand_ids'  => array()
            );

            $options = ArrayTools::BuildOptions($options, $defaults);

            $options['domain_ids'] = ArrayTools::clean($options['domain_ids'], true);
            $options['brand_ids']  = ArrayTools::clean($options['brand_ids'], true);

            $db = Application::GetDb();

            $select = $db->select();
            $select->from(array('d' => self::$table), array());

            if (count($options['domain_ids']) > 0) {
                $select->where('d.domain_id in (?)', $options['domain_ids']);
            }

            if (count($options['brand_ids']) > 0) {
                $select->where('d.brand_id in (?)', $options['brand_ids']);
            }

            return $select;
        }

        public static function ProcessSortForm(Application_Brand $brand, Zend_Controller_Request_Abstract $request)
        {
            $ids     = ArrayTools::Clean($request->getPost('domain'), true);
            $domains = $brand->getDomains();

            if (count($ids) != count($domains)) {
                throw new Exception('Invalid data');
            }

            $queries = array();
            $ranking = 0;

            foreach ($ids as $id) {
                if (!array_key_exists($id, $domains)) {
                    throw new Exception('Invalid data');
                }

                $queries[] = sprintf('update %s set ranking = %d where domain_id = %d', self::$table, ++$ranking, $id);
            }

            if (count($queries) == 0) {
                return;
            }

            $db = Application::GetDb();
            $db->begin();

            foreach ($queries as $query) {
                $db->query($query);
            }

            $db->commit();
        }

        /**
         * Getters / setters / checkers
         */

        public function check($k, $v)
        {
            if (!parent::check($k, $v));

            if ($k == 'url')
                return HTTP_URL::isValidUrl($v);

            return true;
        }

        public function __set($k, $v)
        {
            if ($k == 'url')
                $v = HTTP_URL::NormalizeURL($v, true);

            return parent::__set($k, $v);
        }
    }
?>
