<?php
    class Application_Client extends DatabaseObject
    {
        const STATUS_ACTIVE   = 'A';
        const STATUS_INACTIVE = 'I';
        const STATUS_LOCKED   = 'L'; // not inactive but users can't log in
        const STATUS_DISABLED = 'D'; //front-end inactive, but users can log in to control panel

        public static $statuses = array(
            self::STATUS_ACTIVE   => 'Active',
            self::STATUS_INACTIVE => 'Inactive',
            self::STATUS_LOCKED   => 'Locked',
            self::STATUS_DISABLED => 'Disabled'
        );

        public static $table = 'clients';
        public $profile;

        protected $_autoInstallSiteName    = null;
        protected $_autoInstallDefaultSite = true;
        protected $_autoDeployPermissions  = true;

        protected $_brand;
        protected $_domains;
        protected $_roles;
        protected $_cache;
        protected $_modules;

        private $_deleteEnabled = false;

        public function __construct()
        {
            parent::__construct(Application::GetDb(), self::$table, 'client_id');

            $this->add('brand_id');
            $this->add('status', self::STATUS_ACTIVE);
            $this->add('client_name');
            $this->add('ts_created', time(), self::TYPE_TIMESTAMP);

            $this->profile = new Application_Client_Profile($this->db);
        }

        public function __destruct()
        {
            parent::__destruct();
            unset($this->profile, $this->_brand, $this->_domains, $this->_roles, $this->_cache, $this->_modules);
        }

        /**
         * Callbacks
         */

        protected function postLoad()
        {
            $this->profile->setClientId($this->getId());
            $this->profile->load();
        }


        protected function postInsert()
        {
            $this->profile->setClientId($this->getId());
            $this->profile->save();

            Application::SetClient($this);

            // deploy default modules and drivers as per brand settings
            $brand = $this->getBrand();

            foreach (Application_Driver_Manager::GetDrivers($brand) as $driver) {
                Application_Driver_Manager::AssignDriver($this, $driver);
            }

            Application_Module_Manager::AssignModules($this, Application_Module_Manager::GetModules($brand), true);

            if ($this->_autoDeployPermissions) {
                $pm = Application_User_Permissions_Manager::GetInstance();
                $pm->deployDefaults($this);
            }

            Application_Auditor::Record('Clients: Client created', $this->getId());

            $event = new Application_Event('/core/client/postinsert');
            $event->set('client', $this);
            $event->trigger();

            return true;
        }

        public function setAutoDeployPermissions($bool)
        {
            $this->_autoDeployPermissions = (bool) $bool;
        }

        public function enableAutoInstall($bool)
        {
            $this->_autoInstallDefaultSite = (bool) $bool;
        }

        public function setAutoInstallSite($name)
        {
            $this->_autoInstallSiteName = $name;
        }

        public function autoInstallDefaultSite()
        {
            if (!$this->_autoInstallDefaultSite) {
                throw new Exception('Installing default site is disabled');
            }

            $brand = $this->getBrand();

            $dumpName = $this->_autoInstallSiteName;

            if (strlen($dumpName) == 0) {
                $dumpName = trim($brand->getSetting('client.default_site.auto_install'));
            }

            if (strlen($dumpName) == 0) {
                throw new Exception('Dump not specified');
            }

            $dumpPath = sprintf('%s/%s', Module_Backup_Dump::GetLibraryDumpsPath(), $dumpName);

            $options = new Module_Backup_Importer_Options();
            $options->setDump(new Module_Backup_Dump($dumpPath))
                    ->setClient($this)
                    ->setImportCustomSettings(true)
                    ->setCustomOption('autoUpdateTemplates', false);

            $importer = new Module_Backup_Importer();

            return $importer->run($options);
        }

        protected function preDelete()
        {
            // we're only allowing client data to be deleting has specifically been enabled

            if (!$this->_deleteEnabled) {
                return false;
            }

            Application::SetClient($this);
            Application_Driver_Manager::DisableAccessChecks(true);
            Application_Auditor::Disable();

            $event = new Application_Event('/core/client/predelete');
            $event->set('client', $this)
                  ->verbose()
                  ->trigger();

            $this->profile->delete();

            Application_Auditor::Enable();

            // clear pending maintenance actions. This was in preDelete event but can be reliable there.
            Application_Maintenance::ClearForClient($this->getId());

            return true;
        }

        public function enableDelete($flag)
        {
            $this->_deleteEnabled = $flag;
        }

        protected function postUpdate()
        {
            $this->profile->save();

            Application_Auditor::Record('Clients: Client updated', $this->getId());

            $event = new Application_Event('/core/client/postupdate');
            $event->set('client', $this);
            $event->trigger();

            return true;
        }

        protected function preUpdate()
        {
            $event = new Application_Event('/core/client/preupdate');
            $event->set('client', $this);
            $event->trigger();

            return true;
        }

        protected function postDelete()
        {
            Application_Auditor::Disable();

            $event = new Application_Event('/core/client/postdelete');
            $event->set('id', $this->getDeletedId())
                  ->verbose(true)
                  ->trigger();

            Application_Auditor::Enable();

            $msg = new Application_Auditor_Message('Clients: Client deleted');
            $msg->addMemo('client_name', $this->client_name)
                ->record();

            Application_Driver_Manager::DisableAccessChecks(false);

            $fullpath = sprintf('%s/clients/%d', Application::GetDataDir(), $this->getDeletedId());

            if (file_exists($fullpath)) {
                File::RmdirRecursive($fullpath);
            }

            return true;
        }

        /**
         * Loaders
         */


        // @todo    figure out why this is happy to match only a portion of the string
        public function loadRecordForDomain($domain = '', $dav = false)
        {
            if (strlen($domain) == 0)
                $domain = HTTP_URL::GenerateCurrentURL(true);

            $select = $this->db->select();
            $select->from(array('c' => $this->getTable()), $this->getSelectFields())
                   ->joinInner(array('cd' => $this->getTable() . '_domains'), 'c.client_id = cd.client_id', array())
                   ->where('lower(url) = substring(lower(?), 1, length(url))', $domain)
                   ->where('coalesce(is_dav, false) = ?', new Zend_Db_Expr($dav ? 'true' : 'false'));


            if ($this->_load($select)) {
                return true;
            }

            $domain = HTTP_URL::GenerateCurrentURL(true, true);

            $select = $this->db->select();
            $select->from(array('c' => $this->getTable()), $this->getSelectFields())
                   ->joinInner(array('cd' => $this->getTable() . '_domains'), 'c.client_id = cd.client_id', array())
                   ->where('lower(url) = substring(lower(?), 1, length(url))', $domain)
                   ->where('coalesce(is_dav, false) = ?', new Zend_Db_Expr($dav ? 'true' : 'false'));

            return $this->_load($select);
        }

        public function loadRecordForDavDomain($domain = '')
        {
            return $this->loadRecordForDomain($domain, true);
        }

        /**
         * Misc functions
         */

        public function __toString()
        {
            return (string) $this->client_name;
        }

        public function setBrand(Application_Brand $brand)
        {
            $this->_brand = $brand;
            $this->brand_id = $brand->getId();
            return $this;
        }

        public function getBrand()
        {
            if (!$this->_brand instanceof Application_Brand) {
                $this->_brand = new Application_Brand();
                $this->_brand->load($this->brand_id);
            }

            return $this->_brand;
        }

        public function getModules()
        {
            return Application_Module_Manager::GetModules($this);
        }

        public function hasModule($module)
        {
            if ($module instanceof Application_Module_Interface) {
                $module = $module->getName();
            }

            return array_key_exists($module, $this->getModules());
        }

        /**
         * Domain functions
         */

        public function domainAdd($url, $dav = false)
        {
            $domain = new Application_Client_Domain();
            if (!$domain->set('url', $url)) {
                throw new Exception('Invalid URL');
            }

            if ($domain->domainExists($domain->url)) {
                throw new Exception('Domain already in use: ' . $domain->url);
            }

            $domain->client_id = $this->getId();

            if ($dav) {
                $domain->is_dav = true;
            }

            $domain->save();

            $this->_domains = null;
        }

        public function domainAddDav($url)
        {
            return $this->domainAdd($url, true);
        }

        public function getAllDomains()
        {
            if (!is_array($this->_domains)) {
                $this->_domains = Application_Client_Domain::GetDomains(array('client_ids' => $this->getId()));
            }

            return $this->_domains;
        }

        public function getDomains()
        {
            $ret = array();

            foreach ($this->getAllDomains() as $domain) {
                if (!$domain->isDav()) {
                    $ret[$domain->getId()] = $domain;
                }
            }

            return $ret;
        }

        public function getDavDomains()
        {
            $ret = array();

            foreach ($this->getAllDomains() as $domain) {
                if ($domain->isDav()) {
                    $ret[$domain->getId()] = $domain;
                }
            }

            return $ret;
        }

        public function deleteDomains()
        {
            foreach ($this->getAllDomains() as $domain_id => $domain) {
                $domain->delete();
            }

            $this->_domains = array();
        }

        public function getPrimaryDomain($force = null)
        {
            $primary = null;

            foreach ($this->getDomains() as $domain) {
                if (strlen($force) > 0 && $domain->url == $force) {
                    return $force;
                }

                if (strlen($primary) == 0) {
                    $primary = $domain->url;
                }
            }

            return $primary;
        }

        public function getControlPanelDomain()
        {
            $brand = $this->getBrand();

            return $brand->getPrimaryDomain();
        }

        public function getClientTemplateData()
        {
            $ret = array(
                'id'    => $this->getId(),
                'title' => $this->client_name
            );

            return $ret;
        }

        public function getRoles()
        {
            if (!is_array($this->_roles)) {
                $options = array(
                    'client_ids'    => $this->getId(),
                    'directory_ids' => null
                );

                $this->_roles = Application_User_Role::GetRoles($options);
            }

            return $this->_roles;
        }

        public function getDataCache()
        {
            if (!$this->_cache instanceof Zend_Cache_Core) {
                $options = new Application_Cache_Options();
                $options->setMaxLifetime(3600);

                $this->_cache = Application_Cache::GetCache('_clientdata', null, $options);
            }

            return $this->_cache;
        }

        /**
         * @return  Profile_Field_FieldList_Field
         */
        public function getCustomSetting($name)
        {
            $settings = Application_Client_Settings_Custom::GetSettings();

            foreach ($settings as $setting) {
                if ($setting->setting_key == $name) {
                    return $setting->getField();
                }
            }

            throw new Exception('Invalid setting');
        }

        public function getCustomSettings()
        {
            $settings = Application_Client_Settings_Custom::GetSettings();

            $fieldList = new Profile_Field_FieldList();

            foreach ($settings as $setting) {
                $fieldList->add($setting->getField());
            }

            return $fieldList;
        }

        public function setSetting($name, $val)
        {
            $this->profile->set(Profile_Field::getFormKey($name), $val);
            return $this;
        }

        public function getSetting($name, $defaultIfEmpty = false)
        {
            $ret = $this->profile->get(Profile_Field::getFormKey($name));

            if (empty($ret) && $defaultIfEmpty) {
                // find the setting name in the configs and return the default value
                try {
                    $ret = Application_Settings_Manager::GetDefaultForSetting($this, $name);
                }
                catch (Exception $ex) {
                    $ret = null;
                }
            }

            return $ret;
        }

        public function getStatusLabel()
        {
            if (strlen($this->status) == 0) {
                $this->status = self::STATUS_ACTIVE;
            }

            if (array_key_exists($this->status, self::$statuses)) {
                return self::$statuses[$this->status];
            }

            return '';
        }

        public function setStatus($status)
        {
            if (!array_key_exists($status, self::$statuses)) {
                throw new Exception('Invalid status');
            }

            $this->status = $status;
            return $this;
        }

        public function isActive()
        {
            if (strlen($this->status) == 0) {
                $this->status = self::STATUS_ACTIVE;
            }

            return $this->status == self::STATUS_ACTIVE;
        }

        public function isLocked()
        {
            return $this->status == self::STATUS_LOCKED;
        }

        public function isDisabled()
        {
            return $this->status == self::STATUS_DISABLED;
        }

        /**
         * Static methods
         */

        public function GetClientIds()
        {
            $query = sprintf('select client_id from clients');

            return Application::GetDb()->fetchCol($query);
        }

        public static function GetClientsCount($options)
        {
            $select = self::_GetBaseQuery($options);
            $select->from(null, 'count(*)');

            return (int) Application::GetDb()->fetchOne($select);
        }

        public static function GetClients($options = array())
        {
            $defaults = array(
                'order'  => array('lower(c.client_name)', 'c.client_id'),
                'offset' => 0,
                'limit'  => 0
            );

            $options = ArrayTools::BuildOptions($options, $defaults);

            $db = Application::GetDb();

            $select = self::_GetBaseQuery($options);
            $select->from(null, array('c.*'))
                   ->order($options['order']);

            $options['offset'] = max(0, (int) $options['offset']);
            if ($options['limit'] > 0)
                $select->limit($options['limit'], $options['offset']);

            $result = $db->query($select);

            $data = array();
            while ($row = $result->fetch())
                $data[] = $row;

            $clients = parent::BuildMultiple($db, __CLASS__, $data);
            self::_LoadProfile($clients);
            self::_LoadDomains($clients);

            return $clients;
        }

        public static function GetBrandlessClients()
        {
            $select = self::_GetBaseQuery();
            $select->from(null, array('c.*'))
                   ->where('brand_id is null')
                   ->order('lower(client_name)');

            $db = Application::GetDb();
            $result = $db->query($select);

            $data = array();
            while ($row = $result->fetch()) {
                $data[] = $row;
            }

            return parent::BuildMultiple($db, __CLASS__, $data);
        }

        public static function GetProfilingClients()
        {
            $db = Application::GetDb();

            $select = $db->select();
            $select->from(self::$table . '_profile', 'client_id')
                   ->where("profile_key = 'website.profiling'")
                   ->where("profile_value = '1'");

            $client_ids = $db->fetchCol($select);

            if (count($client_ids) == 0)
                return array();

            $options = array(
                'client_ids' => $client_ids
            );

            $select = self::_GetBaseQuery($options);
            $select->from(null, array('c.*'))
                   ->order('lower(client_name)');

            $db = Application::GetDb();

            $result = $db->query($select);

            $data = array();
            while ($row = $result->fetch()) {
                $data[] = $row;
            }

            return parent::BuildMultiple($db, __CLASS__, $data);
        }

        private static function _LoadDomains(&$arr)
        {
            if (count($arr) == 0) {
                return;
            }

            foreach ($arr as $k => $v)
                $arr[$k]->_domains = array();

            $options = array('client_ids' => array_keys($arr));
            $domains = Application_Client_Domain::GetDomains($options);

            foreach ($domains as $k => $v) {
                $arr[$v->client_id]->_domains[$k] = $v;
            }
        }

        protected function _GetBaseQuery($options = array())
        {
            $defaults = array(
                'client_ids' => array(),
                'brand_ids'  => array(),
                'filters'    => array(),
                'statuses'   => array()
            );

            $options = ArrayTools::BuildOptions($options, $defaults);

            $options['client_ids'] = ArrayTools::clean($options['client_ids'], true);
            $options['brand_ids'] = ArrayTools::clean($options['brand_ids'], true);
            $options['filters'] = ArrayTools::clean($options['filters']);
            $options['statuses'] = ArrayTools::clean($options['statuses']);

            $db = Application::GetDb();

            $select = $db->select();
            $select->from(array('c' => self::$table), array());

            if (count($options['client_ids']) > 0) {
                $select->where('c.client_id in (?)', $options['client_ids']);
            }

            if (count($options['brand_ids']) > 0) {
                $select->where('c.brand_id in (?)', $options['brand_ids']);
            }

            if (count($options['statuses']) > 0) {
                $select->where("coalesce(c.status, 'A') in (?)", $options['statuses']);
            }

            foreach ($options['filters'] as $filter) {
                $select->where($filter);
            }

            return $select;
        }

        private function _LoadProfile(&$arr)
        {
            if (count($arr) == 0) {
                return;
            }

            $db    = Application::GetDb();
            $ids   = array_keys($arr);
            $class = __CLASS__ . '_Profile';

            try {
                $profiles = Profile::BuildMultiple($db, $class, array('client_id' => $ids));
            }
            catch (Exception $ex) {
                $profiles = array();
            }

            foreach ($arr as $id => $row) {
                if (array_key_exists($id, $profiles) && $profiles[$id] instanceof $class) {
                    $arr[$id]->profile = $profiles[$id];
                }
                else {
                    $arr[$id]->profile = new $class($db, $id);
                }
            }
        }

        /**
         * Getters / setters / checkers
         */

        public function check($k, $v)
        {
            if (!parent::check($k, $v))
                return false;

            if ($k == 'client_name')
                return strlen(trim($v)) > 0;

            return true;
        }
    }
?>
