<?php
    class Application_Client_Domain extends DatabaseObject
    {
        public static $table = 'clients_domains';

        public function __construct()
        {
            parent::__construct(Application::GetDb(), self::$table, 'domain_id');

            $this->add('client_id');
            $this->add('url');
            $this->add('is_dav', false, self::TYPE_BOOLEAN);
            $this->add('ranking');
        }

        public function preInsert()
        {
            $select = $this->db->select();
            $select->from(self::$table, new Zend_Db_Expr('max(coalesce(ranking, 0)) + 1'))
                   ->where('client_id = ?', $this->client_id)
                   ->where('is_dav = ?', new Zend_Db_Expr($this->is_dav ? 'true' : 'false'));

            $this->ranking = (int) $this->db->fetchOne($select);

            return true;
        }

        public function postInsert()
        {
            $msg = new Application_Auditor_Message('Clients: Domain added');
            $msg->setClientId($this->client_id)
                ->addMemo('url', $this->url)
                ->record();

            $event = new Application_Event('/core/client/domain/postinsert');
            $event->set('domain', $this);
            $event->trigger();

            $this->clearDomainCache();

            return true;
        }

        public function preUpdate()
        {
            $event = new Application_Event('/core/client/domain/preupdate');
            $event->set('domain', $this);
            $event->trigger();

            return true;
        }

        public function postUpdate()
        {
            $msg = new Application_Auditor_Message('Clients: Domain updated');
            $msg->setClientId($this->client_id)
                ->addMemo('url', $this->url)
                ->record();

            $event = new Application_Event('/core/client/domain/postupdate');
            $event->set('domain', $this);
            $event->trigger();

            $this->clearDomainCache();

            return true;
        }

        public function preDelete()
        {
            $event = new Application_Event('/core/client/domain/predelete');
            $event->set('domain', $this);
            $event->trigger();

            return true;
        }

        public function postDelete()
        {
            $msg = new Application_Auditor_Message('Clients: Domain deleted');
            $msg->setClientId($this->client_id)
                ->addMemo('url', $this->url)
                ->record();

            $event = new Application_Event('/core/client/domain/postdelete');
            $event->set('id', $this->getDeletedId())
                  ->set('url', $this->url)
                  ->set('dav', $this->is_dav)
                  ->trigger();

            $this->clearDomainCache();

            return true;
        }

        public function domainExists($url)
        {
            $query = sprintf("select count(*) as num_hits from %s where lower(url) = lower(%s)",
                             $this->getTable(),
                             $this->db->quote($url));

            if ($this->saved())
                $query .= sprintf(' and %s != %d', $this->getIdField(), $this->getId());

            return $this->db->fetchOne($query) > 0;
        }

        public function loadRecordForClient($client_id, $id=NULL)
        {
            if ( empty($client_id) || empty( $id ) ){
                return false;
            }

            $this->set('client_id', $client_id);
            return parent::loadRecordForClient($id);
        }

        public function __toString()
        {
            return $this->url;
        }

        public function isDav()
        {
            return (bool) $this->is_dav;
        }

        public function clearDomainCache()
        {
            $cache = Application_Cache::GetCache('domains');
            $cache->clean();
        }

        /**
         * Static functions
         */

        public static function GetClientIdAndDomainIdForCurrentDomain($dav = false)
        {
            $db      = Application::GetDb();
            $cache   = Application_Cache::GetCache('domains');
            $domain  = HTTP_URL::GenerateCurrentURL(true);
            $cacheId = md5($domain);

            $clientId = (int) $cache->load('client_' . $cacheId);
            $domainId = (int) $cache->load('domain_' . $cacheId);

            if ($clientId > 0 && $domainId > 0) {
                // do a final validation
                $select = $db->select();
                $select->from(self::$table, 'count(*)')
                       ->where('client_id = ?', $clientId)
                       ->where('domain_id = ?', $domainId);

                if ($db->fetchOne($select) > 0) {
                    return array(
                        'client' => $clientId,
                        'domain' => $domainId
                    );
                }
            }

            $clientId = 0;
            $domainId = 0;

            $db = Application::GetDb();

            $select = $db->select();
            $select->from(self::$table, array('domain_id', 'client_id'))
                   ->where('lower(url) = substring(lower(?), 1, length(url))', $domain)
                   ->where('coalesce(is_dav, false) = ?', new Zend_Db_Expr($dav ? 'true' : 'false'));

            $row = $db->fetchRow($select);

            if (is_array($row)) {
                $clientId = (int) $row['client_id'];
                $domainId = (int) $row['domain_id'];
            }

            if ($clientId <= 0 || $domainId <= 0) {
                $domain = HTTP_URL::GenerateCurrentURL(true, true);

                $select = $db->select();
                $select->from(self::$table, array('domain_id', 'client_id'))
                       ->where('lower(url) = substring(lower(?), 1, length(url))', $domain)
                       ->where('coalesce(is_dav, false) = ?', new Zend_Db_Expr($dav ? 'true' : 'false'));

                $row = $db->fetchRow($select);

                if (is_array($row)) {
                    $clientId = (int) $row['client_id'];
                    $domainId = (int) $row['domain_id'];
                }
            }

            if ($clientId <= 0 || $domainId <= 0) {
                throw new Exception('Unable to determine client ID');
            }

            $cache->save($clientId, 'client_' . $cacheId);
            $cache->save($domainId, 'domain_' . $cacheId);

            return array(
                'client' => $clientId,
                'domain' => $domainId
            );
        }

        public static function GetDomains($options)
        {
            $select = self::_GetBaseQuery($options);
            $select->from(null, 'd.*')
                   ->order('d.ranking');

            $db = Application::GetDb();
            $result = $db->query($select);

            $data = array();
            while ($row = $result->fetch()) {
                $data[] = $row;
            }

            return parent::BuildMultiple($db, __CLASS__, $data);
        }

        public static function GetDomainsCount($options)
        {
            $select = self::_GetBaseQuery($options);
            $select->from(null, 'count(*)');

            return (int) Application::GetDb()->fetchOne($select);
        }

        protected static function _GetBaseQuery($options)
        {
            $defaults = array(
                'domain_ids' => array(),
                'client_ids' => array()
            );

            $options = ArrayTools::BuildOptions($options, $defaults);

            $options['domain_ids'] = ArrayTools::clean($options['domain_ids'], true);
            $options['client_ids'] = ArrayTools::clean($options['client_ids'], true);

            $db = Application::GetDb();

            $select = $db->select();
            $select->from(array('d' => self::$table), array());

            if (count($options['domain_ids']) > 0) {
                $select->where('d.domain_id in (?)', $options['domain_ids']);
            }

            if (count($options['client_ids']) > 0) {
                $select->where('d.client_id in (?)', $options['client_ids']);
            }

            return $select;
        }

        public static function ProcessSortForm(Application_Client $client, Zend_Controller_Request_Abstract $request)
        {
            $ids = ArrayTools::Clean($request->getPost('domain'), true);

            if (count($ids) > 0) {
                $domains = $client->getDomains();
            }
            else {
                $ids = ArrayTools::Clean($request->getPost('davdomain'), true);

                if (count($ids) == 0) {
                    return;
                }

                $domains = $client->getDavDomains();
            }

            if (count($ids) != count($domains)) {
                throw new Exception('Invalid data');
            }

            $queries = array();
            $ranking = 0;

            foreach ($ids as $id) {
                if (!array_key_exists($id, $domains)) {
                    throw new Exception('Invalid data');
                }

                $queries[] = sprintf('update %s set ranking = %d where domain_id = %d', self::$table, ++$ranking, $id);
            }

            if (count($queries) == 0) {
                return;
            }

            $db = Application::GetDb();
            $db->begin();

            foreach ($queries as $query) {
                $db->query($query);
            }

            $db->commit();
        }

        /**
         * Getters, setters and checkers
         */

        public function check($k, $v)
        {
            if (!parent::check($k, $v));

            if ($k == 'url')
                return HTTP_URL::IsValidUrl($v);

            return true;
        }

        public function __set($k, $v)
        {
            if ($k == 'url')
                $v = HTTP_URL::NormalizeURL($v, true);

            return parent::__set($k, $v);
        }
    }
?>
