<?php
    /**
     * @author  Quentin Zervaas <quentin@recite.com.au>
     * @since   Recite 1.7.0
     */
    class Application_Client_Settings_Custom extends DatabaseObject
    {
        public static $table = 'clients_customsettings';
        protected $_type;

        protected static $_get_settings = array();

        protected static $_clientTemplateData = array();

        public function __construct()
        {
            parent::__construct(Application::GetDb(), self::$table, 'setting_id');

            $this->add('client_id', getClientId());
            $this->add('title');
            $this->add('setting_key');
            $this->add('setting_type');
            $this->add('setting_value');
            $this->add('params', array(), self::TYPE_ARRAY);
        }

        public function __destruct()
        {
            parent::__destruct();
            unset($this->_type);
        }

        /**
         * Loaders
         */

        public function load($id, $field = NULL)
        {
            return $this->loadRecordForClient($id);
        }

        /**
         * Callbacks
         */

        protected function postInsert()
        {
            $message = new Application_Auditor_Message('Custom setting created');
            $message->addMemo($this->getIdField(), $this->getId())
                    ->addMemo('client_id', $this->client_id)
                    ->addMemo('type', $this->getType())
                    ->record();

            return true;
        }

        protected function postUpdate()
        {
            $message = new Application_Auditor_Message('Custom setting updated');
            $message->addMemo($this->getIdField(), $this->getId())
                    ->addMemo('client_id', $this->client_id)
                    ->addMemo('type', $this->getType())
                    ->record();

            return true;
        }

        protected function postDelete()
        {
            $message = new Application_Auditor_Message('Custom setting deleted');
            $message->addMemo($this->getIdField(), $this->getDeletedId())
                    ->addMemo('client_id', $this->client_id)
                    ->addMemo('type', $this->getType())
                    ->record();

            return true;
        }

        /**
         * Misc methods
         */

        public function __toString()
        {
            return (string) $this->title;
        }

        public function setType(Profile_Field $type)
        {
            $this->_type = $type;
            $this->setting_type = $type->getType();
        }

        public function getType()
        {
            if (!$this->_type instanceof Profile_Field) {
                $this->_type = Profile_Field::Factory($this->setting_type);
                $this->_type->set($this->setting_value);
            }

            return $this->_type;
        }

        public function check($k, $v)
        {
            if ($k == 'setting_key') {
                if (strlen($v) == 0) {
                    return false;
                }

                return preg_match('/^[a-z0-9_]+$/i', $v);
            }

            return parent::check($k, $v);
        }

        public function getField()
        {
            $ret = new Profile_Field_FieldList_Field($this->getType());
            $ret->setName($this->setting_key)
                ->setTitle($this->title);

            return $ret;
        }

        /**
         * Static functions
         */

        public static function GetBundle()
        {
            $ret = new Application_Settings_Bundle();

            $section = new Application_Settings_Section('Custom Settings');

            foreach (self::GetSettings() as $setting) {
                $obj = new Application_Settings_Setting($setting->setting_key);

                $obj->setType($setting->setting_type)
                    ->setDefault($setting->setting_value)
                    ->setLabel($setting->title)
                    ->setTypeParam('setting_id', $setting->getId());

                $section->add($obj);
            }

            $ret->addSection($section);

            return $ret;
        }

        public static function ClearForClient($client_id)
        {
            $query = sprintf('delete from %s where client_id = %d', self::$table, $client_id);
            Application::GetDb()->query($query);
        }

        public static function BuildFromField(Profile_Field_FieldList_Field $field, $setValue = false)
        {
            $ret = new self();
            $ret->title        = $field->getTitle();
            $ret->setting_key  = $field->getName();
            $ret->setting_type = $field->getType()->getType();
            $ret->params       = $field->getParams();

            if ($setValue) {
                $ret->setting_value = $field->get();
            }

            return $ret;
        }

        public static function SetFromFieldList(Profile_Field_FieldList $fieldList)
        {
            $db = Application::GetDb();
            $db->begin();

            foreach (self::GetSettings() as $setting) {
                foreach ($fieldList as $field) {
                    if ($field->getName() == $setting->setting_key) {
                        $setting->setting_value = $field->get();

                        if ($setting->updated('setting_value')) {
                            $setting->save();
                        }

                        continue;
                    }
                }
            }

            $db->commit();
        }

        public static function GetClientTemplateData()
        {
            $clientId = getClientId();
            if (!array_key_exists($clientId, self::$_clientTemplateData)) {
                $settings = self::GetSettings();

                $fields = new Profile_Field_FieldList();

                foreach ($settings as $setting) {
                    $fields->add($setting->getField());
                }

                self::$_clientTemplateData[$clientId] = $fields->getClientTemplateData();
            }

            return self::$_clientTemplateData[$clientId];
        }

        public static function GetSettingsCount($options = array())
        {
            $select = self::_GetBaseQuery($options);
            $select->from(null, 'count(*)');

            return Application::GetDb()->fetchOne($select);
        }

        public static function GetSettings($options = array())
        {
            $defaults = array(
                'order'  => 'lower(s.setting_key)',
                'offset' => 0,
                'limit'  => 0
            );

            $options = ArrayTools::BuildOptions($options, $defaults);

            $clientId = getClientId();

            $hash = $clientId . ',' . md5(serialize($options));

            if (!array_key_exists($hash, self::$_get_settings)) {
                $select = self::_GetBaseQuery($options);
                $select->from(null, 's.*')
                       ->order($options['order']);

                $options['offset'] = max(0, (int) $options['offset']);
                if ($options['limit'] > 0) {
                    $select->limit($options['limit'], $options['offset']);
                }

                $db = Application::GetDb();

                $result = $db->query($select);

                $data = array();
                while ($row = $result->fetch()) {
                    $data[] = $row;
                }

                self::$_get_settings[$hash] = parent::BuildMultiple($db, __CLASS__, $data);
            }

            return self::$_get_settings[$hash];
        }

        protected static function _GetBaseQuery($options)
        {
            $defaults = array(
                'setting_ids' => array(),
            );

            $options = ArrayTools::BuildOptions($options, $defaults);

            $options['setting_ids'] = ArrayTools::clean($options['setting_ids'], true);

            $db = Application::GetDb();

            $select = $db->select();
            $select->from(array('s' => self::$table), array())
                   ->where('s.client_id = ?', getClientId());

            if (count($options['setting_ids']) > 0) {
                $select->where('s.setting_id in (?)', $options['setting_ids']);
            }

            return $select;
        }
    }
?>
