<?php
    class Application_Config
    {
        /**
         * getDefaultConfiguration
         *
         * This function specifies the default configuration options that can
         * be overridden in the application configuration file. Note that for
         * boolean type values you should use 1 and 0 in the ini file as opposed
         * to the literal 'true' or 'false'
         *
         * @return array    The application default configuration
         */
        public static function getDefaultConfiguration()
        {
            return array(
                'database' => array(
                    'dsn'               => '',
                    'persistent'        => false
                ),
                'paths' => array(
                    'logs' => './data/logs'
                ),
                'locale' => array(
                    'timezone'             => '',
                    'locale'               => ''
                ),
                'templates' => array(
                    'force_compile'        => true,
                    'trim_whitespace'      => true
                ),
                'debug' => array(
                    'debug_log_file'       => './data/logs/recite.log',
                    'system_audit'         => true,
                    'system_audit_archive' => true
                )
            );
        }


        /**
         * GetConfig
         *
         * Return the application configuration based on combining the default configuration
         * and the configuration in the specified file. If the specified doesn't exist or
         * cannot be read then only the defaults are returned. This is to ensure that application
         * doesn't break as a result of trying to read a bogus config. If this results in any
         * inadequate values (such as database connection) then the relevant code should find
         * this problem (e.g. the database connector)
         *
         * @param   string  $file       The fullpath of the configuration file
         * @param   string  $section    The configuration section to use
         * @return  Zend_Config         The application configuration
         */
        public static function GetConfig($file, $section)
        {
            if (file_exists($file) && is_readable($file)) {
                try {
                    $_config = new Zend_Config_Ini($file, $section, true);
                    $arr = ArrayTools::buildOptionsRecursive($_config->toArray(), self::getDefaultConfiguration());
                }
                catch (Exception $ex) { }
            }

            if (!isset($arr) || !is_array($arr) || empty($arr))
                $arr = self::getDefaultConfiguration();

            foreach ($arr['paths'] as $k => $v) {
                if ($v{0} == '.')
                    $arr['paths'][$k] = realpath(Application::getBaseDir() . '/' . $v);
            }

            $arr['debug']['debug_log_file'] = Application::getBaseDir() . '/' . $arr['debug']['debug_log_file'];

            return new Zend_Config($arr);
        }


        /**
         * write
         *
         * Write a configuration to an ini file. Uses the difference between
         * the passed in config and the default config
         *
         * @param   string  $file       The fullpath of the ini file to write
         * @param   string  $section    The ini section to write
         * @param   array   $config     The configuration to write
         */
        public static function write($file, $section, $config)
        {
            $default = self::getDefaultConfiguration();

            $arr = ArrayTools::arrayDiffAssocRecursive($config, $default);

            $lines = array();
            foreach (self::_constructIni($arr, array()) as $k => $v) {
                $lines[] = sprintf('%s = "%s"', $k, $v);
            }

            $config = sprintf("[%s]\n\n%s\n", $section, join("\n", $lines));

            file_put_contents($file, $config);
        }

        protected static function _constructIni($arr, $parts)
        {
            $lines = array();
            foreach ($arr as $k => $v) {
                $_parts = $parts;
                $_parts[] = $k;
                if (is_array($v))
                    $lines = array_merge($lines, self::_constructIni($v, $_parts));
                else
                    $lines[join('.', $_parts)] = $v;
            }

            return $lines;
        }
    }
?>