<?php
    class Application_Content_Lock_Manager
    {
        const SHARED    = 1;
        const EXCLUSIVE = 2;

        const TABLE = 'application_content_locks';

        protected static function _GetBaseQuery($type)
        {
            $type = trim($type);

            if (strlen($type) == 0) {
                throw new Application_Content_Lock_Exception('No lock type specified');
            }

            $db = Application::GetDb();
            $helper = $db->getStatementHelper();

            $select = $db->select();
            $select->from(self::TABLE, '*')
                   ->where('client_id = ?', getClientId())
                   ->where('type = ?', $type)
                   ->where('ts_expired > now()');

            return $select;
        }

        protected static function GetLocksFromDatabaseQuery(Zend_Db_Select $select)
        {
            $db  = Application::GetDb();
            $ret = array();

            foreach ($db->fetchAll($select) as $row) {
                $ret[] = self::GetLockFromDatabaseRow($row);
            }

            return $ret;
        }

        protected static function GetLockFromDatabaseRow($array)
        {
            $helper = Application::GetDb()->getStatementHelper();

            $ret = new Application_Content_Lock($array['type']);
            $ret->owner     = $array['owner'];
            $ret->clientId  = $array['client_id'];
            $ret->type      = $array['type'];
            $ret->scope     = $array['scope'];
            $ret->token     = $array['token'];
            $ret->path      = $array['path'];
            $ret->tsCreated = $helper->unixTimestamp($array['ts_created']);
            $ret->tsExpired = $helper->unixTimestamp($array['ts_expired']);

            return $ret;
        }

        public static function GetLocksForPath($type, $path)
        {
            if (rand(1, 100) == 1) {
                Application_Maintenance::Queue('Application_Content_Lock_Cleaner');
            }

            $path = trim($path);

            if (strlen($path) == 0) {
                throw new Application_Content_Lock_Exception('No path specified');
            }

            $select = self::_GetBaseQuery($type);

            $db = Application::GetDb();

            $wheres = array(
                $db->quoteInto('path = ?', $path)
            );

            // check locks for every part in the path.
            $parts = array_filter(explode('/', $path));
            array_pop($parts);

            $currentParts = array();

            foreach ($parts as $part) {
                $currentParts[] = $part;

                $wheres[] = $db->quoteInto('(depth != 0 and path = ?)', '/' . join('/', $currentParts));
            }

            $select->where(join(' or ', $wheres));

            return self::GetLocksFromDatabaseQuery($select);
        }

        public static function UpdateLockByToken(Application_Content_Lock $lock)
        {
            $locks = self::GetLocksForPath($lock->type, $lock->path);

            $exists = false;

            foreach ($locks as $l) {
                if ($lock->token == $l->token) {
                    $exists = true;
                    break;
                }
            }

            $db = Application::GetDb();
            $helper = $db->getStatementHelper();

            if ($exists) {
                $values = array(
                    'type'  => $lock->type,
                    'owner' => $lock->owner,
                    'depth' => $lock->depth,
                    'scope' => $lock->scope,
                    'path'  => $lock->path,
                    'ts_created' => $helper->getTimestamp($lock->tsCreated),
                    'ts_expired' => $helper->getTimestamp($lock->tsExpired)
                );

                $where = array(
                    $db->quoteInto('token = ?', $lock->token)
                );

                try {
                    $where[] = $db->quoteInto('client_id = ?', getClientId());
                }
                catch (Exception $ex) {
                    $where[] = 'client_id is null';
                }

                $db->update(self::TABLE, $values, $where);
            }
            else {
                $values = array(
                    'type'  => $lock->type,
                    'owner' => $lock->owner,
                    'depth' => $lock->depth,
                    'scope' => $lock->scope,
                    'token' => $lock->token,
                    'path'  => $lock->path,
                    'ts_created' => $helper->getTimestamp($lock->tsCreated),
                    'ts_expired' => $helper->getTimestamp($lock->tsExpired)
                );

                try {
                    $values['client_id'] = getClientId();
                }
                catch (Exception $ex) {
                }

                $db->insert(self::TABLE, $values);
            }
        }

        public static function Unlock($path = null, $token = null)
        {
            $path  = trim($path);
            $token = trim($token);

            if (strlen($path) == 0 && strlen($token) == 0) {
                return;
            }

            $db = Application::GetDb();

            $where = array();

            if (strlen($path) > 0) {
                $where[] = $db->quoteInto('path = ?', $path);
            }

            if (strlen($token) > 0) {
                $where[] = $db->quoteInto('token = ?', $token);
            }

            try {
                $where[] = $db->quoteInto('client_id = ?', getClientId());
            }
            catch (Exception $ex) {
                $where[] = 'client_id is null';
            }

            $db->delete(self::TABLE, $where);
        }

        public static function DeleteExpiredLocks()
        {
            $query = sprintf('delete from %s where ts_expired < now()', self::TABLE);
            Application::GetDb()->query($query);
        }
    }
?>
