<?php
    /**
     * This class manages control panel themes
     *
     * @package Application_Cp
     * @author  Quentin Zervaas
     */
    class Application_Cp_Theme_Manager
    {
        protected static $_activeTheme;

        /**
         * Get the active theme
         *
         * @return  Application_Cp_Theme_Interface
         */
        public static function GetActiveTheme()
        {
            if (!self::$_activeTheme instanceof Application_Cp_Theme_Interface) {
                try {
                    if (Zend_Registry::isRegistered('brand')) {
                        $setting = Zend_Registry::get('brand')->profile->theme;
                    }
                    else {
                        $setting = Application_Brand::GetStaticSetting(Application::GetClient()->brand_id, 'theme');
                    }

                    self::$_activeTheme = self::Factory($setting);
                }
                catch (Exception $ex) {
                    self::$_activeTheme = self::GetDefaultTheme();
                }
            }

            return self::$_activeTheme;
        }


        public static function GetPath()
        {
            return Application::GetBaseDir() . '/lib/themes';
        }

        public static function GetDefaultTheme()
        {
            return new Application_Cp_Theme_Default();
        }


        public static function GetThemes()
        {
            $default = self::GetDefaultTheme();

            $ret = array(
                $default->getName() => $default
            );

            foreach (self::GetThemesByPath(self::GetPath()) as $theme) {
                $ret[$theme->getName()] = $theme;
            }

            return $ret;
        }


        public static function GetThemesByPath($path)
        {
            try {
                $fd = new File_Directory($path);
            }
            catch (Exception $ex) {
                return array();
            }

            $ret = array();

            $basepath = self::GetPath();
            $regex    = '/^' . preg_quote($basepath, '/') . '\/(.*?)\/theme\.php$/';

            foreach ($fd->getFiles() as $file) {
                if (substr($file, -4) != '.php') {
                    continue;
                }

                $fullpath = $fd->getPath() . '/' . $file;

                $class = str_replace('/', '_', preg_replace($regex, '$1', $fullpath));

                try {
                    $driver = self::Factory($class);
                    $ret[$driver->getName()] = $driver;

                    return $ret; // drivers can't be in directory of other drivers, so must return here to stop looking
                }
                catch (Exception $ex) {
                    continue;
                }
            }

            foreach ($fd->getDirs() as $dir) {
                foreach (self::GetThemesByPath($path . '/' . $dir) as $name => $driver) {
                    $ret[$name] = $driver;
                }
            }

            return $ret;
        }


        /**
         * Get a theme
         *
         * @param   string|Application_Cp_Theme_Interface   $name   The name of the theme
         * @throws  Application_Cp_Theme_Exception
         * @return  Application_Cp_Theme_Interface
         */
        public static function Factory($name)
        {
            if ($name instanceof Application_Cp_Theme_Interface) {
                return $name;
            }

            if (strlen($name) == 0) {
                throw new Application_Cp_Theme_Exception('No theme name specified');
            }

            if ($name == Application_Cp_Theme_Default::NAME) {
                return new Application_Cp_Theme_Default();
            }

            $path = str_replace('_', '/', preg_replace('/^Theme_/', '', $name));

            $fullpath = sprintf('%s/lib/themes/%s/theme.php', Application::GetBaseDir(), $path);

            if (!file_exists($fullpath)) {
                throw new Application_Cp_Theme_Exception('Theme file does not exist: ' . $fullpath);
            }

            if (!is_file($fullpath)) {
                throw new Application_Cp_Theme_Exception('Theme path is not a file: ' . $fullpath);
            }

            if (!is_readable($fullpath)) {
                throw new Application_Cp_Theme_Exception('Theme file is not readable: ' . $fullpath);
            }

            $class = 'Theme_' . $name;

            require_once($fullpath);

            if (!class_exists($class, false)) {
                throw new Application_Cp_Theme_Exception('Theme class not declared: ' . $class);
            }

            $reflection = new ReflectionClass($class);

            if (!$reflection->isSubclassOf(new ReflectionClass('Application_Cp_Theme_Interface'))) {
                throw new Application_Cp_Theme_Exception('Theme class does not implement Application_Cp_Theme_Interface: ' . $class);
            }

            return new $class;
        }
    }
?>
