<?php
    /**
     * The base abstract the widgets should extend from
     *
     * @author  Quentin Zervaas
     */
    abstract class Application_Cp_Widget_Abstract implements Application_Cp_Widget_Interface
    {
        protected $_request;
        protected $_response;
        protected $_view;
        protected $_reflection;

        protected $_scripts = array();
        protected $_cssPaths = array();

        public function __construct()
        { }

        public function getName()
        {
            $ret = get_class($this);

            return preg_replace('/_controller$/', '', $ret);
        }

        public function getCategories()
        {
            return array('Miscellaneous');
        }

        public function getDescription()
        {
            return 'No description specified';
        }

        public function getDependentModules()
        {
            return array();
        }

        public function getWidgetSize()
        {
            return self::SIZE_SMALL;
        }

        public function canBeManuallyAdded()
        {
            return true;
        }

        public function getPath()
        {
            if (!$this->_reflection instanceof ReflectionObject) {
                $this->_reflection = new ReflectionObject($this);
            }

            return dirname($this->_reflection->getFilename());
        }

        public function setRequest(Application_Cp_Site_Request $request)
        {
            $this->_request = $request;
        }

        public function getRequest()
        {
            return $this->_request;
        }

        public function setResponse(Application_Cp_Site_Response $response)
        {
            $this->_response = $response;
        }

        public function getResponse()
        {
            return $this->_response;
        }

        public function setView(Application_View_Cp $view)
        {
            $this->_view = $view;
        }

        public function getView()
        {
            return $this->_view;
        }

        public function getClient()
        {
            return Application::GetClient();
        }

        protected static function _ParsePath($path)
        {
            $path = trim($path, '/');

            if (strlen($path) == 0) {
                $path = 'index';
            }

            $parts  = explode('/', $path);
            $action = array_shift($parts);
            $action = str_replace('-', ' ', strtolower($action));
            $action = preg_replace('/[^a-z0-9 ]/', '', $action);
            $action = str_replace(' ', '', ucwords($action));

            if (strlen($action) == 0) {
                $action = 'index';
            }

            return $action;
        }

        public function run($path = '')
        {
            $action = self::_ParsePath($path);
            $method = $action . 'Action';

            if (!method_exists($this, $method)) {
                throw new Application_Cp_Widget_Exception('Invalid method');
            }

            $this->_scripts  = array();
            $this->_cssPaths = array();

            $this->preDispatch();
            $this->$method($path);
            $this->postDispatch();

            $view = $this->getView();
            $view->WIDGET = $this;

            $response = $this->getResponse();
            $response->setBody($view->render($this->getTemplatePath() . '/' . $action . '.tpl'));

            if (count($this->_cssPaths) == 0) {
                $cssPath = strtolower($action) . '.css';
                $fullpath = sprintf('%s/files/css/%s', $this->getPath(), $cssPath);

                if (file_exists($fullpath)) {
                    $this->_cssPaths[] = sprintf('/files/widget/%s/css/%s', $this->getName(), $cssPath);
                }
            }
        }

        public function preDispatch()
        { }

        public function postDispatch()
        { }

        public function getScript($path = '')
        {
            if (array_key_exists($path, $this->_scripts)) {
                $ret = $this->_scripts[$path];

                if ($ret instanceof Application_Cp_Widget_Script) {
                    return $ret;
                }

                throw new Exception('Controller script not set');
            }
            else {
                $action = self::_ParsePath($path);

                $scriptPath = strtolower($action) . '.js';
                $scriptName = $this->getName() . '_' . strtolower($action);

                $script = new Application_Cp_Widget_Script($this);
                $script->setPath('/js/' . $scriptPath)
                       ->setName($scriptName);

                return $script;
            }
        }

        public function setScriptForPath($path, Application_Cp_Widget_Script $script = null)
        {
            $this->_scripts[$path] = $script;
            return $this;
        }

        public function getCssPaths()
        {
            return $this->_cssPaths;
        }

        public function sendFile(Application_Cp_Site_Response $response, $path)
        {
            $fullpath = realpath(sprintf('%s/files/%s', $this->getPath(), $path));

            if (strlen($fullpath) == 0) {
                throw new Application_Cp_Widget_Exception('File not found');
            }

            $prefix = realpath($this->getPath());

            if (substr($fullpath, 0, strlen($prefix)) != $prefix) {
                throw new Application_Cp_Widget_Exception('Path not in valid directory');
            }

            $options = array(
                'response' => $response
            );

            HTTP::SendFile($fullpath, $options);
        }

        public function getTemplatePath()
        {
            return sprintf('%s/templates', $this->getPath());
        }

        public function assertPermissions()
        { }
    }
?>