<?php
    /**
     * Specifies methods that must be implemented for drivers
     *
     * @author  Quentin Zervaas <quentin@recite.com.au>
     */
    interface Application_Driver_Interface
    {
        /**
         * Constructor. Must not have any arguments
         */
        public function __construct();


        /**
         * Return the internal name of the driver based on the driver type
         *
         * @return  string  The name of the driver
         */
        public function getName();


        /**
         * Return the full driver name
         *
         * @return  string
         */
        public function getFullName();


        /**
         * Return a description for the type of driver this is
         *
         * @return  string
         */
        public function getDriverTypeDescription();


        /**
         * Return a description of what this driver does
         *
         * @return  string
         */
        public function getDescription();


        /**
         * Get the directory of the driver
         *
         * @return  string
         */
        public function getDriverDirectory();


        /**
         * Return the path of the driver
         *
         * @return string
         */
        public function getPath();


        /**
         * Return the descriptive title of the driver
         *
         * @return  string
         */
        public function __toString();


        /**
         * Return an array of module names that this driver relies on
         *
         * @return  array
         */
        public function getDependentModules();


        /**
         * Return an array of other drivers that this driver relies on
         *
         * @return  array
         */
        public function getDependentDrivers();


        /**
         * Assert the current user has permissions required to use this driver
         *
         * @throws  Application_User_Permissions_Exception  If the user doesn't have access
         */
        public function assertPermissions();


        /**
         * Whether or not the driver can be assigned to clients
         *
         * @return  bool
         */
        public function isClientDriver();


        /**
         * Return the filesystem path of a file that is contained within the driver.
         * This is purely for files that are to be accessed via the control panel directly.
         * This method does not need to ensure the file exists.
         *
         * @return  string
         */
        public function getFileFullPath($path);

        /**
         * Return the web path used to retrieve a driver file through the control panel.
         * This method does not need to ensure the file exists
         *
         * @return  string
         */
        public function getFileWebPath($path);


        /**
         * Send a driver file. Throws an exception if the file isn't found or can't be sent
         *
         * @throws  Application_Driver_Exception
         */
        public function sendFile(Application_Cp_Site_Response $response, $path);


        /**
         * Whether or not the driver supports Control Panel actions
         *
         * @return  bool
         */
        public function isCpController();

        public function setCpRequest(Zend_Controller_Request_Abstract $request);
        public function setCpResponse(Zend_Controller_Response_Abstract $response);
    }
?>