<?php
    /**
     * Class to manage add-on drivers for Recite
     *
     * @since   Recite 1.7.0
     * @author  Quentin Zervaas
     * @package Application
     */
    class Application_Driver_Manager
    {
        const TABLE_CLIENTS = 'clients_drivers';
        const TABLE_BRANDS  = 'brands_drivers';

        private static $_interfaceReflection = array();
        private static $_interfaceReflectionDefaultClassName = 'Application_Driver_Abstract';
        private static $_interfaceReflectionClassName = null;

        private static $_disableAccessChecks = false;

        private static $_driverCache = array(
            'names' => array(),
            'objs'  => array()
        );

        public static function GetPath()
        {
            return realpath(dirname(__FILE__) . '/../../../../lib/drivers');
        }

        public static function DisableAccessChecks($flag)
        {
            self::$_disableAccessChecks = (bool) $flag;
        }


        public static function GetAllDrivers(Application_Client $client = null)
        {
            $drivers = array();

            foreach (Application_Module_Manager::GetModules() as $module) {
                foreach ($module->getDriverTypes() as $type) {
                    foreach (self::GetDriversByType($type, $client) as $driver) {
                        $drivers[] = $driver;
                    }
                }
            }

            foreach (Application_Driver_Standalone_Manager::GetDrivers() as $driver) {
                $drivers[] = $driver;
            }

            return $drivers;
        }

        public static function GetAllClientDrivers()
        {
            $ret = array();

            foreach (self::GetAllDrivers() as $k => $driver) {
                if ($driver->isClientDriver()) {
                    $ret[$k] = $driver;
                }
            }

            return $ret;
        }

        public static function GetDriversByType($type, Application_Client $client = null, $checkAcls = false)
        {
            $path = self::GetPathForDriverType($type);

            if ($client instanceof Application_Client && $client->saved() && !self::$_disableAccessChecks) {
                $drivers = self::GetDriversByPath($path);

                $ret = array();

                if (count($drivers) > 0) {
                    $clientDrivers = self::GetDriverNames($client);

                    foreach ($drivers as $driver) {
                        if (!$driver->isClientDriver()) {
                            continue;
                        }

                        $name = $driver->getFullName();

                        if (!in_array($name, $clientDrivers)) {
                            continue;
                        }
                        if ($checkAcls) {
                            try {
                                $drivers[$name]->assertPermissions();
                            }
                            catch (Exception $ex) {
                                continue;
                            }
                        }

                        $allowed = true;

                        $modules = $client->getModules();

                        if ($checkAcls) {
                            $query = Application_User_Permissions_Query::NewAnd();
                        }

                        foreach ($drivers[$name]->getDependentModules() as $module) {
                            if (!array_key_exists($module, $modules)) {
                                $allowed = false;
                                break;
                            }
                            else if ($checkAcls) {
                                $query->add('module:' . $module);
                            }
                        }

                        if ($checkAcls) {
                            if (!$query->check()) {
                                continue;
                            }
                        }

                        if ($allowed) {
                            foreach ($drivers[$name]->getDependentDrivers() as $dep) {
                                try {
                                    $dep = self::Factory($dep);
                                    self::AssertClientHasDriver($client, $dep);

                                    if ($checkAcls) {
                                        $dep->assertPermissions();
                                    }
                                }
                                catch (Exception $ex) {
                                    $allowed = false;
                                    break;
                                }
                            }
                        }

                        if ($allowed) {
                            $ret[$name] = $drivers[$name];
                        }
                    }
                }
            }
            else {
                $ret = array();

                foreach (self::GetDriversByPath($path) as $driver) {
                    $allowed = true;
                    foreach ($driver->getDependentDrivers() as $dep) {
                        try {
                            self::Factory($dep);
                        }
                        catch (Exception $ex) {
                            $allowed = false;
                            break;
                        }
                    }

                    if ($allowed) {
                        $ret[$driver->getFullName()] = $driver;
                    }
                }
            }

            uasort($ret, array(__CLASS__, '_sortDrivers'));

            return $ret;
        }

        public static function GetPathForDriverType($type)
        {
            $str = str_replace('_', '/', $type);
            return sprintf('%s/%s', self::GetPath(), $str);
        }

        public static function GetDriversByPath($path)
        {
            try {
                $fd = new File_Directory($path);
            }
            catch (Exception $ex) {
                return array();
            }

            $ret = array();

            $basepath = self::GetPath();
            $regex = '/^' . preg_quote($basepath, '/') . '\/(.*?)\.php$/';

            foreach ($fd->getFiles() as $file) {
                if (substr($file, -4) != '.php') {
                    continue;
                }

                $fullpath = $fd->getPath() . '/' . $file;

                $class = 'Driver_' . str_replace('/', '_', preg_replace($regex, '$1', $fullpath));

                try {
                    $driver = self::Factory($class);
                    $ret[$driver->getFullName()] = $driver;

                    return $ret; // drivers can't be in directory of other drivers, so must return here to stop looking
                }
                catch (Exception $ex) {
                    continue;
                }
            }

            foreach ($fd->getDirs() as $dir) {
                foreach (self::GetDriversByPath($path . '/' . $dir) as $name => $driver) {
                    $ret[$name] = $driver;
                }
            }

            return $ret;
        }

        /**
         * Create an instance of a driver
         *
         * @param   string  $driver         The name of the driver to create
         * @param   bool    $checkAccess    Whether or not to check access for the current client
         * @param   bool    $checkAcls      Whether or not to check if current user has access
         * @param   bool    $checkDeps      Whether or not to check for dependencies
         * @throws  Application_Driver_DriverNotFoundException  If specified driver could not be found
         * @throws  Application_Driver_Exception                If unable to find or create the driver
         * @return  Application_Driver_Abstract                 The driver
         */
        public static function Factory($driver, $checkAccess = false, $checkAcls = false, $checkDeps = true)
        {
            if (self::$_disableAccessChecks) {
                $checkAccess = false;
            }

            if ($driver instanceof Application_Driver_Interface) {
                if ($checkAccess) {
                    self::AssertClientHasDriver(Application::GetClient(), $driver, $checkAcls);
                }

                if ($checkDeps) {
                    // check for other drivers this particular drivers depends upon
                    $deps = $driver->getDependentDrivers();

                    if (!is_array($deps)) {
                        throw new Application_Driver_Exception('Dependency list must be an array');
                    }

                    $old = self::GetInterfaceReflectionClassName();
                    self::ResetInterfaceReflectionClassName();

                    foreach ($deps as $dep) {
                        $dep = self::Factory($dep);

                        if ($checkAccess) {
                            self::AssertClientHasDriver(Application::GetClient(), $dep, $checkAcls);
                        }
                    }

                    self::SetInterfaceReflectionClassName($old);
                }

                return $driver;
            }
            else {
                $driver = (string) $driver;
            }

            if (preg_match('/[^a-z0-9_]/i', $driver)) {
                throw new Application_Driver_Exception('Invalid driver name: ' . $driver);
            }

            $path = preg_replace('/^Driver_/', '', $driver);

            $fullpath = sprintf('%s/%s.php', self::GetPath(), str_replace('_', '/', $path));

            if (!file_exists($fullpath)) {
                throw new Application_Driver_DriverNotFoundException('Driver file does not exist: ' . $fullpath);
            }

            if (!is_file($fullpath)) {
                throw new Application_Driver_DriverNotFoundException('Driver path is not a file: ' . $fullpath);
            }

            if (!is_readable($fullpath)) {
                throw new Application_Driver_DriverNotFoundException('Driver file is not readable: ' . $fullpath);
            }

            if ($checkAccess) {
                self::AssertClientHasDriver(Application::GetClient(), $driver, $checkAcls);
            }

            $class = $driver;

            require_once($fullpath);

            if (!class_exists($class, false)) {
                throw new Application_Driver_DriverNotFoundException('Driver class not declared: ' . $class);
            }

            $reflection = new ReflectionClass($class);
            $interfaceReflection = self::_GetInterfaceReflection();

            if ($interfaceReflection->isInterface()) {
                if (!$reflection->implementsInterface($interfaceReflection)) {
                    throw new Application_Driver_DriverNotFoundException(sprintf('Driver class does not implement %s: %s', $interfaceReflection->getName(), $class));
                }
            }
            else if ($interfaceReflection->isAbstract()) {
                if (!$reflection->isSubclassOf($interfaceReflection)) {
                    throw new Application_Driver_DriverNotFoundException(sprintf('Driver class does not extend %s: %s', $interfaceReflection->getName(), $class));
                }
            }
            else {
                throw new Application_Driver_DriverNotFoundException(sprintf('Interface is not a valid class: %s', $interfaceReflection->getName()));
            }

            $driver = new $class();

            if ($checkDeps) {
                // check for other drivers this particular drivers depends upon
                $deps = $driver->getDependentDrivers();

                if (!is_array($deps)) {
                    throw new Application_Driver_Exception('Dependency list must be an array');
                }

                $old = self::GetInterfaceReflectionClassName();
                self::ResetInterfaceReflectionClassName();

                foreach ($deps as $dep) {
                    $dep = self::Factory($dep);

                    if ($checkAccess) {
                        self::AssertClientHasDriver(Application::GetClient(), $dep, $checkAcls);
                    }
                }

                self::SetInterfaceReflectionClassName($old);
            }

            return $driver;
        }

        private static function _GetInterfaceReflection()
        {
            $className = self::GetInterfaceReflectionClassName();

            if (!array_key_exists($className, self::$_interfaceReflection)) {
                self::$_interfaceReflection[$className] = new ReflectionClass($className);
            }

            return self::$_interfaceReflection[$className];
        }

        public static function SetInterfaceReflectionClassName($class)
        {
            if ($class == self::$_interfaceReflectionDefaultClassName) {
                self::ResetInterfaceReflectionClassName();
            }
            else {
                self::$_interfaceReflectionClassName = $class;
            }
        }

        public static function GetInterfaceReflectionClassName()
        {
            if (strlen(self::$_interfaceReflectionClassName) > 0) {
                return self::$_interfaceReflectionClassName;
            }

            return self::$_interfaceReflectionDefaultClassName;
        }

        public static function ResetInterfaceReflectionClassName()
        {
            self::$_interfaceReflectionClassName = null;
        }

        public static function SetDrivers($item, $drivers)
        {
            $db = Application::GetDb();
            $db->begin();

            self::ClearDrivers($item);

            foreach (ArrayTools::clean($drivers) as $driver) {
                try {
                    self::AssignDriver($item, $driver);
                }
                catch (Exception $ex)
                { }
            }

            $db->commit();
        }

        public static function ClearDrivers($item)
        {
            if ($item instanceof Application_Brand) {
                $table  = self::TABLE_BRANDS;
                $column = 'brand_id';
            }
            else if ($item instanceof Application_Client) {
                $table  = self::TABLE_CLIENTS;
                $column = 'client_id';
            }
            else {
                throw new Application_Driver_Exception(sprintf('Invalid item passed to %s::%s', __CLASS__, __FUNCTION__));
            }

            $db = Application::GetDb();

            $where = array(
                $db->quoteInto($column . ' = ?', $item->getId())
            );

            $db->delete($table, $where);

            $key = strtolower(get_class($item));

            unset(self::$_driverCache['names'][$key][$item->getId()]);
            unset(self::$_driverCache['objs'][$key][$item->getId()]);
        }

        public static function AssignDriver($item, $driver)
        {
            $driver = self::Factory($driver);

            if (!$driver->isClientDriver()) {
                return;
            }

            if ($item instanceof Application_Brand) {
                $table  = self::TABLE_BRANDS;
                $column = 'brand_id';
            }
            else if ($item instanceof Application_Client) {
                $table  = self::TABLE_CLIENTS;
                $column = 'client_id';
            }
            else {
                throw new Application_Driver_Exception(sprintf('Invalid item passed to %s::%s', __CLASS__, __FUNCTION__));
            }

            $existing = self::GetDrivers($item);

            if (array_key_exists($driver->getFullName(), $existing)) {
                return;
            }

            $values = array(
                $column       => $item->getId(),
                'driver_name' => $driver->getFullName()
            );

            $db = Application::GetDb();
            $db->insert($table, $values);

            $key = strtolower(get_class($item));
            self::$_driverCache['names'][$key][$item->getId()][] = $driver->getFullname();
            self::$_driverCache['objs'][$key][$item->getId()][$driver->getFullName()] = $driver;
        }

        public static function GetDrivers($item)
        {
            $key = strtolower(get_class($item));

            if (!array_key_exists($key, self::$_driverCache['objs'])) {
                self::$_driverCache['objs'][$key] = array();
            }

            if (array_key_exists($item->getId(), self::$_driverCache['objs'][$key])) {
                return self::$_driverCache['objs'][$key][$item->getId()];
            }

            if ($item instanceof Application_Brand) {
                $table  = self::TABLE_BRANDS;
                $column = 'brand_id';
            }
            else if ($item instanceof Application_Client) {
                $table  = self::TABLE_CLIENTS;
                $column = 'client_id';
            }
            else {
                throw new Application_Driver_Exception(sprintf('Invalid item passed to %s::%s', __CLASS__, __FUNCTION__));
            }

            self::$_driverCache['objs'][$key][$item->getId()] = array();

            $old = self::GetInterfaceReflectionClassName();
            self::ResetInterfaceReflectionClassName();

            foreach (self::GetDriverNames($item) as $driverName) {
                try {
                    $driver = self::Factory($driverName);
                    self::$_driverCache['objs'][$key][$item->getId()][$driver->getFullName()] = $driver;
                }
                catch (Exception $ex)
                {}
            }

            self::SetInterfaceReflectionClassName($old);

            return self::$_driverCache['objs'][$key][$item->getId()];
        }

        public static function GetDriverNames($item)
        {
            $key = strtolower(get_class($item));

            if (!array_key_exists($key, self::$_driverCache['names'])) {
                self::$_driverCache['names'][$key] = array();
            }

            if (array_key_exists($item->getId(), self::$_driverCache['names'][$key])) {
                return self::$_driverCache['names'][$key][$item->getId()];
            }

            if ($item instanceof Application_Brand) {
                $table  = self::TABLE_BRANDS;
                $column = 'brand_id';
            }
            else if ($item instanceof Application_Client) {
                $table  = self::TABLE_CLIENTS;
                $column = 'client_id';
            }
            else {
                throw new Application_Driver_Exception(sprintf('Invalid item passed to %s::%s', __CLASS__, __FUNCTION__));
            }

            $db = Application::GetDb();
            $query = sprintf('select driver_name from %s where %s = %d', $table, $column, $item->getId());

            self::$_driverCache['names'][$key][$item->getId()] = $db->fetchCol($query);

            return self::$_driverCache['names'][$key][$item->getId()];
        }

        public static function AssertClientHasDriver(Application_Client $client, $driver, $checkAcls = false)
        {
            if (self::$_disableAccessChecks) {
                return;
            }

            $driver = self::Factory($driver);

            $dependentModules = ArrayTools::Clean($driver->getDependentModules());

            foreach ($dependentModules as $module) {
                if (!$client->hasModule($module)) {
                    throw new Application_Driver_NoPermissionsException('Client does not have access to module: ' . $module);
                }
                else if ($checkAcls) {
                    Application_User_Permissions_Query_Item::BuildAndAssert('module:' . $module);
                }
            }

            if (!self::ClientHasDriver($client, $driver)) {
                throw new Application_Driver_NoPermissionsException('Client does not have access to driver: ' . $driver->getFullName());
            }
            else if ($checkAcls) {
                $driver->assertPermissions();
            }
        }

        public static function ClientHasDriver(Application_Client $client, $driver)
        {
            if ($driver instanceof Application_Driver_Abstract) {
                $driver = $driver->getFullName();
            }

            return in_array((string) $driver, self::GetDriverNames($client));
        }

        public static function GetObserverDirectories()
        {
            $path = self::GetPath();
            $len  = strlen($path);

            $dp = new File_Directory($path);

            $paths = array();

            foreach ($dp->getDirs() as $dir) {
                $paths = array_merge($paths, self::_GetObserverDirectories($path . DIRECTORY_SEPARATOR . $dir));
            }

            $ret = array();

            foreach ($paths as $path) {
                $ret[] = array(
                    'path'  => $path,
                    'class' => 'Driver_' . trim(str_replace(DIRECTORY_SEPARATOR, '_', substr($path, $len)), '_')
                );
            }

            return $ret;
        }

        public static function _GetObserverDirectories($dir)
        {
            $dp = new File_Directory($dir);

            $ret = array();

            foreach ($dp->getDirs() as $child) {
                if ($child == 'observers') {
                    $ret[] = $dir . DIRECTORY_SEPARATOR . $child;
                }
                else {
                    $ret = array_merge($ret, self::_GetObserverDirectories($dir . DIRECTORY_SEPARATOR . $child));
                }
            }

            return $ret;
        }

        /**
         * Used for sorting the array of drivers by title
         */
        public static function _sortDrivers($a, $b)
        {
            return strcmp((string) $a, (string) $b);
        }
    }
?>
