<?php
    class Application_Event_Manager
    {
        protected static $_enabled = true;
        protected static $_updateQueued = false;

        protected static $_callStack = array();

        /**
         * This is true if an event is verbose. This prevents other
         * events from being verbose even if they wanted to
         */
        protected static $_verbose = false;

        public static function Enable()
        {
            self::$_enabled = true;
        }

        public static function Disable()
        {
            self::$_enabled = false;
        }

        public static function Trigger(Application_Event $event)
        {
            if ($event->getResponds()) {
                return self::TriggerResponder($event);
            }

            if (!self::$_enabled) {
                return;
            }

            if (array_key_exists($event->getName(), self::$_callStack)) {
                return;
            }

            // this line is useful for debugging which events are triggered:
            //echo $event->getName() . "\n";
            //$logger = Application::GetLogger();
            //$logger->info($event->getName());

            $log = $event->isVerbose() && !self::$_verbose && IS_CLI;

            if ($log) {
                self::$_verbose = true;
            }

            self::$_callStack[$event->getName()] = $event;

            $observers = self::GetObserversForEvent($event->getName());

            if ($log) {
                echo sprintf("Running event: %s\n", $event->getName());
                $progress = new Application_CommandLine_ProgressBar(count($observers));
            }

            $i = 0;

            foreach ($observers as $observer) {
                if ($log) {
                    $progress->notify($i++);
                }

                $observer->notify($event);
            }

            if ($log) {
                $progress->complete();
                echo sprintf("\n");
            }

            unset(self::$_callStack[$event->getName()]);

            if ($log) {
                self::$_verbose = false;
            }
        }

        public static function TriggerResponder(Application_Event $event)
        {
            $collection = new Application_Event_ResponseCollection();

            if (!self::$_enabled) {
                return $collection;
            }

            if (array_key_exists($event->getName(), self::$_callStack)) {
                return $collection;
            }

            self::$_callStack[$event->getName()] = $event;

            $observers = self::GetObserversForEvent($event->getName());

            foreach ($observers as $observer) {
                try {
                    $response = $observer->notify($event);

                    if ($response instanceof Application_Event_Response) {
                        $collection->add($response);
                    }
                }
                catch (Exception $ex) {

                }
            }

            unset(self::$_callStack[$event->getName()]);

            return $collection;
        }

        public static function GetObserversForEvent($name)
        {
            if (IN_DEVELOPMENT) {
                return self::GetObserversFromSearch($name);
            }

            try {
                return self::GetObserversFromCache($name);
            }
            catch (Exception $ex) {
                self::QueueCacheUpdate();
                return self::GetObserversFromSearch($name);
            }
        }

        public static function GetObserversFromCache($name)
        {
            $name  = trim($name, '/');
            $cache = self::GetCache();

            $primed = (bool) $cache->load('primed');

            if (!$primed) {
                throw new Exception('Cache is not primed');
            }

            $cacheId = md5($name);
            $observers = $cache->load($cacheId);

            if (!is_array($observers)) {
                return array();
            }

            $ret = array();

            foreach ($observers as $observer) {
                $fullpath = coalesce($observer['f']);

                if (!preg_match('|' . preg_quote($name, '|') . '|', $fullpath)) {
                    throw new Exception('Path does not match event name');
                }

                if (!file_exists($fullpath)) {
                    throw new Exception('Invalid path');
                }

                $class = $observer['c'];

                require_once($fullpath);

                if (!class_exists($class, false)) {
                    throw new Exception('Class ' . $class . ' is not defined');
                }

                $ret[] = new $class();
            }

            return $ret;
        }

        public static function GetObserversFromSearch($name)
        {
            $name = trim($name, '/');

            $ret = array();

            // get any core observers

            $path = sprintf('%s/Observer/Core/%s.php', dirname(__FILE__), $name);

            if (file_exists($path) && is_file($path)) {
                $class = sprintf('Application_Event_Observer_Core_%s', str_replace('/', '_', $name));

                require_once($path);

                if (class_exists($class, false)) {
                    $ret[] = new $class();
                }
            }


            // get any module observers

            foreach (Application_Module_Manager::GetModules() as $module) {
                $path = sprintf('%s/include/Observers/%s.php', $module->getPath(), $name);

                if (!file_exists($path) || !is_file($path)) {
                    continue;
                }

                $class = sprintf('Module_%s_Observers_%s', ucfirst($module->getName()), str_replace('/', '_', $name));

                require_once($path);

                if (!class_exists($class, false)) {
                    throw new Application_Exception(sprintf('Class %s not defined', $class));
                }

                $ret[] = new $class();
            }


            // get any driver observers

            foreach (Application_Driver_Manager::GetObserverDirectories() as $baseDir) {
                $path = sprintf('%s/%s.php', $baseDir['path'], $name);

                if (!file_exists($path) || !is_file($path)) {
                    continue;
                }

                $class = $baseDir['class'] . '_' . str_replace('/', '_', $name);

                require_once($path);

                if (!class_exists($class, false)) {
                    throw new Application_Exception(sprintf('Class %s not defined', $class));
                }

                $ret[] = new $class();
            }

            return $ret;
        }

        public static function GetEveryObserver()
        {
            $ret = array();

            $path = sprintf('%s/Observer/Core', dirname(__FILE__));

            foreach (self::_GetEveryObserver($path, $path, 'Application_Event_Observer_Core') as $observer) {
                $event = $observer['e'];
                if (!array_key_exists($event, $ret)) {
                    $ret[$event] = array();
                }

                $ret[$event][] = array('c' => $observer['c'], 'f' => $observer['f']);
            }

            foreach (Application_Module_Manager::GetModules() as $module) {
                $path = sprintf('%s/include/Observers', $module->getPath());

                if (!file_exists($path) || !is_dir($path)) {
                    continue;
                }

                $baseClass = sprintf('Module_%s_Observers', ucfirst($module->getName()));

                foreach (self::_GetEveryObserver($path, $path, $baseClass) as $observer) {
                    $event = $observer['e'];
                    if (!array_key_exists($event, $ret)) {
                        $ret[$event] = array();
                    }

                    $ret[$event][] = array('c' => $observer['c'], 'f' => $observer['f']);
                }
            }

            foreach (Application_Driver_Manager::GetAllDrivers() as $driver) {
                $path = sprintf('%s/observers', $driver->getDriverDirectory());

                if (!file_exists($path) || !is_dir($path)) {
                    continue;
                }

                $baseClass = preg_replace('/_[^_]+$/', '_observers', get_class($driver));

                foreach (self::_GetEveryObserver($path, $path, $baseClass) as $observer) {
                    $event = $observer['e'];
                    if (!array_key_exists($event, $ret)) {
                        $ret[$event] = array();
                    }

                    $ret[$event][] = array('c' => $observer['c'], 'f' => $observer['f']);
                }
            }

            return $ret;
        }

        private static function _GetEveryObserver($path, $basePath, $baseClass)
        {
            $basePath = rtrim($basePath, '/');
            $dp = dir($path);

            $ret = array();

            while (($file = $dp->read()) !== false) {
                if ($file[0] == '.') {
                    continue;
                }

                $fullpath = $dp->path . '/' . $file;

                if (is_dir($fullpath)) {
                    $ret = array_merge($ret, self::_GetEveryObserver($fullpath, $basePath, $baseClass . '_' . $file));
                }
                else if (preg_match('|^' . preg_quote($basePath) . '/(.*)\.php$|', $fullpath, $matches)) {
                    require_once($fullpath);

                    $class = $baseClass . '_' . substr($file, 0, -4);

                    if (!class_exists($class, false)) {
                        throw new Exception('Class ' . $class . ' is not defined');
                    }

                    $ret[] = array(
                        'c' => $class,
                        'e' => trim($matches[1], '/'),
                        'f' => $fullpath
                    );
                }
            }

            return $ret;
        }

        public static function GetCache()
        {
            $options = new Application_Cache_Options();
            $options->setUseGlobalCache(true);

            return Application_Cache::GetCache('events', null, $options);
        }

        public static function ClearCache()
        {
            $cache = self::GetCache();
            $cache->clean(Zend_Cache::CLEANING_MODE_ALL);
        }

        public static function QueueCacheUpdate()
        {
            if (self::$_updateQueued) {
                return;
            }

            self::$_updateQueued = true;

            $queueItem = new Application_Maintenance_QueueItem('Application_Event_CachePrimer');
            $queueItem->setCanHaveDuplicates(false)
                      ->queue();
        }
    }
?>
