<?php
    class Application_HttpdConfig
    {
        const TOUCH_FILE      = 'recite.lock';
        const FILENAME_BRAND  = 'brand-%d-%s.conf';
        const FILENAME_CLIENT = 'client-%d-%s.conf';

        public static function GetConfigDir()
        {
            $ret = sprintf('%s%sconf', Application::GetDataDir(), DIRECTORY_SEPARATOR);

            if (!file_exists($ret)) {
                File::CreateDirStructure($ret);
            }

            return $ret;
        }

        public static function Rebuild($item, $touch = true)
        {
            if (!$item instanceof Application_Brand && !$item instanceof Application_Client) {
                throw new Exception('Item must be a brand or a client');
            }

            if (!$item->saved()) {
                throw new Exception('Item is not a saved record');
            }

            self::Remove($item, false);

            $build = (bool) $item->profile->httpd_conf_generate;

            if ($build) {
                if ($item instanceof Application_Brand) {
                    $filename = sprintf(self::FILENAME_BRAND, $item->getId(), Text::Urlize($item));
                }
                else {
                    $filename = sprintf(self::FILENAME_CLIENT, $item->getId(), Text::Urlize($item));
                }

                $data = self::GetConfig($item);

                $path = sprintf('%s%s%s', self::GetConfigDir(), DIRECTORY_SEPARATOR, $filename);

                file_put_contents($path, $data);
            }

            if ($touch) {
                self::Touch();
            }
        }

        public static function GetConfig($item)
        {
            if (!$item instanceof Application_Brand && !$item instanceof Application_Client) {
                throw new Exception('Item must be a brand or a client');
            }

            if (!$item->saved()) {
                throw new Exception('Item is not a saved record');
            }

            if ($item instanceof Application_Brand) {
                $template = trim($item->getSetting('httpd.config.brand'));
            }
            else {
                $template = trim($item->getSetting('httpd.config'));

                if (strlen($template) == 0) {
                    $brand    = $item->getBrand();
                    $template = trim($brand->getSetting('httpd.config.client'));
                }
            }

            return self::_GetConfig($item, $template);
        }

        public static function Remove($item, $touch = true)
        {
            if (!$item instanceof Application_Brand && !$item instanceof Application_Client) {
                throw new Exception('Item must be a brand or a client');
            }

            if (!$item->saved()) {
                throw new Exception('Item is not a saved record');
            }

            if ($item instanceof Application_Brand) {
                self::RemoveBrand($item->getId(), $touch);
            }
            else {
                self::RemoveClient($item->getId(), $touch);
            }
        }

        public static function RemoveBrand($brandId, $touch = true)
        {
            $filename = sprintf(self::FILENAME_BRAND, $brandId, '*');
            $path = sprintf('%s%s%s', self::GetConfigDir(), DIRECTORY_SEPARATOR, $filename);

            foreach (glob($path) as $file) {
                @unlink($file);
            }

            if ($touch) {
                self::Touch();
            }
        }

        public static function RemoveClient($clientId, $touch = true)
        {
            $filename = sprintf(self::FILENAME_CLIENT, $clientId, '*');
            $path = sprintf('%s%s%s', self::GetConfigDir(), DIRECTORY_SEPARATOR, $filename);

            foreach (glob($path) as $file) {
                @unlink($file);
            }

            if ($touch) {
                self::Touch();
            }
        }

        public static function Touch()
        {
            touch(self::GetTouchPath());
        }

        public static function GetTouchPath()
        {
            return sprintf('%s%s%s', self::GetConfigDir(), DIRECTORY_SEPARATOR, self::TOUCH_FILE);
        }

        protected static function _GetClientVars($client)
        {
            $vars = array(
                'ip'         => $_SERVER['SERVER_ADDR'],
                'docroot'    => realpath(sprintf('%s/client/htdocs', Application::GetApplicationDir())),
                'davdocroot' => realpath(sprintf('%s/dav/htdocs', Application::GetApplicationDir())),
                'logroot'    => realpath(sprintf('%s/logs', Application::GetDataDir())),
                'client'     => Text::Urlize($client),
                'title'      => (string) $client,
                'id'         => $client->getId(),
                'domain'     => '',
                'domains'    => array(),
                'davdomain'  => '',
                'davdomains' => array()
            );

            $i = 0;

            foreach ($client->getDomains() as $domain) {
                $url = preg_replace('|^http(s)?://|i', '', $domain->url);

                if ($i == 0) {
                    $vars['domain'] = $url;
                }
                else {
                    $vars['domains'][] = $url;
                }

                $i++;
            }

            if (strlen($vars['domain']) == 0 && count($vars['domains']) > 0) {
                $vars['domain'] = array_shift($vars['domains']);
            }

            $i = 0;

            foreach ($client->getDavDomains() as $domain) {
                $url = preg_replace('|^http(s)?://|i', '', $domain->url);

                if ($i == 0) {
                    $vars['davdomain'] = $url;
                }
                else {
                    $vars['davdomains'][] = $url;
                }

                $i++;
            }

            if (strlen($vars['davdomain']) == 0 && count($vars['davdomains']) > 0) {
                $vars['davdomain'] = array_shift($vars['davdomains']);
            }

            return $vars;
        }

        protected static function _GetBrandVars($brand)
        {
            $vars = array(
                'ip'      => $_SERVER['SERVER_ADDR'],
                'docroot' => realpath(sprintf('%s/cp/htdocs', Application::GetApplicationDir())),
                'logroot' => realpath(sprintf('%s/logs', Application::GetDataDir())),
                'brand'   => Text::Urlize($brand),
                'title'   => (string) $brand,
                'id'      => $brand->getId(),
                'domain'  => '',
                'domains' => array()
            );

            $i = 0;

            foreach ($brand->getDomains() as $domain) {
                $url = preg_replace('|^http(s)?://|i', '', $domain->url);
                if ($i == 0) {
                    $vars['domain'] = $url;
                }
                else {
                    $vars['domains'][] = $url;
                }

                $i++;
            }

            if (strlen($vars['domain']) == 0 && count($vars['domains']) > 0) {
                $vars['domain'] = array_shift($vars['domains']);
            }

            return $vars;
        }

        protected static function _GetConfig($item, $template)
        {
            if ($item instanceof Application_Brand) {
                $vars = self::_GetBrandVars($item);
            }
            else {
                $vars = self::_GetClientVars($item);
            }

            $view = new Application_View_String();

            foreach ($vars as $k => $v) {
                $view->assign($k, $v);
            }

            $config = $view->fetch($template);

            return (string) $config;
        }

        public static function RebuildAll()
        {
            $ret = array(
                'messages' => array(),
                'errors'   => array()
            );

            $clients = Application_Client::GetClients();

            foreach ($clients as $client) {
                try {
                    self::Rebuild($client, false);
                    $ret['messages'][] = sprintf('Client %s: Config rebuilt', $client);
                }
                catch (Exception $ex) {
                    $ret['errors'][] = sprintf('Client %s: %s', $client, $ex->getMessage());
                }
            }

            $brands = Application_Brand::GetBrands();

            foreach ($brands as $brand) {
                try {
                    self::Rebuild($brand, false);
                    $ret['messages'][] = sprintf('Brand %s: Config rebuilt', $client);
                }
                catch (Exception $ex) {
                    $ret['errors'][] = sprintf('Brand %s: %s', $client, $ex->getMessage());
                }
            }

            self::Touch();

            return $ret;
        }
    }
?>