<?php
    /**
     * Used for installing the application fresh
     *
     * @package     Application
     */
    class Application_Installer
    {
        protected static $_downloadPortalUser;

        private function __construct()
        { }

        public static function InstallFromRequest($request)
        {
            $report = self::ValidateInstallFromRequest($request);

            if (count($report['errors']) > 0) {
                return $report;
            }

            $db = $report['db']['connection'];
            $db->begin();

            Application::SetDb($db);
            Application_Event_Manager::Disable();

            try {
                Application_Installer_Database::installCoreSchema($db);
            }
            catch (Exception $ex) {
                $db->rollback();
                $report['errors'][] = $ex->getMessage();
                return $report;
            }

            $config = array(
                'database' => array(
                    'dsn'    => $db->getDsn()
                )
            );

            Application_Upgrader_Manager::SetDatabaseVersion();

            $report['user'] = self::_CreateAdminUser($options['user']);
            self::_CreateBranding();

            Application_Config::write(Application::getConfigFilename(), Application::getConfigSection(), $config);

            Application_SystemStatus::EnableMaintenanceMode();
            Application_Event_Manager::Enable();

            if (is_array(self::$_downloadPortalUser)) {
                $user = self::$_downloadPortalUser;

                if ($user['create_user']) {
                    try {
                        $newUser = Application_Upgrader_Manager::CreateAccount($user['name'], $user['email']);
                        $user['username'] = $newUser['username'];
                        $user['password'] = $newUser['password'];
                    }
                    catch (Exception $ex) {
                        // something failed, don't stop user from installing
                    }
                }

                if (strlen($user['username']) > 0 && strlen($user['password']) > 0) {
                    Application_Upgrader_Manager::SetSettings(
                        $user['username'],
                        $user['password'],
                        true,
                        array('stable')
                    );
                }
            }

            return $report;
        }

        protected static function _CreateAdminUser()
        {
            $username = 'administrator';
            $password = Text_Password::Create(8);

            $sendWelcomeEmail = false;

            $user = new Application_User();
            $user->username = $username;
            $user->setPassword($password);

            if (is_array(self::$_downloadPortalUser) && strlen(self::$_downloadPortalUser['email']) > 0) {
                $user->profile->email = self::$_downloadPortalUser['email'];
                $sendWelcomeEmail = true;
            }

            $user->save();

            if ($sendWelcomeEmail) {
                try {
                    $template = Smarty::Readfile(sprintf('%s/admin/templates/install/admin-welcome-email.tpl', Application::GetApplicationDir()));
                    $lines = explode("\n", $template);

                    $subject = trim(array_shift($lines));
                    $body    = trim(join("\n", $lines));

                    $options = new Application_Mail_Options();
                    $options->assign('url', HTTP_URL::GenerateCurrentUrl(true));
                    $options->setTemplate(
                        $subject,
                        $body
                    );

                    $options->setFrom(new Application_Mail_Recipient('support@recitecms.com', 'Recite CMS Support'));

                    $user->sendEmail($options);
                }
                catch (Exception $ex) {
                }
            }

            $role = new Application_User_Role();
            $role->title = Application_Admin_Auth::$role;
            $role->save();

            $user->roleAdd($role->getId());

            return array(
                'username' => $username,
                'password' => $password
            );
        }

        protected static function _CreateBranding()
        {
            $brand = new Application_Brand();
            $brand->brand_name = Application::NAME;
            $brand->save();
        }

        public static function GetInstallationReport()
        {
            $php = array(
                'has'   => phpversion(),
                'needs' => '5.2.4'
            );

            $php['valid'] = version_compare($php['has'], $php['needs']) >= 0;

            $db = array(
                'has'       => Application_Installer_Database::getAvailableAdapters(),
                'available' => Application_Installer_Database::getAllAdapters()
            );

            $report = array(
                'db'         => $db,
                'php'        => $php,
                'canInstall' => $php['valid'] && count($db['has']) > 0,
                'license'    => file_get_contents(sprintf('%s/../../../license.txt', dirname(__FILE__)))
            );

            return $report;
        }

        public static function ValidateInstallFromRequest($request)
        {
            $ret = array(
                'errors' => array()
            );

            $report = Application_Installer::GetInstallationReport();

            if (!$report['canInstall']) {
                $ret['errors'][] = 'Server does not meet pre-requisites';
            }
            else {
                $accountType = $request->getPost('acct_type');

                $dpUser = array(
                    'create_user' => false,
                    'name'        => '',
                    'username'    => '',
                    'password'    => '',
                    'first_name'  => '',
                    'last_name'   => '',
                    'email'       => ''
                );

                if ($accountType == 'new') {
                    $accountName  = $request->getPost('dp_name');
                    $accountEmail = $request->getPost('dp_email');

                    $dpUser['email'] = $accountEmail;

                    $createUser = true;

                    if (strlen($accountName) == 0 && strlen($accountEmail) == 0) {
                        $ret['errors'][] = 'Please enter your name and email address for the download portal';
                        $createUser = false;
                    }
                    else if (strlen($accountName) == 0) {
                        $ret['errors'][] = 'Please enter your name for the download portal';
                        $createUser = false;
                    }
                    else if (strlen($accountEmail) == 0) {
                        $ret['errors'][] = 'Please enter your email address for the download portal';
                        $createUser = false;
                    }

                    if (strlen($accountEmail) > 0) {
                        $validator = new Zend_Validate_EmailAddress();

                        if (!$validator->isValid($accountEmail)) {
                            $ret['errors'][] = 'Please enter a valid email address for the download portal';
                            $createUser = false;
                        }
                        else {
                            $dpUser['email'] = $accountEmail;
                        }
                    }

                    if ($createUser) {
                        $dpUser['name'] = $accountName;
                        $dpUser['create_user'] = true;
                    }
                }
                else if ($accountType == 'existing') {
                    $accountUsername = $request->getPost('dp_username');
                    $accountPassword = $request->getPost('dp_password');

                    $dpUser['username'] = $accountUsername;
                    $dpUser['password'] = $accountPassword;

                    $canVerify = true;

                    if (strlen($accountUsername) == 0 && strlen($accountPassword) == 0) {
                        $ret['errors'][] = 'Please enter your download portal username and password';
                        $canVerify = false;
                    }
                    else if (strlen($accountUsername) == 0) {
                        $ret['errors'][] = 'Please enter your download portal username';
                        $canVerify = false;
                    }
                    else if (strlen($accountPassword) == 0) {
                        $ret['errors'][] = 'Please enter your download portal password';
                        $canVerify = false;
                    }

                    if ($canVerify) {
                        try {
                            $details = Application_Upgrader_Manager::GetAccountDetails($accountUsername, $accountPassword);

                            if (array_key_exists('email', $details)) {
                                $dpUser['email'] = $details['email'];
                            }
                        }
                        catch (Exception $ex) {
                            if ($ex->getCode() == 401) {
                                $ret['errors'][] = 'Please check your download portal username and password';
                            }
                            else {
                                // something failed, don't stop user from installing
                            }
                        }
                    }
                }
                else {
                    $ret['errors'][] = 'Please select an account type';
                }

                $options = array(
                    'adapter'  => $request->getPost('db_adapter'),
                    'dbname'   => $request->getPost('db_name'),
                    'username' => $request->getPost('db_username'),
                    'password' => $request->getPost('db_password'),
                    'host'     => $request->getPost('db_host')
                );

                $canConnectToDatabase = true;

                if (strlen($options['adapter']) == 0) {
                    $ret['errors'][] = 'Please select the database server type';
                    $canConnectToDatabase = false;
                }

                if (strlen($options['dbname']) == 0) {
                    $ret['errors'][] = 'Please enter the name of the database';
                    $canConnectToDatabase = false;
                }

                $ret['db'] = array(
                    'options'    => $options,
                    'connection' => null
                );

                if ($canConnectToDatabase) {
                    try {
                        $db = Application_Installer_Database::getDatabase($options);

                        if (!$db->isValidEncoding()) {
                            throw new Exception(sprintf('Invalid database encoding (%s)', $db->getEncoding()));
                        }

                        $ret['db']['connection'] = $db;
                    }
                    catch (Exception $ex) {
                        $ret['errors'][] = $ex->getMessage();
                    }
                }

                $license = (bool) $request->getPost('license');

                if (!$license) {
                    $ret['errors'][] = 'You must agree to the Recite CMS End User License Agreement';
                }

                if (count($ret['errors']) == 0) {
                    self::$_downloadPortalUser = $dpUser;
                }
            }

            return $ret;
        }
    }
?>