<?php
    /**
     * Manage database installation tasks
     *
     * @package     Application
     */
    class Application_Installer_Database
    {
        protected static $_dbAdapters = array(
            'Db_Adapter_mysql'      => 'MySQL',
            //'Db_Adapter_mssql'      => 'MSSQL',
            'Db_Adapter_postgresql' => 'PostgreSQL'
        );

        public static function getAllAdapters()
        {
            return self::$_dbAdapters;
        }

        public static function getAvailableAdapters()
        {
            $ret = array();

            foreach (self::$_dbAdapters as $class => $title) {
                $add = false;
                switch ($class) {
                    case 'Db_Adapter_mysql':
                        $add = extension_loaded('mysql');
                        break;

                    case 'Db_Adapter_postgresql':
                        $add = extension_loaded('pgsql');
                        break;

                    case 'Db_Adapter_mssql':
                        $add = extension_loaded('mssql');
                        break;
                }

                if ($add) {
                    $ret[$class] = $title;
                }
            }

            return $ret;
        }

        public static function getDatabase($options)
        {
            $class = coalesce($options['adapter']);

            if (!array_key_exists($class, self::$_dbAdapters)) {
                throw new Exception('Unknown adapter: ' . $adapter);
            }

            unset($options['adapter']);

            $ret = new $class($options);
            $ret->getConnection();

            return $ret;
        }

        public static function installCoreSchema($db)
        {
            $dir = realpath(dirname(__FILE__) . '/Database/Schema');
            self::installSchemaFromDirectory($db, $dir, __CLASS__ . '_Schema');
        }

        public static function getCoreSchemaTables($db)
        {
            $dir = realpath(dirname(__FILE__) . '/Database/Schema');

            return self::getSchemaTables($db, $dir, __CLASS__ . '_Schema');
        }

        public static function installSchemaFromDirectory($db, $dir, $baseClass)
        {
            $queue = self::getSchemaTables($db, $dir, $baseClass);

            $tries = 0;

            while (count($queue) > 0) {
                $table = array_shift($queue);

                if ($table->exists())
                    continue;

                if ($table->create()) {
                    $tries = 0;
                }
                else {
                    $tries++;
                    $queue[] = $table;
                }

                if ($tries > count($queue)) {
                    throw new Exception('Unable to install schema');
                }
            }

            $db->commit();
        }

        public static function getSchemaTables($db, $dir, $baseClass)
        {
            try {
                $files = new File_DirectoryListing($dir);
            }
            catch (Exception $ex) {
                return array();
            }

            $tables = array();
            foreach ($files->files as $file) {
                if (!preg_match('/^(.*)\.php$/', $file, $matches))
                    continue;

                $fullpath = sprintf('%s/%s', $files->dir, $file);

                require_once($fullpath);

                $class = sprintf('%s_%s', $baseClass, $matches[1]);

                if (!class_exists($class, false)) {
                    throw new Exception(sprintf('Class %s not found in %s', $class, $fullpath));
                }

                $table = new $class($db);

                if ($table->ignore()) {
                    continue;
                }

                $tables[$table->getName()] = $table;
            }

            return $tables;
        }

        public static function getAllTables($db)
        {
            $tables = array(
                '_core' => Application_Installer_Database::getCoreSchemaTables($db)
            );

            foreach (Application_Module_Manager::GetModules() as $module) {
                $tables[$module->getName()] = $module->getDatabaseTables($db);
            }

            return $tables;
        }

        public static function getTable($name)
        {
            foreach (self::getAllTables(Application::GetDb()) as $section) {
                foreach ($section as $table) {
                    if ($table->getName() == $name)
                        return $table;
                }
            }

            throw new Exception(sprintf('Table is not valid (%s)', $name));

        }
    }
?>