<?php
    class Application_License_Manager
    {
        const TYPE_DEVELOPER    = 'developer';
        const TYPE_UNRESTRICTED = 'unrestricted';

        public static $types = array(
            self::TYPE_DEVELOPER    => 'Developer',
            self::TYPE_UNRESTRICTED => 'Unrestricted'
        );

        public static function GetPathForDomain($domain)
        {
            $domain = preg_replace('|^[a-z]+://|', '', strtolower($domain));

            if (strlen($domain) == 0) {
                throw new Exception('Invalid domain');
            }

            $ret = sprintf('%s/%s', self::GetPath(), Text::Urlize($domain));

            if (!file_exists($ret)) {
                File::CreateDirStructure($ret);
            }

            return $ret;
        }

        public static function GetPath()
        {
            $ret = sprintf('%s/licenses', Application::GetDataDir());

            if (!file_exists($ret)) {
                File::CreateDirStructure($ret);
            }

            return $ret;
        }

        public static function GetLoaderVersion()
        {
            if (function_exists('ioncube_loader_version')) {
                return ioncube_loader_version();
            }

            throw new Exception('Unable to determine loader version');
        }

        public static function IsSupported()
        {
            if (IN_DEVELOPMENT) {
                return false;
            }

            return function_exists('ioncube_file_is_encoded') && ioncube_file_is_encoded();
        }

        public static function AssertLicensingSupported()
        {
            if (!self::IsSupported()) {
                throw new Exception('Licensing not supported');
            }
        }

        public static function GetInstalledLicenses()
        {
            $rii = new RecursiveIteratorIterator(new RecursiveDirectoryIterator(self::GetPath()), RecursiveIteratorIterator::CHILD_FIRST);
            $ret = array();

            foreach ($rii as $fullpath => $obj) {
                if (!$obj->isFile()) {
                    continue;
                }

                if (!preg_match('/\.key$/', $fullpath)) {
                    continue;
                }

                try {
                    $properties = self::GetLicenseProperties($fullpath);
                    $properties['filename'] = sprintf('%s/%s', $properties['domain'], $properties['product']);
                }
                catch (Exception $ex) {
                    continue;
                }

                $ret[] = $properties;
            }

            return $ret;
        }

        public static function GetLicensePath($filename)
        {
            $parts = explode('/', $filename, 2);
            $domain  = trim(coalesce($parts[0]));
            $product = trim(coalesce($parts[1]));

            if (strlen($domain) == 0 || strlen($product) == 0) {
                throw new Application_License_Exception('Invalid license filename');
            }

            $ret = sprintf('%s/%s/%s.key', self::GetPath(), Text::Urlize($domain), $product);

            if (!file_exists($ret)) {
                throw new Application_License_Exception('Unable to find license file');
            }

            return $ret;
        }

        public static function GetLicensePathsForDomainAndProduct($domain, $product)
        {
            $domain = preg_replace('|^[a-z]+://|', '', strtolower($domain));
            $url    = Text::Urlize($domain);

            $domains = array($url);

            if (substr($domain, 0, 4) == 'www.') {
                $domains[] = substr($url, 4);
            }
            else {
                $domains[] = 'www-' . $url;
            }

            return self::GetLicensePathsForDomainsAndProduct($domains, $product);
        }

        public static function GetLicensePathsForClientAndProduct(Application_Client $client, $product)
        {
            $domains = array();

            foreach ($client->getDomains() as $domain) {
                $url = Text::Urlize(
                    preg_replace('|^[a-z]+://|', '', strtolower($domain->url))
                );

                $domains[] = $url;
            }

            return self::GetLicensePathsForDomainsAndProduct($domains, $product);
        }

        public static function GetLicensePathsForDomainsAndProduct($domains, $product)
        {
            $ret = array();

            foreach ($domains as $domain) {
                try {
                    $ret[] = self::GetLicensePath(sprintf('%s/%s', $domain, $product));
                }
                catch (Exception $ex) {
                }
            }

            return $ret;
        }

        public static function GetLicensePathsForDomain($domain)
        {
            $domain = preg_replace('|^[a-z]+://|', '', strtolower($domain));
            $url    = Text::Urlize($domain);

            $domains = array($url);

            if (substr($domain, 0, 4) == 'www.') {
                $domains[] = substr($url, 4);
            }
            else {
                $domains[] = 'www-' . $url;
            }

            $ret = array();

            foreach ($domains as $domain) {
                try {
                    $path = self::GetPathForDomain($domain);
                }
                catch (Exception $ex) {
                    continue;
                }

                foreach (glob($path . '/*.key') as $fullpath) {
                    $ret[] = $fullpath;
                }
            }

            return $ret;
        }

        public static function GetLicenseProperties($fullpath)
        {
            if (IN_DEVELOPMENT) {
                $ret = array(
                    'name'         => 'Recite Developer',
                    'domain'       => 'www.dev.recite.com.au',
                    'producttitle' => 'Recite CMS Version 2',
                    'product'      => 'recite-cms-2',
                    'type'         => 'developer',
                    'expiry'       => null
                );
            }
            else {
                if (!file_exists($fullpath)) {
                    throw new Application_License_Exception('License file not found');
                }

                if (!is_file($fullpath)) {
                    throw new Application_License_Exception('License path is not a file');
                }

                if (!is_readable($fullpath)) {
                    throw new Application_License_Exception('License file is not readable');
                }

                $wasEncrypted = false;
                $data = ioncube_read_file($fullpath, $wasEncrypted);

                if (!$wasEncrypted) {
                    throw new Application_License_Exception('Unable to read license file');
                }

                $ret = array();
                foreach (explode("\n", $data) as $line) {
                    $pair = explode('=', $line, 2);
                    if (!is_array($pair) || count($pair) != 2) {
                        continue;
                    }

                    $k = trim($pair[0]);
                    $v = trim($pair[1]);

                    if (strlen($k) > 0 && strlen($v) > 0) {
                        $ret[$k] = $v;
                    }
                }
            }

            if (file_exists($fullpath)) {
                $ret['license'] = file_get_contents($fullpath);
            }
            else {
                $ret['license'] = '[no license data found]';
            }

            $ret['filename'] = sprintf('%s/%s', $ret['domain'], $ret['product']);

            return $ret;
        }

        public static function InstallLicense($data)
        {
            self::AssertLicensingSupported();

            $tmp = tempnam(Application::GetTmpDir(), 'recite-license-check');

            if ($tmp === false) {
                throw new Application_License_Exception('Unable to check license');
            }

            file_put_contents($tmp, $data);

            try {
                $info = self::GetLicenseProperties($tmp);
            }
            catch (Exception $ex) {
                unlink($tmp);
                throw $ex;
            }

            if (!array_key_exists('domain', $info)) {
                throw new Application_License_Exception('Unable to find domain data in license');
            }

            $domains   = explode(',', $info['domain']);
            $product   = $info['product'];
            $fullpaths = array();

            foreach ($domains as $domain) {
                $fullpath = sprintf('%s/%s.key', self::GetPathForDomain($domain), $product);

                if (file_exists($fullpath)) {
                    throw new Application_License_Exception(sprintf('License file already exists for domain %s', $info['domain']));
                }

                $fullpaths[] = $fullpath;
            }

            foreach ($fullpaths as $fullpath) {
                file_put_contents($fullpath, $data);
            }

            unlink($tmp);

            return $info;
        }

        public static function Delete($name)
        {
            $fullpath = self::GetLicensePath($name);
            unlink($fullpath);
        }

        public static function IsValidLicenseType($type)
        {
            return array_key_exists($type, self::$types);
        }

        public static function GetClientSummary()
        {
            if (!self::IsSupported()) {
                return array();
            }

            $clients = Application_Client::GetClients();

            $ret = array();

            foreach ($clients as $client) {
                $ret[] = array(
                    'client'   => $client,
                    'licenses' => self::GetClientLicenseSummary($client)
                );
            }

            return $ret;
        }

        public static function GetClientLicenseSummary(Application_Client $client)
        {
            $needsLicense = array(
                'recite-cms-2' => 'Recite CMS'
            );

            $ret = array();

            $paths = array();

            foreach ($client->getDomains() as $domain) {
                foreach (self::GetLicensePathsForDomain($domain) as $path) {
                    $paths[] = $path;
                }
            }

            foreach (array_unique($paths) as $path) {
                $props = self::GetLicenseProperties($path);
                $productName = $props['product'];

                if (!array_key_exists($productName, $ret)) {
                    $ret[$productName] = array(
                        'title'      => $props['producttitle'],
                        'hasActive'  => false,
                        'hasExpired' => false
                    );
                }

                if ($ret[$productName]['hasActive']) {
                    continue;
                }

                $ts = (int) $props['expiryts'];

                if ($ts == 0 || $ts > time()) {
                    $ret[$productName]['hasActive']  = true;
                    $ret[$productName]['hasExpired'] = false;
                }
                else {
                    $ret[$productName]['hasExpired'] = true;
                }
            }

            foreach ($needsLicense as $k => $v) {
                if (!array_key_exists($k, $ret)) {
                    $ret[$k] = array(
                        'title'      => $v,
                        'hasActive'  => false,
                        'hasExpired' => false
                    );
                }
            }

            $ret['core'] = $ret['recite-cms-2'];

            return $ret;
        }
    }
?>