<?php
    /**
     * Class used to build options for sending an email
     */
    class Application_Mail_Options
    {
        protected $_view;
        protected $_viewData = array();
        protected $_recipients = array();
        protected $_headers = array();
        protected $_testMode = false;
        protected $_attachments = array();
        protected $_template;
        protected $_attachInlineImages = false;

        public function __construct()
        { }


        /**
         * Set the view used to build the email body
         *
         * @param   Application_View    $view
         * @return  self
         */
        public function setView(Application_View $view)
        {
            $this->_view = $view;
            return $this;
        }


        public function getView()
        {
            if (!$this->_view instanceof Application_View) {
                throw new Application_Mail_Exception('View not set');
            }

            return $this->_view;
        }


        /**
         * Set the template to use for sending an email. This can be either
         * a sample or an actual template from the template manager
         *
         * @param   Module_Templates_Template|Module_Templates_Template_Sample  $template
         * @return  self
         */
        public function setTemplate($template, $textBody = null, $htmlBody = null)
        {
            if ($template instanceof Module_Templates_Template) {
                $type = $template->getType();

                if (!$type instanceof Module_Templates_Template_Type_Email) {
                    throw new Application_Mail_Exception('Template must be of type Module_Templates_Template_Type_Email');
                }

                $this->_template = $template;
            }
            else if ($template instanceof Module_Templates_Template_Sample) {
                $type = new Module_Templates_Template_Type_Email(null, null);

                $segments = array();

                foreach ($type->getSegments() as $segment) {
                    $segments[$segment->getName()] = false;
                }

                foreach ($type->getSegments() as $segment) {
                    if (!array_key_exists($segment->getName(), $segments)) {
                        throw new Application_Mail_Exception('Invalid segment found');
                    }
                    else {
                        $segments[$segment->getName()] = true;
                    }
                }

                foreach ($segments as $bool) {
                    if (!$bool) {
                        throw new Application_Mail_Exception('Invalid segments found');
                    }
                }

                $this->_template = $template;
            }
            else {
                $this->_template = array(
                    'subject' => $template,
                    'text'    => $textBody,
                    'html'    => $htmlBody
                );
            }

            return $this;
        }

        public function getTemplate()
        {
            return $this->_template;
        }


        /**
         * Set data that will be assigned to the view when the email is created
         *
         * @param   string  $k
         * @param   string  $v
         * @return  self
         */
        public function assign($k, $v)
        {
            $this->_viewData[$k] = $v;
            return $this;
        }


        public function getAssignedData()
        {
            return $this->_viewData;
        }


        /**
         * Set who the email is from. This can be either a mail recipient or a user directory
         *
         * @param   Application_Mail_Recipient|Application_User_Directory   $from
         * @return  self
         */
        public function setFrom($from)
        {
            if ($from instanceof Application_Mail_Recipient) {
                $this->_from = $from;
            }
            else if ($from instanceof Application_User_Directory) {
                $this->_from = $from;
            }
            else if ($from instanceof Application_Client) {
                $this->_from = $from;
            }
            else {
                throw new Application_Mail_Exception('Invalid from object');
            }

            return $this;
        }

        public function getFrom()
        {
            if (is_null($this->_from)) {
                throw new Application_Mail_Exception('From not set');
            }

            return $this->_from;
        }

        public function addTo($to)
        {
            $this->addRecipient($to, Application_Mail_Recipient::TYPE_TO);
            return $this;
        }

        public function addCc($to)
        {
            $this->addRecipient($to, Application_Mail_Recipient::TYPE_CC);
            return $this;
        }

        public function addBcc($to)
        {
            $this->addRecipient($to, Application_Mail_Recipient::TYPE_BCC);
            return $this;
        }

        public function addRecipient($to, $type)
        {
            if ($to instanceof Application_User) {
                $to = new Application_Mail_Recipient($to->getEmailAddress(), $to->getFullName());
            }
            else if (!$to instanceof Application_Mail_Recipient) {
                $to = new Application_Mail_Recipient($to);
            }
            $to->setType($type);
            $this->_recipients[] = $to;
            return $this;
        }

        public function getNumRecipients()
        {
            return count($this->_recipients);
        }

        public function getRecipients()
        {
            return $this->_recipients;
        }

        public function clearRecipients()
        {
            $this->_recipients = array();
            return $this;
        }

        public function addHeader($k, $v)
        {
            $this->_headers[$k] = $v;
            return $this;
        }

        public function getHeaders()
        {
            return $this->_headers;
        }

        public function enableTestMode($flag = true)
        {
            $this->_testMode = (bool) $flag;
            return $this;
        }

        public function inTestMode()
        {
            return $this->_testMode;
        }

        public function addAttachment($attachment)
        {
            if ($attachment instanceof Module_Assets_Asset) {
                $file = $attachment;
                $fp = fopen($file->getFullPath(), 'r');

                if ($fp === false) {
                    throw new Exception('Unable to open file');
                }

                $attachment = new Zend_Mime_Part($fp);
                $attachment->filename    = (string) $file;
                $attachment->disposition = Zend_Mime::DISPOSITION_ATTACHMENT;
                $attachment->encoding    = Zend_Mime::ENCODING_BASE64;
            }

            if (!$attachment instanceof Zend_Mime_Part) {
                throw new Exception('Attachment must be an instanceof Zend_Mime_Part or Module_Assets_Asset');
            }

            $this->_attachments[] = $attachment;
            return $this;
        }

        public function getAttachments()
        {
            return $this->_attachments;
        }

        public function setAttachInlineImages($flag = true)
        {
            $this->_attachInlineImages = (bool) $flag;
            return $this;
        }

        public function attachInlineImages()
        {
            return $this->_attachInlineImages;
        }
    }
?>