<?php
    class Application_Mail_Sender
    {
        private function __construct()
        { }


        /**
         * Send an email
         *
         * @param   Application_Mail_Options    $options
         * @return  Application_Mail_Response   $response
         */
        public static function Send(Application_Mail_Options $options)
        {
            if ($options->getNumRecipients() == 0) {
                throw new Application_Mail_Exception('No recipient email addresses specified');
            }

            $mail = new Application_Mail();
            $response = new Application_Mail_Response($mail);

            try {
                $from = $options->getFrom();

                if ($from instanceof Application_User_Directory) {
                    $mail->fromDirectory($from);
                }
                else if ($from instanceof Application_Client) {
                    $mail->fromClient($from);
                }
                else {
                    $mail->setFrom($from->getEmail(), $from->getName());
                }
            }
            catch (Exception $ex)
            { }

            foreach ($options->getRecipients() as $recipient) {
                $response->addRecipient($recipient);
                switch ($recipient->getType()) {
                    case Application_Mail_Recipient::TYPE_TO:
                        $mail->addTo($recipient->getEmail(), $recipient->getName());
                        break;

                    case Application_Mail_Recipient::TYPE_CC:
                        $mail->addCc($recipient->getEmail(), $recipient->getName());
                        break;

                    case Application_Mail_Recipient::TYPE_BCC:
                        $mail->addBcc($recipient->getEmail());
                        break;
                }
            }

            foreach ($options->getHeaders() as $k => $v) {
                if (is_array($v)) {
                    foreach ($v as $_v) {
                        $mail->addHeader($k, $v, true);
                    }
                }
                else {
                    $mail->addHeader($k, $v);
                }
            }

            // attach files as required
            foreach ($options->getAttachments() as $attachment) {
                $mail->addAttachment($attachment);
            }

            // now build the email subject and body
            $template = $options->getTemplate();

            try {
                $view = $options->getView();
            }
            catch (Exception $ex) {
                $view = new Application_View_Cp();
                $view->getEngine()->unregister_outputfilter('trimwhitespaceimproved');
                $view->getEngine()->unregister_outputfilter('trimwhitespace');
            }

            foreach ($options->getAssignedData() as $k => $v) {
                $view->__set($k, $v);
            }

            if ($template instanceof Module_Templates_Template_Sample || $template instanceof Module_Templates_Template) {
                if ($template instanceof Module_Templates_Template && !$template->isLive()) {
                    throw new Application_Mail_Exception('Specified template is marked as deleted');
                }

                foreach ($template->getSegments() as $segment) {
                    switch ($segment->getName()) {
                        case 'subject':
                            $subject = trim($view->fetchFromString($segment->getData()));
                            $mail->setSubject($subject);
                            $response->setSubject($subject);
                            break;

                        case 'text':
                            $text = trim($view->fetchFromString($segment->getData()));

                            if (strlen($text) > 0) {
                                $mail->setBodyText($text);
                                $response->setBodyText($text);
                            }
                            break;

                        case 'html':
                            $html = trim($view->fetchFromString($segment->getData()));
                            $html = self::_SetBodyHtmlAndImages($mail, $html, $options->attachInlineImages());

                            if (strlen($html) > 0) {
                                $response->setBodyHtml($html);
                            }
                            break;
                    }
                }
            }
            else if (is_array($template)) {
                $subject = trim($view->fetchFromString($template['subject']));
                $mail->setSubject($subject);
                $response->setSubject($subject);

                $text = trim($view->fetchFromString($template['text']));
                if (strlen($text) > 0) {
                    $mail->setBodyText($text);
                    $response->setBodyText($text);
                }

                $html = trim($view->fetchFromString($template['html']));
                if (strlen($html) > 0) {
                    $mail->setBodyHtml($html);
                    $response->setBodyHtml($html);
                }
            }

            if ($options->inTestMode()) {
                $response->setTestMode();
            }
            else {
                try {
                    $mail->send();
                }
                catch (Exception $ex) {
                    $response->addError($ex->getMessage());
                }
            }

            return $response;
        }

        protected static function _SetBodyHtmlAndImages(Application_Mail $mail, $html, $attach = true)
        {
            $html = trim($html);

            if (strlen($html) == 0) {
                return;
            }

            // only process images for actual clients
            try {
                Application::GetClient();
            }
            catch (Exception $ex) {
                $mail->setBodyHtml($html);
                return $html;
            }

            // find all the images
            preg_match_all('|<img([^>]*)src="(.*?)"[^>]*>|i', $html, $matches);

            $_images = array_unique($matches[2]);
            $images = array();
            foreach ($_images as $image) {
                if (!preg_match('|^http(s?)://|', $image)) {
                    $images[] = $image;
                }
            }

            if (count($images) == 0) {
                $mail->setBodyHtml($html);
                return $html;
            }

            $map   = array();
            $files = array();

            foreach ($images as $path) {
                $file = new Module_Assets_Asset();
                $file->loadRecordByPath(preg_replace('/^assets/', '', trim($path, '/')));

                if (!$file->saved()) {
                    continue;
                }

                $type = $file->getType();
                if (!$type->isImage()) {
                    continue;
                }

                $files[$path] = $file;
            }

            if (count($images) == 0) {
                $mail->setBodyHtml($html);
                return $html;
            }

            if ($attach) {
                $mail->setType(Zend_Mime::MULTIPART_RELATED);

                $i = 0;
                foreach ($files as $path => $file) {
                    $fp = fopen($file->getFullPath(), 'r');

                    if ($fp === false) {
                        continue;
                    }

                    $id = $path;//sprintf('recite-%d', ++$i);

                    $attachment = new Zend_Mime_Part($fp);
                    $attachment->encoding    = Zend_Mime::ENCODING_BASE64;
                    $attachment->id          = $id;
                    $attachment->filename    = (string) $file;
                    $attachment->location    = $path;
                    $attachment->disposition = Zend_Mime::DISPOSITION_INLINE;

                    $mail->addAttachment($attachment);

                    $map[$path] = $id;
                }
            }
            else {
                $client = Application::GetClient();
                $url    = $client->getPrimaryDomain();

                foreach ($files as $path => $file) {
                    $map[$path] = sprintf('%s%s', $url, $file->getWebPath());
                }
            }

            foreach ($map as $src => $dst) {
                $html = str_replace($src, $dst, $html);
            }

            if ($attach) {
                $mail->setBodyHtml($html, null, Zend_Mime::MULTIPART_RELATED);
            }
            else {
                $mail->setBodyHtml($html);
            }

            return $html;
        }
    }
?>