<?php
    /**
     * Manages any maintenance tasks that need to be run
     *
     * @since   Recite 1.7.0
     * @author  Quentin Zervaas
     */
    class Application_Maintenance
    {
        const TABLE   = 'application_maintenance';
        const MAX_AGE = 1200;

        protected static $_db;

        private function __construct()
        { }

        public static function getDb()
        {
            if (!self::$_db instanceof Db_Abstract) {
                self::$_db = Zend_Registry::get('db');
            }

            return self::$_db;
        }

        public static function Factory($class)
        {
            if ($class instanceof Application_Maintenance_Abstract) {
                return $class;
            }

            if (strlen($class) == 0) {
                throw new Application_Maintenance_Exception('Maintenance action class not specified');
            }

            Application_Loader::AssertCanLoadClass($class);

            if (!class_exists($class)) {
                throw new Application_Maintenance_Exception('Class not found: ' . $class);
            }

            $reflection = new ReflectionClass($class);

            if (!$reflection->isSubclassOf('Application_Maintenance_Abstract')) {
                throw new Application_Maintenance_Exception('Class does not extend Application_Maintenance_Abstract: ' . $class);
            }

            return new $class();
        }

        public static function Queue($item)
        {
            if (!$item instanceof Application_Maintenance_QueueItem) {
                $item = new Application_Maintenance_QueueItem($item);
            }

            try {
                $controller = $item->getController();
            }
            catch (Exception $ex) {
                return;
            }

            if (!$item->canHaveDuplicates() && self::Queued($item)) {
                return;
            }

            try {
                $item->setClient(Application::GetClient());
            }
            catch (Exception $ex)
            { }

            if ($controller->accept($item->getParams())) {
                $message = new Application_Auditor_Message('Application: Maintenance action queued');
                $message->addMemo('action', $item->getActionName())
                        ->record();

                $values = array(
                    'action_name' => $item->getActionName(),
                    'params'      => serialize($item->getParams()),
                    'ts_created'  => new Zend_Db_Expr('now()')
                );

                $db = self::getDb();

                $ts = (int) $item->getCreatedTimestamp();

                if ($ts <= 0) {
                    $values['ts_created'] = new Zend_Db_Expr('now()');
                }
                else {
                    $values['ts_created'] = $db->getStatementHelper()->getTimestamp($ts);
                }

                try {
                    $values['client_id'] = $item->getClient()->getId();
                }
                catch (Exception $ex)
                { }

                $db->insert(self::getTable(), $values);
            }
        }

        public static function Queued(Application_Maintenance_QueueItem $item)
        {
            $db = self::getDb();

            $select = $db->select();
            $select->from(self::getTable(), '*')
                   ->where('action_name = ?', $item->getActionName())
                   ->where('ts_running is null');

            try {
                $select->where('client_id = ?', $item->getClient()->getId());
            }
            catch (Exception $ex) {
                $select->where('client_id is null');
            }

            $result = $db->query($select);

            while ($row = $result->fetch()) {
                if (strlen($row->params) == 0) {
                    $row->params = array();
                }
                else {
                    $row->params = unserialize($row->params);
                }

                if (!is_array($row->params)) {
                    $row->params = array();
                }

                if (count(array_diff($item->getParams(), $row->params)) == 0) {
                    return true;
                }
            }

            return false;
        }

        public static function Get($id)
        {
            $query = sprintf('select * from %s where update_id = %d', self::getTable(), $id);
            $result = self::getDb()->query($query);

            if ($row = $result->fetch()) {
                if (strlen($row->params) == 0) {
                    $params = array();
                }
                else {
                    $params = unserialize($row->params);
                }

                if (!$params) {
                    $params = array();
                }

                $ret = new Application_Maintenance_QueueItem($row->action_name);
                $ret->setCreatedTimestamp($row->ts_created)
                    ->setRunningTimestamp($row->ts_running)
                    ->setParams($params)
                    ->setId($row->update_id);

                $client = new Application_Client();
                $client->load($row->client_id);

                if ($client->saved()) {
                    $ret->setClient($client);
                }

                return $ret;
            }

            throw new Application_Maintenance_Exception(sprintf('System Maintenance: Unable to find queue item: %d', $id));
        }

        public static function GetNextPendingQueueItem()
        {
            $db = self::getDb();
            $select = $db->select();
            $select->from(self::getTable(), '*')
                   ->where('ts_running is null')
                   ->where('ts_created <= now()')
                   ->order('ts_created')
                   ->order('update_id')
                   ->limit(1);

            $result = $db->query($select);

            if ($row = $result->fetch()) {
                if (strlen($row->params) == 0) {
                    $params = array();
                }
                else {
                    $params = unserialize($row->params);
                }

                if (!$params) {
                    $params = array();
                }

                try {
                    $ret = new Application_Maintenance_QueueItem($row->action_name);
                }
                catch (Exception $ex) {
                    self::Unqueue($row->update_id);
                    throw new Application_Maintenance_Exception('Invalid action');
                }

                $ret->setCreatedTimestamp($row->ts_created)
                    ->setRunningTimestamp($row->ts_running)
                    ->setParams($params)
                    ->setId($row->update_id);

                $client = new Application_Client();
                $client->load($row->client_id);

                if ($client->saved()) {
                    $ret->setClient($client);
                }

                return $ret;
            }

            throw new Application_Maintenance_Exception('No queue items remaining');
        }

        public static function Unqueue($id)
        {
            $message = new Application_Auditor_Message('Application: Maintenance action unqueued');
            $message->addMemo('id', $id)
                    ->record();

            $query = sprintf('delete from %s where update_id = %d', self::getTable(), $id);
            self::getDb()->query($query);

            return true;
        }

        public static function Run()
        {
            self::deleteStaleEntries();

            $performed = array();

            while (1) {
                try {
                    $item = self::GetNextPendingQueueItem();
                }
                catch (Exception $ex) {
                    break;
                }

                self::markAsStarted($item->getId());

                $hash = md5($item->getActionName() . ',' . serialize($item->getParams()));

                try {
                    $hash = $item->getClient()->getId() . ',' . $hash;
                }
                catch (Exception $ex)
                { }

                // don't perform the same action twice
                if (!$item->canHaveDuplicates() && array_key_exists($hash, $performed)) {
                    self::markAsFinished($item->getId());
                    continue;
                }

                $performed[$hash] = $item;

                try {
                    $controller = $item->getController();
                }
                catch (Exception $ex) {
                    self::markAsFinished($item->getId());
                    continue;
                }

                try {
                    $client = $item->getClient();

                    if (!$client->saved()) {
                        throw new Exception('Client not loaded');
                    }

                    Application::SetClient($client);
                }
                catch (Exception $ex) {
                    Application::ClearClient();
                }

                $message = new Application_Auditor_Message('Application: Performing maintenance action: ' . $controller);
                $message->record();

                try {
                    ob_start();
                    $controller->run($item->getParams());
                    $performed[$hash]->output = ob_get_clean();

                }
                catch (Exception $ex) {
                    ob_end_clean();
                    $performed[$hash]->output = $ex->getMessage();
                }

                self::markAsFinished($item->getId());

                Application::ClearClient();
            }

            if (count($performed) > 0) {
                self::SendReport($performed);
            }

            $profile = new Application_Settings();
            $profile->load();
            $profile->maintenance_last_ts = time();
            $profile->save();
        }

        public static function GetLastRunTs()
        {
            $profile = new Application_Settings();
            $profile->load();

            $ts = trim($profile->maintenance_last_ts);

            if (strlen($ts) == 0) {
                throw new Application_Maintenance_Exception('Maintenance queue never run');
            }

            return $ts;
        }

        public static function GetLastRunElapsed()
        {
            return max(0, time() - self::GetLastRunTs());
        }

        public static function getTable()
        {
            return self::TABLE;
        }

        public static function DeleteStaleEntries()
        {
            $db = self::getDb();
            $helper = self::$_db->getStatementHelper();

            $where = array(
                'ts_running is not null',
                $helper->getOlderThanExpression('ts_running', self::MAX_AGE, Zend_Date::SECOND)
            );

            $db->delete(self::getTable(), $where);
        }


        public static function Running()
        {
            $query = sprintf('select count(*) as num_hits from %s where ts_running is not null', self::getTable());
            return self::getDb()->fetchOne($query) > 0;
        }

        public static function GetQueue($pendingOnly = false)
        {
            $db = self::getDb();

            $select = $db->select();
            $select->from(self::getTable(), '*');

            if ($pendingOnly) {
                $select->where('ts_running is null');
            }

            $result = self::getDb()->query($select);

            $clients = Application_Client::GetClients();

            $ret = array();

            while ($row = $result->fetch()) {
                if (strlen($row->params) == 0) {
                    $params = array();
                }
                else {
                    $params = unserialize($row->params);
                }

                if (!$params) {
                    $params = array();
                }

                try {
                    $item = new Application_Maintenance_QueueItem($row->action_name);
                }
                catch (Exception $ex) {
                    self::Unqueue($row->update_id);
                    continue;
                }

                $item->setCreatedTimestamp(is_null($row->ts_created) ? null : strtotime($row->ts_created))
                     ->setRunningTimestamp(is_null($row->ts_running) ? null : strtotime($row->ts_running))
                     ->setParams($params)
                     ->setId($row->update_id);

                if (array_key_exists($row->client_id, $clients)) {
                    $item->setClient($clients[$row->client_id]);
                }

                $ret[$row->update_id] = $item;
            }

            return $ret;
        }

        protected static function MarkAsStarted($id)
        {
            $query = sprintf('update %s set ts_running = now() where update_id = %d', self::getTable(), $id);
            self::getDb()->query($query);
        }

        protected static function MarkAsFinished($id)
        {
            $query = sprintf('delete from %s where update_id = %d', self::getTable(), $id);
            self::getDb()->query($query);
        }

        public static function ClearForClient($clientId)
        {
            $query = sprintf('delete from %s where client_id = %d', self::getTable(), $clientId);
            self::getDb()->query($query);
        }

        protected static function SendReport($actions)
        {
            return;

            if (IN_DEVELOPMENT)
                return;

            $to = Zend_Registry::get('config')->admin->email;

            if (!isValidEmail($to))
                return;

            $ret = '';

            foreach ($actions as $action) {
                $ret .= sprintf("%s: %s\n\n", $action->action_name, $action->output);
            }

            $mail = new Application_Mail();
            $mail->addTo($to);
            $mail->setSubject(Application::GetTitle() . ' maintenance report');
            $mail->setBodyText($ret);

            $mail->send();
        }

        public static function GetProcessScriptPath()
        {
            return sprintf(
                '%s%stools%smaintenance%sprocess.php',
                Application::GetApplicationDir(),
                DIRECTORY_SEPARATOR,
                DIRECTORY_SEPARATOR,
                DIRECTORY_SEPARATOR
            );
        }

        public static function GetCliPath()
        {
            if (IS_WINDOWS) {
                return 'C:\php\php.exe';
            }
            else {
                return '/usr/local/bin/php';
            }
        }
    }
?>