<?php
    /**
     * The base abstract class that modules should extend from
     *
     * @package Application_Module
     * @author  Quentin Zervaas
     */
    abstract class Application_Module_Abstract implements Application_Module_Interface
    {
        private static $_allDepsStack = array();

        /**
         * Constructor
         */
        public function __construct()
        { }


        /**
         * Return the name of the driver
         *
         * @return string
         */
        public function getName()
        {
            preg_match('/^Module_(.+)$/', get_class($this), $matches);
            return $matches[1];
        }

        /**
         * Return the path of the driver
         *
         * @return string
         */
        public function getPath()
        {
            $reflection = new ReflectionObject($this);

            return realpath(dirname($reflection->getFileName()));
        }

        public function sendFileToCp(Application_Cp_Site_Response $response, $path)
        {
            $fullpath = $this->getFullPathForCpFile($path);

            $options = array(
                'response'    => $response,
                'sendexpires' => true
            );

            HTTP::SendFile($fullpath, $options);
        }

        public function sendFileToClient(Application_Client_Site_Response $response, $path)
        {
            $fullpath = $this->getFullPathForCpFile($path);

            $options = array(
                'response'    => $response,
                'sendexpires' => true
            );

            HTTP::SendFile($fullpath, $options);
        }

        public function getFullPathForCpFile($path)
        {
            $fullpath = realpath(sprintf('%s/files/%s', $this->getPath(), $path));

            if (strlen($fullpath) == 0) {
                throw new Application_Module_Exception('Unable to find file');
            }

            if (!is_readable($fullpath)) {
                throw new Application_Module_Exception('Unable to read file');
            }

            return $fullpath;
        }

        public function getTemplateTypes()
        {
            return array();
        }

        public function getTemplateType($name = null)
        {
            $fullName = get_class($this);

            if (strlen($name) > 0) {
                $fullName .= ':' . $name;
            }

            foreach ($this->getTemplateTypes() as $type) {
                if ($type->getName() == $fullName) {
                    return $type;
                }
            }

            throw new Module_Templates_Exception('Template type not found');
        }


        public function getDatabaseTables($db)
        {
            $path  = sprintf('%s/include/Schema', $this->getPath());
            $class = sprintf('Module_%s_Schema', ucfirst(strtolower($this->getName())));

            return Application_Installer_Database::getSchemaTables($db, $path, $class);
        }


        public function canInstallDatabaseTables($db)
        {
            $tables = self::getDatabaseTables($db);

            $deps = array();
            foreach ($tables as $name => $table) {
                $deps = array_merge($deps, $table->getDependencies());
            }

            $deps = array_diff(array_unique($deps), array_keys($tables));

            foreach ($deps as $table) {
                if (!$db->hasTable($table)) {
                    return false;
                }
            }
            return true;
        }

        public function installDatabaseTables($db)
        {
            $path = sprintf('%s/include/Schema', $this->getPath());
            $class = sprintf('Module_%s_Schema', ucfirst(strtolower($this->getName())));
            Application_Installer_Database::installSchemaFromDirectory($db, $path, $class);
        }

        public function getDependentModules()
        {
            return array();
        }

        public function getDriverTypes()
        {
            return array();
        }

        public function getDrivers()
        {
            $ret = array();

            foreach ($this->getDriverTypes() as $type) {
                foreach (Application_Driver_Manager::GetDriversByType($type) as $driver) {
                    $ret[$driver->getFullName()] = $driver;
                }
            }

            return $ret;
        }

        public function isClientModule()
        {
            return true;
        }

        final public function getAllDependentModules()
        {
            if (array_key_exists($this->getName(), self::$_allDepsStack)) {
                throw new Application_Module_Exception(sprintf('Circular dependency: %s', $this->getName()));
            }

            self::$_allDepsStack[$this->getName()] = true;
            $ret = array();

            foreach ($this->getDependentModules() as $name) {
                if ($name == $this->getName()) {
                    continue;
                }

                $ret[] = $name;

                $module = Application_Module_Manager::Factory($name);

                array_merge($ret, $module->getAllDependentModules());
            }

            unset(self::$_allDepsStack[$this->getName()]);
            return array_unique($ret);
        }
    }
?>
