<?php
    /**
     * The base interface that all modules must implement
     *
     * @package Application_Module
     * @author  Quentin Zervaas
     */
    interface Application_Module_Interface
    {
        /**
         * Constructor. Must not have any arguments
         */
        public function __construct();

        /**
         * Returns the title of the module
         *
         * @return  string
         */
        public function __toString();


        /**
         * Returns the internal name of the driver
         *
         * @return  string
         */
        public function getName();


        /**
         * Returns the path where the module lives. This must return the directory
         * in which the class implementing this interface resides
         *
         * @return  string
         */
        public function getPath();


        /**
         * Returns a list of modules this module relies on to function. This should
         * only be modules that would result in this module not working at all
         * without. The module may make use of other modules. Each element in the
         * returned array should be the internal name of the module (as returned by
         * the getName() method)
         *
         * @return  array
         */
        public function getDependentModules();


        /**
         * Returns a list of all modules this module relies on. This includes any
         * modules from getDependentModules() and any modules those modules
         * depend on
         *
         * @return array
         */
        public function getAllDependentModules();


        /**
         * Get a list of driver types this module makes use of
         *
         * @return  array
         */
        public function getDriverTypes();


        /**
         * Get a list of drivers this module has
         *
         * @return  array
         */
        public function getDrivers();


        /**
         * Get a list of template types for this module (this doesn't include any
         * template types for drivers that belong to this module since the driver
         * returns those separately)
         *
         * @return  Module_Templates_Template_Type[]
         */
        public function getTemplateTypes();


        /**
         * Get a template type based on its name
         *
         * @param   string  $name   The name of the template to return
         * @throws  Module_Templates_Exception      If the type can't be found
         * @return  Module_Templates_Template_Type
         */
        public function getTemplateType($name = null);


        /**
         * Whether or not the module can be assigned to clients
         *
         * @return  bool
         */
        public function isClientModule();


        /**
         * Get a list of database tables this module uses
         *
         * @return  Db_Schema_Table_Abstract[]
         */
        public function getDatabaseTables($db);


        /**
         * Whether or not the database tables can be created
         *
         * @return  bool
         */
        public function canInstallDatabaseTables($db);


        /**
         * Install database tables
         *
         */
        public function installDatabaseTables($db);


        /**
         * Send a file belonging to the module to the control panel user
         *
         * @param   Application_Cp_Site_Response    $response   The response object
         * @param   string                          $path       The path to send
         * @throws  Application_Module_Exception    If the file doesn't exist or can't be sent
         */
        public function sendFileToCp(Application_Cp_Site_Response $response, $path);
    }
?>