<?php
    /**
     * This class is used to manage modules, including fetching modules,
     * assigning and unassigning to and from clients
     *
     * @package Application_Module
     * @author  Quentin Zervaas
     */
    class Application_Module_Manager
    {
        const TABLE_BRANDS  = 'brands_modules';
        const TABLE_CLIENTS = 'clients_modules';

        private static $_interfaceReflection;
        private static $_cache = array();

        /**
         * Constructor. This class can't be instantiated
         */
        protected function __construct()
        { }


        /**
         * Get the path where modules are stored
         *
         * @return  string
         */
        public static function GetPath()
        {
            return realpath(dirname(__FILE__) . '/../../../../lib/modules');
        }


        /**
         * Get all installed modules
         *
         * @param   Application_Brand|Application_Client    $item   If specified only loads modules that belong to the brand or client
         * @return  Application_Module_Interface[]
         */
        public static function GetModules($item = null)
        {
            try {
                $fd = new File_Directory(self::GetPath());
            }
            catch (Exception $ex) {
                return array();
            }

            if ($item instanceof DatabaseObject) {
                $type = get_class($item);
                $key  = $item->getId();
            }
            else {
                $type = null;
            }

            if (strlen($type) > 0) {
                if (!array_key_exists($type, self::$_cache)) {
                    self::$_cache[$type] = array();
                }

                if (array_key_exists($key, self::$_cache[$type])) {
                    return self::$_cache[$type][$key];
                }
            }

            if ($item instanceof Application_Brand) {
                $db = Application::GetDb();
                $select = $db->select();
                $select->from(self::TABLE_BRANDS, 'module')
                       ->where('brand_id = ?', $item->getId());

                $allowed = $db->fetchCol($select);
            }
            else if ($item instanceof Application_Client) {
                $db = Application::GetDb();
                $select = $db->select();
                $select->from(self::TABLE_CLIENTS, 'module')
                       ->where('client_id = ?', $item->getId());

                $allowed = $db->fetchCol($select);
            }
            else {
                $allowed = null;
            }

            if (is_array($allowed) && count($allowed) == 0) {
                return array();
            }

            $ret = array();

            foreach ($fd->getDirs() as $dir) {
                if (is_array($allowed) && !in_array($dir, $allowed)) {
                    continue;
                }

                try {
                    $module = self::Factory($dir);

                    if ($item instanceof Application_Client && !$module->isClientModule()) {
                        continue;
                    }

                    $ret[$module->getName()] = $module;
                }
                catch (Exception $ex)
                { }
            }

            uasort($ret, array(__CLASS__, '_sortModules'));

            if (strlen($type) > 0) {
                self::$_cache[$type][$key] = $ret;
            }

            return $ret;
        }


        /**
         * Get items not assigned to the object passed
         *
         * @param   Application_Brand|Application_Client    $item
         * @return  Application_Module_Interface[]
         */
        public static function GetModulesNotAssigned($item)
        {
            if (!$item instanceof Application_Brand && !$item instanceof Application_Client) {
                throw new Application_Module_Exception(sprintf('Invalid item passed to %s::%s', __CLASS__, __FUNCTION__));
            }

            $modules = self::GetModules();

            foreach (self::GetModules($item) as $module) {
                unset($modules[$module->getName()]);
            }

            if ($item instanceof Application_Client) {
                foreach ($modules as $module) {
                    if (!$module->isClientModule()) {
                        unset($modules[$module->getName()]);
                    }
                }
            }

            return $modules;
        }


        public static function SetModules($item, $modules)
        {
            $db = Application::GetDb();
            $db->begin();

            $newModules = array();

            foreach (ArrayTools::clean($modules) as $module) {
                $module = self::Factory($module);
                $newModules[$module->getName()] = $module;
            }

            $alreadyAssigned = self::GetModules($item);

            $add    = array();
            $remove = array();

            foreach ($newModules as $module) {
                if (!array_key_exists($module->getName(), $alreadyAssigned)) {
                    $add[] = $module;
                }
            }

            foreach ($alreadyAssigned as $module) {
                if (!array_key_exists($module->getName(), $newModules)) {
                    $remove[] = $module;
                }
            }

            foreach ($remove as $module) {
                self::UnassignModule($item, $module);
            }

            if (count($add) > 0) {
                self::AssignModules($item, $add);
            }

            $db->commit();
        }



        /**
         * Assign a module to the object passed
         *
         * @param   Application_Brand|Application_Client    $item
         * @param   Application_Module_Interface|string     $module The module to assign
         * @throws  Application_Module_Exception            If the item already has the module
         */
        public static function AssignModule($item, $module, $deployPermissions = true)
        {
            $module = self::Factory($module);
            $db = Application::GetDb();

            if ($item instanceof Application_Brand) {
                $table  = self::TABLE_BRANDS;
                $column = 'brand_id';
            }
            else if ($item instanceof Application_Client) {
                $table  = self::TABLE_CLIENTS;
                $column = 'client_id';
            }
            else {
                throw new Application_Module_Exception(sprintf('Invalid item passed to %s::%s', __CLASS__, __FUNCTION__));
            }

            $select = $db->select();
            $select->from($table, 'count(*)')
                   ->where('module = ?', $module->getName())
                   ->where($column . ' = ?', $item->getId());

            if ($db->fetchOne($select) > 0) {
                throw new Application_Module_Exception(sprintf('Item already has module %s', $module));
            }

            $values = array(
                $column  => $item->getId(),
                'module' => $module->getName()
            );

            $db->begin();
            $db->insert($table, $values);

            self::_ClearCache($item);

            if ($item instanceof Application_Brand) {
                $event = new Application_Event('/core/brand/moduleassigned');
                $event->set('brand', $item)
                      ->set('module', $module)
                      ->trigger();
            }
            else if ($item instanceof Application_Client) {
                if ($deployPermissions) {
                    $pm = Application_User_Permissions_Manager::GetInstance();
                    $pm->deployModulePermissions($item, $module);
                }

                $event = new Application_Event('/core/client/moduleassigned');
                $event->set('client', $item)
                      ->set('module', $module)
                      ->trigger();
            }

            $db->commit();
        }

        /**
         * Assign multiple module to the object passed
         *
         * @param   Application_Brand|Application_Client    $item
         * @param   Application_Module_Interface[]|string[] $modules The modules to assign
         * @throws  Application_Module_Exception            If unable to assign modules
         */
        public static function AssignModules($item, $modules, $deployPermissions = true)
        {
            if (!is_array($modules)) {
                $modules = array();
            }

            if (count($modules) == 0) {
                return;
            }

            $_modules = array();

            foreach ($modules as $module) {
                $module = self::Factory($module);
                $_modules[$module->getName()] = $module;
            }

            $db = Application::GetDb();

            if ($item instanceof Application_Brand) {
                $table  = self::TABLE_BRANDS;
                $column = 'brand_id';
            }
            else if ($item instanceof Application_Client) {
                $table  = self::TABLE_CLIENTS;
                $column = 'client_id';
            }
            else {
                throw new Application_Module_Exception(sprintf('Invalid item passed to %s::%s', __CLASS__, __FUNCTION__));
            }

            $select = $db->select();
            $select->from($table, 'module')
                   ->where('module in (?)', array_keys($_modules))
                   ->where($column . ' = ?', $item->getId());

            $alreadyAssigned = $db->fetchCol($select);
            $newlyAssigned   = array();

            $db->begin();

            foreach ($_modules as $module) {
                if (in_array($module->getName(), $alreadyAssigned)) {
                    continue;
                }

                $values = array(
                    $column  => $item->getId(),
                    'module' => $module->getName()
                );

                $db->insert($table, $values);

                $newlyAssigned[] = $module;
            }

            self::_ClearCache($item);

            if (count($newlyAssigned) > 0) {
                if ($item instanceof Application_Brand) {
                    $event = new Application_Event('/core/brand/moduleassigned');
                    $event->set('brand', $item)
                          ->set('modules', $newlyAssigned)
                          ->trigger();
                }
                else if ($item instanceof Application_Client) {
                    if ($deployPermissions) {
                        $pm = Application_User_Permissions_Manager::GetInstance();
                        $pm->deployModulePermissions($item, $module);
                    }

                    $event = new Application_Event('/core/client/moduleassigned');
                    $event->set('client', $item)
                          ->set('modules', $newlyAssigned)
                          ->trigger();
                }
            }

            $db->commit();
        }

        private static function _ClearCache($item)
        {
            if (!$item instanceof DatabaseObject) {
                return;
            }

            $type = get_class($item);

            if (!array_key_exists($type, self::$_cache)) {
                return;
            }

            unset(self::$_cache[$type][$item->getId()]);
        }


        /**
         * Unassign a module from the object passed
         *
         * @param   Application_Brand|Application_Client    $item
         * @param   Application_Module_Interface|string     $module The module to unassign
         * @throws  Application_Module_Exception            If the item is invalid
         */
        public static function UnassignModule($item, $module)
        {
            $module = self::Factory($module);
            $db = Application::GetDb();

            if ($item instanceof Application_Brand) {
                $table  = self::TABLE_BRANDS;
                $column = 'brand_id';
            }
            else if ($item instanceof Application_Client) {
                $table  = self::TABLE_CLIENTS;
                $column = 'client_id';
            }
            else {
                throw new Application_Module_Exception(sprintf('Invalid item passed to %s::%s', __CLASS__, __FUNCTION__));
            }

            $select = $db->select();
            $select->from($table, 'count(*)')
                   ->where('module = ?', $module->getName())
                   ->where($column . ' = ?', $item->getId());

            if ($db->fetchOne($select) <= 0) {
                throw new Application_Module_Exception(sprintf('Item does not have module %s', $module));
            }

            $where = array(
                $db->quoteInto($column . ' = ?', $item->getId()),
                $db->quoteInto('module = ?', $module->getName())
            );

            $db->delete($table, $where);

            self::_ClearCache($item);

            if ($item instanceof Application_Brand) {
                $event = new Application_Event('/core/brand/moduleunassigned');
                $event->set('brand', $item)
                      ->set('module', $module)
                      ->trigger();
            }
            else if ($item instanceof Application_Client) {
                $event = new Application_Event('/core/client/moduleunassigned');
                $event->set('client', $item)
                      ->set('module', $module)
                      ->trigger();
            }
        }


        public static function ClearModules($item)
        {
            $db = Application::GetDb();

            if ($item instanceof Application_Brand) {
                $table  = self::TABLE_BRANDS;
                $column = 'brand_id';
            }
            else if ($item instanceof Application_Client) {
                $table  = self::TABLE_CLIENTS;
                $column = 'client_id';
            }
            else {
                throw new Application_Module_Exception(sprintf('Invalid item passed to %s::%s', __CLASS__, __FUNCTION__));
            }

            $where = array(
                $db->quoteInto($column . ' = ?', $item->getId())
            );

            $db->delete($table, $where);

            self::_ClearCache($item);

            if ($item instanceof Application_Brand) {
                $event = new Application_Event('/core/brand/allmodulesunassigned');
                $event->set('brand', $item)
                      ->trigger();
            }
            else if ($item instanceof Application_Client) {
                $event = new Application_Event('/core/client/allmodulesunassigned');
                $event->set('client', $item)
                      ->trigger();
            }
        }


        /**
         * Get a module by its name
         *
         * @param   Application_Module_interface|string $module     The name of the module
         * @return  Application_Module_Interface
         */
        public static function Factory($module)
        {
            if ($module instanceof Application_Module_Interface) {
                return $module;
            }

            $fullpath = sprintf('%s/%s/module.php', self::GetPath(), $module);

            if (!file_exists($fullpath)) {
                throw new Application_Module_Exception('Module file does not exist: ' . $fullpath);
            }

            if (!is_file($fullpath)) {
                throw new Application_Module_Exception('Module path is not a file: ' . $fullpath);
            }

            if (!is_readable($fullpath)) {
                throw new Application_Module_Exception('Module file is not readable: ' . $fullpath);
            }

            $class = sprintf('Module_%s', $module);

            require_once($fullpath);

            if (!class_exists($class, false)) {
                throw new Application_Module_Exception('Module class not declared: ' . $class);
            }

            $reflection = new ReflectionClass($class);

            if (!$reflection->isSubclassOf(self::_GetInterfaceReflection())) {
                throw new Application_Module_Exception('Module class does not implement Application_Module_Interface: ' . $class);
            }

            return new $class;
        }


        /**
         * Get the reflection class for the base module interface
         *
         * @return  ReflectionClass
         */
        private static function _GetInterfaceReflection()
        {
            if (!self::$_interfaceReflection instanceof ReflectionClass) {
                self::$_interfaceReflection = new ReflectionClass('Application_Module_Interface');
            }

            return self::$_interfaceReflection;
        }

        /**
         * Used for sorting the array of modules by title
         */
        public static function _sortModules($a, $b)
        {
            return strcmp((string) $a, (string) $b);
        }


        /**
         * Get a full summary of modules and drivers assigned to the object passed
         */
        public static function GetFullSummary($item)
        {
            if (!$item instanceof Application_Brand && !$item instanceof Application_Client) {
                throw new Application_Module_Exception(sprintf('Invalid item passed to %s', __FUNCTION__));
            }

            $modules = self::GetModules();

            $ret = array();

            if ($item instanceof Application_Brand) {
                $table1 = self::TABLE_BRANDS;
                $table2 = Application_Driver_Manager::TABLE_BRANDS;
                $column = 'brand_id';
            }
            else {
                $table1 = self::TABLE_CLIENTS;
                $table2 = Application_Driver_Manager::TABLE_CLIENTS;
                $column = 'client_id';
            }

            $db = Application::GetDb();
            $select = $db->select();
            $select->from($table1, 'module')
                   ->where($column . ' = ?', $item->getId());

            $assignedModules = $db->fetchCol($select);

            $db = Application::GetDb();
            $select = $db->select();
            $select->from($table2, 'driver_name')
                   ->where($column . ' = ?', $item->getId());

            $assignedDrivers = $db->fetchCol($select);

            foreach ($modules as $module) {
                if (!$module->isClientModule()) {
                    continue;
                }

                $name = $module->getName();

                $ret[$name] = array(
                    'module'  => $module,
                    'has'     => in_array($name, $assignedModules),
                    'drivers' => array(),
                    'categories' => array()
                );

                $drivers = $module->getDrivers();

                foreach ($drivers as $driver) {
                    $driverName = $driver->getFullname();

                    $ret[$name]['drivers'][$driverName] = array(
                        'driver' => $driver,
                        'has'    => $ret[$name]['has'] && in_array($driverName, $assignedDrivers)
                    );

                    $category = $driver->getDriverTypeDescription();

                    if (!array_key_exists($category, $ret[$name]['categories'])) {
                        $ret[$name]['categories'][$category] = array();
                    }

                    $ret[$name]['categories'][$category][] = $ret[$name]['drivers'][$driverName];
                }
            }

            // get standalone drivers
            $drivers = Application_Driver_Standalone_Manager::GetDrivers();

            if (count($drivers) > 0) {
                $row = array(
                    'module'     => 'Standalone',
                    'has'        => false,
                    'drivers'    => array(),
                    'categories' => array()
                );

                foreach ($drivers as $driver) {
                    $driverName = $driver->getFullName();

                    $row['drivers'][$driverName] = array(
                        'driver' => $driver,
                        'has'    => in_array($driverName, $assignedDrivers)
                    );

                    $category = $driver->getDriverTypeDescription();

                    if (!array_key_exists($category, $row['categories'])) {
                        $row['categories'][$category] = array();
                    }

                    $row['categories'][$category][] = $row['drivers'][$driverName];
                }

                $ret['_standalone'] = $row;
            }

            return $ret;
        }

        public static function SetModulesAndDrivers($item, $modules, $drivers)
        {
            if (!$item instanceof Application_Brand && !$item instanceof Application_Client) {
                throw new Application_Module_Manager(sprintf('Invalid item passed to %s', __FUNCTION__));
            }

            $modules = ArrayTools::Clean($modules);
            $drivers = ArrayTools::Clean($drivers);

            $moduleNames = array();
            $driverNames = array();

            $summary = self::GetFullSummary($item, $modules);

            foreach ($summary as $name => $module) {
                if ($module['module'] instanceof Application_Module_Abstract) {
                    if (!in_array($name, $modules)) {
                        continue;
                    }

                    $moduleNames[] = $name;
                }

                foreach ($module['drivers'] as $driverName => $driver) {
                    if (!in_array($driverName, $drivers)) {
                        continue;
                    }

                    $driverNames[] = $driverName;
                }
            }

            self::SetModules($item, $moduleNames);
            Application_Driver_Manager::SetDrivers($item, $driverNames);
        }
    }
?>
