<?php
    /**
     * Used to manage the system status, including putting the application into
     * and out of maintenance mode, and also for checking for any problems
     * with the application
     *
     * @since   Recite 1.7.0
     */
    class Application_SystemStatus
    {
        const LOCK_FILE      = 'maintenance.lock';
        const LOCK_FILE_ITEM = 'maintenance.lock.%s_%d';

        public static function EnableMaintenanceMode($flag = true)
        {
            if (!$flag) {
                return self::DisableMaintenaceMode();
            }

            @touch(self::GetLockFullpath());
        }

        public static function EnableItemMaintenanceMode(Application_Client $client, $message = null)
        {
            @file_put_contents(
                self::GetItemLockFullpath($client),
                trim($message)
            );
        }

        public static function DisableMaintenanceMode()
        {
            if (!self::InMaintenanceMode()) {
                throw new Exception('Not currently in maintenance mode');
            }

            $report = self::GetStatusReport();

            $critical = false;
            foreach ($report->getItems() as $item) {
                if ($item->isCritical()) {
                    $critical = true;
                    break;
                }
            }

            if ($critical) {
                throw new Exception('Unable to disable maintenance while there are pending critical maintenance actions');
            }

            unlink(self::GetLockFullpath());
        }

        public static function DisableItemMaintenanceMode(Application_Client $client)
        {
            @unlink(self::GetItemLockFullpath($client));
        }

        public static function ItemInMaintenanceMode(Application_Client $client)
        {
            return file_exists(self::GetItemLockFullpath($client));
        }

        public static function InMaintenanceMode()
        {
            return file_exists(self::GetLockFullpath());
        }

        protected static function GetLockFullpath()
        {
            return Application::GetDataDir() . '/' . self::LOCK_FILE;
        }

        protected static function GetItemLockFullpath(Application_Client $client)
        {
            return Application::GetDataDir() . '/' . sprintf(self::LOCK_FILE_ITEM, get_class($client), $client->getId());
        }

        public static function GetStatusReport()
        {
            $report = new Application_SystemStatus_Report();

            $brands = Application_Brand::GetBrands();

            try {
                $dbReport = Application_Upgrader_Database::getDifferences();

                $item = new Application_SystemStatus_Report_Item();

                if (count($dbReport['missing']) > 0) {
                    $item->addMessage(sprintf('%s table%s missing', count($dbReport['missing']), count($dbReport['missing']) != 1 ? 's' : ''));
                }

                if (count($dbReport['incorrect']) > 0) {
                    $item->addMessage(sprintf('%s table%s incorrect', count($dbReport['incorrect']), count($dbReport['incorrect']) != 1 ? 's' : ''));
                }

                if (count($item->getMessages()) > 0) {
                    $item->setFixUrl('/system/database/index')
                         ->setCritical();
                    $report->add($item);
                }
            }
            catch (Exception $ex)
            { }

            if (!Application_Cache::CachingIsEnabled()) {
                $report->add(sprintf('Caching is currently disabled'))
                       ->setFixUrl('/system/cache/index');
            }

            // check that the maintenance script is scheduled to run
            $url = '/system/maintenance/help';

            try {
                $ts = Application_Maintenance::GetLastRunElapsed();

                if ($ts > 3600 * 30) {
                    $report->add(sprintf('System maintenance queue has not run for %s', Calendar_Tools::Duration($ts)))
                           ->setFixUrl($url);
                }
            }
            catch (Exception $ex) {
                $report->add('System maintenance queue never processed')
                       ->setFixUrl($url);
            }


            // check for clients that have no brand
            $clients = Application_Client::GetBrandlessClients();

            foreach ($clients as $client) {
                $item = $report->add(sprintf("Client `%s' has no branding set", $client));

                if (count($brands) == 0) {
                    $item->setFixUrl('/brands/index/edit');
                }
                else {
                    $item->setFixUrl('/clients/index/edit?id=' . $client->getId());
                }
            }

            $packages = Application_Upgrader_Manager::GetRegisteredPackages();

            if (count($packages) == 0) {
                $report->add('No packages have been installed')
                       ->setFixUrl('/upgrade');
            }

            // check for brands that have no CP url
            foreach ($brands as $brand) {
                if (strlen($brand->getPrimaryDomain()) == 0) {
                    $report->add(sprintf("Brand `%s' has no Control Panel URL set", $brand))
                           ->setFixUrl('/brands/index/view?id=' . $brand->getId());
                }
            }

            $clients = Application_Client::GetClients();

            // check for clients that have no URL
            foreach ($clients as $client) {
                if (strlen($client->getPrimaryDomain()) == 0) {
                    $report->add(sprintf("Client `%s' has no site URL set", $client))
                           ->setFixUrl('/clients/index/view?id=' . $client->getId());
                }
            }

            $db = Application::GetDb();

            // check for clients that have no users
            $select = $db->select();
            $select->from(array('ur' => Application_User_Role::$table), 'client_id')
                   ->distinct()
                   ->where('ur.directory_id is null')
                   ->where('ur.client_id is not null')
                   ->join(array('utr' => Application_User::$rolesTable), 'utr.role_id = ur.role_id', array());

            $clientIds = $db->fetchCol($select);

            foreach ($clients as $client) {
                if (in_array($client->getId(), $clientIds)) {
                    continue;
                }

                $report->add(sprintf("Client `%s' has no users assigned", $client))
                       ->setFixUrl('/users');
            }

            $showNoClientWarning = true;

            if (count($packages) > 0) {

                // check for brands with no default drivers
                $select = $db->select();
                $select->from(Application_Driver_Manager::TABLE_BRANDS, 'brand_id')
                       ->distinct();

                $brandIdsDrivers = $db->fetchCol($select);

                $select = $db->select();
                $select->from(Application_Module_Manager::TABLE_BRANDS, 'brand_id')
                       ->distinct();

                $brandIdsModules = $db->fetchCol($select);

                foreach ($brands as $brand) {
                    if (in_array($brand->getId(), $brandIdsModules) && in_array($brand->getId(), $brandIdsDrivers)) {
                        continue;
                    }

                    $showNoClientWarning = false;

                    $report->add(sprintf("Brand `%s' has no default modules/drivers set", $brand))
                           ->setFixUrl('/brands/drivers/index?id=' . $brand->getId());
                }

                // check for brands that have no default widget layout
                $select = $db->select();
                $select->from(Application_Cp_Tab_Manager::TABLE_DEFAULTS_BRANDS, 'brand_id')
                       ->distinct();

                $brandIds = $db->fetchCol($select);

                foreach ($brands as $brand) {
                    if (in_array($brand->getId(), $brandIds)) {
                        continue;
                    }

                    $showNoClientWarning = false;

                    $report->add(sprintf("Brand `%s' has no default widgets set", $brand))
                           ->setFixUrl('/brands/widgets/index?id=' . $brand->getId());
                }

                // check for clients that have no drivers
                $select = $db->select();
                $select->from(Application_Driver_Manager::TABLE_CLIENTS, 'client_id')
                       ->distinct();

                $clientIdsDrivers = $db->fetchCol($select);

                $select = $db->select();
                $select->from(Application_Module_Manager::TABLE_CLIENTS, 'client_id')
                       ->distinct();

                $clientIdsModules = $db->fetchCol($select);

                foreach ($clients as $client) {
                    if (in_array($client->getId(), $clientIdsModules) && in_array($client->getId(), $clientIdsDrivers)) {
                        continue;
                    }

                    $report->add(sprintf("Client `%s' has no modules/drivers set", $client))
                           ->setFixUrl('/clients/drivers/index?id=' . $client->getId());
                }

                if ($showNoClientWarning && count($clients) == 0) {
                    $report->add('No clients have been created')
                           ->setFixUrl('/clients/index/edit');
                }

                if (self::InMaintenanceMode()) {
                    $report->add('Currently in maintenance mode')
                           ->setFixUrl('/system/index');
                }
            }

            if (Application_License_Manager::IsSupported()) {
                // check the license status for clients
                $status = Application_License_Manager::GetClientSummary();
                $url    = '/clients/licenses';

                foreach ($status as $client) {
                    foreach ($client['licenses'] as $name => $license) {
                        if ($name == 'core') {
                            continue;
                        }

                        if ($license['hasExpired']) {
                            $report->add(sprintf("Client `%s' has an expired license for `%s'", $client['client'], $license['title']))
                                   ->setFixUrl($url);
                        }
                        else if (!$license['hasActive']) {
                            $report->add(sprintf("Client `%s' does not have a license for `%s'", $client['client'], $license['title']))
                                   ->setFixUrl($url);
                        }
                    }
                }
            }

            return $report;
        }
    }
?>
