<?php
    /**
     * Manage CMS upgrade tasks
     */
    class Application_Upgrader_Manager
    {
        const URL = 'http://download.recite.com.au/services';

        private static $_url;

        const REGISTRY_TABLE = 'application_packages';
        const LOCK_FILE = 'upgrade.lock';
        protected static $_lockPassword;

        protected static $_upgradeSummary;

        protected static $_installQueue = array();

        protected static $_loggerStream;
        protected static $_loggerLineFormat = "%s\n";

        protected static $_packageInstallCallback;

        private function __construct()
        { }

        public static function GetUrl()
        {
            if (strlen(self::$_url) == 0) {
                $path = sprintf('%s/releases-url.txt', Application::GetBaseDir());

                $ret = self::URL;

                if (file_exists($path)) {
                    $url = trim(file_get_contents($path));
                    $url = rtrim($url, '/');

                    if (HTTP_URL::IsValidUrl($url)) {
                        $ret = $url;
                    }
                }

                self::$_url = $ret;
            }

            return self::$_url;
        }

        public static function GetUpgradeSummary()
        {
            if (!is_array(self::$_upgradeSummary)) {
                $postData = array(
                    'recite_version' => Application::VERSION,
                    'compat_version' => Application::COMPAT_VERSION,
                    'channels'       => join(',', self::GetActiveChannels()),
                    'installed'      => join(',', self::GetRegisteredPackages())
                );

                $ch = curl_init(self::GetUrl() . '/packages/query');

                curl_setopt($ch, CURLOPT_USERAGENT, sprintf('%s %s', Application::NAME, Application::VERSION));
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                curl_setopt($ch, CURLOPT_POSTFIELDS, $postData);
                curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_DIGEST);
                curl_setopt($ch, CURLOPT_USERPWD, sprintf('%s:%s', self::GetUsername(), self::GetPassword()));

                $response = curl_exec($ch);

                if ($response === false) {
                    $errno  = curl_errno($ch);
                    $errmsg = curl_error($ch);
                    curl_close($ch);

                    throw new Exception(sprintf('Error getting summary: %d - %s', $errno, $errmsg), $errno);
                }

                self::_AssertValidResponseFromReleaseServer($ch);
                curl_close($ch);

                $json = json_decode($response, true);

                if (!is_array($json)) {
                    throw new Exception('Invalid response');
                }

                self::$_upgradeSummary = $json;
            }

            return self::$_upgradeSummary;
        }


        /**
         * Check if the given username and password are valid credentials for the download portal
         *
         * @param   string  $username   The username to check
         * @param   string  $password   The password to check
         * @return  bool                True if the username and password are valid
         * @throws  Exception           If unable to check credentials
         */
        public static function GetAccountDetails($username, $password)
        {
            $postData = array(
                'recite_version' => Application::VERSION,
                'compat_version' => Application::COMPAT_VERSION,
                'channels'       => 'stable',
                'installed'      => ''
            );

            $ch = curl_init(self::GetUrl() . '/account/details');

            curl_setopt($ch, CURLOPT_USERAGENT, sprintf('%s %s', Application::NAME, Application::VERSION));
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $postData);
            curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_DIGEST);
            curl_setopt($ch, CURLOPT_USERPWD, sprintf('%s:%s', $username, $password));

            $response = curl_exec($ch);

            if ($response === false) {
                $errno  = curl_errno($ch);
                $errmsg = curl_error($ch);
                curl_close($ch);

                throw new Exception(sprintf('Error checking credentials: %d - %s', $errno, $errmsg), $errno);
            }

            $info = curl_getinfo($ch);
            curl_close($ch);

            $code = (int) $info['http_code'];

            if ($code < 200 || $code >= 300) {
                throw new Exception('Unexpected response', $code);
            }

            $json = json_decode($response, true);

            if (!is_array($json)) {
                throw new Exception('Unable to decode response');
            }

            return $json;
        }


        /**
         * Create a new user for the given name and email address
         *
         * @param   string  $name   The name for the account
         * @param   string  $email  The email address for the account
         * @return  array           The details of the newly created account
         * @throws  Exception       If unable to create the account
         */
        public static function CreateAccount($name, $email)
        {
            $postData = array(
                'recite_version' => Application::VERSION,
                'compat_version' => Application::COMPAT_VERSION,
                'name'           => $name,
                'email'          => $email
            );

            $postData['checksum'] = md5(serialize($postData));

            $ch = curl_init(self::GetUrl() . '/account/create');

            curl_setopt($ch, CURLOPT_USERAGENT, sprintf('%s %s', Application::NAME, Application::VERSION));
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $postData);

            $response = curl_exec($ch);

            if ($response === false) {
                $errno  = curl_errno($ch);
                $errmsg = curl_error($ch);
                curl_close($ch);

                throw new Exception(sprintf('Error creating account: %d - %s', $errno, $errmsg), $errno);
            }

            $info = curl_getinfo($ch);
            curl_close($ch);

            $code = (int) $info['http_code'];

            if ($code < 200 || $code >= 300) {
                throw new Exception('Unexpected response', $code);
            }

            $json = json_decode($response, true);

            if (!is_array($json)) {
                throw new Exception('Unable to decode response');
            }

            return $json;
        }

        public static function CanAutoCheck()
        {
            $profile = Application_Settings::GetInstance();

            return (bool) $profile->upgrade_auto_check;
        }

        public static function SetSettings($username, $password, $enabled, $channels)
        {
            $profile = Application_Settings::GetInstance();
            $profile->upgrade_auto_check = $enabled ? 1 : 0;
            $profile->upgrade_username   = trim($username);
            $profile->upgrade_password   = trim($password);

            $allChannels = self::GetChannels();

            $newChannels = array();

            if (is_array($channels)) {
                foreach ($channels as $channel) {
                    if (array_key_exists($channel, $allChannels)) {
                        $newChannels[] = $channel;
                    }
                }
            }
            $profile->upgrade_channels = join(',', $newChannels);

            $profile->save();
        }


        /**
         * Allowing this function may be insecure...
         */
        public static function GetUsername()
        {
            $profile = Application_Settings::GetInstance();
            $ret = $profile->upgrade_username;

            if (strlen($ret) == 0) {
                throw new Exception('Username not specified');
            }

            return $ret;
        }


        /**
         * Allowing this function may be insecure...
         */
        public static function GetPassword()
        {
            $profile = Application_Settings::GetInstance();
            $ret = $profile->upgrade_password;

            if (strlen($ret) == 0) {
                throw new Exception('Password not specified');
            }

            return $ret;
        }


        public static function GetChannels()
        {
            $ret = array(
                'stable' => array(
                    'name'    => 'stable',
                    'title'   => 'Stable',
                    'enabled' => false
                ),
                'alpha' => array(
                    'name'    => 'alpha',
                    'title'   => 'Alpha',
                    'enabled' => false
                ),
                'beta' => array(
                    'name'    => 'beta',
                    'title'   => 'Beta',
                    'enabled' => false
                )
            );

            $profile = Application_Settings::GetInstance();

            foreach (explode(',', $profile->upgrade_channels) as $name) {
                if (array_key_exists($name, $ret)) {
                    $ret[$name]['enabled'] = true;
                }
            }

            return $ret;
        }

        public static function GetActiveChannels()
        {
            $ret = array();

            foreach (self::GetChannels() as $channel) {
                if ($channel['enabled']) {
                    $ret[] = $channel['name'];
                }
            }

            return $ret;
        }

        public static function InstallAllPackages()
        {
            $summary = self::GetUpgradeSummary();

            if ($summary['error']) {
                throw new Exception($summary['errmsg'], $summary['errnum']);
            }

            $names = array();

            foreach ($summary['packages']['a'] as $package) {
                $names[] = sprintf('%s-%s', $package['n'], $package['v']);
            }

            $password = self::Lock();

            $ret = self::InstallPackages($names, false, $password);
            self::Unlock();

            return $ret;
        }

        public static function UpgradeAllPackages()
        {
            $summary = self::GetUpgradeSummary();

            if ($summary['error']) {
                throw new Exception($summary['errmsg'], $summary['errnum']);
            }

            $names   = array();

            foreach ($summary['packages']['i'] as $package) {
                if ($package['u']) {
                    $names[] = sprintf('%s-%s', $package['n'], $package['v']);
                }
            }

            $password = self::Lock();

            $ret = self::InstallPackages($names, false, $password);
            self::Unlock();

            return $ret;
        }

        public static function InstallPackages($names, $force = false, $password = '')
        {
            $unlock = false;
            if (strlen($password) == 0) {
                $unlock = true;
                $password = self::Lock();
            }

            self::AssertLockIsValid($password);

            $installed = array();

            foreach ($names as $name) {
                $done = false;

                foreach ($installed as $p) {
                    if ($name == $p->getName()) {
                        $done = true;
                        break;
                    }
                }

                if (!$done) {
                    $installed = array_merge($installed, self::InstallPackage($name, true, $force, $password));
                }
            }

            if ($unlock) {
                self::Unlock();
            }

            return $installed;
        }

        public static function SetPackageInstallCallback($callback)
        {
            if (is_callable($callback)) {
                self::$_packageInstallCallback = $callback;
            }
            else {
                self::$_packageInstallCallback = null;
            }
        }

        public static function InstallPackage($name, $installDeps = true, $force = false, $password = '')
        {
            $unlock = false;

            if (strlen($password) == 0) {
                $unlock = true;
                $password = self::Lock();
            }

            self::AssertLockIsValid($password);

            $package = self::GetPackage($name);
            $channel = $package->getChannel();

            if (!$channel->isLocal()) {
                if (!Application_SystemStatus::InMaintenanceMode()) {
                    throw new Exception('Must be in maintenance mode to install or upgrade packages');
                }

                if (IN_DEVELOPMENT) {
                    throw new Exception('Package installer is disabled while in development mode');
                }
            }

            // check if the package has been installed since it might've been installed by a previous dependency
            $reinstalling = false;
            if (self::PackageIsRegistered($channel, $package->getName(), $package->getVersion())) {
                if ($force) {
                    $reinstalling = true;
                }
                else {
                    if (count(self::$_installQueue) == 0) {
                        self::Log(sprintf("Package `%s' is already installed", $package), true);
                    }

                    return array();
                }
            }

            self::Log(sprintf("Installing package `%s'", $name), true);

            $hash = md5(sprintf('%s,%s', $channel->getName(), $package->getName()));

            if (array_key_exists($hash, self::$_installQueue)) {
                throw new Exception(sprintf("Recursive dependency detected on package `%s'", $package));
            }

            self::$_installQueue[$hash] = $package;

            self::Log(sprintf('Asserting package is compatible with %s %s', Application::NAME, Application::VERSION), true);
            if (!$package->isCompatible()) {
                throw new Exception('Package is not compatible with this version of Recite');
            }

            $deps = $package->getDependencies();
            $installedPackages = array();

            if ($deps > 0) {
                self::Log('Checking dependencies ...', true);

                $depInstall = array();

                foreach ($deps as $dep) {
                    $name = $dep->getName() . '-' . $dep->getVersion();
                    self::Log(sprintf("Package `%s'", $name));

                    if ($dep->isInstalled()) {
                        self::Log(' already installed', true);
                        continue;
                    }

                    if ($dep->isUpgrade()) {
                        self::Log(' needs upgrading', true);
                    }
                    else {
                        self::Log(' needs installing', true);
                    }

                    $depInstall[] = $dep;
                }

                if ($installDeps) {
                    foreach ($depInstall as $dep) {
                        $installedPackages = array_merge($installedPackages, self::InstallPackage($dep, true, false, $password));
                    }
                }
                else {
                    throw new Exception('Required dependencies not installed');
                }

                self::Log(sprintf("Dependencies satisfied, proceeding to install `%s'", $package), true);
            }

            $package->install();

            Application_Cache::ClearAllCaches();

            unset(self::$_installQueue[$hash]);

            $installedPackages[] = $package;

            if (!is_null(self::$_packageInstallCallback)) {
                call_user_func(self::$_packageInstallCallback, $package);
            }

            if ($unlock) {
                self::Unlock();
            }

            return $installedPackages;
        }

        public static function UninstallPackages($names)
        {
            $uninstalled = array();

            $packages = array();

            foreach ($names as $name) {
                $package = self::GetRegisteredPackage($name);
                self::AssertPackageCanBeUninstalled($package);

                $packages[] = $package;
            }

            foreach ($packages as $package) {
                if (in_array($package->getName(), $uninstalled)) {
                    continue;
                }

                $uninstalled = array_merge($uninstalled, self::UninstallPackage($package));
            }

            return $uninstalled;
        }


        public static function AssertPackageCanBeUninstalled(Application_Upgrader_Package_Registered $package)
        {
            $deps = $package->getDependents();

            if (count($deps) == 0) {
                return;
            }

            throw new Exception(sprintf('The following packages rely on this package (%s): %s', join(', ', $deps), $package));
        }

        public static function UninstallPackage(Application_Upgrader_Package_Registered $package)
        {
            $package->uninstall();
            return array($package);
        }

        public static function SetLoggerStream($stream)
        {
            if (!is_resource($stream)) {
                $stream = null;
            }
            else if (get_resource_type($stream) != 'stream') {
                $stream = null;
            }

            self::$_loggerStream = $stream;
        }

        public static function SetLoggerLineFormat($format)
        {
            $format = trim($format);

            if (strlen($format) > 0) {
                self::$_loggerLineFormat = $format;
            }
        }

        public static function Log($msg, $lf = false)
        {
            $msg = trim($msg, "\n");

            if ($lf) {
                $msg = sprintf(self::$_loggerLineFormat, $msg);
            }

            if (self::$_loggerStream != null) {
                fwrite(self::$_loggerStream, $msg);
            }
        }

        public static function GetPackage($name, $channel = null)
        {
            if ($name instanceof Application_Upgrader_Package) {
                return $name;
            }

            // check if the package is on the local filesystem
            if (substr($name, -4) == '.zip' && file_exists($name)) {
                $channel = Application_Upgrader_Channel::GetLocalChannel();
                $_name   = substr(basename($name), 0, -4);
                $info    = Application_Upgrader_Package::GetFilenameInfo($_name);

                $ret = new Application_Upgrader_Package($info['name'], $info['version']);
                $ret->setChannel($channel);
                $ret->setLocalPath($name);

                return $ret;
            }

            $info = Application_Upgrader_Package::GetFilenameInfo($name);

            $summary = self::GetUpgradeSummary();

            foreach ($summary['packages']['a'] as $package) {
                if ($package['n'] == $info['name']) {
                    $ret = new Application_Upgrader_Package($package['n'], $package['v']);
                    $ret->setChannel(new Application_Upgrader_Channel($package['c']));
                    $ret->setChecksum($package['h']);
                    return $ret;
                }
            }

            foreach ($summary['packages']['i'] as $package) {
                if ($package['n'] == $info['name']) {
                    $ret = new Application_Upgrader_Package($package['n'], $package['v']);
                    $ret->setChannel(new Application_Upgrader_Channel($package['c']));
                    $ret->setChecksum($package['h']);
                    return $ret;
                }
            }
/*
// this has been removed since the channel is unknown here now,
// breaking the call to PackageIsRegistered
            // this is checked last just in case there's an update with the same compatibility version
            if (self::PackageIsRegistered($info['name'], $info['version'])) {
                return new Application_Upgrader_Package($info['name'], $info['version']);
            }
*/
            throw new Exception('Package not found');
        }

        public static function DownloadPackage($name, $version)
        {
            $postData = array(
                'name'    => $name,
                'version' => $version
            );

            return self::_Download('/packages/download', $postData, sprintf('%s-%s', $name, $version));
        }

        public static function DownloadApp($version)
        {
            $postData = array(
                'version' => $version
            );

            return self::_Download('/app/download', $postData, sprintf('recite-%s', $version));
        }

        protected static function _Download($url, $postData, $packageName)
        {
            self::Log(sprintf("Downloading package `%s' ...", $packageName));

            $dir = Application::GetTmpDir() . '/packages';

            if (!file_exists($dir)) {
                @mkdir($dir, 0777);
            }

            if (!file_exists($dir)) {
                self::Log(' error', true);
                throw new Exception(sprintf('Unable to prepare package download path (%s)', $dir));
            }

            $dstPath = sprintf('%s/%s.zip', $dir, $packageName);

            if (file_exists($dstPath)) {
                unlink($dstPath);
            }

            $fp = fopen($dstPath, 'w');
            $ch = curl_init(self::GetUrl() . $url);

            curl_setopt($ch, CURLOPT_USERAGENT, sprintf('%s %s', Application::NAME, Application::VERSION));
            curl_setopt($ch, CURLOPT_POSTFIELDS, $postData);
            curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_DIGEST);
            curl_setopt($ch, CURLOPT_USERPWD, sprintf('%s:%s', self::GetUsername(), self::GetPassword()));
            curl_setopt($ch, CURLOPT_FILE, $fp);

            $response = curl_exec($ch);
            fclose($fp);
            unset($fp);

            if (!$response) {
                $errno  = curl_errno($ch);
                $errmsg = curl_error($ch);
                curl_close($ch);

                throw new Exception(sprintf("Error downloading package `%s': %d - %s", $packgeName, $errno, $errmsg), $errno);
            }

            try {
                self::_AssertValidResponseFromReleaseServer($ch);
            }
            catch (Exception $ex) {
                self::Log(' error', true);
                throw $ex;
            }

            $info = curl_getinfo($ch);

            curl_close($ch);

            $secs = (int) $info['total_time'];
            self::Log(sprintf(" complete (downloaded %d bytes in %d second%s)\n", $info['size_download'], $secs, $secs != 1 ? 's' : ''), true);

            return $dstPath;
        }

        public static function Cleanup($downloadPath, $extractPath)
        {
            Application_Upgrader_Manager::Log("Cleaning up package", true);

            if (strlen($extractPath) > 0 && file_exists($extractPath) && is_dir($extractPath)) {
                try {
                    File::RmdirRecursive($extractPath);
                }
                catch (Exception $ex) {
                }
            }

            if (strlen($downloadPath) > 0 && file_exists($downloadPath)) {
                @unlink($downloadPath);
            }
        }

        public static function GetRegisteredPackage($name)
        {
            $db = Application::GetDb();

            $select = $db->select();
            $select->from(self::REGISTRY_TABLE, '*')
                   ->where('name = ?', $name);

            $rows = $db->fetchAll($select);

            if (count($rows) == 0) {
                throw new Exception(sprintf('Package %s is not registered', $name));
            }

            foreach ($rows as $row) {
                $ret = new Application_Upgrader_Package_Registered($row['name'], $row['version']);
                $ret->setData(json_decode($row['data'], true));
                $ret->setChannel($row['channel']);

                return $ret;
            }
        }

        public static function GetRegisteredPackages()
        {
            $db = Application::GetDb();

            $select = $db->select();
            $select->from(self::REGISTRY_TABLE, '*')
                   ->order('lower(name)');

            $result = $db->query($select);

            $ret = array();

            while ($row = $result->fetch()) {
                if (array_key_exists($row->name, $ret)) {
                    $cmp = version_compare($row->version, $ret[$row->name]->getVersion());

                    if ($cmp <= 0) {
                        continue;
                    }
                }

                $package = new Application_Upgrader_Package_Registered($row->name, $row->version);
                $package->setChannel($row->channel);
                $package->setData(json_decode($row->data, true));

                $ret[$row->name] = $package;
            }

            return $ret;
        }

        public static function PackageIsRegistered(Application_Upgrader_Channel $channel, $name, $version)
        {
            foreach (self::GetRegisteredPackages() as $package) {
                if ($package->getName() == $name && $package->getVersion() == $version) {
                    if ($channel->isSameEndpoint($package->getChannel())) {
                        return true;
                    }
                }
            }

            return false;
        }

        protected static function _AssertValidResponseFromReleaseServer($curl, $closeOnError = true)
        {
            $info = curl_getinfo($curl);

            $code = (int) $info['http_code'];

            if ($code < 200 || $code >= 300) {
                $message = Zend_Http_Response::responseCodeAsText($code);

                if ($closeOnError) {
                    curl_close($curl);
                }
                throw new Exception(sprintf('Invalid HTTP Response (%d): %s', $code, $message), $code);
            }
        }

        public static function UpgradeApp($force = false)
        {
            try {
                if (!Application_SystemStatus::InMaintenanceMode()) {
                    throw new Exception('Must be in maintenance mode to install or upgrade packages');
                }

                if (IN_DEVELOPMENT) {
                    throw new Exception('Package installer is disabled while in development mode');
                }

                $summary = self::GetUpgradeSummary();

                if (!array_key_exists('recite', $summary)) {
                    throw new Exception('Invalid summary data');
                }

                if (!$summary['recite']['u'] && !$force) {
                    throw new Exception('Upgrade not required');
                }

                $oldVersion = Application::VERSION;
                $newVersion = $summary['recite']['n'];

                $downloadPath = self::DownloadApp($newVersion);

                $checksum = md5(file_get_contents($downloadPath));

                if ($checksum != $summary['recite']['h']) {
                    self::Cleanup($downloadPath, null);
                    throw new Exception(sprintf('Downloaded package file in %s not valid (%s != %s)', $downloadPath, $checksum, $summary['recite']['h']));
                }

                $extractPath = self::Extract($downloadPath);

                $xmlPath = $extractPath . DIRECTORY_SEPARATOR . 'package.xml';

                if (!file_exists($xmlPath)) {
                    self::Cleanup($downloadPath, $extractPath);
                    throw new Exception('Unable to find package.xml file');
                }

                $xml = new SimpleXmlElement($xmlPath, null, true);
                $xml->registerXpathNamespace('ns', Application_Upgrader_Package::XML_NAMESPACE);

                $result = $xml->xpath('/ns:package/ns:items/ns:item');

                if (!is_array($result)) {
                    self::Cleanup($downloadPath, $extractPath);
                    throw new Exception('Unable to find installation items in package.xml');
                }

                $paths = array();
                $preserve = array();

                foreach ($result as $node) {
                    $path = (string) $node;

                    try {
                        if (strlen($path) == 0) {
                            throw new Exception('Empty path');
                        }

                        $fullpath = realpath(sprintf('%s%s%s', $extractPath, DIRECTORY_SEPARATOR, str_replace('/', DIRECTORY_SEPARATOR, $node)));

                        if (strlen($fullpath) == 0) {
                            throw new Exception(sprintf('Cannot find path %s', $path));
                        }

                        $paths[] = $path;
                    }
                    catch (Exception $ex) {
                        self::Cleanup($downloadPath, $extractPath);
                        throw $ex;
                    }
                }

                $result = $xml->xpath('/ns:package/ns:items/ns:preserve');

                if (is_array($result)) {
                    foreach ($result as $node) {
                        $path = (string) $node;

                        try {
                            if (strlen($path) == 0) {
                                throw new Exception('Empty path');
                            }

                            $fullpath = realpath(sprintf('%s%s%s', $extractPath, DIRECTORY_SEPARATOR, str_replace('/', DIRECTORY_SEPARATOR, $node)));

                            if (strlen($fullpath) == 0) {
                                throw new Exception(sprintf('Cannot find path %s', $path));
                            }

                            $preserve[] = array(
                                'path'      => $path,
                                'recursive' => (bool) $node['recursive']
                            );
                        }
                        catch (Exception $ex) {
                            self::Cleanup($downloadPath, $extractPath);
                            throw $ex;
                        }
                    }
                }

                // get paths to preserve from packages
                foreach (Application_Upgrader_Package_Registered::GetPreservePaths() as $path) {
                    $preserve[] = array(
                        'path'      => $path,
                        'recursive' => false
                    );
                }

                if (count($paths) == 0) {
                    self::Cleanup($downloadPath, $extractPath);
                    throw new Exception('Found no paths to install');
                }

                $baseDir = Application::getBaseDir();

                $preUpgradeScriptPath = sprintf('%s/application/tools/internal/upgrade/pre-upgrade.php', $extractPath);
                if (file_exists($preUpgradeScriptPath) && is_readable($preUpgradeScriptPath)) {
                    $GLOBALS['RECITE_APP_OLD_VERSION'] = $oldVersion;
                    $GLOBALS['RECITE_APP_NEW_VERSION'] = $newVersion;
                    $GLOBALS['RECITE_APP_INSTALL_PATH'] = $baseDir;
                    $GLOBALS['RECITE_APP_INSTALL_ERROR'] = false;

                    include($preUpgradeScriptPath);

                    if ($GLOBALS['RECITE_APP_INSTALL_ERROR']) {
                        self::Cleanup($downloadPath, $extractPath);
                        throw new Exception('Received error from pre-upgrade script');
                    }

                    $GLOBALS['RECITE_APP_OLD_VERSION'] = null;
                    $GLOBALS['RECITE_APP_NEW_VERSION'] = null;
                    $GLOBALS['RECITE_APP_INSTALL_PATH'] = null;
                }

                foreach ($paths as $path) {
                    self::_InstallPath($baseDir, $extractPath, $path, $preserve);
                }

                self::FixPermissions($baseDir . DIRECTORY_SEPARATOR . 'application');
                self::FixPermissions($baseDir . DIRECTORY_SEPARATOR . 'lib');

                self::Cleanup($downloadPath, $extractPath);

                self::SetDatabaseVersion($newVersion);
                Application_Upgrader::RunUpdateTasks($oldVersion, $newVersion);

                Application_Upgrader_Manager::Log("Upgrade complete", true);

                Application_Cache::ClearAllCaches();

                try {
                    File::RmdirRecursive(Application::GetTmpDir() . '/templates_c');
                }
                catch (Exception $ex) {
                }
            }
            catch (Exception $ex) {
                self::Log($ex->getMessage(), true);
            }
        }

        protected static function _InstallPath($appPath, $basePath, $path, $preserve)
        {
            $needsPreserving     = false;
            $pathNeedsPreserving = false;
            $preserveRecursive   = false;

            $regex = '|^' . preg_quote($path) . '|';
            $len   = strlen($path);

            foreach ($preserve as $p) {
                if ($path == $p['path']) {
                    $needsPreserving = true;
                    if ($p['recursive']) {
                        $preserveRecursive = true;
                    }
                }

                if ($len < strlen($p['path']) && preg_match($regex, $p['path'])) {
                    $pathNeedsPreserving = true;
                }
            }

            $localPath = str_replace('/', DIRECTORY_SEPARATOR, $path);

            if ($pathNeedsPreserving) {
                if (is_dir($basePath . DIRECTORY_SEPARATOR . $localPath)) {
                    // loop over children
                    // check if child needs preserving. if so process it
                    // if not, remove the existing directory and process it

                    $dp = new File_Directory(sprintf('%s%s%s', $basePath, DIRECTORY_SEPARATOR, $localPath));

                    foreach ($dp->getFiles() as $file) {
                        self::_InstallPath($appPath, $basePath, $path . '/' . $file, $preserve);
                    }

                    foreach ($dp->getDirs() as $dir) {
                        self::_InstallPath($appPath, $basePath, $path . '/' . $dir, $preserve);
                    }
                }
                else {
                    // path is a file, nothing to do
                }
            }
            else if ($needsPreserving) {
                self::Log(sprintf("Installing `%s'", $path), true);

                if (is_dir($appPath . DIRECTORY_SEPARATOR . $localPath)) {
                    if (!$preserveRecursive) {
                        // remove all files in install dir
                        $dp = new File_Directory(sprintf('%s%s%s', $appPath, DIRECTORY_SEPARATOR, $localPath));

                        foreach ($dp->getFiles() as $file) {
                            //self::Log(sprintf("Removing `%s'", $file), true);
                            @unlink($dp . DIRECTORY_SEPARATOR . $file);
                        }

                        foreach ($dp->getDirs() as $dir) {
                            //self::Log(sprintf("Removing `%s'", $dir), true);
                            try {
                                File::RmdirRecursive($dp . DIRECTORY_SEPARATOR . $dir);
                            }
                            catch (Exception $ex) {
                            }
                        }
                    }

                    // move in children from src dir
                    $dp = new File_Directory(sprintf('%s%s%s', $basePath, DIRECTORY_SEPARATOR, $localPath));

                    foreach ($dp->getFiles() as $file) {
                        //self::Log(sprintf("Installing `%s'", $file), true);
                        $dstPath = $appPath . DIRECTORY_SEPARATOR . $localPath . DIRECTORY_SEPARATOR . $file;
                        if (file_exists($dstPath)) {
                            File::RmdirRecursive($dstPath);
                        }
                        rename($dp . DIRECTORY_SEPARATOR . $file, $dstPath);
                    }

                    foreach ($dp->getDirs() as $dir) {
                        //self::Log(sprintf("Installing `%s'", $dir), true);
                        $dstPath = $appPath . DIRECTORY_SEPARATOR . $localPath . DIRECTORY_SEPARATOR . $dir;
                        if (file_exists($dstPath)) {
                            File::RmdirRecursive($dstPath);
                        }
                        rename($dp . DIRECTORY_SEPARATOR . $dir, $dstPath);
                    }
                }
                else {
                    // preserving a file, do nothing
                }
            }
            else {
                $fullpath = sprintf('%s%s%s', $appPath, DIRECTORY_SEPARATOR, trim($localPath, DIRECTORY_SEPARATOR));

                self::Log(sprintf("Installing `%s' ...", $path));

                try {
                    File::RmdirRecursive($fullpath);
                }
                catch (Exception $ex) {
                }
                File::CreateDirStructure(dirname($fullpath));
                rename($basePath . DIRECTORY_SEPARATOR . $path, $fullpath);

                self::Log(' done', true);
            }
        }

        public static function FixPermissions($path)
        {
            if (is_dir($path)) {
                $iterator = new RecursiveIteratorIterator(
                    new RecursiveDirectoryIterator($path),
                    RecursiveIteratorIterator::SELF_FIRST
                );

                foreach($iterator as $item) {
                    if (substr($item->getFilename(), 0, 1) == '.') {
                        continue;
                    }

                    @chmod($item, $item->getPerms() | 0551);
                }
            }
            else if (is_file($path)) {
                @chmod($path, fileperms($path) | 0551);
            }
        }

        public static function Extract($downloadPath)
        {
            Application_Upgrader_Manager::Log('Extracting package ...');

            if (IS_WINDOWS) {
                $unzip = '';
            }
            else {
                try {
                    $unzip = File::GetBinaryPath('unzip');
                }
                catch (Exception $ex) {
                    $unzip = '';
                }
            }

            if (strlen($unzip) == 0 && !class_exists('ZipArchive', false)) {
                Application_Upgrader_Manager::Log(' error', true);
                throw new Exception('Either unzip tool or ZipArchive PHP extension is required');
            }

            $dstPath = Application::GetTmpDir() . DIRECTORY_SEPARATOR . 'packages' . DIRECTORY_SEPARATOR . substr(md5(rand()), 0, 10);

            if (strlen($unzip) > 0) {
                $cmd = sprintf(
                    '%s -qq %s -d %s',
                    escapeshellcmd($unzip),
                    escapeshellarg($downloadPath),
                    escapeshellarg($dstPath)
                );
                $retval = 0;

                system($cmd, $retval);

                if ($retval != 0) {
                    Application_Upgrader_Manager::Log(' error', true);
                    throw new Exception('Unable to extract package');
                }
            }
            else {
                $zip = new ZipArchive();
                $result = $zip->open($downloadPath);

                if ($result !== true) {
                    Application_Upgrader_Manager::Log(' error', true);
                    throw new Exception(sprintf('Unable to open package (error code %d)', $result));
                }

                $result = $zip->extractTo($dstPath);

                if (!$result) {
                    Application_Upgrader_Manager::Log(' error', true);
                    throw new Exception('Unable to extract package');
                }
            }

            Application_Upgrader_Manager::Log(' done', true);

            return $dstPath;
        }

        public static function SetDatabaseVersion($forceVersion = null)
        {
            if (strlen($forceVersion) == 0) {
                $forceVersion = Application::VERSION;
            }

            $profile = Application_Settings::GetInstance();
            $profile->database_version = $forceVersion;
            $profile->save();
        }

        public static function GetDatabaseVersion()
        {
            $profile = Application_Settings::GetInstance();

            $version = trim($profile->database_version);
            if (strlen($version) == 0) {
                $version = '0.0.0';
            }

            return $version;
        }

        public static function GetInstallInfo()
        {
            $cmd = sprintf('%s%stools%srecite%supgrade.php', Application::GetApplicationDir(), DIRECTORY_SEPARATOR, DIRECTORY_SEPARATOR, DIRECTORY_SEPARATOR);

            $ret = array(
                'script' => array(
                    'fullpath' => $cmd,
                    'filename' => basename($cmd),
                    'dirname'  => dirname($cmd)
                ),
                'commands' => array(),
                'paths' => array(
                    'data'  => Application::GetDataDir(),
                    'app'   => Application::GetApplicationDir(),
                    'lib'   => Application::GetBaseDir() . DIRECTORY_SEPARATOR . 'lib'
                )
            );

            $ret['paths']['tools'] = $ret['paths']['app'] . DIRECTORY_SEPARATOR . 'tools';

            if (IS_WINDOWS) {
                $ret['commands'][] = substr($ret['script']['fullpath'], 0, 2);
            }

            $bin = basename(Application_Maintenance::GetCliPath());
            $ret['commands'][] = sprintf('cd %s', $ret['script']['dirname']);
            $ret['commands'][] = sprintf('%s %s', $bin, $ret['script']['filename']);

            return $ret;
        }

        /**
         * Manage locking of upgrading
         */

        public static function Lock()
        {
            if (self::IsLocked()) {
                throw new Exception('Already locked');
            }

            $path = self::GetLockFullPath();
            @unlink($path);
            @touch($path);

            if (!self::IsLocked()) {
                throw new Exception('Unable to lock for upgrade');
            }

            $password = uniqid();
            self::$_lockPassword = $password;

            return $password;
        }

        public static function AssertLockIsValid($password)
        {
            if (strlen(self::$_lockPassword) == 0) {
                throw new Exception('No lock password set');
            }

            if ($password != self::$_lockPassword) {
                throw new Exception('Lock password is invalid');
            }
        }

        public static function Unlock()
        {
            if (!self::IsLocked()) {
                throw new Exception('Not locked');
            }

            $path = self::GetLockFullPath();

            $ret = file_get_contents($path);

            $succ = @unlink($path);

            if (!$succ) {
                throw new Exception('Unable to unlock');
            }

            self::$_lockPassword = null;

            return $ret;
        }

        public static function GetInstallLog()
        {
            $path = self::GetLockFullpath();

            $ret = '';

            if (file_exists($path)) {
                $ret = file_get_contents($path);
            }

            return $ret;
        }

        public static function IsLocked()
        {
            return file_exists(self::GetLockFullpath());
        }

        protected static function GetLockFullpath()
        {
            return Application::GetDataDir() . '/' . self::LOCK_FILE;
        }
    }
?>
