<?php
    class Application_Upgrader_Package
    {
        const XML_NAMESPACE = 'http://www.recite.com.au/schema/package';

        protected $_name;
        protected $_version;
        protected $_checksum;
        protected $_downloadPath;
        protected $_localPath;
        protected $_path;
        protected $_oldPackage;
        protected $_channel;

        public function __construct($name, $version)
        {
            $this->_name = $name;
            $this->_version = $version;
        }

        public function __toString()
        {
            return sprintf('%s-%s', $this->_name, $this->_version);
        }

        public function setChecksum($checksum)
        {
            $this->_checksum = $checksum;
            return $this;
        }

        public function getName()
        {
            return $this->_name;
        }

        public function getVersion()
        {
            return $this->_version;
        }

        public function setChannel(Application_Upgrader_Channel $channel)
        {
            $this->_channel = $channel;
            return $this;
        }

        public function setLocalPath($path)
        {
            $this->_localPath = $path;
            return $this;
        }

        public function getReciteVersion()
        {
            $xml    = $this->getXml();
            $result = $xml->xpath('/ns:package/ns:info/ns:recite_version');

            if (count($result) == 0) {
                throw new Exception('Unable to determine required Recite version');
            }

            foreach ($result as $node) {
                return (string) $node;
            }
        }

        public function getDependencies()
        {
            $xml    = $this->getXml();
            $result = $xml->xpath('/ns:package/ns:dependencies/ns:package');

            if (!is_array($result)) {
                return array();
            }

            $ret = array();

            foreach ($result as $dep) {
                $ret[] = Application_Upgrader_Manager::GetPackage($dep['name'] . '-' . $dep['version'], $dep['channel']);
            }

            return $ret;
        }

        public function isInstalled()
        {
            return Application_Upgrader_Manager::PackageIsRegistered($this->getChannel(), $this->getName(), $this->getVersion());
        }

        public function isUpgrade()
        {
            try {
                $oldPackage = $this->getOldPackage();
                $oldVersion = $oldPackage->getVersion();

                if (version_compare($oldVersion, $this->getVersion()) < 0) {
                    return true;
                }
            }
            catch (Exception $ex) {
            }

            return false;
        }

        public function isCompatible()
        {
            $version = $this->getReciteVersion();

            return version_compare($version, Application::COMPAT_VERSION) >= 0 && version_compare(Application::VERSION, $version) >= 0;
        }

        public function getDescription()
        {
            $xml    = $this->getXml();
            $result = $xml->xpath('/ns:package/ns:info/ns:description');

            foreach ($result as $node) {
                return trim($node);
            }

            return '';
        }

        public function getChannel()
        {
            if (!$this->_channel instanceof Application_Upgrader_Channel) {
                $xml    = $this->getXml();
                $result = $xml->xpath('/ns:package/ns:info/ns:version');

                $name = '';

                if (is_array($result)) {
                    foreach ($result as $node) {
                        $name = trim($node['channel']);
                        break;
                    }
                }

                if (strlen($name) == 0) {
                    throw new Exception('Channel not specified');
                }

                $this->_channel = new Application_Upgrader_Channel($name);
            }

            return $this->_channel;
        }

        public function getXml()
        {
            if (is_null($this->_xml)) {
                $this->_downloadAndExtractPackage();

                $this->_xml = new SimpleXmlElement(sprintf('%s/package.xml', $this->_path), null, true);
                $this->_xml->registerXpathNamespace('ns', self::XML_NAMESPACE);

                // update version number just in case it's a newer version with same compatibility version
                $xml    = $this->getXml();
                $result = $xml->xpath('/ns:package/ns:info/ns:version');

                foreach ($result as $node) {
                    $this->_version = (string) $node;
                    break;
                }
            }

            return $this->_xml;
        }

        protected function _downloadAndExtractPackage()
        {
            if (strlen($this->_path) == 0) {
                if (strlen($this->_localPath) > 0) {
                    $path = $this->_localPath;
                }
                else {
                    $this->_downloadPath = Application_Upgrader_Manager::DownloadPackage($this->_name, $this->_version);
                    $path = $this->_downloadPath;
                }

                if (strlen($this->_checksum) > 0) {
                    $checksum = md5_file($path);

                    if ($checksum != $this->_checksum) {
                        throw new Exception(sprintf('Downloaded package file in %s not valid (%s != %s)', $this->_downloadPath, $checksum, $this->_checksum));
                    }
                }

                $this->_path = Application_Upgrader_Manager::Extract($path);
            }

            return $this->_path;
        }

        protected function _cleanup()
        {
            Application_Upgrader_Manager::Cleanup($this->_downloadPath, $this->_path);
        }

        public function getOldPackage()
        {
            if (is_null($this->_oldPackage)) {
                $this->_oldPackage = Application_Upgrader_Manager::GetRegisteredPackage($this->getName());
            }

            return $this->_oldPackage;
        }

        public static function GetItemsToInstall($xml)
        {
            $ret = array();

            $itemsNodes = $xml->xpath('/ns:package/ns:items');

            if (!is_array($itemsNodes)) {
                return $ret;
            }

            foreach ($itemsNodes as $node) {
                $nodes = $node->xpath('child::*');

                if (!is_array($nodes)) {
                    continue;
                }

                if (isset($node['basedir'])) {
                    $baseDir = (string) $node['basedir'];
                }
                else {
                    $baseDir = '';
                }

                foreach ($nodes as $n) {
                    $ret[] = array(
                        'path'     => (string) $n,
                        'basedir'  => $baseDir,
                        'preserve' => isset($n['preserve']) ? (bool) $n['preserve'] : false
                    );
                }
            }

            return $ret;
        }

        public function install()
        {
            try {
                try {
                    $oldPackage = $this->getOldPackage();
                    $oldVersion = $oldPackage->getVersion();

                    $oldPackage->removeItems();
                    $oldPackage->unregister();

                    $upgrade = true;
                }
                catch (Exception $ex) {
                    $oldVersion = null;

                    $upgrade = false;
                }

                $installedItems = array();

                $items = self::GetItemsToInstall($this->getXml());

                foreach ($items as $item) {
                    $this->_installItem($item['path'], $item['basedir']);
                    $installedItems[] = $item;
                }

                $this->_runTasks($oldVersion, $upgrade);
                $this->_register($installedItems);
                $this->_cleanup();
            }
            catch (Exception $ex) {
                $this->_cleanup();
                throw $ex;
            }
        }

        protected function _runTasks($oldVersion, $upgrade)
        {
            $xml = $this->getXml();

            if (!$upgrade || $oldVersion == $this->getVersion()) {
                $xpath = '/ns:package/ns:install/ns:task';

                $install    = true;
                $version    = $this->getVersion();

                Application_Upgrader_Manager::Log('Running install tasks', true);
            }
            else {
                $xpath = '/ns:package/ns:upgrade/ns:task';

                $install    = false;
                $version    = $this->getVersion();

                Application_Upgrader_Manager::Log('Running upgrade tasks', true);
            }

            $tasks = $xml->xpath($xpath);

            if (!is_array($tasks)) {
                return;
            }

            foreach ($tasks as $task) {
                try {
                    $task = Application_Upgrader_Package_Task::FromXml($task);

                    if ($install) {
                        Application_Upgrader_Manager::Log(sprintf('Running install task: %s (version %s)', $task, $version), true);
                        $task->runInstall($version);
                    }
                    else {
                        Application_Upgrader_Manager::Log(sprintf('Running upgrade task: %s (version %s -> %s)', $task, $oldVersion, $version), true);
                        $task->runUpgrade($oldVersion, $version);
                    }
                }
                catch (Exception $ex) {
                    Application_Upgrader_Manager::Log(sprintf("Error: %s", $ex->getMessage()), true);
                    continue;
                }
            }
        }

        protected function _register($installedItems)
        {
            Application_Upgrader_Manager::Log(sprintf("Registering package `%s'", $this), true);

            $db = Application::GetDb();

            $where = array(
                $db->quoteInto('name = ?', $this->getName())
            );

            $db->delete(Application_Upgrader_Manager::REGISTRY_TABLE, $where);

            $values = array(
                'channel'    => $this->getChannel()->getName(),
                'name'       => $this->getName(),
                'version'    => $this->getVersion(),
                'data'       => json_encode(array('items' => $installedItems)),
                'ts_created' => new Zend_Db_Expr('now()')
            );

            $db->insert(
                Application_Upgrader_Manager::REGISTRY_TABLE,
                $values
            );
        }

        protected function _installItem($item, $basePath = '')
        {
            $item = (string) $item;

            Application_Upgrader_Manager::Log(sprintf("Installing `%s'", $item), true);

            $basePath = str_replace('/', DIRECTORY_SEPARATOR, trim($basePath, '/'));
            $basePath = realpath(Application::GetBaseDir() . DIRECTORY_SEPARATOR . $basePath);

            if (strlen($basePath) == 0) {
                throw new Exception('Invalid application base path');
            }

            $fullpath = str_replace('/', DIRECTORY_SEPARATOR, trim($item, '/'));
            $fullpath = sprintf('%s%s%s', $basePath, DIRECTORY_SEPARATOR, $fullpath);

            if (!preg_match('|^' . preg_quote($basePath . DIRECTORY_SEPARATOR, '|') . '|', $fullpath)) {
                throw new Exception('Invalid package item');
            }

            $srcpath = sprintf('%s%s%s', $this->_path, DIRECTORY_SEPARATOR, str_replace('/', DIRECTORY_SEPARATOR, $item));

            if (!file_exists($srcpath)) {
                throw new Exception('Package item not found in package');
            }

            // @todo check if this package can indeed write here: no if installing and path exists, yes if upgrading and previously owns the path

            if (file_exists($fullpath)) {
                File::RmdirRecursive($fullpath);
            }

            File::CreateDirStructure(dirname($fullpath));

            rename($srcpath, $fullpath);

            Application_Upgrader_Manager::FixPermissions($fullpath);

            return $item;
        }


        public static function GetFilenameInfo($filename)
        {
            if (strlen($filename) == 0) {
                throw new Exception('Filename not specified');
            }

            if (!preg_match('/^([a-z0-9-]+)-(\d+\.\d+\.\d+(-?(a|b|rc|dev)\d*)?)(\.zip)?$/i', $filename, $matches)) {
                throw new Exception('Invalid filename: ' . $filename);
            }

            return array(
                'name'    => $matches[1],
                'version' => $matches[2]
            );
        }

        public static function GetInstallInfo($packages)
        {
            if (!is_array($packages)) {
                $packages = array();
            }

            $path = sprintf('%s%stools%spackages', Application::GetApplicationDir(), DIRECTORY_SEPARATOR, DIRECTORY_SEPARATOR);

            $cmds = array(
                'install'    => 'install.php',
                'installAll' => 'install-all.php',
                'upgradeAll' => 'upgrade-all.php'
            );

            $ret = array(
                'sample' => ''
            );

            $baseCmds = array();

            if (IS_WINDOWS) {
                $baseCmds[] = substr($path, 0, 2);
            }

            $baseCmds[] = sprintf('cd %s', $path);


            foreach ($cmds as $cmd => $script) {
                $ret[$cmd] = array(
                    'fullpath' => $path . DIRECTORY_SEPARATOR . $script,
                    'filename' => $script,
                    'dirname'  => $path,
                    'commands' => $baseCmds
                );
            }

            //if (count($packages) == 0) {
                $ret['sample'] = 'package-name-x.y.z';
                $packages = array($ret['sample']);
            //}

            $bin = basename(Application_Maintenance::GetCliPath());
            $ret['install']['commands'][] = sprintf('%s %s %s', $bin, $ret['install']['filename'], join(' ', $packages));
            $ret['installAll']['commands'][] = sprintf('%s %s', $bin, $ret['installAll']['filename']);
            $ret['upgradeAll']['commands'][] = sprintf('%s %s', $bin, $ret['upgradeAll']['filename']);

            return $ret;
        }
    }
?>