<?php
    class Application_User extends DatabaseObject
    {
        public static $table = 'users';
        public static $rolesTable = 'users_to_roles';

        const STATUS_UNAPPROVED = 'U';
        const STATUS_ACTIVE     = 'A';
        const STATUS_INACTIVE   = 'I';
        const STATUS_DISABLED   = 'D';

        public static $statuses = array(
            self::STATUS_UNAPPROVED => 'Unapproved',
            self::STATUS_ACTIVE     => 'Active',
            self::STATUS_INACTIVE   => 'Inactive',
            self::STATUS_DISABLED   => 'Disabled'
        );

        const ACTIVATE_CODE_MAX_AGE_DAYS = 2;

        public $profile;
        public $directory;

        protected $_roles;

        protected $_newPassword;
        protected $_storeNewPassword = false;
        protected $_audit = true;
        protected $_disable_notification = false;
        protected $_reindex = true;
        protected $_fieldList;
        protected $_auto_subscribe = true;
        protected $_publicIdentity;

        protected static $_getRolesCache = array();

        public function __construct()
        {
            parent::__construct(Application::GetDb(), self::$table, 'user_id');

            $this->add('directory_id');
            $this->add('username');
            $this->add('password');
            $this->add('status', self::STATUS_ACTIVE);
            $this->add('ts_created', time(), self::TYPE_TIMESTAMP);
            $this->add('ts_updated', null, self::TYPE_TIMESTAMP);
            $this->add('ts_last_login', null, self::TYPE_TIMESTAMP);

            $this->profile = new Application_User_Profile($this->db);
            $this->directory = new Application_User_Directory();
        }

        public function __destruct()
        {
            parent::__destruct();
            unset($this->profile, $this->directory);
        }

        /**
         * Callbacks
         */

        protected function postLoad()
        {
            try {
                $this->directory = DatabaseObject_Registry::Load('Application_User_Directory', $this->directory_id);
            }
            catch (Exception $ex) {
            }

            $this->profile->setUserId($this->getId());
            $this->profile->load();
        }

        protected function preInsert()
        {
            $this->ts_created = time();
            $this->ts_updated = time();

            if ($this->_storeNewPassword) {
                $this->password = Application_Auth::GetAuthHash($this->username, $this->_newPassword);
            }

            return true;
        }

        protected function preUpdate()
        {
            $this->ts_updated = time();

            if ($this->updated('status') && $this->status == self::STATUS_ACTIVE) {
                // ensure a new password is set when account is reactivated
                if (strlen($this->_newPassword) == 0) {
                    $this->setPassword(Text_Password::create(8));
                }
            }

            if ($this->_storeNewPassword) {
                $this->password = Application_Auth::GetAuthHash($this->username, $this->_newPassword);
            }

            return true;
        }

        protected function postInsert()
        {
            $this->profile->setUserId($this->getId());
            $this->profile->save();

            $event = new Application_Event('/module/users/user/postinsert');
            $event->set('user', $this);
            $event->trigger();

            if ($this->_audit) {
                $message = new Application_Auditor_Message('Users: User created');
                $message->addMemo('user_id', $this->getId())
                        ->addMemo('username', $this->username)
                        ->addMemo('directory_id', $this->directory_id)
                        ->record();
            }

            if ($this->status == self::STATUS_ACTIVE) {
                $this->sendNewAccountNotification();
            }

            return true;
        }

        protected function postUpdate()
        {
            $this->profile->save();

            $event = new Application_Event('/module/users/user/postupdate');
            $event->set('user', $this);
            $event->trigger();

            $this->_publicIdentity = null;

            if ($this->_audit) {
                $message = new Application_Auditor_Message('Users: User updated');
                $message->addMemo('user_id', $this->getId())
                        ->addMemo('username', $this->username)
                        ->addMemo('directory_id', $this->directory_id)
                        ->record();
            }

            if ($this->updated('status') && $this->status == self::STATUS_ACTIVE) {
                $this->sendNewAccountNotification();
            }

            return true;
        }

        protected function preDelete()
        {
            $event = new Application_Event('/module/users/user/predelete');
            $event->set('user', $this);
            $event->trigger();

            $this->clearRoles();

            $this->profile->delete();

            return true;
        }

        protected function postDelete()
        {
            $event = new Application_Event('/module/users/user/postdelete');
            $event->set('id', $this->getDeletedId())
                  ->set('directory_id', $this->directory_id);

            $event->trigger();

            if ($this->_audit) {
                $message = new Application_Auditor_Message('Users: User deleted');
                $message->addMemo('user_id', $this->getDeletedId())
                        ->addMemo('username', $this->username)
                        ->addMemo('directory_id', $this->directory_id)
                        ->record();
            }

            return true;
        }

        /**
         * Loaders
         */

        public function loadForDirectory($directoryId, $userId, $statuses = null)
        {
            $userId      = (int) $userId;
            $directoryId = (int) $directoryId;

            if ($userId <= 0) {
                return false;
            }

            $select = $this->db->select();
            $select->from($this->getTable(), $this->getSelectFields())
                   ->where('user_id = ?', $userId);

            if ($directoryId > 0) {
                $select->where('directory_id = ?', $directoryId);
            }
            else {
                $select->where('directory_id is null');
            }

            $statuses = ArrayTools::clean($statuses);
            if (count($statuses) > 0) {
                $select->where('status in (?)', $statuses);
            }

            return $this->_load($select);
        }

        public function loadByUsername($directory_id, $username, $statuses = null)
        {
            $username     = trim($username);
            $directory_id = (int) $directory_id;

            if (strlen($username) == 0) {
                return false;
            }

            $select = $this->db->select();
            $select->from($this->getTable(), $this->getSelectFields())
                   ->where('lower(username) = lower(?)', $username);

            if ($directory_id > 0) {
                $select->where('directory_id = ?', $directory_id);
            }
            else {
                $select->where('directory_id is null');
            }

            $statuses = ArrayTools::clean($statuses);
            if (count($statuses) > 0) {
                $select->where('status in (?)', $statuses);
            }

            return $this->_load($select);
        }

        public function loadRecordForClientDirectory($client_id, $user_id=NULL)
        {
            $client_id = (int) $client_id;
            $user_id   = (int) $user_id;

            if ( empty($client_id) || empty( $user_id ) ) {
                return false;
            }

            $select = $this->db->select();

            $select->from(array('u' => $this->getTable()), $this->getSelectFields('u.'))
                   ->joinInner(array('d' => Application_User_Directory::$table), 'u.directory_id = d.directory_id', array())
                   ->where('u.user_id = ?', $user_id)
                   ->where('d.client_id = ?', $client_id);

            return $this->_load($select);
        }

        public function loadControlPanelUser($client_id, $user_id)
        {
            $client_id = (int) $client_id;
            $user_id   = (int) $user_id;

            if ($client_id == 0 || $user_id == 0)
                return false;

            $select = $this->db->select();

            $select->from(array('u' => $this->getTable()), $this->getSelectFields('u.'))
                   ->joinInner(array('utr' => self::$rolesTable), 'utr.user_id = u.user_id', array())
                   ->joinInner(array('ur' => Application_User_Role::$table), 'utr.role_id = ur.role_id', array())
                   ->where('u.directory_id is null')
                   ->where('ur.directory_id is null')
                   ->where('ur.client_id = ?', $client_id)
                   ->where('u.user_id = ?', $user_id)
                   ->limit(1);

            return $this->_load($select);
        }

        public function loadControlPanelUserByUsername($client_id, $username, $statuses = null)
        {
            $client_id = (int) $client_id;
            $username  = trim($username);

            if ($client_id <= 0 || strlen($username) == 0)
                return false;

            $select = $this->db->select();

            $select->from(array('u' => $this->getTable()), $this->getSelectFields('u.'))
                   ->joinInner(array('utr' => self::$rolesTable), 'utr.user_id = u.user_id', array())
                   ->joinInner(array('ur' => Application_User_Role::$table), 'utr.role_id = ur.role_id', array())
                   ->where('u.directory_id is null')
                   ->where('ur.directory_id is null')
                   ->where('ur.client_id = ?', $client_id)
                   ->where('lower(u.username) = lower(?)', $username)
                   ->limit(1);

            $statuses = ArrayTools::clean($statuses);
            if (count($statuses) > 0) {
                $select->where('u.status in (?)', $statuses);
            }

            return $this->_load($select);
        }

        public function loadSysadminByUsername($username, $statuses = null)
        {
            $username = trim($username);

            if (strlen($username) == 0) {
                return false;
            }

            $select = $this->db->select();
            $select->from(array('u' => $this->getTable()), $this->getSelectFields('u.'))
                   ->where('u.directory_id is null')
                   ->where('lower(u.username) = lower(?)', $username)
                   ->join(array('utr' => self::$rolesTable), 'utr.user_id = u.user_id', array())
                   ->join(array('ur' => Application_User_Role::$table), 'utr.role_id = ur.role_id', array())
                   ->where('ur.directory_id is null')
                   ->where('ur.title = ?', 'sysadmin')
                   ->limit(1);

            $statuses = ArrayTools::clean($statuses);
            if (count($statuses) > 0) {
                $select->where('u.status in (?)', $statuses);
            }

            return $this->_load($select);
        }

        /**
         * Role management
         */

        public function getUserRoles()
        {
            if (!is_array($this->_roles)) {
                $options = array(
                    'user_ids'     => $this->getId(),
                    'load_clients' => true
                );
                $this->_roles = Application_User_Role::GetRoles($options);
            }

            return $this->_roles;
        }

        public function getUserRolesForClient($client_id = null)
        {
            $client_id = (int) $client_id;
            if ($client_id <= 0) {
                $client_id = getClientId();
            }

            $ret = array();

            foreach ($this->getUserRoles() as $role) {
                if ($role->client_id == $client_id) {
                    $ret[$role->getId()] = $role;
                }
            }

            return $ret;
        }

        public function roleSet($roleIds)
        {
            $roleIds = ArrayTools::Clean($roleIds, true);

            if (count($roleIds) == 0) {
                return;
            }

            $add      = array();
            $remove   = array();
            $existing = self::GetRoles($this->getId());

            foreach ($existing as $roleId => $role) {
                if (!in_array($roleId, $roleIds)) {
                    $remove[] = $roleId;
                }
            }

            foreach ($roleIds as $roleId) {
                if (!array_key_exists($roleId, $existing)) {
                    $add[] = $roleId;
                }
            }

            foreach ($remove as $id) {
                $this->roleRemove($id);
            }

            foreach ($add as $id) {
                $this->roleAdd($id);
            }
        }

        public function roleAdd($role_id)
        {
            $role = new Application_User_Role();
            if (!$role->loadForDirectory($this->directory_id, $role_id)) {
                return false;
            }

            $roles = $this->getUserRoles();
            if (array_key_exists($role_id, $roles)) {
                return true;
            }

            $this->db->begin();

            $values = array(
                'user_id' => $this->getId(),
                'role_id' => $role->getId()
            );

            $this->db->insert(self::$rolesTable, $values);

            if ($role->isControlPanelRole()) {
                try {
                    Application_User_Cp_Tab::DeployWidgets($this, $role->getClient());
                }
                catch (Exception $ex) {
                }
            }

            if ($this->_audit) {
                $message = new Application_Auditor_Message('Users: Role added to user');
                $message->addMemo('user_id', $this->getId())
                        ->addMemo('username', $this->username)
                        ->addMemo('role_id', $role->getId())
                        ->record();
            }

            $this->db->commit();

            unset(self::$_getRolesCache[$this->getId()]);

            return true;
        }

        public function roleRemove($role_id)
        {
            $role = new Application_User_Role();
            if (!$role->loadForDirectory($this->directory_id, $role_id))
                return false;

            $query = sprintf('delete from %s where user_id = %d and role_id = %d',
                             self::$rolesTable,
                             $this->getId(),
                             $role_id);

            $this->db->query($query);

            if ($this->_audit) {
                $message = new Application_Auditor_Message('Users: Role removed from user');
                $message->addMemo('user_id', $this->getId())
                        ->addMemo('username', $this->username)
                        ->addMemo('role_id', $role->getId())
                        ->record();
            }

            unset(self::$_getRolesCache[$this->getId()]);
        }

        public function clearRoles()
        {
            $query = sprintf('delete from %s where user_id = %d',
                             self::$rolesTable,
                             $this->getId());

            $this->db->query($query);

            $this->_roles = array();

            unset(self::$_getRolesCache[$this->getId()]);
        }

        public function getControlPanelRole($client_id)
        {
            if ($client_id instanceof Application_Client) {
                $client_id = $client_id->getId();
            }

            foreach ($this->getUserRoles() as $role) {
                if (!is_null($role->directory_id))
                    continue;

                if ($role->client_id == $client_id)
                    return $role;
            }

            throw new Exception('No control panel role found');
        }

        public function removeControlPanelRoles($client_id)
        {
            $select = $this->db->select();
            $select->from(array('ur' => self::$rolesTable), 'role_id')
                   ->join(array('r' => Application_User_Role::$table), 'ur.role_id = r.role_id', array())
                   ->where('ur.user_id = ?', $this->getId())
                   ->where('r.client_id = ?', $client_id)
                   ->where('r.directory_id is null');

            $role_ids = $this->db->fetchCol($select);

            if (count($role_ids) > 0) {
                $where = array(
                    $this->db->quoteInto('role_id in (?)', $role_ids),
                    $this->db->quoteInto('user_id = ?', $this->getId())
                );

                $this->db->delete(self::$rolesTable, $where);

                $this->_roles = null;
                unset(self::$_getRolesCache[$this->getId()]);
            }
        }

        /**
         * Misc functions
         */

        public function __toString()
        {
            try {
                $str = $this->getFullname();

                if (strlen($str) == 0) {
                    throw new Exception('No name found');
                }
            }
            catch (Exception $ex) {
                $str = $this->username;
            }

            return $str;
        }

        public function setStatus($status)
        {
            if (!array_key_exists($status, self::$statuses)) {
                return;
            }

            if ($this->status != self::STATUS_ACTIVE && $status == self::STATUS_ACTIVE) {
                $password = $this->profile->_newPassword;
                unset($this->profile->_newPassword);

                if (strlen($password) > 0)
                    $this->_newPassword = $password;
            }

            $this->status = $status;
        }

        public function isActive()
        {
            return $this->status == self::STATUS_ACTIVE;
        }

        public function getNewPassword()
        {
            return $this->profile->_newPassword;
        }

        public function getPublicIdentityData()
        {
            if (!is_array($this->_publicIdentity)) {
                $identity = array(
                    'user_id'  => $this->getId(),
                    'username' => $this->username,
                    'roles'    => array(
                        'ids'   => array(),
                        'names' => array()
                    ),
                    'custom' => array()
                );

                foreach (self::GetRoles($this->getId()) as $role) {
                    $name = Text::Urlize($role->title);
                    $identity['roles']['names'][$name] = $role->title;
                    $identity['roles']['ids'][$role->role_id] = $role->title;
                }

                if (!array_key_exists('email', $identity)) {
                    try {
                        $identity['email'] = $this->getEmailAddress();
                    }
                    catch (Exception $ex)
                    { }
                }

                if (!array_key_exists('full_name', $identity)) {
                    $identity['full_name'] = $this->getFullName();
                }

                if (!array_key_exists('first_name', $identity)) {
                    $identity['first_name'] = $this->getFirstName();
                }

                if (!array_key_exists('last_name', $identity)) {
                    $identity['last_name'] = $this->getLastName();
                }

                if ($this->directory->saved()) {
                    $identity['directory'] = array(
                        'id'    => $this->directory->getId(),
                        'title' => (string) $this->directory
                    );

                    $custom = array();

                    $event = new Application_Event('/module/users/user/getauthidentity');
                    $event->responds()
                          ->set('user', $this);

                    $responseCollection = $event->trigger();

                    foreach ($responseCollection->getResponses() as $response) {
                        foreach ($response->getData() as $k => $v) {
                            $custom[$k] = $v;
                        }
                    }

                    $identity['custom'] = $custom;
                }

                $identity['fields'] = $this->directory->getClientTemplateData();

                foreach ($this->directory->getEditableProfileFields() as $field) {
                    $key = $field->getName();

                    if (!array_key_exists($key, $identity)) {
                        $identity[$key] = $this->profile->get($key);
                    }
                }

                $identity['ts_generated'] = time();

                $this->_publicIdentity = $identity;
            }

            return $this->_publicIdentity;
        }

        public function getStatusDescription()
        {
            if (array_key_exists($this->status, self::$statuses)) {
                return self::$statuses[$this->status];
            }

            return '';
        }

        public function checkPassword($password, $hashed = false)
        {
            $hashed = (bool) $hashed;

            if ($hashed) {
                return strtolower($password) == strtolower($this->password);
            }
            else {
                return strtolower(Application_Auth::GetAuthHash($this->username, $password)) == strtolower($this->password);
            }
        }

        public function getUserHash()
        {
            $parts = array(
                $this->username,
                $this->getId(),
            );

            try {
                $parts[] = $this->getEmailAddress();
            }
            catch (Exception $ex)
            { }

            return md5(join(', ', $parts));
        }

        public function getClientTemplateData()
        {
            $ret = array(
                'id'         => $this->getId(),
                'title'      => (string) $this,
                'username'   => $this->username,
                'first_name' => $this->getFirstName(),
                'last_name'  => $this->getLastName(),
                'name'       => $this->getFullName(),
                'email'      => '',
                'hash'       => $this->getUserHash(),
                'linked'     => array()
            );

            try {
                $ret['email'] = $this->getEmailAddress();
            }
            catch (Exception $ex)
            { }

            if (isset($this->_newPassword) && strlen($this->_newPassword) > 0) {
                $ret['password'] = $this->_newPassword;
            }

            $ret['fields'] = $this->getDirectory()->getEditableProfileFields(true)->getClientTemplateData($this->profile);

            $evt = new Application_Event('/module/users/user/gctd');
            $evt->responds()
                ->set('user', $this);

            $responseCollection = $evt->trigger();

            foreach ($responseCollection->getResponses() as $response) {
                foreach ($response->getData() as $k => $v) {
                    $ret['linked'][$k] = $v;
                }
            }

            return $ret;
        }

        public function getDirectory()
        {
            return $this->directory;
        }

        public function setDirectory(Application_User_Directory $directory)
        {
            $this->directory = $directory;
            $this->directory_id = $directory->getId();
            return $this;
        }

        public function generateNewPassword()
        {
            $password = Text_Password::create(8);
            $this->setPassword($password);
            return $password;
        }


        public function setPassword($password, $hashed = false)
        {
            if ($hashed) {
                parent::__set('password', strtolower($password));
            }
            else {
                if ($this->status != self::STATUS_ACTIVE) {
                    $this->profile->_newPassword = $password;
                }
                $this->_newPassword      = $password;
                $this->_storeNewPassword = true;
            }

            return true;
        }

        public function userExists($username, $directory_id = null)
        {
            $query = sprintf("select count(*) as num_hits from %s where lower(username) = lower(%s)",
                             $this->getTable(),
                             $this->db->quote($username));

            if ($this->saved())
                $query .= sprintf(' and %s != %d', $this->getIdField(), $this->getId());

            $directory_id = (int) $directory_id;
            if ($directory_id > 0)
                $query .= sprintf(' and directory_id = %d', $directory_id);
            else
                $query .= sprintf(' and directory_id is null');

            return $this->db->fetchOne($query) > 0;
        }

        public function getStatusLabel()
        {
            $status = $this->status;

            if (array_key_exists($status, self::$statuses)) {
                return self::$statuses[$status];
            }

            return null;
        }

        public function getClientIds()
        {
            $options = array(
                'user_ids'     => $this->getId(),
                'filters'      => array('r.client_id is not null', 'r.client_id > 0')
            );
            $roles = Application_User_Role::GetRoles($options);

            $client_ids = array();
            foreach ($roles as $role)
                $client_ids[$role->client_id] = (int) $role->client_id;

            return $client_ids;
        }

        public function getFirstName()
        {
            $fieldList = $this->getDirectory()->getEditableProfileFields();

            try {
                $field = $fieldList->getFirstNameField();
                return trim($this->profile->get($field->getName()));
            }
            catch (Exception $ex)
            {
                return '';
            }
        }

        public function getLastName()
        {
            $fieldList = $this->getDirectory()->getEditableProfileFields();

            try {
                $field = $fieldList->getLastNameField();
                return trim($this->profile->get($field->getName()));
            }
            catch (Exception $ex)
            {
                return '';
            }
        }

        public function getFullname()
        {
            $vals = array(
                'first' => '',
                'last'  => '',
                'full'  => ''
            );

            $fieldList = $this->getDirectory()->getEditableProfileFields();

            foreach ($fieldList as $field) {
                $title = $field->getTitle();
                $name  = $field->getName();

                if (!preg_match('/name/i', $title)) {
                    continue;
                }

                $value = trim($this->profile->get($field->getName()));

                if (strlen($value) == 0) {
                    continue;
                }


                if (preg_match('/first|given/i', $title) && strlen($vals['first']) == 0) {
                    $vals['first'] = $value;
                }
                else if (preg_match('/last|sur|family/i', $title) && strlen($vals['last']) == 0) {
                    $vals['last'] = $value;
                }
                else {
                    $vals['full'] = $value;
                }
            }

            if (strlen($vals['first']) && strlen($vals['last']) > 0) {
                return $vals['first'] . ' ' . $vals['last'];
            }
            else if (strlen($vals['full']) > 0) {
                return $vals['full'];
            }
            else if (strlen($vals['first']) > 0) {
                return $vals['first'];
            }
            else if (strlen($vals['last']) > 0) {
                return $vals['last'];
            }

            return $this->username;
        }

        public function getEmailAddress()
        {
            $directory = $this->getDirectory();

            if ($directory->usernameIsEmail()) {
                $email = $this->username;
            }
            else {
                $field = $directory->getEmailAddressField();
                $type  = $field->getType();

                $email = $this->profile->get($field->getName());
            }

            if (strlen($email) > 0) {
                $validator = new Zend_Validate_EmailAddress();

                if ($validator->isValid($email)) {
                    return $email;
                }
            }

            throw new Exception('No email address found');
        }

        public function getEmailAddressField()
        {
            return $this->getDirectory()->getEmailAddressField();
        }

        public function getProfileFields()
        {
            if (!$this->_fieldList instanceof Profile_Field_FieldList) {
                $this->_fieldList = clone $this->getDirectory()->getEditableProfileFields();
                $this->_fieldList->setValuesFromProfile($this->profile);
            }

            return $this->_fieldList;
        }

        public function disableNotification($flag = true)
        {
            $this->_disable_notification = (bool) $flag;
        }

        public function sendNewAccountNotification()
        {
            if ($this->_disable_notification) {
                return;
            }

            $directory = $this->getDirectory();

            // only do this for new users on client directories
            if (!$directory->isClientDirectory() || !$directory->profile->send_welcome_email) {
                return;
            }

            $client = $directory->getClient();

            $options = new Application_Mail_Options();
            $options->setView(new Application_View_Client_Email($client))
                    ->setFrom($directory);

            $template = new Module_Templates_Template();

            if ($template->load($directory->profile->template_id)) {
                $options->setTemplate($template);
            }
            else {
                Application_Loader::AssertCanLoadClass('Module_Forms_Driver_Manager');

                $driver = Module_Forms_Driver_Manager::Factory('user_register');
                $tt = $driver->getEmailTemplateType();

                $options->setTemplate($tt->getSample('generic-registration-complete.tpl'));
            }

            $options->assign('client', $client->getClientTemplateData())
                    ->assign('directory', $directory->getClientTemplateData());

            $this->sendEmail($options);
        }

        public function sendEmail(Application_Mail_Options $options)
        {
            $options->addTo($this)
                    ->assign('user', $this->getClientTemplateData());

            $response = Application_Mail_Sender::Send($options);
            Application_User_Email::Record($this, $response);
        }

        /**
         * deleteForControlPanel
         *
         * This method will try to delete the current record if it is a control panel
         * user and only belongs to the client that is passed (i.e. doesn't have any
         * other roles). If it does have other roles then only the role for the given
         * client shall be removed.
         *
         * @param   int     $client_id
         */
        public function deleteForControlPanel($client_id)
        {
            if (!is_null($this->directory_id))
                return false;

            $roles = $this->getUserRoles();

            $remove = array();
            $delete = true;

            foreach ($roles as $role) {
                if ($role->client_id == $client_id)
                    $remove[] = $role->getId();
                else
                    $delete = false;
            }

            if ($delete)
                $this->delete();
            else foreach ($remove as $role_id) {
                $this->roleRemove($role_id);
            }

            return true;
        }

        /**
         * Password reset functions
         */

        public function setActivateCode()
        {
            $seed = time() . uniqid();
            $this->profile->activate_code = md5($seed);
            $this->profile->activate_code_time = time();
        }

        public function fetchPassword($send = true)
        {
            $this->setActivateCode();
            $this->save();

            if (!$send) {
                return true;
            }

            $type = new Module_Templates_Template_Type_Email('email', 'Email');
            $type->addSamplesPath(sprintf('%s/cp/templates/default/auth/email', Application::GetApplicationDir()));

            $options = new Application_Mail_Options();
            $options->setTemplate($type->getSample('reset-password.tpl'))
                    ->assign('activate_code', $this->profile->activate_code);

            try {
                $options->assign('brand', Application::GetBrand()->getClientTemplateData());
            }
            catch (Exception $ex) {
            }

            $this->sendEmail($options);
        }

        public function fetchAdminPassword($url)
        {
            $this->setActivateCode();
            $this->save();

            $path = sprintf('%s/admin/templates/default/auth/email/', Application::GetApplicationDir());

            $options = new Application_Mail_Options();

            $options->assign(
                'url',
                sprintf('%s?u=%s&a=%s', $url, $this->username, $this->profile->activate_code)
            );

            $options->setTemplate(
                file_get_contents($path . 'reset-password.tpl.subject'),
                file_get_contents($path . 'reset-password.tpl.text')
            );

            $this->sendEmail($options);
        }

        public function resetPassword($password)
        {
            $this->setPassword($password);
            unset($this->profile->activate_code);
            unset($this->profile->activate_code_time);
            return $this->save();
        }

        public function checkActivateCode($code)
        {
            $code = trim($code);
            if (strlen($code) == 0)
                return false;

            if (!isset($this->profile->activate_code) || !isset($this->profile->activate_code_time))
                return false;

            if ($code != $this->profile->activate_code)
                return false;

            $ts = (int) $this->profile->activate_code_time;
            if ($ts <= 0)
                return false;

            if (time() - self::ACTIVATE_CODE_MAX_AGE_DAYS * 86400 > $ts)
                return false;

            return true;
        }

        /**
         * Auth related methods
         */

        public function loginSuccess()
        {
            $this->ts_last_login = time();

            $query = sprintf('update %s set ts_last_login = now() where user_id = %d', self::$table, $this->getId());
            $this->db->query($query);

            $msg = new Application_Auditor_Message('Users: Successful login');
            $msg->setUserId($this->getId());
            $msg->record();
        }

        static public function LoginFailure($username)
        {
            $message = new Application_Auditor_Message('Users: Failed login');
            $message->addMemo('username', $username)->record();
        }

        /**
         * Static methods
         */

        public static function GetMappedStatus($status)
        {
            switch ($status) {
                case self::STATUS_UNAPPROVED:
                    return Application_Content_Status::UNAPPROVED;

                case self::STATUS_ACTIVE:
                    return Application_Content_Status::ACTIVE;

                case self::STATUS_INACTIVE:
                    return Application_Content_Status::INACTIVE;

                case self::STATUS_DISABLED:
                    return Application_Content_Status::DISABLED;

            }

            throw new Exception('Invalid status');
        }

        public static function GetRoles($userId)
        {
            if (!array_key_exists($userId, self::$_getRolesCache)) {
                $db = Application::GetDb();

                $select = $db->select();
                $select->from(array('utr' => self::$rolesTable), array())
                       ->where('utr.user_id = ?', $userId)
                       ->join(array('ur' => Application_User_Role::$table), 'utr.role_id = ur.role_id', array('ur.role_id', 'ur.title', 'ur.client_id'))
                       ->join(array('u' => self::$table), 'utr.user_id = u.user_id', array())
                       ->where('coalesce(ur.directory_id, 0) = coalesce(u.directory_id, 0)');

                $result = $db->query($select);

                $ret = array();
                while ($row = $result->fetch()) {
                    $ret[$row->role_id] = $row;
                }

                self::$_getRolesCache[$userId] = $ret;
            }

            return self::$_getRolesCache[$userId];
        }

        public function GetUsersCount($options)
        {
            $select = self::_GetBaseQuery($options);
            $select->from(null, 'count(*)');

            return (int) Application::GetDb()->fetchOne($select);
        }

        public function GetUsers($options)
        {
            $defaults = array(
                'order'  => array('lower(u.username)'),
                'sorter' => null,
                'offset' => 0,
                'limit'  => 0
            );

            $options = ArrayTools::buildOptions($options, $defaults);

            $db = Application::GetDb();

            $select = self::_GetBaseQuery($options);
            $select->from(null, 'u.*');

            if ($options['sorter'] instanceof Profile_Field_FieldList_Sorter) {
                foreach ($options['sorter']->get() as $k => $field) {
                    $alias = 'up' . $k;

                    $cond = array(
                        'u.user_id = ' . $alias . '.user_id',
                        $db->quoteInto($alias . '.profile_key = ?', $field->getName())
                    );

                    $select->joinLeft(array($alias => self::$table . '_profile'), '(' . join(' and ', $cond) . ')', array())
                           ->order($field->get($db, $alias . '.profile_value'));
                }
            }

            $select->order($options['order']);

            $options['offset'] = max(0, (int) $options['offset']);
            if ($options['limit'] > 0) {
                $select->limit($options['limit'], $options['offset']);
            }

            $result = $db->query($select);
            $data = array();
            while ($row = $result->fetch()) {
                $data[] = $row;
            }

            $users = parent::BuildMultiple($db, __CLASS__, $data);
            self::_LoadDirectories($users);
            self::_LoadProfile($users);
            self::_LoadRoles($users);

            return $users;
        }

        private function _LoadProfile(&$arr)
        {
            if (count($arr) == 0) {
                return;
            }

            $db    = Application::GetDb();
            $ids   = array_keys($arr);
            $class = 'Application_User_Profile';

            try {
                $profiles = Profile::BuildMultiple($db, $class, array('user_id' => $ids));
            }
            catch (Exception $ex) {
                $profiles = array();
            }

            foreach ($arr as $id => $row) {
                if (array_key_exists($id, $profiles) && $profiles[$id] instanceof $class) {
                    $arr[$id]->profile = $profiles[$id];
                }
                else {
                    $arr[$id]->profile = new $class($db, $id);
                }
            }
        }

        private function _LoadDirectories(&$arr)
        {
            if (count($arr) == 0)
                return;

            $directory_ids = array();
            foreach ($arr as $row) {
                $directory_ids[$row->directory_id] = $row->directory_id;
            }

            $directory_ids = ArrayTools::Clean($directory_ids, true);

            if (count($directory_ids) == 0)
                return;

            $options = array(
                'directory_ids' => $directory_ids
            );

            $directories = Application_User_Directory::GetDirectories($options);

            foreach ($arr as $k => $v) {
                if (array_key_exists($v->directory_id, $directories)) {
                    $arr[$k]->directory = $directories[$v->directory_id];
                }
            }
        }

        private function _LoadRoles(&$arr)
        {
            if (count($arr) == 0) {
                return;
            }

            $db = Application::GetDb();

            $select = $db->select();
            // *** danh - 09/07/2015 - correcting query for following references to ->client_id below ***
            //$select->from(self::$rolesTable, '*')
            //       ->where('user_id in (?)', array_keys($arr));
            $select->from(array('r' => self::$rolesTable), null)
                ->joinLeft(array('u' => self::$table), '(r.user_id=u.user_id)', null)
                ->joinLeft(array('d' => Application_User_Directory::$table), '(d.directory_id=u.directory_id)', null)
                ->from(null, array('r.*', 'd.client_id'))
                ->where('r.user_id in (?)', array_keys($arr));
            // *** end danh - 09/07/2015 - correcting query for following references to ->client_id below ***

            $role_ids = array();
            $data = array();
            $result = $db->query($select);

            while ($row = $result->fetch()) {
                if (!array_key_exists($row->client_id, $data))
                    $data[$row->client_id] = array();

                $data[$row->client_id][$row->role_id] = $row->role_id;
                $role_ids[$row->role_id] = $row->role_id;
            }

            if (count($role_ids) == 0)
                return;

            $options = array(
                'role_ids' => $role_ids
            );

            $roles = Application_User_Role::GetRoles($options);

            // *** danh - 09/07/2015 - $users is undefined?!? ***
            // Also, what is this loop even for... there are no class instantiations,
            // no cache values set... commenting out for now; perhaps incomplete function?
            //foreach ($arr as $user_id => $user) {
            //    $users[$user_id]->_roles = array();

            //    if (!array_key_exists($user_id, $data))
            //        continue;

            //    foreach ($data[$user_id] as $role_id) {
            //        if (!array_key_exists($role_id, $roles))
            //            continue;

            //        $users[$user_id]->_roles[$role_id] = $roles[$role_id];
            //    }
            //}
            // *** end danh - 09/07/2015 - $users is undefined?!? ***
        }

        public static function _GetBaseQuery($options)
        {
            $defaults = array(
                'user_ids'       => array(),
                'usernames'      => array(),
                'directory_ids'  => array(),
                'client_ids'     => array(),
                'role_ids'       => array(),
                'any_role_only'  => false,
                'no_role_only'   => false,
                'statuses'       => array(),
                'default_directory_only' => false,
                'exclude_user_ids' => array(),
                'filters'        => null
            );

            $options = ArrayTools::buildOptions($options, $defaults);

            $options['user_ids']      = ArrayTools::clean($options['user_ids'], true);
            $options['usernames']     = ArrayTools::clean($options['usernames']);
            $options['exclude_user_ids'] = ArrayTools::clean($options['exclude_user_ids'], true);
            $options['directory_ids'] = ArrayTools::clean($options['directory_ids'], true);
            $options['client_ids']    = ArrayTools::clean($options['client_ids'], true);
            $options['role_ids']      = ArrayTools::clean($options['role_ids'], true);
            $options['statuses']      = ArrayTools::clean($options['statuses']);

            $db = Application::GetDb();

            $select = $db->select();
            $select->from(array('u' => self::$table), array());

            if (count($options['user_ids']) > 0) {
                $select->where('u.user_id in (?)', $options['user_ids']);
            }

            if (count($options['usernames']) > 0) {
                $select->where('u.username in (?)', $options['usernames']);
            }

            if (count($options['exclude_user_ids']) > 0) {
                $select->where('u.user_id not in (?)', $options['exclude_user_ids']);
            }

            if ($options['default_directory_only']) {
                $options['directory_ids'] = array();

                $select->where('u.directory_id is null');
            }

            if (count($options['directory_ids']) > 0) {
                $select->where('u.directory_id in (?)', $options['directory_ids']);
            }

            if (count($options['client_ids']) > 0 || count($options['role_ids']) > 0) {
                $select->joinInner(array('utr' => self::$rolesTable), 'utr.user_id = u.user_id', array())
                       ->joinInner(array('ur' => 'users_roles'), 'utr.role_id = ur.role_id', array());
            }

            if (count($options['client_ids']) > 0) {
                $select->where('u.directory_id is null')
                       ->where('ur.directory_id is null')
                       ->where('ur.client_id in (?)', $options['client_ids']);
            }

            if (count($options['role_ids']) > 0) {
                $select->where('ur.role_id in (?)', $options['role_ids']);
            }
            else if ($options['any_role_only']) {
                $select->where('u.user_id in (select distinct user_id from users_to_roles)');
            }
            else if ($options['no_role_only']) {
                $select->where('u.user_id not in (select distinct user_id from users_to_roles)');
            }

            if (count($options['statuses']) > 0) {
                $select->where('u.status in (?)', $options['statuses']);
            }

            if ($options['filters'] instanceof Profile_Field_FieldList) {
                $i = 0;
                foreach ($options['filters'] as $filter) {
                    if ($filter->getName() == 'username') {
                        switch ($filter->getParam('operator')) {
                            case 'eq':
                                $select->where('u.username = ?', $filter->getParam('filterValue'));
                                break;

                            case 'neq':
                                $select->where('u.username != ?', $filter->getParam('filterValue'));
                                break;

                            case 'like':
                                $select->where('u.username like ?', '%' . $filter->getParam('filterValue') . '%');
                                break;
                        }

                        continue;
                    }

                    $table = 'up' . $i++;
                    $select->joinLeft(
                        array($table => self::$table . '_profile'),
                        'u.user_id = ' . $table . '.user_id and ' . $db->quoteInto($table . '.profile_key = ?', $filter->getName()),
                        array()
                    );
                    $filter->getType()->compareInSql($select, $filter->getParam('operator'), $table . '.profile_value', $filter->getParam('filterValue'));
                }
            }

            return $select;
        }

        public function check($k, $v)
        {
            if ($k == 'username') {
                $directory = $this->getDirectory();
                if ($directory->usernameIsEmail()) {
                    $validator = new Zend_Validate_EmailAddress();
                    return $validator->isValid($v);
                }
                else {
                    return preg_match('/^[a-z0-9]([a-z0-9@_.-]{1,98})?[a-z0-9]$/i', $v);
                }
            }

            return parent::check($k, $v);
        }
    }
?>
