<?php
    class Application_User_Cp_Tab extends DatabaseObject
    {
        public static $table = 'users_cp_tabs';
        public $profile;

        protected $_template;

        public function __construct()
        {
            parent::__construct(Application::GetDb(), self::$table, 'tab_id');

            $this->add('user_id');
            $this->add('client_id');
            $this->add('url');
            $this->add('ranking');

            $this->profile = new Application_User_Cp_Tab_Profile($this->db);
        }

        public function __destruct()
        {
            parent::__destruct();
            unset($this->profile);
        }

        /**
         * Callbacks
         */

        protected function postLoad()
        {
            $this->profile->setTabId($this->getId());
            $this->profile->load();
        }

        protected function preInsert()
        {
            $select = $this->db->select();
            $select->from($this->getTable(), new Zend_Db_Expr('COALESCE(MAX(ranking), 0) + 1'))
                   ->where('client_id = ?', $this->client_id)
                   ->where('user_id = ?', $this->user_id);

            $this->ranking = (int) $this->db->fetchOne($select);

            if ($this->_template instanceof Application_Cp_Tab_Template) {
                if ($this->_template->isLegacyModule()) {
                    $this->profile->legacy_module = $this->_template->getLegacyModule()->getName();
                }
                else {
                    $this->setLayout($this->_template->getLayout());
                }
            }

            return true;
        }

        protected function postInsert()
        {
            $this->profile->setTabId($this->getId());
            $this->profile->save();

            if ($this->_template instanceof Application_Cp_Tab_Template && !$this->_template->isLegacyModule()) {
                $col = 0;
                foreach ($this->_template->getColumns() as $column) {
                    $col++;

                    $ranking = 0;

                    foreach ($column->getWidgets() as $driver) {
                        $widget = new Application_User_Cp_Widget();
                        $widget->tab_id    = $this->getId();
                        $widget->col       = $col;
                        $widget->ranking   = ++$ranking;
                        $widget->driver    = $driver->getName();
                        $widget->user_id   = getUserId();
                        $widget->client_id = getClientId();
                        $widget->save();
                    }
                }
            }

            return true;
        }

        protected function postUpdate()
        {
            $this->profile->save();
            return true;
        }

        protected function preDelete()
        {
            $this->deleteWidgets();
            $this->profile->delete();
            return true;
        }

        /**
         * Loaders
         */

        public function loadForClientAndUser($client_id, $user_id, $tab_id)
        {
            $client_id = (int) $client_id;
            $user_id   = (int) $user_id;
            $tab_id    = (int) $tab_id;

            if ($client_id == 0 || $user_id == 0 || $tab_id == 0)
                return false;

            $select = $this->db->select();
            $select->from($this->getTable(), $this->getSelectFields())
                   ->where($this->getIdField() . ' = ?', $tab_id)
                   ->where('user_id = ?', $user_id)
                   ->where('client_id = ?', $client_id);

            return $this->_load($select);
        }

        /**
         * Misc functions
         */

        public function __toString()
        {
            return $this->getTitle();
        }

        public function getTitle()
        {
            return $this->profile->title;
        }

        public function setTitle($title)
        {
            $title = trim($title);

            if (strlen($title) == 0) {
                $title = 'Untitled';
            }

            $this->profile->title = $title;
            $this->url = Text::Urlize($title);
        }

        public function getUrl()
        {
            return $this->url;
        }

        public function __set($k, $v)
        {
            if ($k == 'url') {
                $v = $this->_makeUniqueUrl($v);
            }

            parent::__set($k, $v);
        }

        protected function _makeUniqueUrl($url)
        {
            $select = $this->db->select();
            $select->from($this->getTable(), 'url')
                   ->where('client_id = ?', $this->client_id)
                   ->where('user_id = ?', $this->user_id)
                   ->where('url like ?', $url . '%');

            $urls = $this->db->fetchCol($select);

            if (count($urls) == 0)
                return $url;

            $i = 2;
            do {
                $newUrl = $url . $i++;
            } while (in_array($newUrl, $urls));

            return $newUrl;
        }

        public function getLayout()
        {
            $layout = $this->profile->layout;

            if (!in_array($layout, Application_Cp_Tab_Manager::$_customLayouts)) {
                $layout = Application_Cp_Tab_Manager::GetDefaultLayout();
            }

            return $layout;
        }

        public function setLayout($layout)
        {
            if (!in_array($layout, Application_Cp_Tab_Manager::$_customLayouts)) {
                $layout = Application_Cp_Tab_Manager::GetDefaultLayout();
            }

            $this->profile->layout = $layout;
        }

        public function getWidgetDrivers()
        {
            $widgets = Application_User_Cp_Widget::GetWidgetsForTab($this->getId());

            $ret = array();

            foreach ($widgets as $widget) {
                try {
                    $ret[] = $widget->getDriver();
                }
                catch (Exception $ex) {
                }
            }

            return $ret;
        }

        public function getColumns()
        {
            $ret = array();

            $layout = $this->getLayout();

            $numColumns = strlen($layout);

            for ($i = 0; $i < $numColumns; $i++) {
                $ret[] = new Application_Cp_Tab_Column(substr($layout, $i, 1));
            }

            $widgets = Application_User_Cp_Widget::GetWidgetsForTab($this->getId());

            foreach ($widgets as $widget) {
                $col = $widget->col - 1;

                if (!array_key_exists($col, $ret)) {
                    $col = count($ret) - 1;
                }

                $ret[$col]->addWidget($widget);
            }

            return $ret;
        }

        public function getNumColumns()
        {
            return strlen($this->getLayout());
        }


        /**
         * This method determines where in a tab a new widget should be placed. If
         * the preferredColumn argument is set to 0 then the column is calculated automatically
         *
         * @return  Application_Cp_Tab_WidgetPosition
         */
        public function getPositionForNewWidget(Application_Cp_Widget_Interface $driver = null, $preferredColumn = null)
        {
            $column = max(0, (int) $preferredColumn);

            $layout = $this->getLayout();
            $numCols = strlen($layout);

            if ($column > $numCols) {
                $column = 0;
            }

            $ret = new Application_Cp_Tab_WidgetPosition();

            if ($column == 0) {
                $min = 0;
                $max = 0;

                for ($i = 0; $i < $numCols; $i++) {
                    $span = (int) $layout[$i];

                    if ($min == 0) {
                        $min = $span;
                    }

                    if ($max == 0) {
                        $max = $span;
                    }

                    $max = max($max, $span);
                }

                $avg = ($min + $max) / 2;

                $diffs = array();
                $smallestDiff = $max;

                $cols = array(
                    Application_Cp_Widget_Interface::SIZE_SMALL  => array(),
                    Application_Cp_Widget_Interface::SIZE_MEDIUM => array(),
                    Application_Cp_Widget_Interface::SIZE_LARGE  => array()
                );

                for ($i = 0; $i < $numCols; $i++) {
                    $span = (int) $layout[$i];

                    if ($span == $min) {
                        $cols[Application_Cp_Widget_Interface::SIZE_SMALL][] = $i + 1;
                    }
                    if ($span == $max) {
                        $cols[Application_Cp_Widget_Interface::SIZE_LARGE][] = $i + 1;
                    }

                    // weight more towards using a bigger span rather than smaller for medium size widgets
                    if ($span < $avg) {
                        $diff = $avg - $span + 0.5;
                    }
                    else {
                        $diff = $span - $avg;
                    }

                    $smallestDiff = min($smallestDiff, $diff);

                    $diffs[$i] = $diff;
                }

                foreach ($diffs as $col => $diff) {
                    if ($diff == $smallestDiff) {
                        $cols[Application_Cp_Widget_Interface::SIZE_MEDIUM][] = $col + 1;
                    }
                }

                if (array_key_exists($driver->getWidgetSize(), $cols) && count($cols[$driver->getWidgetSize()]) > 0) {
                    $counts = array();

                    foreach ($cols[$driver->getWidgetSize()] as $num) {
                        $counts[$num] = 0;
                    }

                    $select = $this->db->select();
                    $select->from(Application_User_Cp_Widget::$table, array('col', 'count(*)'))
                           ->where('tab_id = ?', $this->getId())
                           ->where('col in (?)', array_keys($counts))
                           ->group('col');

                    foreach ($this->db->fetchPairs($select) as $col => $count) {
                        $counts[$col] = $count;
                    }

                    $arr = array();

                    foreach ($counts as $col => $count) {
                        $arr[] = array($col, $count);
                    }

                    usort($arr, array(__CLASS__, '_sortCounts'));

                    foreach ($arr as $v) {
                        $ret->setColumn($v[0]);
                        break;
                    }
                }
                else {
                    $ret->setColumn($this->getNumColumns());
                }
            }
            else {
                $ret->setColumn($column);
            }

            $db = Application::GetDb();
            $select = $db->select();
            $select->from(Application_User_Cp_Widget::$table, array(
                       'ranking' => new Zend_Db_Expr('COALESCE(MAX(ranking), 0) + 1')
                   ))
                   ->where('tab_id = ?', $this->getId())
                   ->where('col = ?', $ret->getColumn());

            $ret->setRanking($db->fetchOne($select));

            return $ret;
        }

        public static function _sortCounts($a, $b)
        {
            if ($a[1] < $b[1]) { return -1; }
            if ($b[1] < $a[1]) { return 1; }
            if ($a[0] < $b[0]) { return -1; }
            if ($b[0] < $a[0]) { return 1; }
            return 0;
        }

        public function fromTemplate(Application_Cp_Tab_Template $template)
        {
            $this->_template = $template;
            return $this;
        }

        public function isLegacy()
        {
            return strlen($this->profile->legacy_module) > 0;
        }

        public function getLegacyModule()
        {
            return $this->profile->legacy_module;
        }

        public function updateColumnOrder($column, $widget_ids)
        {
            $widget_ids = ArrayTools::Clean($widget_ids, true);

            if (count($widget_ids) == 0) {
                return;
            }

            $numColumns = $this->getNumColumns();

            $column = (int) $column;

            if ($column < 1 || $column > $numColumns) {
                throw new Application_Cp_Widget_Exception('Invalid column');
            }

            $options = array(
                'widget_ids' => $widget_ids
            );

            $widgets = Application_User_Cp_Widget::GetWidgetsForTab($this->getId(), $options);

            $ranking = 1;

            $this->db->begin();

            foreach ($widget_ids as $widget_id) {
                if (!array_key_exists($widget_id, $widgets)) {
                    continue;
                }

                $position = new Application_Cp_Tab_WidgetPosition();
                $position->setColumn($column)
                         ->setRanking($ranking++);

                $widgets[$widget_id]->setPosition($position);
                $widgets[$widget_id]->save();
            }

            $this->db->commit();
        }

        public function deleteWidgets()
        {
            foreach (Application_User_Cp_Widget::GetWidgetsForTab($this->getId()) as $widget) {
                $widget->delete();
            }
        }

        public function getSuggestedWidgets()
        {
            $existing   = Application_User_Cp_Widget::GetWidgetsForTab($this->getId());
            $categories = array();

            $usedDrivers = array();

            foreach ($existing as $widget) {
                $driver = $widget->getDriver();
                $cat    = $driver->getCategories();

                if (!is_array($cat)) {
                    continue;
                }

                $usedDrivers[] = $driver->getName();

                $categories = array_merge($categories, $cat);
            }

            $categories = array_unique($categories);

            if (count($categories) == 0) {
                return array();
            }

            $options = array(
                'addableOnly' => true,
                'checkPerms'  => true,
                'categories'  => $categories
            );

            $widgets = Application_Cp_Widget_Manager::GetDrivers($options);

            $ret = array();

            foreach ($widgets as $widget) {
                if (in_array($widget->getName(), $usedDrivers)) {
                    continue;
                }

                $ret[] = $widget;
            }

            usort($ret, array(__CLASS__, '_SortSuggested'));

            return $ret;
        }

        public static function _SortSuggested($a, $b)
        {
            return strcmp((string) $a, (string) $b);
        }

        /**
         * Static methods
         */

        public static function GetTabs($options = array())
        {
            $defaults = array(
                'order' => 't.ranking'
            );

            $options = ArrayTools::BuildOptions($options, $defaults);

            $db = Application::GetDb();

            $select = self::_GetBaseQuery($options);
            $select->from(null, 't.*')
                   ->order($options['order']);

            $options['offset'] = max(0, (int) array_key_exists('offset', $options) ? $options['offset'] : 0);
            if (array_key_exists('limit', $options) && $options['limit'] > 0)
                $select->limit($options['limit'], $options['offset']);

            $result = $db->query($select);
            $data = array();
            while ($row = $result->fetch()) {
                $data[] = $row;
            }

            $tabs = parent::BuildMultiple($db, __CLASS__, $data);
            self::_LoadProfile($tabs);

            return $tabs;
        }

        private static function _LoadProfile(&$arr)
        {
            if (count($arr) == 0) {
                return;
            }

            $db    = Application::GetDb();
            $ids   = array_keys($arr);
            $class = __CLASS__ . '_Profile';

            try {
                $profiles = Profile::BuildMultiple($db, $class, array('tab_id' => $ids));
            }
            catch (Exception $ex) {
                $profiles = array();
            }

            foreach ($arr as $id => $row) {
                if (array_key_exists($id, $profiles) && $profiles[$id] instanceof $class) {
                    $arr[$id]->profile = $profiles[$id];
                }
                else {
                    $arr[$id]->profile = new $class($db, $id);
                }
            }
        }

        public static function GetTabsCount($options)
        {
            $select = self::_GetBaseQuery($options);
            $select->from(null, 'count(*)');

            return (int) Application::GetDb()->fetchOne($select);
        }

        public static function _GetBaseQuery($options)
        {
            $defaults = array(
                'user_ids'   => array(),
                'client_ids' => array()
            );

            $options = ArrayTools::buildOptions($options, $defaults);

            $options['user_ids']   = ArrayTools::clean($options['user_ids'], true);
            $options['client_ids'] = ArrayTools::clean($options['client_ids'], true);

            $db = Application::GetDb();

            $select = $db->select();
            $select->from(array('t' => self::$table), array());

            if (count($options['user_ids']) > 0)
                $select->where('t.user_id in (?)', $options['user_ids']);

            if (count($options['client_ids']) > 0)
                $select->where('t.client_id in (?)', $options['client_ids']);

            return $select;
        }

        public static function ClearForUser($userIds)
        {
            $userIds = ArrayTools::Clean($userIds, true);

            if (count($userIds) == 0) {
                return;
            }

            $db = Application::GetDb();
            $db->begin();

            Application_User_Cp_Widget::ClearForUser($userIds);

            $query  = sprintf('select tab_id from %s where user_id in (%s)', self::$table, $db->quote($userIds));
            $tabIds = $db->fetchCol($query);

            if (count($tabIds) > 0) {
                $where = array(
                    $db->quoteInto('tab_id in (?)', $tabIds)
                );

                $db->delete(self::$table . '_profile', $where);

                $db->query(sprintf('delete from %s where user_id in (%s)', self::$table, $db->quote($userIds)));
            };

            $db->commit();
        }

        public static function ClearForDirectory($directoryId)
        {
            $directoryId = (int) $directoryId;

            if ($directoryId < 0) {
                return;
            }

            $db = Application::GetDb();
            $db->begin();

            Application_User_Cp_Widget::ClearForDirectory($directoryId);

            $query  = sprintf(
                'select tab_id from %s where user_id in (select user_id from %s where directory_id = %d)',
                self::$table,
                Application_User::$table,
                $directoryId
            );
            $tabIds = $db->fetchCol($query);

            if (count($tabIds) > 0) {
                $where = array(
                    $db->quoteInto('tab_id in (?)', $tabIds)
                );

                $db->delete(self::$table . '_profile', $where);

                $db->query(
                    sprintf(
                        'delete from %s where user_id in (select user_id from %s where directory_id = %d)',
                        self::$table,
                        Application_User::$table,
                        $directoryId
                    )
                );
            };

            $db->commit();
        }

        public static function ClearForClient($clientId)
        {
            $db = Application::GetDb();
            $db->begin();

            Application_User_Cp_Widget::ClearForClient($clientId);

            $query  = sprintf('select tab_id from %s where client_id = %d', self::$table, $clientId);
            $tabIds = $db->fetchCol($query);

            if (count($tabIds) > 0) {
                $where = array(
                    $db->quoteInto('tab_id in (?)', $tabIds)
                );

                $db->delete(self::$table . '_profile', $where);

                $db->query(sprintf('delete from %s where client_id = %d', self::$table, $clientId));
            };

            $db->commit();
        }

        public static function DeployWidgets(Application_User $user, Application_Client $client)
        {
            // first check if the user already has widgets for the given client, and if so don't continue deploying them

            $db = Application::GetDb();

            $select = $db->select();
            $select->from(Application_User_Cp_Widget::$table, 'count(*)')
                   ->where('client_id = ?', $client->getId())
                   ->where('user_id = ?', $user->getId());

            if ($db->fetchOne($select) > 0) {
                return;
            }

            // now we retrieve the default widget layout to deploy.
            $tabs = Application_Cp_Tab_Manager::GetDefaults($client->getBrand());

            $db->begin();

            foreach ($tabs as $tab) {
                $obj = new self();
                $obj->user_id   = $user->getId();
                $obj->client_id = $client->getId();
                $obj->setTitle($tab['label']);
                $obj->setLayout($tab['layout']);
                $obj->save();

                $column = 0;
                foreach ($tab['columns'] as $col) {
                    $ranking = 0;
                    $column++;

                    foreach ($col as $widget) {
                        $widgetObj = new Application_User_Cp_Widget();
                        $widgetObj->user_id = $user->getId();
                        $widgetObj->client_id = $client->getId();
                        $widgetObj->tab_id    = $obj->getId();
                        $widgetObj->driver    = $widget;
                        $widgetObj->col       = $column;
                        $widgetObj->ranking   = ++$ranking;
                        $widgetObj->save();
                    }
                }
            }

            $db->commit();
        }

        public static function SetActiveTab(self $tab)
        {
            $db = Application::GetDb();
            $select = $db->select();
            $select->from(self::$table, 'tab_id')
                   ->where('client_id = ?', getClientId())
                   ->where('user_id = ?', getUserId())
                   ->where('tab_id != ?', $tab->getId());

            $tabIds = $db->fetchCol($select);

            if (count($tabIds) > 0) {
                $where = array(
                    $db->quoteInto('tab_id in (?)', $tabIds),
                    $db->quoteInto('profile_key = ?', 'is_active')
                );

                $db->delete(self::$table . '_profile', $where);
            }

            $tab->profile->is_active = true;
            $tab->save();
        }

        public static function GetActiveTabId()
        {
            $db = Application::GetDb();
            $select = $db->select();
            $select->from(self::$table, 'tab_id')
                   ->where('client_id = ?', getClientId())
                   ->where('user_id = ?', getUserId());

            $tabIds = $db->fetchCol($select);

            $ret = 0;

            if (count($tabIds) > 0) {
                $select = $db->select();
                $select->from(self::$table . '_profile', 'tab_id')
                       ->where('tab_id in (?)', $tabIds)
                       ->where('profile_key = ?', 'is_active');

                $ret = (int) $db->fetchOne($select);
            }

            return $ret;
        }
    }
?>
