<?php
    class Application_User_Cp_Widget extends DatabaseObject
    {
        public static $table = 'users_cp_tabs_widgets';
        public $profile;

        protected $_driver;
        protected $_user;

        public function __construct()
        {
            parent::__construct(Application::GetDb(), self::$table, 'widget_id');

            $this->add('user_id');
            $this->add('client_id');
            $this->add('tab_id');
            $this->add('driver');
            $this->add('col');
            $this->add('ranking');

            $this->profile = new Application_User_Cp_Widget_Profile($this->db);
        }

        public function __destruct()
        {
            parent::__destruct();
            unset($this->profile, $this->_driver, $this->_user);
        }

        /**
         * Callbacks
         */

        protected function postLoad()
        {
            $this->profile->setWidgetId($this->getId());
            $this->profile->load();
        }

        protected function preInsert()
        {
            if (!$this->updated('ranking')) {
                $select = $this->db->select();
                $select->from($this->getTable(), new Zend_Db_Expr('COALESCE(MAX(ranking), 0) + 1'))
                       ->where('client_id = ?', $this->client_id)
                       ->where('user_id = ?', $this->user_id)
                       ->where('tab_id = ?', $this->tab_id)
                       ->where('col = ?', $this->col);

                $this->ranking = (int) $this->db->fetchOne($select);
            }

            return true;
        }

        protected function postInsert()
        {
            $this->profile->setWidgetId($this->getId());
            $this->profile->save(false);
            return true;
        }

        protected function postUpdate()
        {
            $this->profile->save(false);
            return true;
        }

        protected function preDelete()
        {
            $this->profile->delete(false);
            return true;
        }

        /**
         * Loaders
         */

        public function loadWidget($client_id, $user_id, $widget_id)
        {
            $client_id = (int) $client_id;
            $user_id   = (int) $user_id;
            $widget_id = (int) $widget_id;

            if ($client_id <= 0 && $user_id <= 0 && $widget_id <= 0)
                return false;

            $select = $this->db->select();
            $select->from(array('w' => $this->getTable()), $this->getSelectFields())
                   ->where('w.client_id = ?', $client_id)
                   ->where('w.user_id = ?', $user_id)
                   ->where('w.widget_id = ?', $widget_id);

            return $this->_load($select);
        }

        /**
         * Misc functions
         */

        public function __toString()
        {
            return (string) $this->getDriver();
        }

        public function setPosition(Application_Cp_Tab_WidgetPosition $position)
        {
            $this->col = $position->getColumn();
            $this->ranking = $position->getRanking();
        }

        public function getDriver()
        {
            if (!$this->_driver instanceof Application_User_Cp_Widget_Interface) {
                $this->_driver = Application_Cp_Widget_Manager::Factory($this->driver);
            }

            return $this->_driver;
        }

        public function setParam($k, $v)
        {
            $this->profile->set('_param_' . $k, $v);
            return $this;
        }

        public function getParam($k)
        {
            return $this->profile->get('_param_' . $k);
        }

        public function getParams()
        {
            $ret = array();

            foreach ($this->profile as $k => $v) {
                if (preg_match('/^_param_(.*)$/', $k, $matches)) {
                    $ret[$matches[1]] = $v;
                }
            }

            return $ret;
        }

        public function setParamsFromRequest(Zend_Controller_Request_Abstract $request)
        {
            foreach ($request->getPost() as $k => $v) {
                $this->setParam($k, $v);
            }

            $this->save();
        }


        /**
         * Static methods
         */

        public static function Create(Application_User_Cp_Tab $tab, $driver, $params, $column = null)
        {
            try {
                $driver = Application_Cp_Widget_Manager::Factory($driver);
            }
            catch (Exception $ex) {
                throw new Application_Cp_Widget_Exception(sprintf('Widget not found (%s)', $driver));
            }

            $widget = new self();
            $widget->user_id   = getUserId();
            $widget->client_id = getClientId();
            $widget->tab_id    = $tab->getId();
            $widget->driver    = $driver->getName();
            $widget->setPosition($tab->getPositionForNewWidget($driver, $column));

            foreach ($params as $k => $v) {
                $widget->setParam($k, $v);
            }

            $widget->save();

            return $widget;
        }

        public static function GetWidgetsForTab($tab_id, $options = array())
        {
            $defaults = array(
                'widget_ids' => array()
            );

            $options = ArrayTools::BuildOptions($options, $defaults);

            $db = Application::GetDb();

            $select = $db->select();
            $select->from(self::$table, '*')
                   ->where('tab_id = ?', $tab_id)
                   ->order('col')
                   ->order('ranking');

            $options['widget_ids'] = ArrayTools::Clean($options['widget_ids'], true);

            if (count($options['widget_ids']) > 0) {
                $select->where('widget_id in (?)', $options['widget_ids']);
            }

            $result = $db->query($select);
            $data = array();
            while ($row = $result->fetch()) {
                $data[] = $row;
            }

            $widgets = parent::BuildMultiple($db, __CLASS__, $data);
            self::_LoadProfile($widgets);

            return $widgets;
        }

        private static function _LoadProfile(&$arr)
        {
            if (count($arr) == 0) {
                return;
            }

            $db    = Application::GetDb();
            $ids   = array_keys($arr);
            $class = __CLASS__ . '_Profile';

            try {
                $profiles = Profile::BuildMultiple($db, $class, array('widget_id' => $ids));
            }
            catch (Exception $ex) {
                $profiles = array();
            }

            foreach ($arr as $id => $row) {
                if (array_key_exists($id, $profiles) && $profiles[$id] instanceof $class) {
                    $arr[$id]->profile = $profiles[$id];
                }
                else {
                    $arr[$id]->profile = new $class($db, $id);
                }
            }
        }

        public static function ClearForUser($userIds)
        {
            $userIds = ArrayTools::Clean($userIds, true);

            if (count($userIds) == 0) {
                return;
            }

            $db = Application::GetDb();
            $db->begin();

            $query  = sprintf('select widget_id from %s where user_id in (%s)', self::$table, $db->quote($userIds));
            $widgetIds = $db->fetchCol($query);

            if (count($widgetIds) > 0) {
                $where = array(
                    $db->quoteInto('widget_id in (?)', $widgetIds)
                );

                $db->delete(self::$table . '_profile', $where);
                $db->query(sprintf('delete from %s where user_id in (%s)', self::$table, $db->quote($userIds)));
            };

            $db->commit();
        }

        public static function ClearForDirectory($directoryId)
        {
            $directoryId = (int) $directoryId;

            if ($directoryId <= 0) {
                return;
            }

            $db = Application::GetDb();
            $db->begin();

            $query  = sprintf(
                'select widget_id from %s where user_id in (select user_id from %s where directory_id = %d)',
                self::$table,
                Application_User::$table,
                $directoryId
            );
            $widgetIds = $db->fetchCol($query);

            if (count($widgetIds) > 0) {
                $where = array(
                    $db->quoteInto('widget_id in (?)', $widgetIds)
                );

                $db->delete(self::$table . '_profile', $where);
                $db->query(
                    sprintf(
                        'delete from %s where user_id in (select user_id from %s where directory_id = %d)',
                        self::$table,
                        Application_User::$table,
                        $directoryId
                    )
                );
            };

            $db->commit();
        }

        public static function ClearForClient($clientId)
        {
            $db = Application::GetDb();
            $db->begin();

            $query  = sprintf('select widget_id from %s where client_id = %d', self::$table, $clientId);
            $widgetIds = $db->fetchCol($query);

            if (count($widgetIds) > 0) {
                $where = array(
                    $db->quoteInto('widget_id in (?)', $widgetIds)
                );

                $db->delete(self::$table . '_profile', $where);
                $db->query(sprintf('delete from %s where client_id = %d', self::$table, $clientId));
            };

            $db->commit();
        }
    }
?>
