<?php
    class Application_User_Directory extends DatabaseObject
    {
        public static $table = 'users_directories';

        public $profile;

        protected $_client;
        protected $_roles;
        protected $_num_users = -1;

        private static $_fieldsCache = array();

        public function __construct()
        {
            parent::__construct(Application::GetDb(), self::$table, 'directory_id');

            $this->add('client_id');
            $this->add('directory_type', '');
            $this->add('title');

            $this->profile = new Application_User_Directory_Profile($this->db);
        }

        public function __destruct()
        {
            parent::__destruct();
            unset($this->profile);
        }

        /**
         * Loaders
         */

        public function loadRecordForClient($client_id, $directory_id=NULL)
        {
            $client_id    = (int) $client_id;
            $directory_id = (int) $directory_id;

            if ( empty( $client_id ) || empty( $directory_id ) ){
                return false;
            }

            $select = $this->db->select();
            $select->from($this->getTable(), $this->getSelectFields())
                   ->where('client_id = ?', $client_id)
                   ->where('directory_id = ?', $directory_id);

            return $this->_load($select);
        }

        public function loadRecordForClientByTitle($client_id, $title)
        {
            $client_id = (int) $client_id;
            $title     = trim($title);

            if ($client_id == 0 || strlen($title) == 0)
                return false;

            $select = $this->db->select();
            $select->from($this->getTable(), $this->getSelectFields())
                   ->where('client_id = ?', $client_id)
                   ->where('lower(title) = lower(?)', $title);

            return $this->_load($select);
        }

        /**
         * Callbacks
         */

        public function __toString()
        {
            if ($this->saved()) {
                return (string) $this->title;
            }

            return 'Control Panel Users';
        }

        protected function postLoad()
        {
            $this->profile->setDirectoryId($this->getId());
            $this->profile->load();
        }

        protected function postInsert()
        {
            $this->profile->setDirectoryId($this->getId());
            $this->profile->save();

            $message = new Application_Auditor_Message('Users: Directory created');
            $message->addMemo('directory_id', $this->getId())
                    ->addMemo('client_id', $this->client_id)
                    ->record();

            return true;
        }

        protected function postUpdate()
        {
            $this->profile->save(false);

            $message = new Application_Auditor_Message('Users: Directory updated');
            $message->addMemo('directory_id', $this->getId())
                    ->addMemo('client_id', $this->client_id)
                    ->record();

            return true;
        }

        protected function preDelete()
        {
            $event = new Application_Event('/module/users/directory/predelete');
            $event->set('id', $this->getId());
            $event->trigger();

            $this->profile->delete();
            $this->deleteEditableProfileFields();

            foreach ($this->getRoles() as $role) {
                $role->delete();
            }

            $query = sprintf('delete from %s where directory_id = %d', Application_User_Permissions_Permission::$table, $this->getId());
            $this->db->query($query);

            $options = array(
                'directory_ids' => $this->getId()
            );

            $options = new Application_User_Bulk_Options();
            $options->setTargetDirectory($this)
                    ->enableAll();

            Application_User_Bulk::Delete($options);

            return true;
        }

        protected function postDelete()
        {
            $event = new Application_Event('/module/users/directory/postdelete');
            $event->set('id', $this->getDeletedId());
            $event->trigger();

            $message = new Application_Auditor_Message('Users: Directory deleted');
            $message->addMemo('directory_id', $this->getDeletedId())
                    ->addMemo('client_id', $this->client_id)
                    ->record();

            return true;
        }

        /**
         * User profile related functions
         */

        public function getEditableProfileFields($unprotectedOnly = false, $mustBeEditable = false)
        {
            $key = sprintf('%d,%d,%d', $this->getId(), $unprotectedOnly, $mustBeEditable);

            $ret = new Profile_Field_FieldList();

            if ($this->saved()) {
                if (!array_key_exists($this->getId(), self::$_fieldsCache)) {
                    $select = $this->db->select();
                    $select->from($this->getTable() . '_fields', '*')
                           ->where($this->getIdField() . ' = ' . $this->getId())
                           ->order('ranking')
                           ->order('lower(title)');

                    self::$_fieldsCache[$this->getId()] = $this->db->fetchAll($select);
                }

                foreach (self::$_fieldsCache[$this->getId()] as $row) {
                    $field = new Profile_Field_FieldList_Field($row['type']);

                    $field->setTitle($row['title'])
                          ->setName($row['name'])
                          ->setParams(unserialize($row['params']));

                    if ($unprotectedOnly && $field->getProtected()) {
                        continue;
                    }

                    if ($mustBeEditable && $field->getNoEdit()) {
                        continue;
                    }

                    $ret->add($field);
                }
            }
            else {
                $ret->add('String')->setTitle('First Name');
                $ret->add('String')->setTitle('Last Name');
                $ret->add('Email')->setTitle('Email');
            }

            return $ret;
        }

        public function deleteEditableProfileFields()
        {
            if (!$this->saved()) {
                return;
            }

            unset(self::$_fieldsCache[$this->getId()]);

            $query = sprintf('delete from %s_fields where %s = %d',
                             $this->getTable(),
                             $this->getIdField(),
                             $this->getId());

            $this->db->query($query);
        }

        public function setEditableProfileFields($fields)
        {
            if (!$this->saved()) {
                return;
            }

            $this->db->begin();

            $this->deleteEditableProfileFields();

            foreach ($fields as $field) {
                $values = array(
                    $this->getIdField() => $this->getId(),
                    'title'             => $field->getTitle(),
                    'name'              => $field->getName(),
                    'type'              => $field->getType()->getType(),
                    'params'            => serialize($field->getParams()),
                    'ranking'           => $field->getRanking()
                );

                $this->db->insert(self::$table . '_fields', $values);
            }

            $this->db->commit();
        }

        public function getEmailAddressField()
        {
            return $this->getEditableProfileFields()->getEmailAddressField();
        }

        public function usernameIsEmail()
        {
            return (bool) $this->profile->username_is_email;
        }

        public function setUsernameIsEmail($flag)
        {
            $this->profile->username_is_email = (bool) $flag;
            return $this;
        }

        /**
         * Misc methods
         */

        public function isClientDirectory()
        {
            $client_id = (int) $this->client_id;
            return $client_id > 0;
        }

        public function getClient()
        {
            if (!$this->_client instanceof Application_Client || !$this->_client->saved()) {
                $this->_client = new Application_Client();
                $this->_client->load($this->client_id);
            }

            return $this->_client;
        }

        public function getClientTemplateData()
        {
            $ret = array(
                'id'    => $this->getId(),
                'title' => $this->title
            );

            return $ret;
        }

        public function requiresManualApproval()
        {
            return (bool) $this->profile->manual_approval;
        }

        public function getRoles()
        {
            if (!is_array($this->_roles)) {
                $options = array(
                    'directory_ids' => $this->getId()
                );

                $this->_roles = Application_User_Role::GetRoles($options);
            }

            return $this->_roles;
        }

        public function getNumUsers()
        {
            if ($this->_num_users < 0) {
                $options = array(
                    'directory_ids' => $this->getId()
                );

                $this->_num_users = Application_User::GetUsersCount($options);
            }

            return $this->_num_users;
        }

        public function getUsers()
        {
            $options = array(
                'directory_ids' => $this->getId()
            );

            return Application_User::GetUsers($options);
        }

        /**
         * Static functions
         */

        public static function GetDirectoriesCount($options = array())
        {
            $select = self::_GetBaseQuery($options);
            $select->from(null, 'count(*)');

            return Application::GetDb()->fetchOne($select);
        }

        public static function GetDirectories($options = array())
        {
            $defaults = array(
                'order'  => array('lower(d.title)', 'd.directory_id'),
                'offset' => 0,
                'limit'  => 0
            );

            $options = ArrayTools::BuildOptions($options, $defaults);

            $db = Application::GetDb();

            $select = self::_GetBaseQuery($options);
            $select->from(null, array('d.*', 'c.client_name'))
                   ->order($options['order']);

            $options['offset'] = max(0, (int) $options['offset']);
            if ($options['limit'] > 0)
                $select->limit($options['limit'], $options['offset']);

            $result = $db->query($select);
            $data = array();
            while ($row = $result->fetch()) {
                $data[] = $row;
            }

            $directories = parent::BuildMultiple($db, __CLASS__, $data);
            self::_LoadProfile($directories);
            self::_GetUserCount($directories);

            return $directories;
        }

        protected static function _GetBaseQuery($options)
        {
            $defaults = array(
                'client_ids'    => array(),
                'directory_ids' => array()
            );

            $options = ArrayTools::BuildOptions($options, $defaults);

            $options['client_ids'] = ArrayTools::clean($options['client_ids'], true);
            $options['directory_ids'] = ArrayTools::clean($options['directory_ids'], true);

            $db = Application::GetDb();

            $select = $db->select();
            $select->from(array('d' => self::$table), array())
                   ->where('d.directory_id is not null');

            if (count($options['client_ids']) > 0) {
                $select->where('d.client_id in (?)', $options['client_ids']);
            }

            if (count($options['directory_ids']) > 0) {
                $select->where('d.directory_id in (?)', $options['directory_ids']);
            }

            $select->joinRight(array('c' => 'clients'), 'd.client_id = c.client_id', array());

            return $select;
        }

        private static function _GetUserCount(&$arr)
        {
            if (count($arr) == 0)
                return;

            $directory_ids = array_keys($arr);

            $db = Application::GetDb();

            $select = $db->select();
            $select->from(Application_User::$table, array('directory_id', 'count(*)'))
                   ->where('directory_id in (?)', $directory_ids)
                   ->group('directory_id');

            $counts = $db->fetchPairs($select);

            foreach ($arr as $k => $v) {
                if (array_key_exists($k, $counts)) {
                    $arr[$k]->_num_users = $counts[$k];
                }
                else {
                    $arr[$k]->_num_users = 0;
                }
            }
        }

        private static function _LoadProfile(&$arr)
        {
            if (count($arr) == 0)
                return;

            $db    = Application::GetDb();
            $ids   = array_keys($arr);
            $class = __CLASS__ . '_Profile';

            try {
                $profiles = Profile::BuildMultiple($db, $class, array('directory_id' => $ids));
            }
            catch (Exception $ex) {
                $profiles = array();
            }

            foreach ($arr as $id => $row) {
                if (array_key_exists($id, $profiles) && $profiles[$id] instanceof $class) {
                    $arr[$id]->profile = $profiles[$id];
                }
                else {
                    $arr[$id]->profile = new $class($db, $id);
                }
            }
        }
    }
?>
