<?php
    class Application_User_Directory_Importer
    {
        protected $_directory;
        protected $_db;

        protected $_invalid = array();
        protected $_validCount = 0;
        protected $_skipped = 0;
        protected $_created = 0;
        protected $_updated = 0;

        public function __construct(Application_User_Directory $directory)
        {
            $this->_directory = $directory;
            $this->_db = $directory->getDb();
        }

        public function importFromCsvFile($fullpath)
        {
            $fp = @fopen($fullpath, 'r');

            if (!$fp) {
                throw new Exception('Unable to open CSV file for import: ' . $fullpath);
            }

            $fields = $this->_directory->getEditableProfileFields();

            $map = null;
            $usernameIdx = -1;
            $passwordIdx = -1;

            $i = 0;
            $this->_validCount = 0;

            $this->_db->begin();

            $event = new Application_Event('/module/users/user/bulkcreatebegin');
            $event->set('directory', $this->_directory);
            $event->trigger();

            Application_Event_Manager::Disable();

            $userIds = array();

            while (($row = fgetcsv($fp)) !== false) {
                if (count(ArrayTools::clean(array_map('trim', $row))) == 0) {
                    continue;
                }

                if (!is_array($map)) {
                    $map = array();
                    foreach ($fields as $field) {
                        $name = $field->getTitle();

                        foreach ($row as $k => $v) {
                            if ($name === $v) {
                                $map[$name] = $k;
                            }
                        }
                    }

                    foreach ($row as $k => $v) {
                        if (strtolower($v) == 'username') {
                            $usernameIdx = $k;
                        }
                        else if (strtolower($v) == 'password') {
                            $passwordIdx = $k;
                        }
                    }

                    continue;
                }

                $user = new Application_User();

                Module_Backup_Logger::Debug(sprintf('Importing row %d', ++$i));
                Module_Backup_Logger::StartBlock();

                $fieldList = clone $fields;

                $errors = array();

                foreach ($fieldList as $field) {
                    $name = $field->getTitle();
                    if (!array_key_exists($name, $map)) {
                        continue;
                    }
                    $type = $field->getType();

                    $idx = $map[$name];

                    $value = trim($row[$idx]);

                    if ($type->isEmpty($value)) {
                        if ($field->getRequired()) {
                            $msg = sprintf('Required field %s is empty', $field->getTitle());
                            $errors[] = $msg;
                        }
                    }
                    else if (!$type->set($value)) {
                        $msg = sprintf('Invalid field %s: %s', $field->getTitle(), $value);
                        $errors[] = $msg;
                    }
                }

                if ($usernameIdx < 0) {
                    $username = Application_User_UsernameGenerator::Generate($this->_directory, $fieldList, true);
                }
                else {
                    $username = trim(coalesce($row[$usernameIdx]));
                }

                if (strlen($username) == 0) {
                    $errors[] = 'Username not specified';
                }
                else if (!$user->check('username', $username)) {
                    $errors[] = 'Username is not valid';
                }
                else if ($user->userExists($username, $this->_directory->getId())) {
                    $errors[] = 'Username is already in use';
                }

                if ($passwordIdx < 0) {
                    $password = Text_Password::Create(8);
                }
                else {
                    $password = trim(coalesce($row[$passwordIdx]));
                }

                if (strlen($password) == 0) {
                    $errors[] = 'Password not specified';
                }
                else if (preg_match('/^[0-9a-f]{32}$/', $password)) {
                    $user->setPassword($password, true);
                }
                else {
                    $user->setPassword($password);
                }

                if (count($errors) > 0) {
                    Module_Backup_Logger::EndBlock();
                    $this->_invalid[] = array(
                        'line'   => $i,
                        'errors' => $errors,
                        'row'    => $row
                    );

                    continue;
                }
                else {
                    $this->_validCount++;
                    $this->_created++;
                }

                $user->disableNotification(true);

                $user->directory_id = $this->_directory->getId();
                $user->username = $username;
                $user->status   = Application_User::STATUS_ACTIVE;

                foreach ($fieldList as $field) {
                    $user->profile->set($field->getName(), $field->get());
                }

                $user->save();

                $userIds[] = $user->getId();

                Module_Backup_Logger::EndBlock();
            }

            Application_Event_Manager::Enable();

            if (count($this->_invalid) == 0) {
                $event = new Application_Event('/module/users/user/bulkcreatecommit');
                $event->set('directory', $this->_directory)
                      ->set('users', $userIds);
                $event->trigger();

                $this->_db->commit();
            }
            else {
                $event = new Application_Event('/module/users/user/bulkcreaterollback');
                $event->set('directory', $this->_directory);
                $event->trigger();

                $this->_db->rollback();
                $this->_created = 0;
            }
        }

        public function getInvalidRows()
        {
            return $this->_invalid;
        }

        public function getValidRowCount()
        {
            return $this->_validCount;
        }

        public function getSkippedCount()
        {
            return $this->_skipped;
        }

        public function getCreatedCount()
        {
            return $this->_created;
        }

        public function getUpdatedCount()
        {
            return $this->_updated;
        }
    }
?>
