<?php
    /**
     * This class is used for sending emails to users, as well as
     * archiving the sent messages for later review (by admin or
     * the user as necessary)
     *
     * @author  Quentin Zervaas
     */
    class Application_User_Email extends DatabaseObject
    {
        public static $table = 'users_email_history';

        protected $_mail;

        public function __construct()
        {
            parent::__construct(Application::GetDb(), self::$table, 'email_id');

            $this->add('user_id');
            $this->add('headers', array(), self::TYPE_ARRAY);
            $this->add('body');
            $this->add('body_html');
            $this->add('success', false, self::TYPE_BOOLEAN);
            $this->add('ts_sent', null, self::TYPE_TIMESTAMP);
        }

        public function __destruct()
        {
            parent::__destruct();
            unset($this->_mail);
        }

        public function loadForUser($user_id, $email_id)
        {
            $user_id  = (int) $user_id;
            $email_id = (int) $email_id;

            if ($user_id <= 0 || $email_id <= 0) {
                return false;
            }

            $select = $this->db->select();
            $select->from(self::$table, $this->getSelectFields())
                   ->where('user_id = ?', $user_id)
                   ->where('email_id = ?', $email_id);

            return $this->_load($select);
        }

        public function __toString()
        {
            return (string) $this->getSubject();
        }

        public function getRecipients()
        {
            $headers = $this->headers;

            $ret = array();

            foreach (array('To', 'Cc', 'Bcc') as $type) {
                if (!array_key_exists($type, $headers)) {
                    continue;
                }

                foreach ($headers[$type] as $k => $v) {
                    if (!preg_match('/^\d+$/', $k)) {
                        continue;
                    }

                    $ret[] = $v;
                }
            }

            return array_unique($ret);
        }

        public function getFrom()
        {
            $headers = $this->headers;

            if (!array_key_exists('From', $headers)) {
                return '';
            }

            return coalesce($headers['From'][0]);
        }

        public function getSubject()
        {
            $headers = $this->headers;

            if (!array_key_exists('Subject', $headers)) {
                return '';
            }

            return join(', ', $headers['Subject']);
        }


        public function Record(Application_User $user, Application_Mail_Response $response)
        {
            $email = new self();
            $email->user_id   = $user->getId();
            $email->body      = (string) $response->getBodyText();
            $email->body_html = (string) $response->getBodyHtml();
            $email->headers   = $response->getMail()->getHeaders();
            $email->ts_sent   = time();
            $email->save();

            return $email;
        }

        protected static function _AddRecipient($user, $mail)
        {
            try {
                $mail->addTo($user->getEmailAddress());
                return true;
            }
            catch (Exception $ex) {
                return false;
            }
        }

        /**
         * Misc functions
         */

        /**
         * Misc static functions
         */

        public static function ClearForUser($userIds)
        {
            $userIds = ArrayTools::Clean($userIds, true);

            if (count($userIds) == 0) {
                return;
            }

            $db = Application::GetDb();
            $query = sprintf('delete from %s where user_id in (%s)', self::$table, $db->quote($userIds));
            $db->query($query);
        }

        public static function ClearForDirectory($directoryId)
        {
            $query = sprintf(
                'delete from %s where user_id in (select user_id from %s where directory_id = %d)',
                self::$table,
                Application_User::$table,
                $directoryId
            );
            Application::GetDb()->query($query);
        }

        public static function GetEmailsCount($options = array())
        {
            $select = self::_GetBaseQuery($options);
            $select->from(null, 'count(*)');

            return (int) Application::GetDb()->fetchOne($select);
        }

        public static function GetEmails($options = array())
        {
            $defaults = array(
                'order'  => array('e.ts_sent', 'e.email_id'),
                'offset' => 0,
                'limit'  => 0
            );

            $options = ArrayTools::BuildOptions($options, $defaults);

            $select = self::_GetBaseQuery($options);
            $select->from(null, '*')
                   ->order($options['order']);

            $options['offset'] = max(0, (int) $options['offset']);
            if ($options['limit'] > 0) {
                $select->limit($options['limit'], $options['offset']);
            }

            $db = Application::GetDb();
            $result = $db->query($select);
            $data = array();
            while ($row = $result->fetch()) {
                $data[] = $row;
            }

            return parent::BuildMultiple($db, __CLASS__, $data);
        }

        public static function _GetBaseQuery($options = array())
        {
            $defaults = array(
                'user_ids' => array()
            );

            $options = ArrayTools::BuildOptions($options, $defaults);

            $db = Application::GetDb();
            $select = $db->select();
            $select->from(array('e' => self::$table), array());

            $options['user_ids'] = ArrayTools::clean($options['user_ids'], true);

            if (count($options['user_ids']) > 0) {
                $select->where('e.user_id in (?)', $options['user_ids']);
            }

            return $select;
        }
    }
?>