<?php
    class Application_User_Permissions_Manager
    {
        protected static $_instance = null;

        static $paths = array(
            'default' => '%s/settings/permissions.xml', // substitute with application dir
            'module'  => '%s/settings/permissions.xml'  // substitute with module dir
        );

        private function __construct()
        {

        }

        public static function getInstance()
        {
            if (null === self::$_instance) {
                self::$_instance = new self();
            }

            return self::$_instance;
        }

        public function deployDefaults(Application_Client $client)
        {
            $fullpath = sprintf(self::$paths['default'], Application::GetApplicationDir());

            if (!file_exists($fullpath)) {
                throw new Exception('Unable to find permissions file in %s', $fullpath);
            }

            if (!is_file($fullpath) || !is_readable($fullpath)) {
                throw new Exception('Unable to read permissions file %s', $fullpath);
            }

            $xml = new SimpleXMLElement($fullpath, LIBXML_NOCDATA, true);

            $db = Application::GetDb();
            $db->begin();

            $this->_deployDefaultRoles($client, $xml);
            $this->_deployPermissions($client, $xml);

            // loop over client's modules and deploy those permissions
            foreach ($client->getModules() as $module) {
                $this->deployModulePermissions($client, $module);
            }

            $db->commit();
        }

        public function deployFromXml(Application_Client $client, $xml)
        {
            $this->_deployDefaultRoles($client, $xml);
            $this->_deployPermissions($client, $xml);
        }

        public function deployModulePermissions(Application_Client $client, $module)
        {
            $module = Application_Module_Manager::Factory($module);

            $fullpath = sprintf(self::$paths['module'], $module->getPath());

            if (file_exists($fullpath) && is_file($fullpath) && is_readable($fullpath)) {
                $xml = new SimpleXMLElement($fullpath, LIBXML_NOCDATA, true);
            }
            else {
                $xml = null;
            }

            $this->_deployPermissions($client, $xml, $module);
        }

        private function _deployDefaultRoles(Application_Client $client, $xml)
        {
            if (!isset($xml->roles))
                return;

            $options = array(
                'directory_ids' => null,
                'client_ids'    => $client->getId()
            );

            $roles = Application_User_Role::GetRoles($options);
            $_roles = array();

            foreach ($roles as $role) {
                $title = Text::Urlize($role->title);
                $_roles[$title] = $role->getId();
            }

            foreach ($xml->roles->role as $node) {
                $title = trim((string) $node->name);

                if (strlen($title) == 0)
                    continue;

                $_title = Text::Urlize($title);

                if (!array_key_exists($_title, $_roles)) {
                    $role = new Application_User_Role();
                    $role->rebuildTree(false);

                    $role->client_id = $client->getId();
                    $role->title     = $title;

                    // @todo    Make descriptions save with roles
                    //$role->profile->description = (string) $node->description;
                    $role->save(false);

                    $roles[$role->getId()] = $role;
                    $_roles[$_title] = $role->getId();
                }
            }

            foreach ($xml->roles->role as $node) {
                $parent = Text::Urlize((string) $node['inherits']);
                if (strlen($parent) == 0)
                    continue;

                $title = trim((string) $node->name);
                if (strlen($title) == 0)
                    continue;

                $_title = Text::Urlize($title);

                if (!array_key_exists($_title, $_roles))
                    continue;

                if (!array_key_exists($parent, $_roles))
                    continue;

                $role_id = $_roles[$_title];
                $parent_id = $_roles[$parent];

                $roles[$role_id]->parent_id = $parent_id;
                $roles[$role_id]->save(false);
            }

            $tree = Application_User_Role_NestedTree::GetInstance();
            $tree->rebuild();
        }

        private function _deployPermissions(Application_Client $client, $xml, $module = null)
        {
            try {
                $module = Application_Module_Manager::Factory($module);
            }
            catch (Exception $ex) {
                $module = null;
            }

            $options = array(
                'directory_ids' => null,
                'client_ids'    => $client->getId()
            );

            $roles = Application_User_Role::GetRoles($options);
            $_roles = array();

            foreach ($roles as $role) {
                $title = Text::Urlize($role->title);
                $_roles[$title] = $role->getId();
            }

            $new = array();

            if ($module instanceof Application_Module_Interface) {
                $key = sprintf('module:%s', $module->getName());
                $new[$key] = array(
                    'allow' => false,
                    'desc'  => (string) $module,
                    'roles' => array(),
                    'finegrained' => false
                );

                foreach ($_roles as $title => $id) {
                    $new[$key]['roles'][$title] = true;
                }
            }

            if ($xml instanceof SimpleXMLElement) {
                foreach ($xml->permission as $node) {
                    $key = (string) $node['key'];

                    // here we allow an empty key if the permissions are being deployed for a module.
                    // this is so the main module setting can be specified on a per-role basis
                    if (!$module instanceof Application_Module && strlen($key) == 0) {
                        continue;
                    }

                    $key = str_replace('.', ':', $key);

                    if (array_key_exists($key, $new))
                        throw new Exception(sprintf('Duplicate key found (%s)', $key));

                    $allow = isset($node['allow']) && $node['allow'];

                    $row = array(
                        'allow'       => $allow,
                        'desc'        => '',
                        'roles'       => array(),
                        'finegrained' => false
                    );

                    if (strlen($key) == 0 && $module instanceof Application_Module_Interface) {
                        $row['desc'] = (string) $module;
                    }

                    if (isset($node->description)) {
                        $row['desc'] = (string) $node->description;
                    }

                    if (isset($node['finegrained'])) {
                        $row['finegrained'] = (bool) $node['finegrained'];
                    }

                    foreach ($node->role as $role) {
                        $name = Text::Urlize((string) $role['name']);
                        if (strlen($name) == 0)
                            continue;

                        $row['roles'][$name] = isset($role['allow']) && $role['allow'];
                    }

                    if ($module instanceof Application_Module_Interface) {
                        $key = sprintf('module:%s:%s', $module->getName(), $key);
                    }

                    $key = rtrim($key, ':');

                    $new[$key] = $row;
                }
            }

            $options = array(
                'client_ids' => $client->getId()
            );

            $permissions = Application_User_Permissions_Permission::GetPermissions(null, $options);
            $hash = array();

            foreach ($permissions as $permission) {
                $hash[$permission->title] = $permission->getId();
            }

            foreach ($new as $key => $row) {
                $exists = array_key_exists($key, $hash);

                if ($exists) {
                    $permission_id = $hash[$key];
                    $permission = $permissions[$permission_id];
                }
                else {
                    $permission = new Application_User_Permissions_Permission();
                    $permission->title = $key;
                    $permission->client_id = $client->getId();
                    $permission->allow = $row['allow'];
                }

                if (strlen($row['desc']) > 0)
                    $permission->description = $row['desc'];

                $permission->finegrained = $row['finegrained'];

                $permission->save();

                if ($exists)
                    continue;

                foreach ($row['roles'] as $role => $allow) {
                    if (!array_key_exists($role, $_roles))
                        continue;

                    $role_id = $_roles[$role];

                    if ($allow)
                        $permission->allow($role_id);
                    else
                        $permission->deny($role_id);
                }

            }
        }

        /**
         * Role module related methods
         */

        public function getModuleSummary(Application_User_Role $role)
        {
            $ret = array();

            $client = $role->getClient();
            if (!$client->saved()) {
                return $ret;
            }

            $permissions = new Application_Cp_Site_Permissions($client);
            $permissions->addUserRole($role->getId());

            foreach (Application_Module_Manager::GetModules($client) as $module) {
                $key = sprintf('module:%s', $module->getName());
                $name = $module->getName();

                $ret[$name] = array(
                    'module' => $module,
                    'used'   => $permissions->isAllowed($key)
                );
            }

            return $ret;
        }

        public function addModulesToRole(Application_User_Role $role, $modules)
        {
            if (!$role->saved()) {
                return 0;
            }

            $modules = ArrayTools::clean($modules);
            $summary = $this->getModuleSummary($role);

            $ret = 0;

            foreach ($modules as $module) {
                $module = Application_Module_Manager::Factory($module);

                if (!array_key_exists($module->getName(), $summary)) {
                    continue;
                }

                if ($summary[$module->getName()]['used']) {
                    continue;
                }

                $permission = new Application_User_Permissions_Permission();
                $permission->loadForDirectoryClientTitle(null, $role->client_id, sprintf('module:%s', $module->getName()));

                if (!$permission->saved()) {
                    continue;
                }

                $permission->allow($role->getId());
                $ret++;
            }

            return $ret;
        }

        public function removeModulesFromRole(Application_User_Role $role, $modules)
        {
            if (!$role->saved()) {
                return 0;
            }

            $modules = ArrayTools::clean($modules);
            $summary = $this->getModuleSummary($role);

            $ret = 0;

            foreach ($modules as $module) {
                $module = Application_Module_Manager::Factory($module);

                if (!array_key_exists($module->getName(), $summary)) {
                    continue;
                }

                if (!$summary[$module->getName()]['used']) {
                    continue;
                }

                $permission = new Application_User_Permissions_Permission();
                $permission->loadForDirectoryClientTitle(null, $role->client_id, sprintf('module:%s', $module->getName()));

                if (!$permission->saved()) {
                    continue;
                }

                $permission->deny($role->getId());
                $ret++;
            }

            return $ret;
        }
    }
?>