<?php
    class Application_User_Permissions_Permission extends DatabaseObject
    {
        public static $table = 'users_roles_permissions';
        public static $rolesTable = 'users_roles_to_permissions';

        const TYPE_ALLOW   = 'allow';
        const TYPE_DENY    = 'deny';
        const TYPE_INHERIT = 'inherit';
        const ROLE_DEFAULT = 'default';

        protected $_roles;

        public function __construct()
        {
            parent::__construct(Application::GetDb(), self::$table, 'permission_id');

            $this->add('directory_id');
            $this->add('client_id');
            $this->add('title');
            $this->add('description');
            $this->add('allow', false, self::TYPE_BOOLEAN);
            $this->add('finegrained', false, self::TYPE_BOOLEAN);
        }

        public function __destruct()
        {
            parent::__destruct();
            unset($this->_roles);
        }

        /**
         * Loaders
         */

        public function loadForDirectoryClientTitle($directory_id, $client_id, $title)
        {
            $directory_id = (int) $directory_id;
            $client_id    = (int) $client_id;
            $title        = trim($title);

            if (strlen($title) == 0)
                return false;

            $select = $this->db->select();
            $select->from($this->getTable(), $this->getSelectFields())
                   ->where('title = ?', $title);

            if ($directory_id > 0)
                $select->where('directory_id = ?', $directory_id);
            else
                $select->where('directory_id is null');

            if ($client_id > 0)
                $select->where('client_id = ?', $client_id);
            else
                $select->where('client_id is null');

            return $this->_load($select);
        }

        public function loadForClient($clientId, $permissionId)
        {
            $clientId = (int) $clientId;
            $permissionId = (int) $permissionId;

            if ($clientId <= 0 || $permissionId <= 0) {
                return false;
            }

            $select = $this->db->select();
            $select->from($this->getTable(), $this->getSelectFields())
                   ->where('client_id = ?', $clientId)
                   ->where('permission_id = ?', $permissionId);

            return $this->_load($select);
        }

        /**
         * Callbacks
         */

        public function preDelete()
        {
            $query = sprintf('delete from %s where permission_id = %d',
                             self::$rolesTable,
                             $this->getId());

            $this->db->query($query);

            return true;
        }

        /**
         * Misc functions
         */

        public function __toString()
        {
            return $this->title;
        }

        public function getRoles()
        {
            if (!is_array($this->_roles)) {
                $query = sprintf('select role_id, grain, allow from %s where permission_id = %d', self::$rolesTable, $this->getId());
                $result = $this->db->query($query);

                $this->_roles = array(
                    'defaults' => array(),
                    'grains'   => array()
                );

                while ($row = $result->fetch()) {
                    if (strlen($row->grain) > 0 && !$this->finegrained)
                        continue;

                    $val = Text::toBoolean($row->allow) ? self::TYPE_ALLOW : self::TYPE_DENY;

                    if (strlen($row->grain) > 0) {
                        if (!array_key_exists($row->grain, $this->_roles['grains']))
                            $this->_roles['grains'][$row->grain] = array();

                        $this->_roles['grains'][$row->grain][$row->role_id] = $val;
                    }
                    else {
                        $this->_roles['defaults'][$row->role_id] = $val;
                    }
                }
            }

            return $this->_roles;
        }

        public function getType($role_id, $grain = '')
        {
            if (!$this->finegrained)
                $grain = '';

            $roles = $this->getRoles();

            $ret = self::TYPE_INHERIT;

            if (strlen($grain) > 0) {
                if (array_key_exists($grain, $roles['grains']) && array_key_exists($role_id, $roles['grains'][$grain]))
                    $ret = $roles['grains'][$grain][$role_id];
            }
            else {
                if (array_key_exists($role_id, $roles['defaults']))
                    $ret = $roles['defaults'][$role_id];
            }

            return $ret;
        }

        public function allow($role_id, $grain = '')
        {
            if (strlen($grain) > 0 && !$this->finegrained)
                return;

            if ($role_id == self::ROLE_DEFAULT) {
                $this->allow = true;
                $this->save();
                return;
            }

            $role_id = (int) $role_id;
            if ($role_id <= 0)
                return;

            $type = $this->getType($role_id, $grain);

            if ($type == self::TYPE_ALLOW)
                return;

            $table = self::$rolesTable;
            if ($type == self::TYPE_DENY) {
                $where = array(
                    $this->db->quoteInto('permission_id = ?', $this->getId()),
                    $this->db->quoteInto('role_id = ?', $role_id),
                    $this->db->quoteInto('grain = ?', $grain)
                );
                $this->db->update($table, array('allow' => 1), $where);
            }
            else {
                $vals = array(
                    'permission_id' => $this->getId(),
                    'role_id'       => $role_id,
                    'allow'         => 1,
                    'grain'         => $grain
                );
                $this->db->insert($table, $vals);
            }
        }

        public function deny($role_id, $grain = '')
        {
            if (strlen($grain) > 0 && !$this->finegrained)
                return;

            if ($role_id == self::ROLE_DEFAULT) {
                $this->allow = false;
                $this->save();
                return;
            }

            $role_id = (int) $role_id;
            if ($role_id <= 0) {
                return;
            }

            $type = $this->getType($role_id, $grain);

            if ($type == self::TYPE_DENY)
                return;

            $table = self::$rolesTable;
            if ($type == self::TYPE_ALLOW) {
                $where = array(
                    $this->db->quoteInto('permission_id = ?', $this->getId()),
                    $this->db->quoteInto('role_id = ?', $role_id),
                    $this->db->quoteInto('grain = ?', $grain)
                );
                $this->db->update($table, array('allow' => 0), $where);
            }
            else {
                $vals = array(
                    'permission_id' => $this->getId(),
                    'role_id'       => $role_id,
                    'allow'         => 0,
                    'grain'         => $grain
                );

                $this->db->insert($table, $vals);
            }
        }

        public function inherit($role_id, $grain = '')
        {
            if (strlen($grain) > 0 && !$this->finegrained)
                return;

            $role_id = (int) $role_id;
            if ($role_id <= 0)
                return;

            $where = array(
                $this->db->quoteInto('permission_id = ?', $this->getId()),
                $this->db->quoteInto('role_id = ?', $role_id),
                $this->db->quoteInto('grain = ?', $grain)
            );

            $this->db->delete(self::$rolesTable, $where);
        }

        public function setPermission($roleId, $setting)
        {
            switch ($setting) {
                case self::TYPE_ALLOW:
                    $this->allow($roleId);
                    break;

                case self::TYPE_DENY:
                    $this->deny($roleId);
                    break;

                case self::TYPE_INHERIT:
                    $this->inherit($roleId);
                    break;

                default:
                    throw new Exception('Invalid setting');
            }
        }

        /**
         * Static functions
         */

        public function GetPermissions($directory_id, $options)
        {
            $defaults = array(
                'client_ids'       => array(),
                'filter'           => null,
                'prefix'           => null,
                'finegrained_only' => false
            );

            $options = ArrayTools::BuildOptions($options, $defaults);

            $db = Application::GetDb();

            $select = $db->select();
            $select->from(self::$table, '*')
                   ->order('lower(title)');

            $directory_id = (int) $directory_id;
            if ($directory_id > 0)
                $select->where('directory_id = ?', $directory_id);
            else
                $select->where('directory_id is null');

            $options['client_ids'] = ArrayTools::clean($options['client_ids'], true);
            if (count($options['client_ids']) > 0)
                $select->where('client_id in (?)', $options['client_ids']);

            if (strlen($options['filter']) > 0)
                $select->where('lower(title) like lower(?)', '%' . $options['filter'] . '%');

            if (strlen($options['prefix']) > 0) {
                $wheres = array(
                    $db->quoteInto('lower(title) like lower(?)', trim($options['prefix'], ':') . ':%'),
                    $db->quoteInto('lower(title) = ?', $options['prefix'])
                );
                $select->where(join(' or ', $wheres));
            }

            if ($options['finegrained_only'])
                $select->where('finegrained = true');

            $result = $db->query($select);
            $data = array();
            while ($row = $result->fetch())
                $data[] = $row;

            $permissions = parent::BuildMultiple($db, __CLASS__, $data);
            self::_LoadRoles($permissions);

            return $permissions;
        }

        protected static function _LoadRoles(&$arr)
        {
            if (count($arr) == 0)
                return;

            foreach ($arr as $k => $v) {
                $arr[$k]->_roles = array(
                    'defaults' => array(),
                    'grains'   => array()
                );
            }

            $permission_ids = array_keys($arr);

            $db = Application::GetDb();

            $select = $db->select();
            $select->from(self::$rolesTable, array('permission_id', 'role_id', 'grain', 'allow'))
                   ->where('permission_id in (?)', $permission_ids);

            $result = $db->query($select);

            while ($row = $result->fetch()) {
                if (!array_key_exists($row->permission_id, $arr)) {
                    continue;
                }

                if (strlen($row->grain) > 0 && !$arr[$row->permission_id]->finegrained) {
                    continue;
                }

                $val = Text::toBoolean($row->allow) ? self::TYPE_ALLOW : self::TYPE_DENY;

                if (strlen($row->grain) > 0) {
                    if (!array_key_exists($row->grain, $arr[$row->permission_id]->_roles['grains']))
                        $arr[$row->permission_id]->_roles['grains'][$row->grain] = array();

                    $arr[$row->permission_id]->_roles['grains'][$row->grain][$row->role_id] = $val;
                }
                else {
                    $arr[$row->permission_id]->_roles['defaults'][$row->role_id] = $val;
                }
            }
        }

        public static function ClearForClient($client_id)
        {
            $query = sprintf('delete from %s where client_id = %d', self::$table, $client_id);
            Application::GetDb()->query($query);
        }

        /**
         * Getters, setters and checkers
         */

        public function __set($k, $v)
        {
            switch ($k) {
                case 'directory_id':
                case 'client_id':
                    $v = (int) $v;
                    if ($v <= 0)
                        $v = null;
                    break;
            }

            parent::__set($k, $v);
        }
    }
?>
