<?php
    /**
     * Class used for a constructing a permissions query. This allows you to check
     * any number of permissions in one hit
     *
     * @author  Quentin Zervaas <quentin@recite.com.au>
     */
    class Application_User_Permissions_Query extends Application_User_Permissions_Query_Item
    {
        /**
         * Constant used to indicate all items must be satisfied
         */
        const OP_AND = 'and';

        /**
         * Constant used to indicate any items must be satisfied
         */
        const OP_OR  = 'or';

        /**
         * Used to hold the boolean operand type
         *
         * @var string
         */
        protected $_op = null;

        /**
         * A list of the permissions to check
         *
         * @var Application_User_Permissions_Query_Item[]
         */
        protected $_items = array();

        /**
         * Constructor. Specify the operand (default 'and')
         *
         * @param   string  $op     The operand, either self::OP_AND (default) or self::OP_OR
         */
        public function __construct($op = null)
        {
            if ($op != self::OP_OR) {
                $op = self::OP_AND;
            }

            $this->_op = $op;
        }

        /**
         * Add an item to the permissions query
         *
         * @param   string|Application_User_Permissions_Query_Item  $item   The item to add
         * @return  self
         */
        public function add($item)
        {
            if (!$item instanceof Application_User_Permissions_Query_Item) {
                $item = new Application_User_Permissions_Query_Item($item);
            }

            $this->_items[] = $item;
            return $this;
        }

        /**
         * Assert the permissions check. If the check fails an exception is thrown.
         * The exception contains a list of permissions that failed.
         *
         * @throws  Application_User_Permissions_Exception
         */
        public function assert()
        {
            if (count($this->_items) == 0) {
                return;
            }

            $failed = array();

            if ($this->_op == self::OP_OR) {
                foreach ($this->_items as $item) {
                    try {
                        $item->assert();
                        return;
                    }
                    catch (Application_User_Permissions_Exception $ex) {
                        foreach ($ex->getPerms() as $perm) {
                            $failed[] = $perm;
                        }
                    }

                    $failed[] = $item;
                }
            }
            else {
                foreach ($this->_items as $item) {
                    try {
                        $item->assert();
                    }
                    catch (Application_User_Permissions_Exception $ex) {
                        foreach ($ex->getPerms() as $perm) {
                            $failed[] = $perm;
                        }
                    }
                }
            }

            if (count($failed) > 0) {
                throw new Application_User_Permissions_Exception($failed);
            }
        }

        /**
         * Create a new 'and' query from a single permission. This is just a small short cut
         * to creating the query, the item, and adding the item to the query.
         *
         * @param   string|Application_User_Permissions_Query_Item  $item
         * @return  Application_User_Permissions_Query
         */
        public static function NewAnd($item = null)
        {
            $ret = new self(self::OP_AND);

            if (!is_null($item)) {
                $ret->add($item);
            }

            return $ret;
        }

        /**
         * Create a new 'or' query from a single permission. This is just a small short cut
         * to creating the query, the item, and adding the item to the query.
         *
         * @param   string|Application_User_Permissions_Query_Item  $item
         * @return  Application_User_Permissions_Query
         */
        public static function NewOr($item = null)
        {
            $ret = new self(self::OP_OR);

            if (!is_null($item)) {
                $ret->add($item);
            }

            return $ret;
        }
    }
?>