<?php
    class Application_User_Role extends DatabaseObject
    {
        public static $table = 'users_roles';
        protected $_rebuildTree = true;
        public $client;

        public function __construct()
        {
            parent::__construct(Application::GetDb(), self::$table, 'role_id');

            $this->add('directory_id');
            $this->add('client_id');
            $this->add('parent_id');
            $this->add('title');
            $this->add('nleft', null, null, false);
            $this->add('nright', null, null, false);
            $this->add('nlevel', null, null, false);
        }

        public function __destruct()
        {
            parent::__destruct();
            unset($this->client);
        }

        /**
         * Loaders
         */

        public function loadForDirectory($directory_id, $role_id)
        {
            $directory_id = (int) $directory_id;
            $role_id = (int) $role_id;

            $query = sprintf('select %s from %s where %s = %d',
                             join(', ', $this->getSelectFields()),
                             $this->getTable(),
                             $this->getIdField(),
                             $role_id);

            if ($directory_id <= 0)
                $query .= ' and directory_id is null';
            else
                $query .= sprintf(' and directory_id = %d', $directory_id);

            return $this->_load($query);
        }

        public function loadClientRole($client_id, $role_id, $directory_id = null)
        {
            $client_id = (int) $client_id;
            $role_id   = (int) $role_id;

            if ($client_id <= 0 || $role_id <= 0)
                return false;

            $select = $this->db->select();
            $select->from(array('ur' => $this->getTable()), $this->getSelectFields())
                   ->where('ur.client_id = ?', $client_id)
                   ->where('ur.role_id = ?', $role_id);

            if (is_null($directory_id))
                $select->where('directory_id is null');
            else
                $select->where('directory_id = ?', $directory_id);

            return $this->_load($select);
        }

        public function loadClientRoleByTitle($client_id, $title, $directory_id = null)
        {
            $client_id = (int) $client_id;
            $title     = trim($title);

            if ($client_id <= 0 || strlen($title) == 0) {
                return false;
            }

            $select = $this->db->select();
            $select->from(array('ur' => $this->getTable()), $this->getSelectFields())
                   ->where('ur.client_id = ?', $client_id)
                   ->where('ur.title = ?', $title);

            if (is_null($directory_id)) {
                $select->where('directory_id is null');
            }
            else {
                $select->where('directory_id = ?', $directory_id);
            }

            return $this->_load($select);
        }

        public function loadSysAdminRole()
        {
            $select = $this->db->select();
            $select->from($this->getTable(), $this->getSelectFields())
                   ->where('directory_id is null')
                   ->where('client_id is null')
                   ->where('parent_id is null')
                   ->where('title = ?', 'sysadmin');

            return $this->_load($select);
        }

        /**
         * Callbacks
         */

        protected function postInsert()
        {
            $message = new Application_Auditor_Message('Users: Role created');
            $message->addMemo('directory_id', $this->directory_id)
                    ->addMemo('client_id', $this->client_id)
                    ->addMemo('role_id', $this->getId())
                    ->record();

            if ($this->_rebuildTree) {
                $tree = Application_User_Role_NestedTree::GetInstance($this->directory_id);
                $tree->rebuild();
            }

            return true;
        }

        protected function postUpdate()
        {
            $message = new Application_Auditor_Message('Users: Role updated');
            $message->addMemo('directory_id', $this->directory_id)
                    ->addMemo('client_id', $this->client_id)
                    ->addMemo('role_id', $this->getId())
                    ->record();

            if ($this->_rebuildTree) {
                $tree = Application_User_Role_NestedTree::GetInstance($this->directory_id);
                $tree->rebuild();
            }

            return true;
        }

        protected function preDelete()
        {
            $event = new Application_Event('/module/users/role/predelete');
            $event->set('role', $this);
            $event->trigger();

            $queries = array();

            // don't delete child roles, just promote them 1 level
            $parent = (int) $this->get('parent_id');
            $queries[] = sprintf('update %s set parent_id = %s where parent_id = %d',
                                 $this->getTable(),
                                 $parent == 0 ? 'null' : $parent,
                                 $this->getId());

            $queries[] = sprintf('delete from users_to_roles where role_id = %d', $this->getId());
            $queries[] = sprintf('delete from users_roles_to_permissions where role_id = %d', $this->getId());

            foreach ($queries as $query)
                $this->db->query($query);

            return true;
        }

        protected function postDelete()
        {
            $event = new Application_Event('/module/users/role/postdelete');
            $event->set('id', $this->getDeletedId());
            $event->trigger();

            $message = new Application_Auditor_Message('Users: Role deleted');
            $message->addMemo('directory_id', $this->directory_id)
                    ->addMemo('client_id', $this->client_id)
                    ->addMemo('role_id', $this->getDeletedId())
                    ->record();

            if ($this->_rebuildTree) {
                $tree = Application_User_Role_NestedTree::GetInstance($this->directory_id);
                $tree->rebuild();
            }

            return true;
        }

        /**
         * Misc functions
         */

        public function __toString()
        {
            return (string) $this->title;
        }

        public function getClient()
        {
            if (!$this->client instanceof Application_Client || !$this->client->saved()) {
                $this->client = new Application_Client();
                $this->client->load($this->client_id);
            }

            return $this->client;
        }

        public function rebuildTree($bool)
        {
            $this->_rebuildTree = (bool) $bool;
        }

        public function getNumUsers()
        {
            $select = $this->db->select();
            $select->from(array('ur' => 'users_to_roles'), 'count(*)')
                   ->where('role_id = ?', $this->getId());

            return (int) $this->db->fetchOne($select);
        }

        public function isControlPanelRole()
        {
            $directoryId = (int) $this->directory_id;

            return $directoryId <= 0;
        }

        /**
         * Static functions
         */

        public static function GetSummaryForUsers($userIds)
        {
            $db = Application::GetDb();
            $select = $db->select();
            $select->from(array('utr' => Application_User::$rolesTable), array('user_id', 'role_id'));

            if ($userIds instanceof Application_User_Directory) {
                $select->join(array('ur' => self::$table), 'ur.role_id = utr.role_id', array())
                       ->where('ur.directory_id = ?', $userIds->getId());
            }
            else {
                $userIds = ArrayTools::Clean($userIds, true);

                if (count($userIds) == 0) {
                    return array();
                }

                $select->where('user_id in (?)', $userIds);
            }

            $ret = array();

            foreach ($db->fetchAll($select) as $row) {
                $userId = $row['user_id'];
                $roleId = $row['role_id'];

                if (!array_key_exists($userId, $ret)) {
                    $ret[$userId] = array();
                }

                $ret[$userId][$roleId] = $roleId;
            }

            return $ret;
        }

        public static function GetSysAdminRole()
        {
            $ret = new self();
            $ret->loadSysAdminRole();

            if (!$ret->saved()) {
                throw new Exception('Unable to load role');
            }

            return $ret;
        }

        public static function GetRoles($options)
        {
            $defaults = array(
                'load_clients' => false,
                'order'        => array('r.nleft'),
                'offset'       => 0,
                'limit'        => 0
            );

            $options = ArrayTools::BuildOptions($options, $defaults);

            $select = self::_GetBaseQuery($options);
            $select->from(null, 'r.*')
                   ->order($options['order']);

            $options['offset'] = max(0, (int) $options['offset']);
            if ($options['limit'] > 0) {
                $select->limit($options['limit'], $options['offset']);
            }

            $db = Application::GetDb();

            $result = $db->query($select);
            $data = array();
            while ($row = $result->fetch()) {
                $data[] = $row;
            }

            $roles = parent::BuildMultiple($db, __CLASS__, $data);

            if ($options['load_clients']) {
                self::_LoadClients($roles);
            }

            return $roles;
        }

        public function GetRolesCount($options)
        {
            $select = self::_GetBaseQuery($options);
            $select->from(null, 'count(*)');

            return Application::GetDb()->fetchOne($select);
        }

        protected function _LoadClients(&$arr)
        {
            $client_ids = array();
            foreach ($arr as $k => $v)
                $client_ids[$v->client_id] = $v->client_id;

            if (count($client_ids) == 0)
                return;

            $options = array('client_ids' => $client_ids);

            $clients = Application_Client::GetClients($options);

            foreach ($arr as $k => $v) {
                if (array_key_exists($v->client_id, $clients)) {
                    $arr[$k]->client = $clients[$v->client_id];
                }
                else {
                    $arr[$k]->client = new Application_Client();
                }
            }
        }

        public function _GetBaseQuery($options)
        {
            $defaults = array(
                'role_ids'      => array(),
                'client_ids'    => array(),
                'user_ids'      => array(),
                'directory_ids' => array(),
                'filters'       => array()
            );

            //allow null values through for client and directory
            //$options = ArrayTools::buildOptions($options, $defaults);
            if (!is_array($options)){
                $options = array();
            }

            foreach ($defaults as $k => $v) {
                //overwrite if not set. Allow null for directory_ids and client_ids
                if (!isset($options[$k]) && (($k != 'directory_ids' && $k != 'client_ids') || !array_key_exists($k, $options))) {
                    $options[$k] = $v;
                }
            }

            $options['role_ids'] = ArrayTools::clean($options['role_ids'], true);
            $options['user_ids'] = ArrayTools::clean($options['user_ids'], true);

            $db = Application::GetDb();

            $select = $db->select();
            $select->from(array('r' => self::$table), array());

            if (count($options['role_ids']) > 0)
                $select->where('r.role_id in (?)', $options['role_ids']);

            if (count($options['user_ids']) > 0) {
                $select->joinInner(array('ur' => 'users_to_roles'), 'ur.role_id = r.role_id', array())
                       ->where('ur.user_id in (?)', $options['user_ids']);
            }

            // some ugly code to deal with null client / directory
            $tmp = array('client_id', 'directory_id');
            foreach ($tmp as $field) {
                $null = false;
                $values = array();

                $key = $field . 's';
                if (!is_array($options[$key]))
                    $options[$key] = array($options[$key]);

                foreach ($options[$key] as $k => $v) {
                    if (is_null($v)) {
                        $null = true;
                        continue;
                    }

                    $v = (int) $v;
                    if ($v > 0)
                        $values[] = $v;
                }

                $clauses = array();
                if ($null)
                    $clauses[] = sprintf('r.%s is null', $field);
                if (count($values) > 0)
                    $clauses[] = sprintf('r.%s in (%s)', $field, join(', ', $values));

                if (count($clauses) > 0)
                    $select->where(join(' or ', $clauses));
            }

            $options['filters'] = ArrayTools::clean($options['filters']);
            foreach ($options['filters'] as $filter)
                $select->where($filter);

            return $select;
        }

        /**
         * Getters, setters and checkers
         */

        public function __set($k, $v)
        {
            if ($k == 'directory_id') {
                $v = (int) $v;
                if ($v <= 0)
                    $v = null;
            }

            parent::__set($k, $v);
        }
    }
?>
