<?php
    /**
     * DatabaseObject
     *
     * Abstract class used to easily manipulate data in a database table
     * via simple load/save/delete methods
     */
    abstract class DatabaseObject
    {
        const TYPE_TIMESTAMP = 1;
        const TYPE_BOOLEAN   = 2;
        const TYPE_ARRAY     = 3;

        protected static $types = array(self::TYPE_TIMESTAMP, self::TYPE_BOOLEAN, self::TYPE_ARRAY);

        private $_id = null;
        private $_deleted_id = null;

        private $_properties = array();

        protected $db;
        protected $_table = '';
        protected $_idField = '';

        protected $_locked = false;

        public function __construct(Db_Adapter_Abstract $db, $table, $idField)
        {
            $this->db = $db;
            $this->_table = $table;
            $this->_idField = $idField;
        }

        public function __destruct()
        {
            unset($this->db);
        }

        public function getTable()
        {
            return $this->_table;
        }

        public function getIdField()
        {
            return $this->_idField;
        }

        public function load($id, $field = null)
        {
            if (strlen($field) == 0) {
                $field = $this->_idField;
            }

            if ($field == $this->_idField) {
                $id = (int) $id;
                if ($id <= 0) {
                    return false;
                }
            }

            $query = sprintf('select %s from %s where %s = ?',
                             join(', ', $this->getSelectFields()),
                             $this->_table,
                             $field);

            $query = $this->db->quoteInto($query, $id);

            return $this->_load($query);
        }

        public function loadRecordForClient($id, $argb=NULL)
        {
            $id = (int) $id;
            $client_id = (int) $this->client_id;
            if ($id == 0 || $client_id == 0)
                return false;

            $select = $this->db->select();
            $select->from($this->_table, $this->getSelectFields())
                   ->where('client_id = ?', $client_id)
                   ->where($this->_idField . ' = ?', $id);

            return $this->_load($select);
        }

        protected function getSelectFields($prefix = '')
        {
            $fields = array($prefix . $this->_idField);
            foreach ($this->_properties as $k => $v)
                $fields[] = $prefix . $k;

            return $fields;
        }

        public function getValues()
        {
            $ret = array();
            foreach ($this->_properties as $k => $v)
                $ret[$k] = $this->get($k);

            return $ret;
        }

        protected function _load($query)
        {
            $result = $this->db->query($query);
            $row = $result->fetch();

            if (!$row)
                return false;

            $this->_init($row);

            $this->postLoad();

            DatabaseObject_Registry::Register($this);

            return true;
        }

        public function _init($row)
        {
            $helper = $this->db->getStatementHelper();

            foreach ($this->_properties as $k => $v) {
                $val = $row->$k;

                switch ($v['type']) {
                    case self::TYPE_TIMESTAMP:
                        $val = $helper->unixTimestamp($val);
                        break;

                    case self::TYPE_BOOLEAN:
                        $val = Text::toBoolean($val);
                        break;

                    case self::TYPE_ARRAY:
                        $val = self::_getArray($val);
                }

                $this->_properties[$k]['value'] = $val;
                $this->_properties[$k]['original'] = $val;
            }
            $field = $this->_idField;
            $this->_id = (int) $row->$field;
        }


        public function save()
        {
            if ($this->_locked) {
                throw new DatabaseObject_Exception('This record is locked');
            }

            $this->_locked = true;

            $update = $this->saved();

            $this->db->begin();

            if ($update)
                $commit = $this->preUpdate();
            else
                $commit = $this->preInsert();

            if (!$commit) {
                $this->db->rollback();
                return false;
            }

            $row = array();

            foreach ($this->_properties as $k => $v) {
                if ($update && (!$v['savechanges'] || !$v['updated'])) {
                    continue;
                }

                switch ($v['type']) {
                    case self::TYPE_TIMESTAMP:
                        if (!is_null($v['value'])) {
                            $helper = $this->db->getStatementHelper();
                            $v['value'] = $helper->getTimestamp($v['value']);
                        }
                        else{
                            $v['value'] = new Zend_Db_Expr('NULL');
                        }
                        break;

                    case self::TYPE_BOOLEAN:
                        $v['value'] = (int) ((bool) $v['value']);
                        break;

                    case self::TYPE_ARRAY:
                        $v['value'] = self::_setArray($v['value']);
                        break;
                }

                $row[$k] = $v['value'];
            }

            if (count($row) > 0) {
                // perform insert/update
                if ($update) {
                    $this->db->update($this->_table, $row, sprintf('%s = %d', $this->_idField, $this->getId()));
                }
                else {
                    $this->db->insert($this->_table, $row);
                    $this->_id = $this->db->lastInsertId($this->_table, $this->_idField);
                }
            }

            // update internal id

            if ($commit) {
                if ($update) {
                    $commit = $this->postUpdate();
                }
                else {
                    $commit = $this->postInsert();
                }
            }

            if ($commit) {
                $this->db->commit();
            }
            else {
                $this->db->rollback();
            }

            $this->_locked = false;

            return $commit;
        }

        public function delete()
        {
            if (!$this->saved()) {
                return false;
            }

            if ($this->_locked) {
                throw new DatabaseObject_Exception('This record is locked');
            }

            $this->_locked = true;

            $this->db->begin();

            $commit = $this->preDelete();

            $id = $this->_id;
            if ($commit) {
                $this->db->delete($this->_table, sprintf('%s = %d', $this->_idField, $this->getId()));
                $this->_deleted_id = $this->_id;
                $this->_id = null;
            }
            else {
                $this->db->rollback();
                $this->_locked = false;
                return false;
            }

            $commit = $this->postDelete($id);

            if ($commit) {
                $this->db->commit();
            }
            else {
                $this->db->rollback();
            }

            $this->_locked = false;
            return $commit;
        }

        public function saved()
        {
            return $this->getId() > 0;
        }

        public function getId()
        {
            return (int) $this->_id;
        }

        public function getDeletedId()
        {
            return (int) $this->_deleted_id;
        }

        public function getDb()
        {
            return $this->db;
        }

        public function __set($name, $value)
        {
            if (array_key_exists($name, $this->_properties)) {
                if ($value !== $this->_properties[$name]['value']) {
                    $this->_properties[$name]['value'] = $value;
                    $this->_properties[$name]['updated'] = true;
                }
                return true;
            }

            return false;
        }

        public function set($name, $value)
        {
            return $this->__set($name, $value);
        }

        public function __get($name)
        {
            return array_key_exists($name, $this->_properties) ? $this->_properties[$name]['value'] : null;
        }

        public function get($name, $default = null)
        {
            return array_key_exists($name, $this->_properties) ? $this->_properties[$name]['value'] : $default;
        }

        public function getOriginal($name, $default = null)
        {
            return array_key_exists($name, $this->_properties) ? $this->_properties[$name]['original'] : $default;
        }

        protected function add($field, $default = null, $type = null, $saveChanges = true)
        {
            $this->_properties[$field] = array(
                'value'    => $default,
                'original' => $default,
                'type'     => in_array($type, self::$types) ? $type : null,
                'updated'  => false,
                'savechanges' => $saveChanges
            );
        }

        public function check($name, $value)
        {
            return array_key_exists($name, $this->_properties);
        }

        public function updated($name = '')
        {
            if (strlen($name) == 0) {
                foreach ($this->_properties as $k => $v) {
                    if ($v['updated'])
                        return true;
                }
                return false;
            }

            return array_key_exists($name, $this->_properties) && $this->_properties[$name]['updated'];
        }

        protected function preInsert()
        {
            return true;
        }

        protected function postInsert()
        {
            return true;
        }

        protected function preUpdate()
        {
            return true;
        }

        protected function postUpdate()
        {
            return true;
        }

        protected function preDelete()
        {
            return true;
        }

        protected function postDelete()
        {
            return true;
        }

        protected function postLoad()
        {
            return true;
        }

        public function getDate($field)
        {
            if (!array_key_exists($field, $this->_properties))
                throw new Exception(sprintf('Invalid field (%s)', $field));

            if ($this->_properties[$field]['type'] != self::TYPE_TIMESTAMP)
                throw new Exception('Not a date field');

            if (is_null($this->_properties[$field]['value']))
                return null;

            $ret = new Zend_Date($this->_properties[$field]['value'], Zend_Date::TIMESTAMP, Application::GetLocale());

            return $ret;
        }

        public static function BuildMultiple($db, $class, $data, $args = null)
        {
            $ret = array();

            if (!class_exists($class))
                throw new Exception('Undefined class specified: ' . $class);

            if (is_null($args))
                $testObj = new $class();
            else {
                if (!is_array($args))
                    $args = array($args);

                $cmd = sprintf('new %s($db, %s)', $class, join(', ', $args));

                eval('$testObj = ' . $cmd . ';');
            }

            if (!$testObj instanceof DatabaseObject)
                throw new Exception('Class does not extend from DatabaseObject');

            foreach ($data as $row) {
                if (is_null($args))
                    $obj = new $class();
                else
                    eval('$obj = ' . $cmd . ';');

                $obj->_init($row);

                $ret[$obj->getId()] = $obj;
                DatabaseObject_Registry::Register($obj);
            }

            return $ret;
        }

        public static function _setArray($v)
        {
            return is_array($v) ? serialize($v) : '';
        }

        public static function _getArray($v)
        {
            $v = unserialize($v);
            if (!is_array($v))
                $v = array();

            return $v;
        }
    }
?>