<?php
    abstract class Db_Adapter_Abstract extends Zend_Db_Adapter_Abstract
    {
        protected $_transactions = 0;

        public function getDsn()
        {
            $hostname = coalesce($this->_config['host'], 'localhost');
            $port     = coalesce($this->_config['port']);
            $database = coalesce($this->_config['dbname']);
            $username = coalesce($this->_config['username']);
            $password = coalesce($this->_config['password']);

            $dsn = sprintf(
                '%s://%s@%s%s/%s',
                $this->getType(),
                strlen($password) > 0 ? $username . ':' . $password : $username,
                $hostname,
                strlen($port) > 0 ? ':' . $port : '',
                $database
            );

            return $dsn;
        }

        abstract function getEncoding();
        abstract function isValidEncoding();

        public function isConnected()
        {
            return (bool) $this->_connection;
        }

        public function begin()
        {
            return $this->beginTransaction();
        }

        public function beginTransaction()
        {
            $this->_transactions++;

            if ($this->_transactions == 1) {
                return parent::beginTransaction();
            }

            return true;
        }

        public function commit()
        {
            if ($this->_transactions == 1) {
                $this->_transactions = 0;
                return parent::commit();
            }

            $this->_transactions = max(0, $this->_transactions - 1);
        }

        public function rollback()
        {
            if ($this->_transactions == 1) {
                $this->_transactions = 0;
                return parent::rollback();
            }

            $this->_transactions = max(0, $this->_transactions - 1);
        }

        public function getType()
        {
            $class = get_class($this);
            $class = preg_replace('/^Db_Adapter_/i', '', $class);

            return strtolower($class);
        }

        public function factory($dsn, $options = array())
        {
            $parts = parse_url($dsn);

            $adapter = strtolower(trim(coalesce($parts['scheme'])));
            if (strlen($adapter) == 0)
                throw new Db_Exception('DB adapter not specified');

            $filename = sprintf('%s/%s.php', dirname(__FILE__), $adapter);
            if (!file_exists($filename)) {
                throw new Db_Exception('Class file not found: ' . $filename);
            }

            require_once($filename);

            $class = 'Db_Adapter_' . $adapter;

            if (!class_exists($class, false))
                throw new Db_Exception(sprintf('DB adapter not found (%s)', $class));

            $_options = array(
                'dbname'   => substr(coalesce($parts['path']), 1),
                'username' => coalesce($parts['user']),
                'password' => coalesce($parts['pass']),
                'host'     => coalesce($parts['host']),
                'port'     => coalesce($parts['port'])
            );

            $options = ArrayTools::clean($options);
            foreach ($options as $k => $v) {
                $_options[$k] = $v;
            }

            return new $class($_options);
        }

        abstract public function __toString();

        public function hasTable($table)
        {
            return in_array($table, $this->listTables());
        }

        public function getConfigSingle($key)
        {
            return coalesce($this->_config[$key]);
        }

        public function getPkFromDescription($description)
        {
            $ret = array();

            foreach ($description as $name => $column) {
                if ($column['PRIMARY'])
                    $ret[] = $name;
            }

            return array_unique($ret);
        }

        public function getStatementHelper()
        {
            $className = 'Db_Statement_Helper_' . $this->getType();

            try {
                Application_Loader::AssertCanLoadClass($className);

                return new $className($this);
            }
            catch (Exception $ex) {
                return new Db_Statement_Helper_Default($this);
            }
        }

        public function getManagerHelper()
        {
            $className = 'Db_Statement_Manager_' . $this->getType();

            try {
                Application_Loader::AssertCanLoadClass($className);

                return new $className($this);
            }
            catch (Exception $ex) {
                return new Db_Statement_Manager_Default($this);
            }
        }
    }
?>
