<?php

function mysqli_result($resource, $row, $field = 0) {
    $resource->data_seek($row);
    $data = $resource->fetch_array();
    return $data[$field];
}

class Db_Adapter_mysqli extends Db_Adapter_Abstract {

    public function __toString() {
        return sprintf('MySQL %s', mysqli_get_server_info($this->_connection));
    }

    public function getServerVersion() {
        return $this->fetchOne('select version()');
    }

    public function getQuoteIdentifierSymbol() {
        return '`';
    }

    protected function _connect() {
        if ($this->_connection) {
            return;
        }

        $persistent = coalesce($this->_config['persistent']);
        $host = ($persistent ? 'p:' : '') . coalesce($this->_config['host'], 'localhost');

        $this->_connection = @mysqli_connect(
            $host,
            coalesce($this->_config['username']),
            coalesce($this->_config['password'])
        );

        if ($this->_connection === false) {
            $msg = trim(mysqli_error());
            $str = 'Unable to connect to database server';

            if (strlen($msg) > 0) {
                $str .= ': ' . $msg;
            }

            throw new Zend_Db_Adapter_Exception($str);
        }

        if (!mysqli_select_db($this->_connection, coalesce($this->_config['dbname']))) {
            $msg = trim(mysqli_error());
            $str = 'Unable to connect to database ' . $this->_config['dbname'];

            if (strlen($msg) > 0) {
                $str .= ': ' . $msg;
            }

            throw new Zend_Db_Adapter_Exception($str);
        }

        //mysqli_set_charset('utf8', $this->_connection);
    }

    public function listTables() {
        return $this->fetchCol('SHOW TABLES');
    }

    /**
     * @see Zend_Db_Adapter_Pdo_Mysql::describeTable()
     */
    public function describeTable($tableName, $schemaName = null) {
        // @todo: use INFORMATION_SCHEMA someday when MySQL's
        // implementation has reasonably good performance and
        // the version with this improvement is in wide use.

        if ($schemaName) {
            $sql = 'DESCRIBE ' . $this->quoteIdentifier("$schemaName.$tableName", true);
        } else {
            $sql = 'DESCRIBE ' . $this->quoteIdentifier($tableName, true);
        }
        $stmt = $this->query($sql);

        // Use FETCH_NUM so we are not dependent on the CASE attribute of the PDO connection
        $result = $stmt->fetchAll(Zend_Db::FETCH_NUM);

        $field   = 0;
        $type    = 1;
        $null    = 2;
        $key     = 3;
        $default = 4;
        $extra   = 5;

        $desc = array();
        $i = 1;
        $p = 1;
        foreach ($result as $row) {
            list($length, $scale, $precision, $unsigned, $primary, $primaryPosition, $identity)
                = array(null, null, null, null, false, null, false);
            if (preg_match('/unsigned/', $row[$type])) {
                $unsigned = true;
            }
            if (preg_match('/^((?:var)?char)\((\d+)\)/', $row[$type], $matches)) {
                $row[$type] = $matches[1];
                $length = $matches[2];
            } else if (preg_match('/^decimal\((\d+),(\d+)\)/', $row[$type], $matches)) {
                $row[$type] = 'decimal';
                $precision = $matches[1];
                $scale = $matches[2];
            } else if (preg_match('/^((?:big|medium|small|tiny)?int)\((\d+)\)/', $row[$type], $matches)) {
                $row[$type] = $matches[1];
                // The optional argument of a MySQL int type is not precision
                // or length; it is only a hint for display width.
            }
            if (strtoupper($row[$key]) == 'PRI') {
                $primary = true;
                $primaryPosition = $p;
                if ($row[$extra] == 'auto_increment') {
                    $identity = true;
                } else {
                    $identity = false;
                }
                ++$p;
            }
            $desc[$this->foldCase($row[$field])] = array(
                'SCHEMA_NAME'      => null, // @todo
                'TABLE_NAME'       => $this->foldCase($tableName),
                'COLUMN_NAME'      => $this->foldCase($row[$field]),
                'COLUMN_POSITION'  => $i,
                'DATA_TYPE'        => $row[$type],
                'DEFAULT'          => $row[$default],
                'NULLABLE'         => (bool) ($row[$null] == 'YES'),
                'LENGTH'           => $length,
                'SCALE'            => $scale,
                'PRECISION'        => $precision,
                'UNSIGNED'         => $unsigned,
                'PRIMARY'          => $primary,
                'PRIMARY_POSITION' => $primaryPosition,
                'IDENTITY'         => $identity
            );
            ++$i;
        }
        return $desc;
    }

    public function compareColumnTypes($a, $b) {
        $map = array(
            'serial'      => 'bigint',
            'timestamptz' => 'datetime',
            'boolean'     => 'tinyint',
            'real'        => 'double'
        );

        if (array_key_exists($a, $map))
            $a = $map[$a];

        if (array_key_exists($b, $map))
            $b = $map[$b];

        return parent::compareColumnTypes($a, $b);
    }

    public function closeConnection() {
        mysqli_close($this->_connection);
        $this->_connection = null;
    }

    public function prepare($sql) {
        $this->_connect();
        $stmt = new Db_Statement_mysql($this, $sql);
        $stmt->setFetchMode($this->_fetchMode);
        return $stmt;
    }

    public function lastInsertId($tableName = null, $primaryKey = null) {
        $result = mysqli_query($this->_connection, 'select last_insert_id()');
        return (int) mysqli_result($result, 0, 0);
    }

    protected function _beginTransaction() {
        $this->query('begin');
    }

    protected function _commit() {
        $this->query('commit');
    }

    protected function _rollBack() {
        $this->query('rollback');
    }

    public function setFetchMode($mode) {

    }

    public function limit($sql, $count, $offset = 0) {
        return sprintf('%s limit %d, %d', $sql, $offset, $count);
    }

    public function supportsParameters($type) {

    }

    /**
     * Copied from the Zend_Db_Adapter_Abstract class, replaces the placeholder
     */
    public function insert($table, array $bind) {
        // extract and quote col names from the array keys
        $cols = array();
        $vals = array();

        $i = 0;
        foreach ($bind as $col => $val) {
            $cols[] = $this->quoteIdentifier($col, true);
            if ($val instanceof Zend_Db_Expr)
                $val = (string) $val;
            else if (is_null($val))
                $val = 'NULL';
            else if (is_bool($val))
                $val = $val ? 'TRUE' : 'FALSE';
            else
                $val = $this->quote($val);

            $vals[] = $val;
        }

        // build the statement
        $sql = "INSERT INTO "
            . $this->quoteIdentifier($table, true)
            . ' (' . implode(', ', $cols) . ') '
            . 'VALUES (' . implode(', ', $vals) . ')';

        // execute the statement and return the number of affected rows
        $stmt = $this->query($sql, array());
        $result = $stmt->rowCount();
        return $result;
    }

    /**
     * Copied from the Zend_Db_Adapter_Abstract class, replaces the placeholder
     */
    public function update($table, array $bind, $where = '') {
        /**
         * Build "col = ?" pairs for the statement,
         * except for Zend_Db_Expr which is treated literally.
         */
        $i = 0;
        $set = array();
        foreach ($bind as $col => $val) {
            if ($val instanceof Zend_Db_Expr)
                $val = $val->__toString();
            else
                $val = $this->quote($val);

            $set[] = $this->quoteIdentifier($col, true) . ' = ' . $val;
        }

        $where = $this->_whereExpr($where);

        /**
         * Build the UPDATE statement
         */
        $sql = "UPDATE "
            . $this->quoteIdentifier($table, true)
            . ' SET ' . implode(', ', $set)
            . (($where) ? " WHERE $where" : '');

        /**
         * Execute the statement and return the number of affected rows
         */

        $stmt = $this->query($sql, array());

        $result = $stmt->rowCount();
        return $result;
    }

    public function getEncoding() {
        return $this->_connection->get_charset()->charset;
        //return mysqli_client_encoding($this->_connection);
    }

    public function isValidEncoding() {
        return true;
        return strtoupper(preg_replace('/[^a-z0-9]/', '', $this->getEncoding())) == 'UTF8';
    }
}
