<?php
    class Db_Adapter_postgresql extends Db_Adapter_Abstract
    {
        public function __toString()
        {
            return trim('PostgreSQL ' . $this->getServerVersion());
        }

        public function getServerVersion()
        {
            if (function_exists('pg_version')) {
                $info = pg_version($this->_connection);
                return $info['server'];
            }

            return '';
        }

        protected function _connect()
        {
            if ($this->_connection)
                return;

            $parts = array();

            $options = array(
                'host'     => 'host',
                'port'     => 'port',
                'dbname'   => 'dbname',
                'user'     => 'username',
                'password' => 'password'
            );

            foreach ($options as $k => $v) {
                $val = coalesce($this->_config[$v]);

                if (strlen($val) > 0) {
                    $parts[] = sprintf('%s=%s', $k, $val);
                }
            }

            $connectionStr = join(' ', $parts);

            $persistent = coalesce($this->_config['persistent']);
            $func = $persistent ? 'pg_pconnect' : 'pg_connect';

            $this->_connection = @$func($connectionStr);

            if ($this->_connection === false) {
                throw new Zend_Db_Adapter_Exception('Unable to connect to database');
            }
        }

        public function getEncoding()
        {
            return pg_parameter_status($this->_connection, 'server_encoding');
        }

        public function isValidEncoding()
        {
            return $this->getEncoding() == 'UTF8';
        }

        public function listTables()
        {
            $query = "select c.relname from pg_catalog.pg_class c left join pg_catalog.pg_namespace n on n.oid = c.relnamespace where c.relkind = 'r' and n.nspname not in ('pg_catalog', 'pg_toast') and pg_catalog.pg_table_is_visible(c.oid)";
            return $this->fetchCol($query);
        }


        /**
         * @see Zend_Db_Adapter_Pdo_Pgsql::describeTable()
         */
        public function describeTable($tableName, $schemaName = null)
        {
            $sql = "SELECT
                    a.attnum,
                    n.nspname,
                    c.relname,
                    a.attname AS colname,
                    t.typname AS type,
                    a.atttypmod,
                    FORMAT_TYPE(a.atttypid, a.atttypmod) AS complete_type,
                    d.adsrc AS default_value,
                    a.attnotnull AS notnull,
                    a.attlen AS length,
                    co.contype,
                    ARRAY_TO_STRING(co.conkey, ',') AS conkey
                FROM pg_attribute AS a
                    JOIN pg_class AS c ON a.attrelid = c.oid
                    JOIN pg_namespace AS n ON c.relnamespace = n.oid
                    JOIN pg_type AS t ON a.atttypid = t.oid
                    LEFT OUTER JOIN pg_constraint AS co ON (co.conrelid = c.oid
                        AND a.attnum = ANY(co.conkey) AND co.contype = 'p')
                    LEFT OUTER JOIN pg_attrdef AS d ON d.adrelid = c.oid AND d.adnum = a.attnum
                WHERE a.attnum > 0 AND c.relname = ".$this->quote($tableName);
            if ($schemaName) {
                $sql .= " AND n.nspname = ".$this->quote($schemaName);
            }
            $sql .= ' ORDER BY a.attnum';

            $stmt = $this->query($sql);

            // Use FETCH_NUM so we are not dependent on the CASE attribute of the PDO connection
            $result = $stmt->fetchAll(Zend_Db::FETCH_NUM);

            $attnum        = 0;
            $nspname       = 1;
            $relname       = 2;
            $colname       = 3;
            $type          = 4;
            $atttypemod    = 5;
            $complete_type = 6;
            $default_value = 7;
            $notnull       = 8;
            $length        = 9;
            $contype       = 10;
            $conkey        = 11;

            $desc = array();
            foreach ($result as $key => $row) {
                if ($row[$type] == 'varchar') {
                    if (preg_match('/character varying(?:\((\d+)\))?/', $row[$complete_type], $matches)) {
                        if (isset($matches[1])) {
                            $row[$length] = $matches[1];
                        } else {
                            $row[$length] = null; // unlimited
                        }
                    }
                }
                list($primary, $primaryPosition, $identity) = array(false, null, false);
                if ($row[$contype] == 'p') {
                    $primary = true;
                    $primaryPosition = array_search($row[$attnum], explode(',', $row[$conkey])) + 1;
                    $identity = (bool) (preg_match('/^nextval/', $row[$default_value]));
                }
                $desc[$this->foldCase($row[$colname])] = array(
                    'SCHEMA_NAME'      => $this->foldCase($row[$nspname]),
                    'TABLE_NAME'       => $this->foldCase($row[$relname]),
                    'COLUMN_NAME'      => $this->foldCase($row[$colname]),
                    'COLUMN_POSITION'  => $row[$attnum],
                    'DATA_TYPE'        => $row[$type],
                    'DEFAULT'          => $row[$default_value],
                    'NULLABLE'         => (bool) ($row[$notnull] != 't'),
                    'LENGTH'           => $row[$length],
                    'SCALE'            => null, // @todo
                    'PRECISION'        => null, // @todo
                    'UNSIGNED'         => null, // @todo
                    'PRIMARY'          => $primary,
                    'PRIMARY_POSITION' => $primaryPosition,
                    'IDENTITY'         => $identity
                );
            }
            return $desc;
        }

        public function compareColumnTypes($a, $b)
        {
            $map = array(
                'serial' => 'int',
                'int4'   => 'int',
                'float4' => 'real',
                'float8' => 'double precision'
            );

            if (array_key_exists($a, $map))
                $a = $map[$a];

            if (array_key_exists($b, $map))
                $b = $map[$b];

            return parent::compareColumnTypes($a, $b);
        }

        public function closeConnection()
        {
            pg_close($this->_connection);
            $this->_connection = null;
        }

        public function prepare($sql)
        {
            $this->_connect();
            $stmt = new Db_Statement_postgresql($this, $sql);
            $stmt->setFetchMode($this->_fetchMode);
            return $stmt;
        }

        protected function returnsOid()
        {
            return version_compare($this->getServerVersion(), '8.1') < 0;
        }

        public function lastInsertId($tableName = null, $primaryKey = null)
        {
            $tableName = trim($tableName);

            if (strlen($tableName) == 0)
                throw new Db_Exception('Table name not specified');

            if (strlen($primaryKey) == 0)
                throw new Db_Exception('Primary key column not specified');

            if (version_compare($this->getServerVersion(), '8.0') < 0)
                $sequence = sprintf('%s_%s_seq', $tableName, $primaryKey);
            else {
                $query = sprintf("select pg_get_serial_sequence(%s, %s)", $this->quote($tableName), $this->quote($primaryKey));
                $sequence = $this->fetchOne($query);

                if (strlen($sequence) == 0) {
                    throw new Db_Exception('Sequence not found');
                }
            }

            $query = sprintf("select currval(%s)", $this->quote($sequence));

            return (int) $this->fetchOne($query);
        }

        protected function _beginTransaction()
        {
            @pg_query($this->_connection, 'begin');
        }

        protected function _commit()
        {
            @pg_query($this->_connection, 'commit');
        }

        protected function _rollBack()
        {
            @pg_query($this->_connection, 'rollback');
        }

        public function setFetchMode($mode)
        {

        }

        public function limit($sql, $count, $offset = 0)
        {
            return sprintf('%s offset %d limit %d', $sql, $offset, $count);
        }

        public function supportsParameters($type)
        {

        }

        /**
         * Copied from the Zend_Db_Adapter_Abstract class, replaces the placeholder
         */
        public function insert($table, array $bind)
        {
            // extract and quote col names from the array keys
            $cols = array();
            $vals = array();

            $i = 0;
            foreach ($bind as $col => $val) {
                $cols[] = $this->quoteIdentifier($col, true);
                if ($val instanceof Zend_Db_Expr) {
                    $vals[] = $val->__toString();
                    unset($bind[$col]);
                } else {
                    $vals[] = '$' . ++$i;
                }
            }

            // build the statement
            $sql = "INSERT INTO "
                 . $this->quoteIdentifier($table, true)
                 . ' (' . implode(', ', $cols) . ') '
                 . 'VALUES (' . implode(', ', $vals) . ')';

            // execute the statement and return the number of affected rows
            $stmt = $this->query($sql, array_values($bind));
            $result = $stmt->rowCount();
            return $result;
        }

        /**
         * Copied from the Zend_Db_Adapter_Abstract class, replaces the placeholder
         */
        public function update($table, array $bind, $where = '')
        {
            /**
             * Build "col = ?" pairs for the statement,
             * except for Zend_Db_Expr which is treated literally.
             */
            $i = 0;
            $set = array();
            foreach ($bind as $col => $val) {
                if ($val instanceof Zend_Db_Expr) {
                    $val = $val->__toString();
                    unset($bind[$col]);
                } else {
                    $val = '$' . ++$i;
                }
                $set[] = $this->quoteIdentifier($col, true) . ' = ' . $val;
            }

            $where = $this->_whereExpr($where);

            /**
             * Build the UPDATE statement
             */
            $sql = "UPDATE "
                 . $this->quoteIdentifier($table, true)
                 . ' SET ' . implode(', ', $set)
                 . (($where) ? " WHERE $where" : '');

            /**
             * Execute the statement and return the number of affected rows
             */
            $stmt = $this->query($sql, array_values($bind));
            $result = $stmt->rowCount();
            return $result;
        }

        public function getTablePkSequence($table)
        {
            $query = sprintf("select c.conname from pg_class r, pg_constraint c where r.oid = c.conrelid and contype = 'p' and r.relname = %s", $this->quote($table));
            $sequence = $this->fetchOne($query);

            if (!$sequence)
                throw new Db_Exception('Unable to find primary key sequence');

            return $sequence;
        }
    }
?>
