<?php
    /**
     * Db_Schema_Table
     *
     * Used to create database tables
     *
     * @package     Db
     */
    abstract class Db_Schema_Table_Abstract
    {
        private $_fields = array();
        private $_pks = array();
        private $_fks = array();
        private $_unique = array();
        private $_indexes = array();

        private $_differences;
        private $_unsupported = array();

        protected $_db;
        protected $_test = false; // this will disable installation of keys and indexes

        public function __construct($db, $testMode = false)
        {
            $this->_db = $db;
            $this->_test = (bool) $testMode;

            $this->init();
        }

        /**
         * getName
         *
         * Returns the name of the database table. This doesn't include
         * the database table prefix.
         *
         * @return  string  The name of the database table
         */
        abstract public static function getName();


        /**
         * init
         *
         * Initialize the fields for the table
         */
        abstract protected function init();


        /**
         * exists
         *
         * Check whether or not the table exists
         *
         * @return  bool    True if the table exists
         */
        public function exists()
        {
            return $this->_db->hasTable($this->getName());
        }


        /**
         * isCorrect
         *
         * Check whether or not the table in the database is the same as
         * the schema specified in this object
         *
         * @return  bool    True if table is correct, false if different
         */
        public function isCorrect()
        {
            $diff = $this->getDifferences();

            foreach ($diff as $v) {
                if (count($v) > 0)
                    return false;
            }

            return true;
        }

        public function getUnsupported()
        {
            return array_unique($this->_unsupported);
        }


        /**
         * getDifferences
         *
         * Determine the differences between the table in the database and
         * the table specified by this class
         *
         * @todo    Check foreign keys
         * @return  array       The differences:
                    + missing       Fields not present
                    + incorrect     Fields with incorrect definition
                    + extra         Fields present that shouldn't be
                    + indexes       Indexes not present
                    + unsupported   Operations not supported by this database type
         */
        public function getDifferences()
        {
            if (is_null($this->_differences)) {
                $this->_unsupported = array();

                $arr = array(
                    'missing'     => array(),
                    'incorrect'   => array(),
                    'extra'       => array(),
                    'pk'          => array(),
                    'unsupported' => array()
                );

                $table = $this->_db->describeTable($this->getName());

                // check field definitions
                foreach ($this->_fields as $name => $field) {
                    if (!array_key_exists($name, $table)) {
                        $arr['missing'][$name] = $name;
                        continue;
                    }

                    try {
                        $field->assertIsCorrect($this->_db, $table[$name]);
                    }
                    catch (Exception $ex) {
                        $arr['incorrect'][$name] = $ex->getMessage();
                    }
                }

                // check for extra columns
                foreach ($table as $name => $column) {
                    if (!array_key_exists($name, $this->_fields)) {
                        $arr['extra'][$name] = $name;
                    }
                }

                // check primary key is correct
                $pk = $this->_db->getPkFromDescription($table);

                if (count(array_diff($this->_pks, $pk)) > 0 || count(array_diff($pk, $this->_pks)) > 0) {
                    $arr['pk'] = $this->_pks;
                }

                // check for missing indexes
                try {
                    $helper  = $this->_db->getManagerHelper();
                    $indexes = $helper->tableGetIndexes($this->getName());

                    foreach ($this->_indexes as $col) {
                        $found = false;

                        foreach ($indexes as $index) {
                            if (count($index['columns']) == 1 && $index['columns'][0] == $col) {
                                $found = true;
                                break;
                            }
                        }

                        if (!$found && !array_key_exists($col, $arr['missing'])) {
                            $arr['indexes'][$col] = $col;
                        }
                    }
                }
                catch (Db_Statement_Exception_NotSupported $ex) {
                    $this->_unsupported[] = 'Checking for missing indexes';
                }

                $this->_differences = $arr;
            }

            return $this->_differences;
        }


        /**
         * getDependencies
         *
         * Retrieve a list of all the tables this table relies on
         *
         * @return  array
         */
        public function getDependencies()
        {
            $ret = array();

            foreach ($this->_fks as $fk) {
                $table = (string) $fk['table'];
                $ret[$table] = $table;
            }

            return array_values($ret);
        }

        /**
         * create
         *
         * Create the table in the database
         *
         * @throws  Exception   If error creating table (not for missing dependency)
         * @return  bool        True if created, false if missing dependency
         */
        public function create()
        {
            foreach ($this->getDependencies() as $table) {
                if ($table != $this->getName() && !$this->_db->hasTable($table)) {
                    return false;
                }
            }

            if ($this->_test) {
                return true;
            }

            if ($this->_db->hasTable($this->getName())) {
                throw new Db_Exception('Table already ' . $this->getName() . ' exists');
            }

            $manager = $this->_db->getManagerHelper();
            $manager->tableCreate($this);

            return true;
        }



        /**
         * createField
         *
         * Create a single field (column) in the table
         *
         * @todo                    Create keys and indexes correctly
         * @throws  Exception       If field not valid; if field already exists; if field can't be created
         * @param   string  $name   The field to create
         */
        public function createField($name)
        {
            if (!array_key_exists($name, $this->_fields)) {
                throw new Exception(sprintf('Field not valid (%s)', $name));
            }

            $diff = $this->getDifferences();

            if (!array_key_exists($name, $diff['missing'])) {
                throw new Exception(sprintf('Field already exists (%s)', $name));
            }

            $field = $this->_fields[$name];
            $field->create($this);

            if (array_key_exists($name, $this->_indexes)) {
                $this->createIndex($name);
            }
        }

        public function dropField($name)
        {
            $diff = $this->getDifferences();

            if (!array_key_exists($name, $diff['extra'])) {
                throw new Exception(sprintf('Not allowed to drop field (%s)', $name));
            }

            $manager = $this->_db->getManagerHelper();
            $manager->fieldDrop($this, $name);
        }

        public function repairField($name)
        {
            $table = $this->_db->describeTable($this->getName());

            if (!array_key_exists($name, $table)) {
                throw new Exception(sprintf('This field could not be found (%s)', $name));
            }

            $field       = $this->getField($name);
            $description = $table[$name];
            $manager     = $this->_db->getManagerHelper();

            // now try and repair the field
            if ($field->getNull() && !$description['NULLABLE']) {
                $manager->fieldMakeNullable($this, $field);
            }
            else if (!$field->getNull() && $description['NULLABLE']) {
                $manager->fieldMakeNotNullable($this, $field);
            }
/*
   @todo Add ability to change field type
   @todo Add ability to change default value

            else if ($description['DATA_TYPE'] != $field->getType()) {
                // @todo this only checks for different types, not for different lengths or precisions

                $queries = array();

                // @todo make this work for MySQL also
                if ($this->_db->getServerVersion() < 8) {
                    $tmp = 'tempCol';

                    $queries[] = sprintf('ALTER TABLE %s ADD %s %s', $this, $tmp, $field->getCreateDefinition(true, false));
                    $queries[] = sprintf('UPDATE %s SET %s = %s', $this, $tmp, $name);
                    $queries[] = sprintf('ALTER TABLE %s DROP %s', $this, $name);
                    $queries[] = sprintf('ALTER TABLE %s RENAME %s to %s', $this, $tmp, $name);
                }
                else {
                    $queries[] = sprintf('ALTER TABLE %s ALTER %s TYPE %s', $this, $name, $field->getCreateDefinition(true, false));
                }

                foreach ($queries as $query) {
                    $this->_db->query($query);
                }
            }
*/
        }

        /**
         * field
         *
         * Add a field to the table
         */
        protected function field($field, $type = null, $precision = null, $null = true, $default = null)
        {
            if (!$field instanceof Db_Schema_Table_Field) {
                $field = new Db_Schema_Table_Field($this->_db, $field);
                $field->setType($type)
                      ->setPrecision($precision)
                      ->setNull($null)
                      ->setDefault($default);
            }

            if (array_key_exists($field->getName(), $this->_fields))
                return;

            $this->_fields[$field->getName()] = $field;

            return $field;
        }

        public function getField($field)
        {
            if (!array_key_exists($field, $this->_fields))
                throw new Exception(sprintf('Unknown field %s in %s', $field, $this->getName()));

            return $this->_fields[$field];
        }

        public function getFields()
        {
            return $this->_fields;
        }


        /**
         * fk
         *
         * Add a foreign key to the table
         */
        protected function fk($fields, $table, $tableFields)
        {
            if ($this->_test)
                return;

            if (!is_array($fields)) {
                $fields = array($fields);
            }

            foreach ($fields as $field) {
                if (!array_key_exists($field, $this->_fields)) {
                    throw new Exception(sprintf('Unknown table field %s called from %s', $field, $this->getName()));
                }
            }

            if (!is_array($tableFields))
                $tableFields = array($tableFields);

            if (count($fields) != count($tableFields))
                throw new Exception('Mismatched foreign key definition');

            if (!$table instanceof self) {
                if (!class_exists($table, false)) {
                    if (!Application_Loader::getAutoloader()->autoload($table)) {
                        throw new Exception(sprintf('Unknown table class specified (%s) in %s', $table, get_class($this)));
                    }
                }

                $table = new $table($this->_db, true);
            }

            $fields = array_values($fields);
            $tableFields = array_values($tableFields);

            foreach ($fields as $k => $field) {

                $this->_fields[$field]->setFk();

                if ($this->_fields[$field]->getType() != Db_Schema_Table_Field::TYPE_FK)
                    continue;

                $this->_fields[$field]->setType($table->getField($tableFields[$k])->getType());
            }

            $this->_fks[] = array(
                'fields'      => $fields,
                'table'       => $table->getName(),
                'tableFields' => $tableFields
            );
        }

        public function rebuildPk()
        {
            $manager = $this->_db->getManagerHelper();
            $manager->tablePkRebuild($this);
        }

        public function createIndex($column)
        {
            $manager = $this->_db->getManagerHelper();
            $manager->indexCreate($this, $column);
        }

        public function getFks()
        {
            return $this->_fks;
        }


        /**
         * pk
         *
         * Set the primary key for the table
         */
        protected function pk()
        {
            if ($this->_test) {
                return;
            }

            $parts = func_get_args();

            $pks = array();
            foreach ($parts as $field) {
                if (is_array($field)) {
                    foreach ($field as $_field)
                        $pks[$_field] = $_field;
                }
                else
                    $pks[$field] = $field;
            }

            foreach ($pks as $pk) {
                if (!array_key_exists($pk, $this->_fields))
                    continue;

                $this->_pks[$pk] = $pk;
                $this->_fields[$pk]->setPk();
            }
        }

        public function getPks()
        {
            return $this->_pks;
        }


        /**
         * unique
         *
         * Set the primary key for the table
         */
        protected function unique()
        {
            if ($this->_test) {
                return;
            }

            $parts = func_get_args();

            $pks = array();
            foreach ($parts as $field) {
                if (is_array($field)) {
                    foreach ($field as $_field)
                        $pks[$_field] = $_field;
                }
                else
                    $pks[$field] = $field;
            }

            foreach ($pks as $pk) {
                if (!array_key_exists($pk, $this->_fields))
                    continue;

                $this->_unique[$pk] = $pk;
            }
        }

        public function getUniques()
        {
            return $this->_unique;
        }


        /**
         * index
         *
         * Create an index on a table
         */
        protected function index($field)
        {
            if ($this->_test) {
                return;
            }

            if (!array_key_exists($field, $this->_fields)) {
                return;
            }

            $this->_indexes[$field] = $field;
        }

        public function getIndexes()
        {
            return $this->_indexes;
        }

        public function __toString()
        {
            return $this->getName();
        }

        public function getDb()
        {
            return $this->_db;
        }

        public function ignore()
        {
            return false;
        }
    }
?>