<?php
    abstract class Db_Statement_Manager_Abstract implements Db_Statement_Manager_Interface
    {
        protected $_db;

        public function __construct(Db_Adapter_Abstract $db)
        {
            $this->_db = $db;
        }

        public function tableCreate(Db_Schema_Table_Abstract $table)
        {
            $query = $this->getTableCreateSql($table);

            if (!is_array($query)) {
                $query = array($query);
            }

            foreach ($query as $q) {
                $result = $this->_db->query($q);

                if ($result->isError()) {
                    $err = $result->errorInfo();
                    throw new Exception(sprintf('Unable to create table %s (%s)', $table->getName(), $err[1]));
                }
            }

            return true;
        }

        public function getTableCreateSql(Db_Schema_Table_Abstract $table)
        {
            $lines = array();

            foreach ($table->getFields() as $field) {
                $lines[] = $this->getFieldCreateSql($field);
            }

            $lines = array_merge($lines, $this->getTableCreatePksSql($table));
            $lines = array_merge($lines, $this->getTableCreateUniquesSql($table));

            $selfFk = array();
            $fks    = $table->getFks();
            foreach ($fks as $fk) {
                if ($fk['table'] == $table->getName()) {
                    $selfFk[] = $fk;
                    continue;
                }

                $lines[] = sprintf(
                    'FOREIGN KEY (%s) REFERENCES %s (%s)',
                    join(', ', $fk['fields']),
                    $fk['table'],
                    join(', ', $fk['tableFields'])
                );
            }

            $ret = array(
                sprintf("CREATE TABLE %s (\n%s\n)", $table->getName(), join(",\n", $lines))
            );

            $ret = array_merge($ret, $this->getTableCreateIndexesSql($table));

            foreach ($selfFk as $fk) {
                $ret[] = sprintf(
                    'ALTER TABLE %s ADD FOREIGN KEY (%s) REFERENCES %s (%s)',
                    $table->getName(),
                    join(', ', $fk['fields']),
                    $table->getName(),
                    join(', ', $fk['tableFields'])
                );
            }

            return $ret;
        }

        public function getTableCreatePksSql(Db_Schema_Table_Abstract $table)
        {
            $ret = array();

            $items = array();

            foreach ($table->getPks() as $fieldName) {
                $items[] = $this->getIndexableField($table, $fieldName);
            }

            if (count($items) > 0) {
                $ret[] = sprintf('PRIMARY KEY (%s)', join(', ', $items));
            }

            return $ret;
        }

        public function getTableCreateUniquesSql(Db_Schema_Table_Abstract $table)
        {
            $items = array();

            foreach ($table->getUniques() as $fieldName) {
                $items[] = $this->getIndexableField($table, $fieldName);
            }

            $ret = array();

            if (count($items) > 0) {
                $ret[] = sprintf('UNIQUE (%s)', join(', ', $items));
            }

            return $ret;
        }

        public function getTableCreateIndexesSql(Db_Schema_Table_Abstract $table)
        {
            $ret = array();

            foreach ($table->getIndexes() as $index) {
                $field = $this->getIndexableField($table, $index);
                $ret[] = sprintf(
                    "CREATE INDEX %s_%s on %s (%s)", $table->getName(), $index, $table->getName(), $field
                );
            }

            return $ret;
        }

        public function getIndexableField(Db_Schema_Table_Abstract $table, $fieldName)
        {
            return $fieldName;
        }

        /**
         * Get the SQL used to create a field
         *
         * @param   Db_Schema_Table_Field   $field
         * @param   bool    $single         True only if the column is being created/updated by itself (i.e. not in a create table statement)
         * @param   bool    $includeName    True if the column name should be included in the statement
         */
        public function getFieldCreateSql(Db_Schema_Table_Field $field, $single = false, $includeName = true)
        {
            $parts = array();

            if ($includeName) {
                $parts[] = $field->getName();
            }

            $fieldName = $this->getFieldTypeName($field->getType(), $field->isPk(), $field->isFk());

            try {
                $precision = $field->getPrecision();
                $parts[] = sprintf('%s(%s)', $fieldName, $precision);
            }
            catch (Exception $ex) {
                $parts[] = $fieldName;
            }

            if (!$field->getNull() && !$single) {
                $parts[] = 'NOT NULL';
            }

            try {
                if (!$single) {
                    $fieldDefault = $field->getDefault();
                    if ($field->getType() != Db_Schema_Table_Field::TYPE_INT) {
                        $fieldDefault = $this->_db->quote($fieldDefault);
                    }
                    $parts[] = 'DEFAULT '.$fieldDefault;
                }
            }
            catch (Exception $ex) {
            }

            return join(' ', $parts);
        }

        public function getFieldTypeName($type, $pk = false, $fk = false)
        {
            return $type;
        }

        public function getFieldTypeNameForComparison($type, $pk = false, $fk = false)
        {
            return $type;
        }

        public function assertFieldIsCorrect(Db_Schema_Table_Field $field, $description)
        {
            if ($field->getNull() != $description['NULLABLE']) {
                throw new Exception($field->getNull() ? 'Expected field to be nullable' : 'Expected field to be not null');
            }

            $typeA = $this->getFieldTypeNameForComparison($field->getType(), $field->isPk(), $field->isFk());

            $typeB = $description['DATA_TYPE'];

            if (strtolower($typeA) == 'boolean') {
                $typeA = 'bool';
            }

            if (strtolower($typeB) == 'boolean') {
                $typeB = 'bool';
            }

            if (preg_match('/unsigned/i', $typeA)) {
                if ($description['UNSIGNED'] === false) {
                    throw new Exception('Field in database should be signed');
                }

                $typeA = trim(preg_replace('/unsigned/i', '', $typeA));
            }
            else {
                if ($description['UNSIGNED'] === true) {
                    throw new Exception('Field in database should be unsigned');
                }
            }

            if (strtolower($typeA) != strtolower($typeB)) {
                throw new Exception(sprintf('Type mismatch (%s != %s)', $typeA, $typeB));
            }
/*
    this is disabled since varchar length in MySQL is returned in 'length', not in precision, so we need to change how this stuff works
            try {
                $precision = $field->getPrecision();
            }
            catch (Exception $precision) {
                $precision = null;
            }

            if ($precision != $description['precision']) {
                throw new Exception('Precision mismatch');
            }
*/
            try {
                $default = (string) $field->getDefault();
            }
            catch (Exception $ex) {
                $default = '';
            }

            if ($default != $description['DEFAULT'] && !preg_match('~^nextval~', $description['DEFAULT'])) {
                throw new Exception(sprintf('Default value mismatch (%s != %s)', $default, $description['DEFAULT']));
            }
        }

        public function fieldCreate(Db_Schema_Table_Abstract $table, Db_Schema_Table_Field $field)
        {
            $this->_db->begin();

            $query = sprintf(
                'alter table %s add %s',
                $this->_db->quoteIdentifier($table),
                $this->getFieldCreateSql($field, true)
            );

            $this->_db->query($query);

            try {
                $default = $this->getFieldCreateDefaultValue($field);

                $values = array(
                    $field->getName() => $default
                );

                $where = array(
                    $this->_db->quoteIdentifier($field->getName()) . ' is null'
                );

                $this->_db->update($table, $values, $where);

                if (!$field->getNull()) {
                    $query = sprintf(
                        'alter table %s alter %s set not null',
                        $this->_db->quoteIdentifier($table),
                        $field
                    );

                    $this->_db->query($query);
                }
            }
            catch (Exception $ex) {
            }

            $this->_db->commit();
        }

        public function fieldDrop($table, $name)
        {
            $query = sprintf(
                'alter table %s drop %s',
                $this->_db->quoteIdentifier($table),
                $this->_db->quoteIdentifier($name)
            );

            $this->_db->query($query);
        }

        public function fieldMakeNotNullable($table, Db_Schema_Table_Field $field)
        {
            $default = $this->getFieldCreateDefaultValue($field, true);

            $values = array(
                $field->getName() => $default
            );

            $where = array(
                $this->_db->quoteIdentifier($field->getName()) . ' is null'
            );

            $this->_db->update($table, $values, $where);

            $query = sprintf(
                'ALTER TABLE %s ALTER %s SET NOT NULL',
                $this->_db->quoteIdentifier($table),
                $this->_db->quoteIdentifier($field)
            );

            $this->_db->query($query);
        }

        public function fieldMakeNullable($table, Db_Schema_Table_Field $field)
        {
            $query = sprintf(
                'ALTER TABLE %s ALTER %s DROP NOT NULL',
                $this->_db->quoteIdentifier($table),
                $this->_db->quoteIdentifier($field)
            );

            $this->_db->query($query);
        }

        public function getFieldCreateDefaultValue(Db_Schema_Table_Field $field, $force = false)
        {
            try {
                $default = $field->getDefault();
            }
            catch (Exception $ex) {
                if ($field->getNull() && !$force) {
                    throw new Exception('No default required');
                }

                $default = null;
            }

            switch ($field->getType()) {
                case Db_Schema_Table_Field::TYPE_TIMESTAMPTZ:
                    $default = new Zend_Db_Expr('now()');
                    break;

                case Db_Schema_Table_Field::TYPE_BOOL:
                    $default = new Zend_Db_Expr($default ? 'true' : 'false');
                    break;

                case Db_Schema_Table_Field::TYPE_INT:
                    $value = (int) $default;
                    break;

                default:
                    $value = (string) $default;
            }

            return $value;
        }

        public function tablePkRebuild(Db_Schema_Table_Abstract $table)
        {
            $this->_db->begin();
            $this->tablePkDrop($table);
            $this->tablePkCreate($table);
            $this->_db->commit();
        }

        public function tablePkCreate(Db_Schema_Table_Abstract $table)
        {
            throw new Db_Statement_Exception_NotSupported();
        }

        public function tablePkDrop(Db_Schema_Table_Abstract $table)
        {
            throw new Db_Statement_Exception_NotSupported();
        }

        public function tableGetIndexes($table)
        {
            throw new Db_Statement_Exception_NotSupported();
        }

        public function indexCreate(Db_Schema_Table_Abstract $table, $column)
        {
            $field = $this->getIndexableField($table, $column);
            $query = sprintf("CREATE INDEX %s_%s on %s (%s)", $table->getName(), $column, $table->getName(), $field);
            $this->_db->query($query);
        }

        public function tableDrop($table)
        {
            throw new Db_Statement_Exception_NotSupported();
        }
    }
?>
