<?php
    class File
    {
        public static function CreateDirStructure($dir)
        {
            if (@file_exists($dir)) {
                return true;
            }

            return @mkdir($dir, 0777, true);
        }

        public static function DetectMimeType($fullpath, $charset = false, $filename = null)
        {
            require_once('File/Mime.php');

            if (strlen($filename) > 0) {
                $extension = self::GetExtension($filename);
            }
            else {
                $extension = self::GetExtension($fullpath);
            }

            $extension = strtolower($extension);

            if (strlen($extension) == 0 || !array_key_exists($extension, File_Mime::$mimes)) {
                return File_Mime::DEFAULT_MIME;
            }

            return File_Mime::$mimes[$extension];
        }

        public static function RmdirRecursive($dir, $native = true)
        {
            $dir = realpath(trim($dir));

            if (!file_exists($dir)) {
                return;
            }

            if (is_file($dir)) {
                unlink($dir);
                return;
            }

            if ($native) {
                if (IS_WINDOWS) {
                    // @todo
                }
                else {
                    try {
                        $bin = self::GetBinaryPath('rm');
                        $cmd = sprintf('%s -rf %s', escapeshellcmd($bin), escapeshellarg($dir));
                        $retval = 0;
                        system($cmd, $retval);

                        if ($retval == 0) {
                            return;
                        }
                    }
                    catch (Exception $ex) {

                    }
                }
            }

            foreach (new DirectoryIterator($dir) as $info) {
                if ($info->isDot()) {
                    continue;
                }

                if ($info->isDir()) {
                    self::RmdirRecursive($info->getPathname(), false);
                    continue;
                }

                unlink($info->getPathname());
            }
        }

        public static function EmptyDir($dir)
        {
            $dp = new File_DirectoryListing($dir);

            while (($file = $dp->nextFile()) !== false)
                @unlink($dir . '/' . $file);

            while (($_dir = $dp->nextDir()) !== false)
                self::RmdirRecursive($dir . '/' . $_dir);
        }

        public static function WriteIniFile($path, $ini)
        {
            $data = '';

            foreach ($ini as $key => $val) {
                if (is_array($val)) {
                    $data .= sprintf("\n[%s]\n", $key);
                    if (count($val) > 0)
                        $data .= "\n";

                    foreach ($val as $k => $v) {
                        $data .= sprintf("%s = %s\n", $k, $v);
                    }
                }
                else {
                    $data .= sprintf("%s = %s\n", $key, $val);
                }
            }

            $fp = fopen($path, 'w');
            if (!$fp)
                return false;

            if (!flock($fp, LOCK_EX))
                return false;

            if (!fwrite($fp, $data)) {
                fclose($fp);
                return false;
            }

            fclose($fp);
            return true;
        }

        /**
         * Format a number of bytes into a human readable format.
         * Optionally choose the output format and/or force a particular unit
         *
         * @param   int     $bytes      The number of bytes to format. Must be positive
         * @param   string  $format     Optional. The output format for the string
         * @param   string  $force      Optional. Force a certain unit. B|KB|MB|GB|TB
         * @return  string              The formatted file size
         */
        public static function FilesizeFormat($bytes, $format = '', $force = 'auto')
        {
            $units = array('B', 'KB', 'MB', 'GB', 'TB', 'PB');
            $force = strtoupper($force);

            $defaultFormat = '%01.1lf %s';
            if (strlen($format) == 0)
                $format = $defaultFormat;

            $bytes = max(0, (int) $bytes);
            $power = array_search($force, $units);

            if ($power === false) {
                $power = $bytes > 0 ? floor(log($bytes, 1024)) : 0;
            }

            $val = $bytes / pow(1024, $power);
            $ret = sprintf($format, $val, $units[$power]);

            // Special case: don't show a file as empty just because it's not at least 1 of the forced unit
            if (strlen($force) > 0 && $bytes > 0) {
                $tmp = self::FilesizeUnformat($ret);
                if ($tmp == 0)
                    $ret = sprintf($format, 1, $units[$power]);
            }

            if ($units[$power] == 'B') {
                $ret = preg_replace('/(\d+)(\.0+)?/', '$1', $ret);
            }

            return $ret;
        }

        public static function FilesizeFormatUnit($bytes)
        {
            $units = array('B', 'KB', 'MB', 'GB', 'TB', 'PB');
            $bytes = max(0, (int) $bytes);
            $power = $bytes > 0 ? floor(log($bytes, 1024)) : 0;

            return $units[$power];
        }

        public static function FilesizeUnformat($str)
        {
            $val = trim($str);
            if (strlen($val) == 0)
                return 0;

            if (preg_match('/^\d+$/', $val))
                return (int) $val;

            $last = strtolower(substr($val, -1));

            switch ($last) {
                case 'g':
                    $val *= 1024;
                case 'm':
                    $val *= 1024;
                case 'k':
                    $val *= 1024;
            }

            return (int) $val;
        }

        public static function CleanPath($path)
        {
            $path = Text::asciify($path);
            $path = preg_replace('|[^0-9a-zA-Z\/ _.-]|', '', $path);
            $path = str_replace('..', '', $path);
            $path = preg_replace('|/{2,}|', '/', $path);
            return $path;
        }

        public static function GetBinaryPath($bin)
        {
            $bin = trim($bin);
            if (strlen($bin) == 0)
                throw new File_Exception('Binary not specified');

            $cmd = sprintf('which %s 2> /dev/null', escapeshellarg($bin));
            $str = trim(`$cmd`);

            if (strlen($str) > 0 && file_exists($str) && is_executable($str))
                return $str;

            $paths = array(Application::GetDataDir() . '/bin', '/usr/local/bin', '/usr/bin', '/bin', '/usr/local/pgsql/bin', '/usr/local/mysql/bin', '/usr/sbin', '/sbin');

            foreach ($paths as $path) {
                $fullpath = $path . '/' . $bin;

                if (file_exists($fullpath) && is_executable($fullpath))
                    return $fullpath;
            }

            throw new File_Exception('Unable to find ' . $bin);
        }

        public static function GetExtension($filename)
        {
            $parts = pathinfo($filename);

            if (isset($parts['extension'])) {
                return $parts['extension'];
            }

            return '';
        }

        public static function GetSystemTmpDir()
        {
            if (function_exists('sys_get_temp_dir'))
                return sys_get_temp_dir();

            if (!empty($_ENV['TMP']))
                return realpath($_ENV['TMP']);

            if (!empty($_ENV['TMPDIR']))
                return realpath($_ENV['TMPDIR']);

            if (!empty($_ENV['TEMP']))
                return realpath($_ENV['TEMP']);

            $tmp = tempnam(md5(uniqid(rand(), true)), '');
            $ret = realpath(dirname($tmp));
            @unlink($tmp);

            return $ret;
        }

        /**
         * GetFilesizeBytes
         *
         * Get the number of bytes for the given filesize and unit.
         *
         * @param   int|double  The filesize to convert
         * @param   string      The unit to convert from (b|mb|kb|gb)
         * @return  int         The number of bytes
         */
        public static function GetFilesizeBytes($size, $unit)
        {
            if ($size == 0) {
                return 0;
            }

            switch (strtolower($unit)) {
                case 'gb':
                    $factor = pow(1024, 3);
                    break;

                case 'mb':
                    $factor = 1024 * 1024;
                    break;

                case 'kb':
                    $factor = 1024;
                    break;

                default:
                    $factor = 1;
            }

            return (int) ($size * $factor);
        }

        /**
         * GetFilesInDir
         *
         * Get all files in a directory
         *
         * @param string        The directory to read
         * @param bool          Whether to read recursively
         * @return array        The files in the directory
         */
        public static function GetFilesInDir($dir, $recursive = true)
        {
            $return = array();
            $files = scandir($dir);

            if (!$files) {
                return false;
            }

            $files = array_diff($files, array('..', '.'));

            foreach ($files as $file) {
                //skip hidden files
                if ($file{0} == '.') {
                    continue;
                }

                $filePath = $dir.DIRECTORY_SEPARATOR.$file;
                if ($recursive && is_dir($filePath)) {
                    $subFiles = self::GetFilesInDir($filePath, true);
                    if ($subFiles) {
                        foreach ($subFiles as $subFile) {
                            $return[] = $subFile;
                        }
                    }
                }
                else {
                    $return[] = $dir.DIRECTORY_SEPARATOR.$file;
                }
            }

            return $return;
        }
    }
?>
