<?php
    /**
     * HTML_Element
     *
     * Used to represent a HTML element
     *
     * @package HTML
     * @author  Quentin Zervaas
     */

    class HTML_Element
    {
        protected $_tag;
        protected $_attributes = array();
        protected $_children = array();
        protected $_styles = array();

        protected $_id = null;

        protected static $_long_tags = array('iframe', 'textarea', 'option');

        public function __construct($tag, $attributes = array())
        {
            $this->_tag = strtolower($tag);
            $this->setAttributes($attributes);
        }

        public function setAttributes($attributes)
        {
            if (is_array($attributes)) {
                foreach ($attributes as $k => $v)
                    $this->setAttribute($k, $v);
            }

            return $this;
        }

        public function setAttribute($k, $v)
        {
            $this->_attributes[$k] = $v;
            return $this;
        }

        public function getAttribute($k, $default = null)
        {
            if (!array_key_exists($k, $this->_attributes)) {
                return $default;
            }

            return $this->_attributes[$k];
        }

        public function addClassName($name)
        {
            $classes = $this->getClassNames();
            $classes[] = $name;

            $classes = array_unique($classes);

            if (count($classes) > 0) {
                $this->setAttribute('class', join(' ', $classes));
            }
            return $this;
        }

        public function getClassNames()
        {
            $class = trim($this->getAttribute('class'));

            return ArrayTools::clean(explode(' ', $class));
        }

        public function __toString()
        {
            $ret = sprintf('<%s', $this->_tag);

            $attrs = $this->_attributes;
            if (strlen($this->_id) > 0)
                $attrs['id'] = $this->_id;

            foreach ($attrs as $k => $v) {
                if (strlen($v) == 0)
                    continue;

                $ret .= sprintf(' %s="%s"', $k, htmlSpecialChars($v));
            }

            if (!array_key_exists('style', $attrs)) {
                $pairs = array();

                foreach ($this->_styles as $k => $v)
                    $pairs[] = sprintf('%s:%s', $k, $v);

                if (count($pairs) > 0)
                    $ret .= sprintf(' style="%s"', htmlSpecialChars(join(';', $pairs)));
            }

            if ($this->isEmpty() && !in_array($this->_tag, self::$_long_tags)) {
                $ret .= ' />';
            }
            else if (count($this->_children) == 1) {
                if ($this->_children[0] instanceof self || $this->_tag == 'script')
                    $ret .= sprintf('>%s</%s>', $this->_children[0], $this->_tag);
                else
                    $ret .= sprintf('>%s</%s>', htmlSpecialChars($this->_children[0]), $this->_tag);
            }
            else {
                $ret .= '>';

                foreach ($this->_children as $child) {
                    if ($child instanceof self || $this->_tag == 'script')
                        $ret .= "\n" . (string) $child;
                    else
                        $ret .= "\n" . htmlSpecialChars((string) $child);
                }

                $ret .= sprintf("\n</%s>", $this->_tag);
            }

            return $ret;
        }

        public function isEmpty()
        {
            return count($this->_children) == 0;
        }

        public function setStyles($styles)
        {
            if (is_array($styles)) {
                foreach ($styles as $k => $v)
                    $this->setStyle($k, $v);
            }

            return $this;
        }

        public function setStyle($k, $v)
        {
            $this->_styles[$k] = $v;
        }

        public function insert($content)
        {
            $this->_children[] = $content;
            return $this;
        }

        public function writeAttribute($k, $v = null)
        {
            if (is_null($v))
                $v = $k;

            $this->_attributes[$k] = $v;
        }

        public function getUniqueId()
        {
            if (is_null($this->_id)) {
                $this->_id = self::GenerateId();
            }

            return $this->_id;
        }

        public static function GenerateId()
        {
            return '__' . substr(md5(uniqid(rand(), true)), 0, 10);
        }
    }
?>