<?php
    /**
     * HTML_Modifier
     *
     * This class is used for modifying HTML code
     *
     * @package HTML
     * @author  Quentin Zervaas
     */
    class HTML_Modifier
    {
        /**
         * LoadJavascript
         *
         * Load a Javascript file in the provided HTML. Adds inbetween
         * the head tags if possible, otherwise just puts it at the end
         * of the source
         *
         * @static
         * @param   string          $html   The HTML to load the Javascript in
         * @param   string|array    $files  One or more files to load
         * @return  string                  The rewritten HTML
         */
        public static function LoadJavascript($html, $files, $endOfBody = false)
        {
            $files = ArrayTools::Clean($files);
            if (count($files) == 0)
                return $html;

            $tpl = '<script type="text/javascript" src="%s"></script>';

            $lines = array();
            foreach ($files as $file)
                $lines[] = sprintf($tpl, $file);

            $newHtml = "\n" . join("\n", $lines) . "\n";

            if ($endOfBody && preg_match('|</body>|i', $html)) {
                $html = preg_replace('|(</body>)|', $newHtml . '$1', $html);
            }
            else if (!$endOfBody && preg_match('|</head>|i', $html)) {
                $html = preg_replace('|(</head>)|', $newHtml . '$1', $html);
            }
            else {
                $html .= $newHtml;
            }

            return $html;
        }

        /**
         * Load a CSS file in the provided HTML. Adds inbetween
         * the head tags if possible, otherwise just puts it at the end
         * of the source
         *
         * @static
         * @param   string          $html   The HTML to load the CSS in
         * @param   string|array    $files  One or more files to load
         * @return  string                  The rewritten HTML
         */
        public static function LoadCss($html, $files)
        {
            $files = ArrayTools::Clean($files);
            if (count($files) == 0)
                return $html;

            $tpl = '<link rel="stylesheet" type="text/css" href="%s" />';

            $lines = array();
            foreach ($files as $file)
                $lines[] = sprintf($tpl, $file);

            $newHtml = "\n" . join("\n", $lines) . "\n";

            if (preg_match('|</head>|i', $html)) {
                $html = preg_replace('|(</head>)|', $newHtml . '$1', $html);
            }
            else {
                $html .= $newHtml;
            }

            return $html;
        }


        public static function WriteMeta($html, $key, $value)
        {
            if (!preg_match('|</head>|i', $html)) {
                throw new Exception('Unable to find head element');
            }

            $elt = new HTML_Element_Meta($key, $value);

            return preg_replace('|(</head>)|', $elt . '$1', $html);
        }


        /**
         * Write the given HTML as the first of the specified element(s)
         *
         * @param   string               $html      The HTML to manipulate
         * @param   string               $selector  Currently this only supports element names
         * @param   string|HTML_Element  $element   The HTML to insert
         * @return  string                          The rewritten HTML
         */
        public static function Prepend($html, $selector, $element)
        {
            $selector = preg_replace('/[^a-z_]/i', '', $selector);

            if (strlen($selector) == 0) {
                throw new Exception('Invalid selector');
            }

            $regex = sprintf('/(<%s[^>]*>)/', preg_quote($selector, '/'));

            if (!preg_match_all($regex, $html, $matches)) {
                throw new Exception('Unable to find selector');
            }

            foreach (array_unique($matches[1]) as $match) {
                $html = str_replace($match, $match . "\n" . (string) $element, $html);
            }

            return $html;
        }
    }
?>