<?php
    /**
     * HTTP_URL
     *
     * Used for various URL manipulation, such as parsing or generating URLs
     */
    class HTTP_URL
    {
        const HTTP_PORT  = 80;
        const HTTPS_PORT = 443;

        /**
         * CreateUrl
         *
         * Creates a URL/DSN based on the provided arguments. Technically this doesn't
         * have to be for HTTP/HTTPS.
         *
         * @param   string  $scheme         The type of DSN being generated (e.g. http or https)
         * @param   string  $hostname       The hostname for the URL
         * @param   string  $path           Optional. The path following the hostname
         * @param   string  $query_string   Optional. The 'get' query string
         * @param   string  $username       Optional. The username
         * @param   string  $password       Optional. The password
         * @return  string                  The generated URL
         */
        public static function CreateUrl($scheme, $hostname, $path = '/', $query_string = null, $username = null, $password = null)
        {
            $url = $scheme . '://';

            if (strlen($username) > 0) {
                $url .= $username;
                if (strlen($password) > 0) {
                    $url .= ':' . $password;
                }
                $url .= '@';
            }

            $url .= $hostname . ltrim('/' . $path, '/');

            if (strlen($query_string) > 0) {
                $url .= '?' . $query_string;
            }

            return $url;
        }


        /**
         * ParseUrl
         *
         * Generates an associative array of each the parts in a URL. Technically you should be able
         * to feed each of these parts back into self::CreateUrl() to regenerate the original URL.
         * This is basically a wrapper to PHP's parse_url() with a few enhancements (such as ensuring
         * all parts are present in the return array)
         *
         * @param   string  $url    The URL to parse
         * @return  array           An array consisting of each part of the URL
         */
        public static function ParseUrl($url)
        {
            $url = trim($url);

            if (strlen($url) == 0) {
                return array();
            }

            $info = parse_url($url);

            if (!isset($info['path']) || strlen($info['path']) == 0) {
                $info['path'] = '/';
            }

            if (!isset($info['scheme'])) {
                $info['scheme'] = null;
            }

            if (!isset($info['query'])) {
                $info['query'] = null;
            }

            if (!isset($info['port']) && $info['scheme'] == 'https') {
                $info['port'] = self::HTTPS_PORT;
            }
            else if (!isset($info['port']) || is_null($info['port'])) {
                $info['port'] = self::HTTP_PORT;
            }

            return $info;
        }


        /**
         * UnparseUrl
         *
         * This method is the opposite of self::ParseUrl() in that it takes
         * an array of URL parts and generates a URL
         */
        public static function UnparseUrl($url, $discardPath = false)
        {
            $keys = array('scheme', 'host', 'port', 'path', 'query');

            foreach ($keys as $key) {
                if (!isset($url[$key])) {
                    $url[$key] = null;
                }
            }

            if (($url['scheme'] == 'http' && $url['port'] == self::HTTP_PORT) || ($url['scheme'] == 'https' && $url['port'] == self::HTTPS_PORT)) {
                $ret = $url['scheme'] . '://' . $url['host'];
            }
            else {
                $ret = $url['scheme'] . '://' . $url['host'] . ':' . $url['port'];
            }

            if (!$discardPath) {
                $ret .= $url['path'];
            }

            if (!$discardPath && strlen($url['query']) > 0) {
                $ret .= '?' . $url['query'];
            }

            return $ret;
        }


        public static function UrlsEqual($link1, $link2)
        {
            return self::NormalizeURL($link1) == self::NormalizeURL($link2);
        }


        public static function NormalizeUrl($val, $discardPath = false)
        {
            return strtolower(self::UnparseURL(self::ParseURL($val), $discardPath));
        }

        public static function GenerateCurrentUrl($discardPath = false, $useServerName = false, $discardQueryString = false)
        {
            if (defined('STDERR')) { // CLI
                return '';
            }

            if ($_SERVER['SERVER_PORT'] == self::HTTPS_PORT) {
                $prefix = 'https://';
            }
            else {
                $prefix = 'http://';
            }

            $host = '';

            if ($useServerName) {
                if (isset($_SERVER['SERVER_NAME'])) {
                    $host = $_SERVER['SERVER_NAME'];
                }
            }
            else {
                if (isset($_SERVER['HTTP_X_FORWARDED_HOST'])) {
                    $host = $_SERVER['HTTP_X_FORWARDED_HOST'];
                }

                if (strlen($host) == 0) {
                    if (isset($_SERVER['HTTP_HOST'])) {
                        $host = $_SERVER['HTTP_HOST'];
                    }
                }
            }

            $url = $prefix . $host;

            if ($discardQueryString) {
                $key = 'REDIRECT_URL';
            }
            else {
                $key = 'REQUEST_URI';
            }

            if (isset($_SERVER[$key])) {
                $url .= $_SERVER[$key];
            }

            return self::NormalizeURL($url, $discardPath);
        }

        public static function IsValidUrl($url, $mustHaveDomain = true)
        {
            if (!$mustHaveDomain && preg_match('|^(/.*)?$|', $url)) {
                return true;
            }

            return preg_match('|^http(s)?://[a-z0-9-]+(\.[a-z0-9-]+)*(:[0-9]+)?(/.*)?$|i', $url);
        }

        public static function GetFullLocalUrl($domain, $currentUrl, $url)
        {
            if (preg_match('|^[a-z]+:|', $url)) {
                if (!preg_match('|^' . preg_quote($domain, '|') . '|', $url)) {
                    throw new Exception('Not a local URL');
                }

                return $url;
            }

            if ($url[0] == '/') {
                return rtrim($domain, '/') . '/' . ltrim($url, '/');
            }

            if (substr($currentUrl, -1) == '/') {
                return $currentUrl . $url;
            }

            return preg_replace('|/[^/]+$|', '/' . $url, $currentUrl);
        }
    }
?>
