<?php
    /**
     * @package NestedTree
     * @since   Recite ?.?.?
     * @author  Quentin Zervaas
     */
    abstract class NestedTree_Abstract
    {
        /**
         * @var Db_Abstract
         */
        protected $_db;


        /**
         * The path separator
         *
         * @var string
         */
        protected $_pathSeparator = '/';


        /**
         * Used to hold data for rebuilding the tree
         */
        protected $_rebuildData;


        /**
         * Used to hold any filters applied to database table
         */
        protected $_filters = array();


        /**
         * Used to hold previously fetched nodes
         *
         * @var NestedTree_Node[]
         */
        protected $_caches = array(
            'nodes'   => array(),
            'paths'   => array(),
            'pathids' => array()
        );


        /**
         * Class constructor
         */
        protected function __construct($db)
        {
            $this->_db = $db;
        }


        /**
         * Return the database connection being used for this tree
         *
         * @return  Db_Abstract
         */
        public function getDb()
        {
            return $this->_db;
        }


        /**
         * Returns the name of the table that holds the tree data, including database prefix
         *
         * @return  string
         */
        abstract public function getTable();


        /**
         * Return a unique name for this tree. This allows the state of any tree to be easily saved
         *
         * @return  string
         */
        public function getTreeName()
        {
            throw new NestedTree_Exception(sprintf('%s::%s not implemented', get_class($this), __FUNCTION__));
        }


        /**
         * Returns the name of the column that holds the primary key / ID column
         *
         * @return string
         */
        abstract public function getIdColumn();


        /**
         * Returns the name of the column that holds the parent ID
         *
         * @return string
         */
        abstract public function getParentColumn();


        /**
         * Returns the name of the column that allows you to load by path
         *
         * @return  string
         */
        abstract public function getPathColumn();


        /**
         * Get the column that holds persistent ranking data used to rebuild the tree
         *
         * @return  string
         * @throws  NestedTree_Exception    If the data isn't stored, meaning the title column will be used to order instead
         */
        public function getRankingColumn()
        {
            throw new NestedTree_Exception('No ranking column specified');
        }


        /**
         * Returns the name of the column that holds the tree left column
         *
         * @return  string
         */
        public function getLeftColumn()
        {
            return 'nleft';
        }


        /**
         * Returns the name of the column that holds the tree right column
         *
         * @return  string
         */
        public function getRightColumn()
        {
            return 'nright';
        }


        /**
         * Returns the name of the column that holds the tree level column
         *
         * @return  string
         */
        public function getLevelColumn()
        {
            return 'nlevel';
        }


        /**
         * Returns the name of the field used as a label for this column
         *
         * @return  string
         */
        public function getTitleColumn()
        {
            return $this->getPathColumn();
        }


        /**
         * Get any additional columns that should be stored with each node
         *
         * @return  array
         */
        public function getAdditionalColumns()
        {
            return array();
        }


        /**
         * Add a filter
         *
         * @param   string  $filter
         * @return  self
         */
        public function addFilter($filter)
        {
            $this->_filters[] = $filter;
            return $this;
        }


        /**
         * Return any additional filters that need to applied to SQL queries
         *
         * @return  string[]
         */
        public function getFilters()
        {
            return $this->_filters;
        }


        /**
         * Whether or not a node can be moved to a new parent
         *
         * @return  bool
         */
        public function canChangeParent()
        {
            return false;
        }


        /**
         * Determine whether or not a child can be moved to the parent.
         * This method will only ever be called if canChangeParent() returns true.
         * The child node may have its own children node. You don't need to check
         * to ensure the parent is within the child as this is handled automatically elsewhere
         *
         * @param   NestedTree_Node|int     $parent     The parent node
         * @param   NestedTree_Node|int     $child      The child node
         * @throws  NestedTree_Exception    If the parent won't accept the child
         * @return  void                    Returning successfully (no exception) means the parent will accept the child
         */
        public function assertParentWillAcceptChild($parent, $child)
        { }


        /**
         * Whether or the order of nodes within a parent can be changed
         *
         * @return  bool
         */
        public function canChangeOrder()
        {
            return false;
        }


        /**
         * Set the path separator character
         *
         * @throws  NestedTree_Exception    If no separator is specified
         * @param   string  $separator      The character used to separate paths
         */
        public function setPathSeparator($separator)
        {
            $separator = trim($separator);
            if (strlen($separator) == 0) {
                throw new NestedTree_Exception('Invalid path separator specified');
            }

            $this->_pathSeparator = $separator;
            return $this;
        }


        /**
         * Retrieve the path separator character
         *
         * @return  string  The path separation character
         */
        public function getPathSeparator()
        {
            return $this->_pathSeparator;
        }


        /**
         * Get a node based on its ID
         *
         * @param   int|NestedTree_Node     $node   The node to fetch
         * @throws  NestedTree_Exception            If the node isn't found
         * @return  NestedTree_Node                 The node to retrieve
         */
        public function getNode($node)
        {
            if ($node instanceof NestedTree_Node) {
                $this->_caches['nodes'][$node->getId()] = $node;
                return $node;
            }

            if (is_null($node)) {
                throw new NestedTree_Exception('Node not found');
            }

            if (array_key_exists($node, $this->_caches['nodes'])) {
                return $this->_caches['nodes'][$node];
            }

            $select = $this->_getBaseQuery();
            $select->where($this->getIdColumn() . ' = ?', $node);

            $row = $this->_db->fetchRow($select);

            if (!$row) {
                throw new NestedTree_Exception('Node not found');
            }

            $node = $this->_nodeFromDatabaseRow($row);

            $this->_caches['nodes'][$node->getId()] = $node;
            return $node;
        }


        /**
         * Get multiple nodes at once
         *
         * @param   int[]|NestedTree_Node[]     $nodes  The nodes to fetch
         * @return  NestedTree_Node[]
         */
        public function getNodes($nodes)
        {
            $ret = array();
            $ids = array();

            foreach ($nodes as $node) {
                try {
                    if ($node instanceof NestedTree_Node) {
                        $this->_caches['nodes'][$node->getId()] = $node;
                        $ret[] = $node;
                    }
                    else if (array_key_exists($node, $this->_caches['nodes'])) {
                        $ret[] = $this->_caches['nodes'][$node];
                    }
                    else if (preg_match('/^\d+$/', $node)) {
                        $ids[] = $node;
                    }
                    else {
                        $ret[] = $this->getNode($node);
                    }
                }
                catch (Exception $ex) {
                }
            }

            if (count($ids) > 0) {
                $select = $this->_getBaseQuery();
                $select->where($this->getIdColumn() . ' in (?)', $ids);

                foreach ($this->_db->fetchAll($select) as $row) {
                    $node = $this->_nodeFromDatabaseRow($row);
                    $this->_caches['nodes'][$node->getId()] = $node;
                    $ret[] = $node;
                }
            }

            return $ret;
        }


        /**
         * Get one or more nodes based on their ID
         *
         * @param   int|array   $nodeIds        The IDs to fetch
         * @param   bool        $mustHaveAll    If this is true then an exception is thrown if not all nodes are found
         * @throws  NestedTree_Exception        If the node isn't found
         * @return  NestedTree_NodeList         The found nodes
         */
        public function getNodesFromIds($nodeIds, $mustHaveAll = false)
        {
            $nodeIds = ArrayTools::clean($nodeIds, true);

            $ret = new NestedTree_NodeList();

            if (count($nodeIds) == 0) {
                return $ret;
            }

            $select = $this->_getBaseQuery();
            $select->where($this->getIdColumn() . ' in (?)', $nodeIds);

            $rows = $this->_db->fetchAll($select);

            if ($mustHaveAll && count($rows) < count($nodeIds)) {
                throw new NestedTree_Exception('Not all IDs were found');
            }

            foreach ($rows as $row) {
                $node = $this->_nodeFromDatabaseRow($row);
                $this->_caches['nodes'][$node->getId()] = $node;
                $ret->add($node);
            }

            return $ret;
        }


        /**
         * Get a node based on its path
         *
         * @param   string  $nodePath       The path of the node to fetch
         * @throws  NestedTree_Exception    If the node isn't found
         * @return  NestedTree_Node         The node to retrieve
         */
        public function getNodeFromPath($nodePath)
        {
            $nodePath = trim($nodePath);

            if (strlen($nodePath) == 0) {
                throw new NestedTree_Exception('Node path not specified');
            }

            if (array_key_exists($nodePath, $this->_caches['pathids'])) {
                if ($this->_caches['pathids'][$nodePath] == 0) {
                    throw new NestedTree_Exception('Node not found');
                }

                return $this->getNode($this->_caches['pathids'][$nodePath]);
            }

            $sep = $this->getPathSeparator();

            $parts = array_filter(explode($sep, trim($nodePath, $sep)));

            if (count($parts) == 0) {
                throw new NestedTree_Exception('Node path not specified');
            }

            $select = $this->_getBaseQuery(false);

            $wheres = array();

            $level = 1;
            foreach ($parts as $part) {
                $wheres[] = $this->_db->quoteInto(sprintf('t.%s = ? and t.%s = %d', $this->getPathColumn(), $this->getLevelColumn(), $level), $part);

                $level++;
            }

            $select->where('(' . join(') or (', $wheres) . ')')
                   ->order($this->getLevelColumn());

            $l = -1;
            $r = -1;

            $level = 1;
            $expectedLevel = count($parts);

            $levelField = $this->getLevelColumn();

            try {
                foreach ($this->_db->fetchAll($select) as $row) {
                    if ($row[$levelField] < $level) {
                        continue;
                    }
                    else if ($row[$levelField] > $level) {
                        throw new NestedTree_Exception(sprintf('Expected level %d, got level %d', $level, $row[$levelField]));
                    }

                    $left  = $row[$this->getLeftColumn()];
                    $right = $row[$this->getRightColumn()];

                    if ($left > $right) {
                        throw new NestedTree_Exception('Invalid tree data');
                    }

                    if ($l < 0 && $r < 0) {
                        $l = $left;
                        $r = $right;
                    }

                    if ($left < $l || $right > $r) {
                        // node at right level but not within tree, skip
                        continue;
                    }

                    if ($row[$levelField] == $expectedLevel) {
                        $node = $this->_nodeFromDatabaseRow($row);
                        $this->_caches['pathids'][$nodePath] = $node->getId();
                        $this->_caches['nodes'][$node->getId()] = $node;
                        return $node;
                    }

                    $l = $left;
                    $r = $right;
                    $level++;
                }
            }
            catch (Exception $ex) { }

            throw new NestedTree_Exception('Node not found');
        }


        /**
         * Returns the path for a node. You can specify offset and length just as you would
         * in PHP functions such as substr() or array_slice()
         *
         * @param   NestedTree_Node|int $node   The node to retrieve the path of
         * @return  NestedTree_Path
         */
        public function getPath($node)
        {
            $node = $this->getNode($node);

            if (array_key_exists($node->getId(), $this->_caches['paths'])) {
                return $this->_caches['paths'][$node->getId()];
            }

            if ($node->getLevel() == 1) {
                $ret = new NestedTree_Path($this->getPathSeparator());
                $ret->add($node);
            }
            else {
                $select = $this->_getBaseQuery();
                $select->where(sprintf('%s <= ?', $this->getLeftColumn()), $node->getL())
                       ->where(sprintf('%s >= ?', $this->getRightColumn()), $node->getR());

                $ret = new NestedTree_Path($this->getPathSeparator());

                foreach ($this->_db->fetchAll($select) as $row) {
                    $ret->add($this->_nodeFromDatabaseRow($row));
                }
            }

            $this->_caches['paths'][$node->getId()] = $ret;

            return $ret;
        }


        /**
         * Returns paths for the specified nodes
         *
         * @param   NestedTree_Node|int $nodes[]    The nodes to retrieve paths for
         * @return  NestedTree_Path[]               The node paths
         */
        public function getPaths($nodes)
        {
            $nodes = $this->getNodes($nodes);
            $pairs = array();

            foreach ($nodes as $node) {
                if (array_key_exists($node->getId(), $this->_caches['paths']) || $node->getLevel() == 1) {
                    continue;
                }

                $add = true;

                foreach ($pairs as $l => $r) {
                    if ($node->getL() > $l && $node->getR() < $r) {
                        unset($pairs[$l]);
                    }
                    else if ($node->getL() < $l && $node->getR() > $r) {
                        $add = false;
                    }
                }

                if ($add) {
                    $pairs[$node->getL()] = $node->getR();
                }
            }

            $wheres = array();
            foreach ($pairs as $l => $r) {
                $wheres[] = sprintf('(%s <= %d and %s >= %d)',
                    $this->getLeftColumn(),
                    $l,
                    $this->getRightColumn(),
                    $r
                );
            }

            $pathNodes = array();

            if (count($wheres) > 0) {
                $select = $this->_getBaseQuery();
                $select->where(join(' or ', $wheres));

                $pathNodes = array();

                foreach ($this->_db->fetchAll($select) as $row) {
                    $node = $this->_nodeFromDatabaseRow($row);
                    $this->_caches['nodes'][$node->getId()] = $node;
                    $pathNodes[] = $node;
                }
            }

            $ret = array();

            foreach ($nodes as $node) {
                if (!array_key_exists($node->getId(), $this->_caches['paths'])) {
                    $path = new NestedTree_Path($this->getPathSeparator());

                    if ($node->getLevel() == 1) {
                        $path->add($node);
                    }
                    else foreach ($pathNodes as $n) {
                        if ($n->getL() <= $node->getL() && $n->getR() >= $node->getR()) {
                            $path->add($n);
                        }
                    }

                    $this->_caches['paths'][$node->getId()] = $path;
                    $ret[] = $path;
                }
                else {
                    $ret[] = $this->_caches['paths'][$node->getId()];
                }
            }

            return $ret;
        }


        /**
         * Retrieve an ancestor node (that is, higher in the tree than the specified node)
         *
         * @param   int     $node           The node to fetch the ancestor of
         * @param   int     $level          If level less than 0 then relative to node, if greater than then its absolute level. E.g. -1 is parent, -2 is grandparent
         * @param   bool    $fallback       Optional. If true, the highest possible node will be returned if the specified level does not exist
         * @throws  NestedTree_Exception    If invalid starting node; if invalid level; if $fallback is false and level out of range
         * @return  NestedTree_Node         The ancestor node
         */
        public function getAncestor($node, $level, $fallback = false)
        {
            $node  = $this->getNode($node);
            $level = (int) $level;

            $select = $this->_getBaseQuery();
            $select->where(sprintf('%s <= ?', $this->getLeftColumn()), $node->getL())
                   ->where(sprintf('%s >= ?', $this->getRightColumn()), $node->getR());


            if ($fallback) {
                $select->order($this->getLevelColumn());
            }
            else {
                $select->where(sprintf('%s = ?', $this->getLevelColumn()), $level);
            }

            $this->_setupGetAncestor($select);

            foreach ($this->_db->fetchAll($select) as $row) {
                return $this->_nodeFromDatabaseRow($row);
            }

            throw new Exception('Ancestor at specified level not found');
        }

        /**
         * Add any custom processing to the query
         */
        public function _setupGetAncestor(Zend_Db_Select $select, NestedTree_Options $options = null)
        { }


        /**
         * Retrieve all ancestor nodes based on the specified options
         *
         * @param   NestedTree_Options      $options
         * @throws  NestedTree_Exception    If invalid starting node
         * @return  NestedTree_NodeList     The list of ancestor nodes
         */
        public function getAncestors(NestedTree_Options $options)
        {
            if (!$options->hasRootPath() && !$options->hasRootId() && !$options->hasRootNode()) {
                $node = null;
            }
            else if ($options->hasRootPath()) {
                $node = $this->getNodeFromPath($options->getRootPath());
            }
            else if ($options->hasRootNode()) {
                $node = $options->getRootNode();
            }
            else {
                $node = $this->getNode($options->getRootId());
            }

            if (!$node instanceof NestedTree_Node) {
                throw new NestedTree_Exception('Invalid starting node');
            }

            $select = $this->_getBaseQuery();

            if ($options->getIncludeSelf()) {
                $select->where($this->getLeftColumn() . ' <= ' . $node->getL())
                       ->where($this->getRightColumn() . ' >= ' . $node->getR());
            }
            else {
                $select->where($this->getLeftColumn() . ' < ' . $node->getL())
                       ->where($this->getRightColumn() . ' > ' . $node->getR());
            }

            if ($options->getMaxLevels() > 0) {
                $_level = $node->getLevel() - $options->getMaxLevels();
                if ($options->getIncludeSelf()) {
                    $_level++;
                }

                $select->where($this->getLevelColumn() . ' >= ?', $_level);
            }

            $this->_setupGetAncestors($select);

            $data = $this->_db->fetchAll($select);

            $ret = new NestedTree_NodeList();

            foreach ($data as $row) {
                $ret->add($this->_nodeFromDatabaseRow($row));
            }

            return $ret;
        }

        /**
         * Add any custom processing to the query
         */
        public function _setupGetAncestors(Zend_Db_Select $select, NestedTree_Options $options = null)
        { }


        /**
         * Retrieve all nodes from starting node
         *
         * @param   NestedTree_Options      $options    The options to specify function behaviour
         * @throws  NestedTree_Exception
         * @return  NestedTree_NodeList                 The list of descendants
         */
        public function getDescendants(NestedTree_Options $options = null)
        {
            if (!$options instanceof NestedTree_Options) {
                $options = new NestedTree_Options();
            }

            if (!$options->hasRootPath() && !$options->hasRootId() && !$options->hasRootNode()) {
                $node = null;
            }
            else if ($options->hasRootNode()) {
                $node = $options->getRootNode();
            }
            else if ($options->hasRootPath()) {
                $node = $this->getNodeFromPath($options->getRootPath());
            }
            else {
                $node = $this->getNode($options->getRootId());
            }

            $db = $this->getDb();

            $ret = new NestedTree_NodeList();

            $select = $this->_getBaseQuery();

            $ranges = array();

            if ($node instanceof NestedTree_Node) {
                if ($options->getIncludeSelf()) {
                    $ret->setId($node->getParentId());
                }
                else {
                    $ret->setId($node->getId());
                }

                $baseRange = new NestedTree_Nrange();

                if ($options->getMaxLevels() > 0) {
                    $_level = $node->getLevel() + $options->getMaxLevels();

                    if ($options->getIncludeSelf()) {
                        $_level--;
                    }

                    $baseRange->setLevel($_level, NestedTree_Nrange::OUTSIDE_INCLUSIVE);
                }

                if ($options->getIncludeSelf()) {
                    $baseRange->setL($node->getL(), NestedTree_Nrange::INSIDE_INCLUSIVE)
                              ->setR($node->getR(), NestedTree_Nrange::INSIDE_INCLUSIVE);
                }
                else {
                    $baseRange->setL($node->getL(), NestedTree_Nrange::INSIDE_EXCLUSIVE)
                              ->setR($node->getR(), NestedTree_Nrange::INSIDE_EXCLUSIVE);
                }

                $ranges[] = $baseRange;
            }
            else if ($options->getMaxLevels() > 0) {
                $baseRange = new NestedTree_Nrange();
                $baseRange->setLevel($options->getMaxLevels(), NestedTree_Nrange::OUTSIDE_INCLUSIVE);

                $ranges[] = $baseRange;
            }

            try {
                $treeState = $options->getTreeState();
                $nodeList  = $treeState->getNodeList();

                foreach ($nodeList->getChildren() as $child) {
                    try {
                        if ($child->getL() <= $baseRange->getL()) {
                            continue;
                        }
                    }
                    catch (Exception $ex)
                    { }

                    try {
                        if ($child->getR() >= $baseRange->getR()) {
                            continue;
                        }
                    }
                    catch (Exception $ex)
                    { }

                    $range = new NestedTree_Nrange();
                    $range->setL($child->getL(), NestedTree_Nrange::INSIDE_INCLUSIVE)
                          ->setR($child->getR(), NestedTree_Nrange::INSIDE_INCLUSIVE)
                          ->setLevel($child->getLevel() + 1, NestedTree_Nrange::OUTSIDE_INCLUSIVE);

                    $ranges[] = $range;
                }
            }
            catch (Exception $ex)
            { }

            $wheres = array();
            foreach ($ranges as $range) {
                $wheres[] = join(' and ', $range->getSqlWhereStatements($this));
            }

            if (count($wheres) > 0) {
                $select->where('(' . join(') or (', $wheres) . ')');
            }

            // process include nodes
            $ids = $options->getIncludeIds();

            if (count($ids) > 0) {
                $nodes = $this->getNodesFromIds($ids);

                $ranges = array();
                $wheres = array();

                if ($options->getIncludeSelf() && $node instanceof NestedTree_Node) {
                    $wheres[] = $db->quoteInto($this->getIdColumn() . ' = ?', $node->getId());
                }

                foreach ($nodes->getChildren() as $n) {
                    if ($node instanceof NestedTree_Node && $n->getId() == $node->getId()) {
                        continue;
                    }

                    $range = new NestedTree_Nrange();
                    $range->setL($n->getL(), NestedTree_Nrange::INSIDE_INCLUSIVE)
                          ->setR($n->getR(), NestedTree_Nrange::INSIDE_INCLUSIVE);

                    $ranges[] = $range;
                }

                foreach ($ranges as $range) {
                    $wheres[] = join(' and ', $range->getSqlWhereStatements($this));
                }

                if (count($wheres) > 0) {
                    $select->where('(' . join(') or (', $wheres) . ')');
                }
            }

            // process exclude nodes
            $ids = $options->getExcludeIds();

            if (count($ids) > 0) {
                $nodes = $this->getNodesFromIds($ids);

                foreach ($nodes->getChildren() as $n) {
                    $range = new NestedTree_Nrange();
                    $range->setL($n->getL(), NestedTree_Nrange::EITHER_SIDE)
                          ->setR($n->getR(), NestedTree_Nrange::EITHER_SIDE);

                    foreach ($range->getSqlWhereStatements($this) as $where) {
                        $select->where($where);
                    }
                }
            }

            if (!$node instanceof NestedTree_Node) {
                $rootId = 0;
            }
            else if ($options->getIncludeSelf()) {
                $rootId = $node->getParentId();
            }
            else {
                $rootId = $node->getId();
            }

            $this->_setupGetDescendants($select, $options);

            $ret = $this->_buildNodesFromQuery($select, $rootId);

            try {
                $ret->setTreeState($options->getTreeState());
            }
            catch (Exception $ex)
            { }

            return $ret;
        }


        /**
         * Add any custom processing to the query
         */
        public function _setupGetDescendants(Zend_Db_Select $select, NestedTree_Options $options = null)
        { }


        /**
         * Retrieve all children nodes from the specified nodes
         *
         * @throws  NestedTree_Exception
         * @return  NestedTree_NodeList     The list of children
         */
        public function getChildren($node)
        {
            throw new NestedTree_Exception(sprintf('Method %s::%s not implemented', __CLASS__, __FUNCTION__));
        }


        /**
         * Get a node's sibling that occurs lower in the tree
         *
         * @param   int|NestedTree_Node $node   The node to find the sibling for
         * @throws  NestedTree_Exception        If the node can't be found or if there is no sibling
         * @return  NestedTree_Node             The sibling node
         */
        public function getSiblingAfterNode($node)
        {
            $node = $this->getNode($node);

            $select = $this->_getBaseQuery();
            $select->where($this->getParentColumn() . ' = ' . $node->getParentId())
                   ->where($this->getLeftColumn() . ' > '. $node->getL())
                   ->limit(1);

            $row = $this->_db->fetchRow($select);

            if (!$row) {
                throw new NestedTree_Exception('Sibling not found');
            }

            return $this->_nodeFromDatabaseRow($row);
        }


        /**
         * Build a base select query which sets up the table and columns
         *
         * @return  Zend_Db_Select
         */
        protected function _getBaseQuery($order = true)
        {
            $select = $this->_db->select();
            $select->from(array('t' => $this->getTable()), $this->_getDatabaseColumns());

            if ($order) {
                $select->order($this->getLeftColumn());
            }

            foreach ($this->getFilters() as $filter) {
                $select->where($filter);
            }

            $this->_setupBaseQuery($select);

            return $select;
        }


        public function _setupBaseQuery(Zend_Db_Select $select)
        {
        }


        /**
         * Returns a list of the columns that should be selected from the database for any tree queries
         *
         * @return  array
         */
        protected function _getDatabaseColumns()
        {
            $ret = array(
                $this->getIdColumn(),
                $this->getParentColumn(),
                $this->getLeftColumn(),
                $this->getRightColumn(),
                $this->getLevelColumn(),
                $this->getPathColumn(),
                $this->getTitleColumn()
            );

            $additional = $this->getAdditionalColumns();

            if (is_array($additional)) {
                foreach ($additional as $name) {
                    $ret[] = $name;
                }
            }

            return array_unique($ret);
        }


        /**
         * Builds a node list from the select query provided
         *
         * @param   Zend_Db_Select|string   $select The query to execute
         * @param   int                     $rootId The ID of the root node
         * @return  NestedTree_NodeList     The list of nodes to return
         */
        protected function _buildNodesFromQuery($select, $rootId = null)
        {
            $data = $this->_db->fetchAll($select);

            $ret = new NestedTree_NodeList();

            $key = $this->getParentColumn();

            $i = 0;

            foreach ($data as $row) {
                $node = $this->_nodeFromDatabaseRow($row);

                if (is_null($rootId) && $i == 0) {
                    $ret->add($node);
                }
                else if ($node->getParentId() == $rootId) {
                    $ret->add($node);
                }
                else {
                    try {
                        $ret->addToNode($node->getParentId(), $node);
                    }
                    catch (Exception $ex) {
                        continue;
                    }
                }

                $i++;
            }

            return $ret;
        }


        protected function _nodeFromDatabaseRow($row)
        {
            $ret = new NestedTree_Node(
                $row[$this->getIdColumn()],
                $row[$this->getLeftColumn()],
                $row[$this->getRightColumn()],
                $row[$this->getLevelColumn()]
            );

            $ret->setTitle($row[$this->getTitleColumn()])
                ->setParentId($row[$this->getParentColumn()])
                ->setPathElement($row[$this->getPathColumn()]);

            $additional = $this->getAdditionalColumns();

            if (is_array($additional)) {
                foreach ($additional as $name) {
                    $ret->setAdditionalField($name, $row[$name]);
                }
            }

            return $ret;
        }


        public function rebuild()
        {
            $select = $this->_getBaseQuery(false);

            try {
                if (!$this->canChangeOrder()) {
                    throw new Exception('Cannot change order');
                }

                $select->order($this->getRankingColumn());
            }
            catch (Exception $ex) {
                $select->order('lower(' . $this->_db->quoteIdentifier($this->getTitleColumn(), true) . ')');
            }

            $this->_rebuildData = array(
                0 => array()
            );

            $result = $this->_db->query($select);

            $parentColumn = $this->getParentColumn();
            $idColumn = $this->getIdColumn();

            while ($row = $result->fetch()) {
                 $parent = (int) $row->$parentColumn;
                 $id     = (int) $row->$idColumn;

                 if (!array_key_exists($parent, $this->_rebuildData)) {
                     $this->_rebuildData[$parent] = array();
                 }

                 if (!array_key_exists($id, $this->_rebuildData)) {
                     $this->_rebuildData[$id] = array();
                 }

                 $this->_rebuildData[$parent][] = $id;
            }

            $n = 1;
            $r = 1;

            $this->_db->begin();

            try {
                foreach ($this->_rebuildData[0] as $id) {
                    $this->_rebuild($id, $n, 1, $r++);
                }

                $this->_db->commit();
            }
            catch (Exception $ex) {
                $this->_db->rollback();
            }

            $this->_rebuildData = null;
        }

        private function _rebuild($id, &$n, $l, $ranking)
        {
            $left = $n++;

            $r = 1;
            foreach ($this->_rebuildData[$id] as $child_id) {
                $this->_rebuild($child_id, $n, $l + 1, $r++);
            }

            $values = array(
                $this->getLeftColumn()  => $left,
                $this->getRightColumn()  => $n++,
                $this->getLevelColumn() => $l
            );

            try {
                $values[$this->getRankingColumn()] = $ranking;
            }
            catch (Exception $ex)
            { }

            $where = array(
                $this->_db->quoteInto($this->_db->quoteIdentifier($this->getIdColumn()) . ' = ?', $id)
            );

            $this->_db->update($this->getTable(), $values, $where);
        }

        public function notifyParentChanged(NestedTree_Node $node, NestedTree_Node $newParent = null)
        {
        }

        public function notifySiblingChanged(NestedTree_Node $node, NestedTree_Node $newSibling, $moveType)
        {
        }


        /**
         * This is used to distinguish between table that have a parent ID that is a foreign
         * key to the table main ID. If it is a FK then a node with no parent must have the
         * parent set to null. If it's not a FK then a node with no parent must have the parent
         * set to 0
         */
        public function parentColumnIsFk()
        {
            return false;
        }
    }
?>