<?php
    class NestedTree_Node_Move_Request
    {
        const BEFORE = 'before';
        const AFTER  = 'after';
        const INSIDE = 'inside';

        public static $moveTypes = array(
            self::BEFORE => 'Before',
            self::AFTER  => 'After',
            self::INSIDE => 'Inside'
        );

        protected $_nodeId          = 0;
        protected $_referenceId     = 0;
        protected $_referenceIsRoot = false;
        protected $_moveType;

        public function __construct($moveType)
        {
            if (!array_key_exists($moveType, self::$moveTypes)) {
                throw new NestedTree_Exception('Invalid move type');
            }

            $this->_moveType = $moveType;
        }

        public function setNodeId($nodeId)
        {
            $this->_nodeId = (int) $nodeId;
            return $this;
        }

        public function getNodeId()
        {
            if ($this->_nodeId <= 0) {
                throw new NestedTree_Exception('Node ID not specified', NestedTree_Node_Move_Response::ERROR_NO_NODE);
            }

            return $this->_nodeId;
        }

        public function setReferenceNodeId($nodeId)
        {
            $this->_referenceId = (int) $nodeId;
            $this->_referenceIsRoot = false;
            return $this;
        }

        public function getReferenceNodeId()
        {
            if ($this->_referenceId <= 0) {
                throw new NestedTree_Exception('Reference node ID not specified');
            }

            return $this->_referenceId;
        }

        public function setReferenceAsRoot()
        {
            $this->_referenceIsRoot = true;
            $this->_referenceId = 0;
            return $this;
        }

        public function referenceIsRoot()
        {
            return $this->_referenceIsRoot;
        }

        public function process(NestedTree_Abstract $tree)
        {
            if (!$tree->canChangeParent() && !$tree->canChangeOrder()) {
                throw new NestedTree_Exception(
                    'Unable to move nodes',
                    NestedTree_Node_Move_Response::ERROR_CANNOT_CHANGE_PARENT | NestedTree_Node_Move_Response::ERROR_CANNOT_CHANGE_SIBLING
                );
            }

            if ($this->referenceIsRoot()) {
                $node    = $tree->getNode($this->getNodeId());
                $refNode = null;
            }
            else {
                $nodeIds = array(
                    $this->getNodeId(),
                    $this->getReferenceNodeId()
                );

                $nodeList = $tree->getNodesFromIds($nodeIds);

                if ($nodeList->size() != 2) {
                    throw new NestedTree_Exception('One or more nodes not found', NestedTree_Node_Move_Response::ERROR_NO_NODE);
                }

                $node    = $nodeList->getChildFromId($this->getNodeId());
                $refNode = $nodeList->getChildFromId($this->getReferenceNodeId());
            }

            if (!$this->referenceIsRoot() && $node->getId() == $refNode->getId()) {
                throw new NestedTree_Exception('Both nodes are the same node', NestedTree_Node_Move_Response::ERROR_NO_NODE);
            }

            switch ($this->_moveType) {
                case self::AFTER:
                case self::BEFORE:
                    if (!$tree->canChangeOrder() || !$refNode instanceof NestedTree_Node) {
                        throw new NestedTree_Exception('Unable to change sibling node', NestedTree_Node_Move_Response::ERROR_CANNOT_CHANGE_SIBLING);
                    }

                    return $this->_changeSibling($tree, $node, $refNode, $this->_moveType);

                case self::INSIDE:
                    if ($this->referenceIsRoot()) {
                        $changingParent = ((int) $node->getParentId()) != 0;
                    }
                    else {
                        $changingParent = ((int) $node->getParentId()) != ((int) $refNode->getId());
                    }

                    if ($changingParent && !$tree->canChangeParent()) {
                        throw new NestedTree_Exception('Unable to change parent node', NestedTree_Node_Move_Response::ERROR_CANNOT_CHANGE_PARENT);
                    }

                    if (!$this->referenceIsRoot() && $refNode->isInsideOf($node)) {
                        throw new NestedTree_Exception('Cannot move to descendant node', NestedTree_Node_Move_Response::ERROR_NO_PARENT);
                    }

                    $tree->assertParentWillAcceptChild($refNode, $node);

                    return $this->_changeParent($tree, $node, $refNode);
            }
        }

        private function _changeParent(NestedTree_Abstract $tree, NestedTree_Node $node, NestedTree_Node $newParent = null)
        {
            try {
                $rankingColumn = $tree->getRankingColumn();
            }
            catch (Exception $ex) {
                $rankingColumn = '';
            }

            $moveToRoot = is_null($newParent);

            $db = $tree->getDb();
            $db->begin();

            $ranking = 1;

            if (strlen($rankingColumn) > 0) {
                $options = new NestedTree_Options();

                if (!$moveToRoot) {
                    $options->setRootNode($newParent);
                }

                $options->setMaxLevels(1)
                        ->setIncludeSelf(false);

                $nodeList = $tree->getDescendants($options);

                foreach ($nodeList->getChildren() as $child) {
                    $values = array(
                        $rankingColumn          => $ranking++,
                        $tree->getLevelColumn() => $moveToRoot ? 1 : $newParent->getLevel() + 1,
                    );

                    $where = array(
                        $db->quoteInto($tree->getIdColumn() . ' = ?', $child->getId())
                    );

                    $db->update($tree->getTable(), $values, $where);
                }
            }

            $rootId = $tree->parentColumnIsFk() ? null : 0;

            $values = array(
                $tree->getParentColumn() => $moveToRoot ? $rootId : $newParent->getId(),
            );

            if (strlen($rankingColumn) > 0) {
                $values[$rankingColumn] = $ranking;
            }

            $where = array(
                $db->quoteInto($tree->getIdColumn() . ' = ?', $node->getId())
            );

            $db->update($tree->getTable(), $values, $where);

            $tree->rebuild();

            $tree->notifyParentChanged($node, $newParent);

            $db->commit();

            $ret = new NestedTree_Node_Move_Response();
            $ret->setNode($node);

            return $ret;
        }

        private function _changeSibling(NestedTree_Abstract $tree, NestedTree_Node $node, NestedTree_Node $newSibling, $moveType)
        {
            if ($moveType != self::AFTER && $moveType != self::BEFORE) {
                throw new NestedTree_Exception('Invalid move type');
            }

            $parentId = (int) $newSibling->getParentId();

            if ($parentId > 0) {
                $newParent = $tree->getNode($newSibling->getParentId());
            }
            else {
                $newParent = null;
            }

            $moveToRoot     = is_null($newParent);
            $changingParent = !$moveToRoot && $node->getParentId() != $newSibling->getParentId();

            if ($changingParent) {
                $tree->assertParentWillAcceptChild($newParent, $node);
            }

            // if there's no ranking column then we can't move sibling
            // and the following call will throw an exception
            $rankingColumn = $tree->getRankingColumn();

            $table = $tree->getTable();

            $options = new NestedTree_Options();

            if (!$moveToRoot) {
                $options->setRootNode($newParent);
            }

            $options->setMaxLevels(1)
                    ->setIncludeSelf(false);

            $nodeList = $tree->getDescendants($options);

            $ranking = 1;

            $db = $tree->getDb();
            $db->begin();

            foreach ($nodeList->getChildren() as $child) {
                if ($child->getId() == $node->getId()) {
                    continue;
                }

                if ($moveType == self::AFTER) {
                    $values = array(
                        $rankingColumn => $ranking++
                    );

                    $where = array(
                        $db->quoteInto($tree->getIdColumn() . ' = ?', $child->getId())
                    );

                    $db->update($table, $values, $where);
                }

                if ($child->getId() == $newSibling->getId()) {
                    $rootId = $tree->parentColumnIsFk() ? null : 0;

                    $values = array(
                        $tree->getParentColumn() => $moveToRoot ? $rootId : $newParent->getId(),
                        $tree->getLevelColumn()  => $moveToRoot ? 1 : $newParent->getLevel() + 1,
                        $rankingColumn           => $ranking++
                    );

                    $where = array(
                        $db->quoteInto($tree->getIdColumn() . ' = ?', $node->getId())
                    );

                    $db->update($table, $values, $where);
                }

                if ($moveType == self::BEFORE) {
                    $values = array(
                        $rankingColumn => $ranking++
                    );

                    $where = array(
                        $db->quoteInto($tree->getIdColumn() . ' = ?', $child->getId())
                    );

                    $db->update($table, $values, $where);
                }
            }

            $tree->rebuild();

            $tree->notifySiblingChanged($node, $newSibling, $moveType);

            $db->commit();

            $ret = new NestedTree_Node_Move_Response();
            $ret->setNode($node);

            return $ret;
        }
    }
?>
