<?php
    /**
     * Class used to specify options for retrieving a nested tree
     *
     * @author  Quentin Zervaas
     */
    class NestedTree_Options
    {
        /**
         * The root node ID
         *
         * @var int
         */
        protected $_rootId = 0;


        /**
         * The root node path
         *
         * @var string
         */
        protected $_rootPath = '';


        /**
         * The root node
         *
         * @var NestedTree_Node
         */
        protected $_rootNode;


        /**
         * Whether or not the root node should be included
         *
         * @var bool
         */
        protected $_includeSelf = false;


        /**
         * Maximum number of levels to return
         *
         * @var int
         */
        protected $_maxLevels = 0;


        /**
         * Holds the IDs to exclude
         *
         * @var int[]
         */
        protected $_excludeIds = array();


        /**
         * Holds the IDs to include
         *
         * @var int[]
         */
        protected $_includeIds = array();


        /**
         * Holds a tree state to also fetch. That is, open and selected nodes.
         * When you call getDescendants() on the tree with this specified then
         * any open or selected nodes will also be fetched (this will override
         * other options such as max levels)
         *
         * @var NestedTree_TreeView_TreeState
         */
        protected $_treeState;


        /**
         * Constructor
         */
        public function __construct()
        { }


        /**
         * Set root node ID
         *
         * @param   int     $id
         * @return  self
         */
        public function setRootId($id)
        {
            $this->_rootId = (int) $id;
            return $this;
        }


        /**
         * Get the root node ID
         *
         * @throws  NestedTree_Exception    If the root ID hasn't been set
         * @return  int
         */
        public function getRootId()
        {
            if ($this->_rootId <= 0) {
                throw new NestedTree_Exception('Root ID not set');
            }

            return $this->_rootId;
        }


        /**
         * Determine whether or not root ID is set
         *
         * @return  bool
         */
        public function hasRootId()
        {
            try {
                $this->getRootId();
                return true;
            }
            catch (Exception $ex) {
                return false;
            }
        }


        /**
         * Set root node path
         *
         * @param   path    $path
         * @return  self
         */
        public function setRootPath($path)
        {
            $this->_rootPath = (string) $path;
            return $this;
        }


        /**
         * Get the root path
         *
         * @return  string
         */
        public function getRootPath()
        {
            if (strlen($this->_rootPath) == 0) {
                throw new NestedTree_Exception('Root path not set');
            }

            return $this->_rootPath;
        }


        /**
         * Determine whether or not the root path is set
         *
         * @return  bool
         */
        public function hasRootPath()
        {
            try {
                $this->getRootPath();
                return true;
            }
            catch (Exception $ex) {
                return false;
            }
        }


        /**
         * Set the root node
         *
         * @param   NestedTree_Node $node
         * @return  self
         */
        public function setRootNode(NestedTree_Node $node)
        {
            $this->_rootNode = $node;
            return $this;
        }


        /**
         * Get the root node
         *
         * @throws  NestedTree_Exception    If the root node isn't set
         * @return  NestedTree_Node
         */
        public function getRootNode()
        {
            if (!$this->_rootNode instanceof NestedTree_Node) {
                throw new NestedTree_Exception('Root node not set');
            }

            return $this->_rootNode;
        }


        /**
         * Return whether or not the root node has been set
         *
         * @return  bool
         */
        public function hasRootNode()
        {
            try {
                $this->getRootNode();
                return true;
            }
            catch (Exception $ex) {
                return false;
            }
        }


        /**
         * Set whether or not the root node should be included
         *
         * @param   bool    $flag
         * @return  self
         */
        public function setIncludeSelf($flag = true)
        {
            $this->_includeSelf = (bool) $flag;
            return $this;
        }


        /**
         * Get whether or not to include the root node
         *
         * @return  bool
         */
        public function getIncludeSelf()
        {
            return $this->_includeSelf;
        }


        /**
         * Set the maximum number of levels
         *
         * @param   int     $num
         * @return  self
         */
        public function setMaxLevels($num)
        {
            $this->_maxLevels = max(0, (int) $num);
            return $this;
        }


        /**
         * Get the maximum number of levels
         *
         * @return  int
         */
        public function getMaxLevels()
        {
            return $this->_maxLevels;
        }


        /**
         * Add IDs to exclude
         *
         * @param   int[]
         * @return  self
         */
        public function addExcludeIds($ids)
        {
            foreach (ArrayTools::clean($ids, true) as $id) {
                $this->_excludeIds[$id] = $id;
            }

            return $this;
        }


        /**
         * Get the IDs to exclude
         *
         * @return  int[]
         */
        public function getExcludeIds()
        {
            return $this->_excludeIds;
        }


        /**
         * Add IDs to include
         *
         * @param   int[]
         * @return  self
         */
        public function addIncludeIds($ids)
        {
            foreach (ArrayTools::clean($ids, true) as $id) {
                $this->_includeIds[$id] = $id;
            }

            return $this;
        }


        /**
         * Get the IDs to include
         *
         * @return  int[]
         */
        public function getIncludeIds()
        {
            return $this->_includeIds;
        }


        /**
         * Set the active tree state to fetch nodes for
         *
         * @param   NestedTree_TreeView_TreeState   $treeState
         * @return  self
         */
        public function setTreeState(NestedTree_TreeView_TreeState $treeState)
        {
            $this->_treeState = $treeState;
            return $this;
        }


        /**
         * Get the active tree state
         *
         * @throws  NestedTree_Exception    If the tree state hasn't been set
         * @return  NestedTree_TreeView_TreeState
         */
        public function getTreeState()
        {
            if (!$this->_treeState instanceof NestedTree_TreeView_TreeState) {
                throw new NestedTree_Exception('Tree state not specified');
            }

            return $this->_treeState;
        }


        /**
         * Retrieve a hash to indicate the options used
         *
         * @return  string
         */
        public function getHash()
        {
            $parts = array(
                $this->_rootId,
                $this->_rootPath,
                $this->_includeSelf,
                $this->_maxLevels,
                $this->_excludeIds,
                $this->_includeIds
            );

            if (!is_null($this->_treeState)) {
                $parts[] = $this->_treeState->getHash();
            }
            else {
                $parts[] = null;
            }

            if (!is_null($this->_rootNode)) {
                $parts[] = $this->_rootNode->getId();
            }
            else {
                $parts[] = null;
            }

            return md5(serialize($parts));
        }
    }
?>
