<?php
    /**
     * Class used to hold the state of a tree, including open and selected nodes
     *
     * @author  Quentin Zervaas
     */
    class NestedTree_TreeView_TreeState
    {
        protected $_tree;
        protected $_nodes = array();


        /**
         * Constructor. Set the tree
         *
         * @param   NestedTree_Abstract $tree
         */
        public function __construct(NestedTree_Abstract $tree)
        {
            $this->_tree = $tree;
        }

        /**
         * Get the tree
         *
         * @return  NestedTree_Abstract
         */
        public function getTree()
        {
            return $this->_tree;
        }

        /**
         * Mark one or more nodes as open
         *
         * @param   int|array   $nodeIds    The nodes to mark as open
         * @return  self
         */
        public function openNodes($nodeIds)
        {
            $nodeList = $this->_tree->getNodesFromIds($nodeIds);

            foreach ($nodeList->getChildren() as $node) {
                if (!array_key_exists($node->getId(), $this->_nodes)) {
                    $this->_nodes[$node->getId()] = $node;
                }

                $this->_nodes[$node->getId()]->setIsOpen(true);
            }


            return $this;
        }


        /**
         * Mark one or more nodes as selected
         *
         * @param   int|array   $nodeIds    The nodes to mark as selected
         * @return  self
         */
        public function selectNodes($nodeIds)
        {
            $nodeList = $this->_tree->getNodesFromIds($nodeIds, true);

            foreach ($nodeList->getChildren() as $node) {
                if (!array_key_exists($node->getId(), $this->_nodes)) {
                    $this->_nodes[$node->getId()] = $node;
                }

                $this->_nodes[$node->getId()]->setIsSelected(true);
            }

            return $this;
        }


        /**
         * Add a node to the tree state
         *
         * @param   NestedTree_Node     $node
         * @return  self
         */
        public function add(NestedTree_Node $node)
        {
            $this->_nodes[$node->getId()] = $node;
            return $this;
        }


        /**
         * Return a node list comprising of all open and/or selected nodes
         *
         * @return  NestedTree_NodeList
         */
        public function getNodeList()
        {
            $nodeList = new NestedTree_NodeList();

            foreach ($this->_nodes as $node) {
                if ($node->isOpen() || $node->isSelected()) {
                    $nodeList->add($node);
                }

            }

            return $nodeList;
        }

        public function hasSelectedNodes()
        {
            foreach ($this->_nodes as $node) {
                if ($node->isSelected()) {
                    return true;
                }
            }

            return false;
        }


        /**
         * Retrieve a hash to indicate the options used
         *
         * @return  string
         */
        public function getHash()
        {
            $parts = array();

            foreach ($this->_nodes as $node) {
                $parts[] = sprintf('%d,%d,%d', $node->getId(), $node->selected(), $node->isOpen());
            }

            return md5(serialize($parts));
        }
    }
?>
