<?php
    abstract class Profile implements Iterator
    {
        const ACTION_NONE   = 0;
        const ACTION_UPDATE = 1;
        const ACTION_INSERT = 2;
        const ACTION_DELETE = 3;

        protected $_db          = null;
        protected $_table       = null;
        protected $_keyField    = 'profile_key';
        protected $_valueField  = 'profile_value';

        protected $_filters     = array();
        public $_properties    = array();
        protected $_customTypes = array();

        public function __construct(Db_Adapter_Abstract $db, $table, array $filters = array())
        {
            $this->_db      = $db;
            $this->_table   = $table;
            $this->_filters = $filters;
        }

        public function load()
        {
            $query = sprintf('select %s, %s from %s',
                             $this->_keyField,
                             $this->_valueField,
                             $this->_table);
            if (count($this->_filters) > 0) {
                $filters = array();
                foreach ($this->_filters as $k => $v)
                    $filters[] = $this->_db->quoteInto($k . ' = ?', $v);

                $query .= sprintf(' where %s', join(' and ', $filters));
            }

            $result = $this->_db->fetchPairs($query);

            foreach ($result as $k => $v) {
                $this->_properties[$k] = array('value'  => $v,
                                               'action' => self::ACTION_NONE);
            }
        }

        public function save()
        {
            $this->_db->begin();

            foreach ($this->_properties as $k => $v) {
                switch ($v['action']) {
                    case self::ACTION_DELETE:
                        $where = array();
                        foreach ($this->_filters as $_k => $_v)
                            $where[] = $this->_db->quoteInto($_k . ' = ?', $_v);
                        $where[] = $this->_db->quoteInto($this->_keyField . ' = ?', $k);
                        $this->_db->delete($this->_table, $where);
                        break;

                    case self::ACTION_INSERT:
                        $values = $this->_filters;
                        $values[$this->_keyField] = $k;
                        $values[$this->_valueField] = $v['value'];
                        $this->_db->insert($this->_table, $values);
                        break;

                    case self::ACTION_UPDATE:
                        $where = array();
                        foreach ($this->_filters as $_k => $_v)
                            $where[] = $this->_db->quoteInto($_k . ' = ?', $_v);
                        $where[] = $this->_db->quoteInto($this->_keyField . ' = ?', $k);
                        $this->_db->update($this->_table, array($this->_valueField => $v['value']), $where);
                        break;

                    case self::ACTION_NONE:
                    default:
                        // do nothing
                }

                if ($v['action'] == self::ACTION_DELETE) {
                    unset($this->_properties[$k]);
                    continue;
                }
                $this->_properties[$k]['action'] = self::ACTION_NONE;
            }

            $this->_db->commit();

            return true;
        }

        public function delete()
        {
            $where = array();
            foreach ($this->_filters as $k => $v)
                $where[] = $this->_db->quoteInto($k . ' = ?', $v);

            $this->_db->delete($this->_table, $where);
            $this->_properties = array();
        }

        public function __set($name, $value)
        {
            if (is_null($value)) {
                $this->unsetValue($name);
                return false;
            }

            if (array_key_exists($name, $this->_properties)) {
                $old = $this->_properties[$name]['value'];
                if ($old != $value) {
                    $this->_properties[$name]['value'] = $value;

                    if ($this->_properties[$name]['action'] != self::ACTION_INSERT) {
                        if (!array_key_exists('old', $this->_properties[$name])) {
                            $this->_properties[$name]['old'] = $old;
                        }
                        $this->_properties[$name]['action'] = self::ACTION_UPDATE;
                    }
                }
            }
            else {
                $this->_properties[$name] = array('value'  => $value,
                                                  'action' => self::ACTION_INSERT);
            }

            return true;
        }

        public function set($name, $value)
        {
            return $this->__set($name, $value);
        }

        public function __get($name)
        {
            if (array_key_exists($name, $this->_properties))
                return $this->_properties[$name]['value'];

            // now do a case-insensitive search
            $name = strtolower($name);

            foreach ($this->_properties as $k => $v) {
                if (strtolower($k) == $name)
                    return $v['value'];
            }

            return null;
        }

        public function get($name, $default = null)
        {
            $name = Profile_Field::getFormKey($name);

            $ret = $this->__get($name);
            if (!is_null($ret))
                return $ret;

            return $default;
        }

        public function __isset($name)
        {
            return $this->has($name);
        }

        public function updated($name)
        {
            if (!array_key_exists($name, $this->_properties)) {
                return false;
            }

            return $this->_properties[$name]['action'] == self::ACTION_UPDATE;
        }

        public function getOldValue($name)
        {
            if (!$this->has($name)) {
                return null;
            }

            if (array_key_exists('old', $this->_properties[$name])) {
                return $this->_properties[$name]['old'];
            }

            return $this->get($name);
        }

        public function has($name)
        {
            if (array_key_exists($name, $this->_properties)) {
                return $this->_properties[$name]['action'] != self::ACTION_DELETE;
            }

            $name = strtolower($name);
            foreach ($this->_properties as $k => $v) {
                if ($name == strtolower($k))
                    return $this->_properties[$name]['action'] != self::ACTION_DELETE;
            }

            return false;
        }

        public function __unset($name)
        {
            if (!array_key_exists($name, $this->_properties))
                return;
            $action = $this->_properties[$name]['action'];

            switch ($action)
            {
                case self::ACTION_NONE:
                case self::ACTION_UPDATE:
                    $this->_properties[$name]['action'] = self::ACTION_DELETE;
                    break;

                case self::ACTION_INSERT:
                    unset($this->_properties[$name]);
                    break;

                case self::ACTION_DELETE:
                default:
            }
        }

        public function unsetValue($name)
        {
            return $this->__unset($name);
        }

        public function getTable()
        {
            return $this->_table;
        }

        public function getKeyField()
        {
            return $this->_keyField;
        }

        public function getValueField()
        {
            return $this->_valueField;
        }

        public function getFilters()
        {
            return $this->_filters;
        }

        public function registerType($name, $class)
        {
            $this->_customTypes[$name] = $class;
        }

        public function getCustomTypes()
        {
            return $this->_customTypes;
        }

        public function getValues()
        {
            $ret = array();

            foreach ($this->_properties as $k => $v) {
                if ($v['action'] == self::ACTION_DELETE) {
                    continue;
                }

                $ret[$k] = $v['value'];
            }

            return $ret;
        }

        // ITERATOR INTERFACE METHODS

        public function rewind()
        {
            reset($this->_properties);
        }

        public function current()
        {
            $ret = current($this->_properties);
            if (!$ret)
                return false;
            return $ret['value'];
        }

        public function key()
        {
            return key($this->_properties);
        }

        public function next()
        {
            $ret = next($this->_properties);
            if (!$ret)
                return false;

            return $ret['value'];
        }

        public function valid()
        {
            return $this->current() !== false;
        }

        public function size()
        {
            return count($this->_properties);
        }

        public function xmlExport(DOMDocument $doc, DOMElement $root, $field = 'profile', $itemField = 'property')
        {
            if (count($this->_properties) == 0) {
                return;
            }

            $node = $doc->createElement($field);
            $root->appendChild($node);

            foreach ($this->_properties as $k => $v) {
                if ($v['action'] == self::ACTION_DELETE) {
                    continue;
                }

                $itemNode = $doc->createElement($itemField);
                $itemNode->setAttribute('key', $k);
                $itemNode->appendChild($doc->createCDATASection($v['value']));

                $node->appendChild($itemNode);
            }
        }

        public function xmlImport($xml, $field = 'property')
        {
            if (!isset($xml->$field)) {
                return;
            }

            foreach ($xml->$field as $node) {
                $name = (string) $node['key'];

                $this->set($name, (string) $node);
            }
        }

        public static function BuildMultiple($db, $class, $filters)
        {
            if (!class_exists($class))
                throw new Exception('Undefined class specified: ' . $class);

            $obj = new $class($db);

            if (!$obj instanceof Profile)
                throw new Exception('Class does not extend from Profile');

            $fields = array_keys($filters);
            $fields[] = $obj->_keyField;
            $fields[] = $obj->_valueField;

            $select = $db->select();
            $select->from($obj->_table, $fields);

            foreach ($filters as $field => $value) {
                $select->where($field . ' in (?)', $value);
            }

            $data = $db->fetchAll($select);

            $ret = array();

            foreach ($data as $row) {
                $key = array();
                foreach ($filters as $field => $value)
                    $key[] = $row[$field];

                $key = join(',', $key);
                if (!array_key_exists($key, $ret))
                    $ret[$key] = new $class($db);

                $k = $row[$obj->_keyField];
                $v = $row[$obj->_valueField];
                $ret[$key]->_properties[$k] = array('value'  => $v,
                                                    'action' => self::ACTION_NONE);

                foreach ($filters as $field => $value) {
                    $ret[$key]->_filters[$field] = $row[$field];
                }
            }

            return $ret;
        }
    }
?>