<?php
    /**
     * Profile_Field
     *
     * A generic class for different profile field types. This allows
     * easy get/set/check on the different types, such as for
     * validating email addresses, etc.
     */
    abstract class Profile_Field
    {
        protected $options   = array();
        protected $_value    = null;
        protected $_type     = null;
        protected $_params   = array();

        public function __construct($options = array())
        {
            $this->_type = preg_replace('/^Driver_forms_fields_/', '', get_class($this));
            $this->options = $options;
        }

        final public function __clone()
        {
            $this->_value = null;
        }

        final public function getType()
        {
            return $this->_type;
        }

        abstract public function __toString();

        public function getInputTemplate($name, $attrs = array())
        {
            return self::GetDefaultInputTemplate($name, $this->get(), $attrs);
        }

        public function getFilterInputTemplate($name, $value)
        {
            $attrs = array(
                'name'  => $name,
                'type'  => 'text',
                'class' => 'profile-string',
                'value' => $value
            );

            $element = new HTML_Element('input', $attrs);

            return (string) $element;
        }

        public function getBulkInputTemplate($name)
        {
            return $this->getInputTemplate($name);
        }

        public function getOptions()
        {
            return new Profile_Field_FieldList();
        }

        public function setParams($params)
        {
            if (!is_array($params)) {
                return;
            }

            foreach ($params as $k => $v) {
                $this->setParam($k, $v);
            }

            return $this;
        }

        public function getParams()
        {
            $ret = array();

            foreach ($this->_params as $k => $v) {
                $ret[$k] = $this->getParam($k);
            }

            return $ret;
        }

        public function getParam($key, $default = null)
        {
            return coalesce($this->_params[$key], $default);
        }

        public function setParam($key, $val)
        {
            $this->_params[$key] = $val;
            return $this;
        }

        public function canBeUsedInSummary()
        {
            return true;
        }

        /**
         * set
         *
         * Set the value for the setting
         */
        public function set($val)
        {
            if (is_array($val)) {
                foreach ($val as $k => $v)
                    $val[$k] = FormProcessor::sanitize($v);
            }
            else {
                $val = FormProcessor::sanitize($val);
            }

            if ($this->isValid($val)) {
                $this->_value = $val;
                return true;
            }
            return false;
        }

        public function init($val)
        {
            $this->_value = $val;
        }

        /**
         * clear
         *
         * Clear the stored value
         */
        public function clear()
        {
            $this->_value = null;
        }


        /**
         * import
         *
         * Set the value as it was recorded during the export process (such as in a client dump)
         * Usually this will just be the normal set method but in some cases the value will first
         * need some modification
         */
        public function import($val)
        {
            return $this->set($val);
        }


        /**
         * isEmpty
         *
         * Check whether or not the value is empty
         */
        public function isEmpty($val)
        {
            if (is_array($val)) {
                return count(array_filter($val)) == 0;
            }

            return strlen(trim($val)) == 0;
        }

        /**
         * get
         *
         * Get the value as it should be written to the profile
         */
        public function get()
        {
            return $this->_value;
        }

        /**
         * export
         *
         * Get the value as it should be written when exporting to a client dump.
         * Typically this will just be the same as the normal get() method but in
         * some special cases a transformation may need to occur
         */
        public function export()
        {
            return $this->get();
        }

        /**
         * preview
         *
         * Return a pretty version of the setting "human consumption"
         */
        public function preview()
        {
            return $this->_value;
        }


        /**
         * Preview multiple values at once. Values should be returned
         * with the key they were passed in with
         *
         * @param   array   The values to preview
         * @return  array
         */
        public function previewBulk($vals)
        {
            $ret = array();

            foreach ($vals as $k => $v) {
                $this->set($v);
                $ret[$k] = $this->preview();
            }

            return $ret;
        }


        /**
         * Get multiple values for the field. This is used to return
         * custom data based on the selected value for the given field type
         *
         * @return  array
         */
        public function getClientTemplateData()
        {
            return array();
        }

        public function getClientTemplateDataBulk($vals)
        {
            $ret = array();

            foreach ($vals as $k => $v) {
                $this->set($v);
                $ret[$k] = $this->getClientTemplateData();
            }

            return $ret;
        }


        /**
         * prepareValue
         *
         * Prepare the value so it can easily be edited
         */
        public function prepareValue($val)
        {
            return $val;
        }

        /**
         * prepare
         *
         * Get data required for editing this value
         */
        public function prepare()
        {
            return array();
        }

        /**
         * isValid
         */
        public function isValid($val)
        {
            return true;
        }

        /**
         * getFormFields
         *
         * Returns an array of form fields to fetch values for.
         * This is primarily for field types with multiple inputs
         */
        public function getFormFields()
        {
            return array();
        }


        /**
         * isPublic
         *
         * This method is used to determine whether or not the field type
         * can be used in custom field lists
         */
        public function isPublic()
        {
            return true;
        }


        /**
         * getComparisonOperators
         *
         * Returns an array containing the different ways two
         * values of this type can be compared.
         */
        public function getComparisonOperators()
        {
            return array(
                'like'   => 'like',
                'eq'     => 'equals',
                'neq'    => "doesn't equal",
                'empty'  => 'empty',
                'nempty' => 'not empty'
            );
        }

        /**
         * compare
         *
         * Compare two different values using the given
         * operator. This should be an operator that is returned
         * by the getComparisonOperators method
         *
         * @see     getComparisonOperators
         */
        public function compare($operator, $a, $b)
        {
            $operator = strtolower($operator);

            switch ($operator) {
                case 'eq':
                    return $a === $b;

                case 'neq':
                    return $a !== $b;
            }

            throw new Exception('Invalid operator for ' . get_class($this));
        }


        public function compareInSql(Zend_Db_Select $select, $operator, $field, $values)
        {
            $operator = strtolower($operator);

            $vals = array();

            foreach (ArrayTools::Clean($values) as $val) {
                $val = str_replace('%', '', $val);
                $vals[] = strtolower($val);
            }

            $vals = ArrayTools::Clean($vals);

            switch ($operator) {
                case 'eq':
                    if (count($vals) == 0) {
                        $select->where('1 = 0');
                    }
                    else {
                        $select->where("lower(coalesce(" . $field . ", '')) in (?)", $vals);
                    }
                    return;

                case 'neq':
                    if (count($vals) == 0) {
                        ; // nothing to do
                    }
                    else {
                        $select->where("lower(coalesce(" . $field . ", '')) not in (?)", $vals);
                    }
                    return;

                case 'like':
                    $wheres = array();
                    $db = $select->getAdapter();

                    foreach ($vals as $val) {
                        $wheres[] = $db->quoteInto("lower(coalesce(" . $field . ", '')) like ?", '%' . $val . '%');
                    }

                    if (count($wheres) > 0) {
                        $select->where(join(' or ', $wheres));
                    }
                    else {
                        $select->where('1 = 0');
                    }
                    return;

                case 'empty':
                    $select->where("length(trim(coalesce(" . $field . ", ''))) = 0");
                    return;

                case 'nempty':
                    $select->where("length(trim(coalesce(" . $field . ", ''))) > 0");
                    return;
            }

            throw new Exception('Invalid operator for ' . get_class($this));
        }

        /**
         * Factory
         */
        public function Factory($type, array $params = array(), array $customTypes = array())
        {
            if ($type instanceof self) {
                return $type;
            }
            else if ($type instanceof Profile_Field_FieldList_Field) {
                return $type->getType();
            }

            $type = (string) $type;

            if (strlen($type) == 0) {
                throw new Exception("Field type not specified");
            }

            if (is_array($customTypes) && array_key_exists($type, $customTypes)) {
                $class = $customTypes[$type];
            }
            else {
                $class = 'Driver_forms_fields_' . strtolower($type);
            }

            $path = sprintf('%s/%s.php', self::GetDriverPath(), strtolower($type));

            if (file_exists($path) && is_file($path)) {
                require_once($path);

                if (class_exists($class, false)) {
                    try {
                        $obj = new $class($params);

                        if ($obj instanceof self) {
                            return $obj;
                        }
                    }
                    catch (Exception $ex) { }
                }
            }

            // couldn't load by name, make a last ditch effort to load by type title

            $types = self::GetAvailableTypes();

            foreach ($types as $t) {
                $title = (string) $t;

                if ($title == $type) {
                    return $t;
                }
            }

            throw new Exception(sprintf("Profile_Field type '%s' not found", $type));
        }

        public static function GetAvailableTypes($publicOnly = false)
        {
            $ret = array();

            foreach (glob(self::GetDriverPath() . '/*.php') as $fullpath) {
                $type = substr(basename($fullpath), 0, -4);
                $class = 'Driver_forms_fields_' . $type;

                $obj = new $class;
                if ($publicOnly && !$obj->isPublic()) {
                    ; // skip
                }
                else {
                    $ret[$type] = new $class;
                }
            }

            return $ret;
        }

        public static function getFormKey($key)
        {
            return strtolower(preg_replace('/[^a-z0-9]/i', '_', $key));
        }

        public function getIndexableField($name)
        {
            return Zend_Search_Lucene_Field::Text(self::getFormKey($name), $this->preview());
        }

        public function getDatabaseType()
        {
            $helper = Application::GetDb()->getManagerHelper();
            return $helper->getFieldTypeName(Db_Schema_Table_Field::TYPE_TEXT);
        }

        public function getDatabaseDefault()
        {
            return new Zend_Db_Expr("''");
        }

        public static function GetDriverPath()
        {
            return sprintf('%s/forms/fields', Application_Driver_Manager::GetPath());
        }


        /**
         * Whether or not the type is a "core" type, or just an occasionally used type
         *
         * @return  bool
         */
        public function isCore()
        {
            return false;
        }


        /**
         * Create array data for managing a field of this type in a field list
         *
         * @return  array
         */
        public function toManagerArray()
        {
            $ret = array(
                'type'      => $this->getType(),
                'typelabel' => (string) $this,
                'o' => array()
            );

            foreach ($this->getOptions() as $field) {
                $ret['o'][] = $field->toManagerArray();
            }

            return $ret;
        }

        public static function GetDefaultInputTemplate($name, $value, $attrs = array())
        {
            if (!is_array($attrs)) {
                $attrs = array();
            }

            $attrs['type']  = 'text';
            $attrs['name']  = $name;
            $attrs['value'] = $value;

            return (string) new HTML_Element('input', $attrs);
        }
    }
?>
