<?php
    class Profile_Field_FieldList_Field
    {
        protected $_type;
        protected $_title;
        protected $_name;
        protected $_index_name; // used when getting client template data
        protected $_ranking = 0;
        protected $_params = array();
        protected $_errors = array();

        public function __construct($type)
        {
            $this->setType($type);
        }

        public function __clone()
        {
            $this->_type   = clone $this->_type;
            $this->_errors = array();
        }

        public function __toString()
        {
            return $this->_title;
        }

        public function setType($type)
        {
            if (!$type instanceof Profile_Field) {
                try {
                    $type = Profile_Field::factory($type);
                }
                catch (Exception $ex) {
                    // this is a fallback so the field can still be constructed if the type is missing
                    $type = Profile_Field::factory('string');
                }
            }

            $this->_type = $type;
        }

        public function getType()
        {
            return $this->_type;
        }

        public function getTypeLabel()
        {
            return (string) $this->getType();
        }

        public function setTitle($title)
        {
            $this->_title = $title;
            return $this;
        }

        public function getTitle()
        {
            return $this->_title;
        }

        public function setName($name)
        {
            $this->_name = Profile_Field::getFormKey($name);
            return $this;
        }

        public function getName()
        {
            if (strlen($this->_name) == 0)
                return Profile_Field::getFormKey($this->getTitle());

            return $this->_name;
        }

        public function setIndexName($name)
        {
            $this->_index_name = $name;
            return $this;
        }

        public function getIndexName()
        {
            return $this->_index_name;
        }

        public function setRequired($bool)
        {
            $this->_params['required'] = (bool) $bool;
            return $this;
        }

        public function getRequired()
        {
            return (bool) coalesce($this->_params['required'], false);
        }

        public function setRanking($ranking)
        {
            $this->_ranking = (int) $ranking;
        }

        public function getRanking()
        {
            return $this->_ranking;
        }

        public function set($vals)
        {
            return $this->_type->set($vals);
        }

        public function get()
        {
            return $this->_type->get();
        }

        public function preview()
        {
            return $this->_type->preview();
        }

        public function setHelp($string)
        {
            $this->setParam('help', Text::Sanitize($string));
            return $this;
        }

        public function getHelp()
        {
            return $this->getParam('help');
        }

        public function setProtected($bool)
        {
            $this->setParam('protected', (bool) $bool);
            return $this;
        }

        public function getProtected()
        {
            return (bool) $this->getParam('protected', false);
        }

        public function setNoEdit($bool)
        {
            $this->setParam('noedit', (bool) $bool);
            return $this;
        }

        public function getNoEdit()
        {
            return (bool) $this->getParam('noedit', false);
        }

        public function setParams($params)
        {
            if (!is_array($params))
                return $this;

            if (array_key_exists('required', $params)) {
                $this->setRequired($params['required']);
                unset($params['required']);
            }

            foreach ($params as $k => $v) {
                $this->setParam($k, $v);
            }

            return $this;
        }

        public function setParam($k, $v)
        {
            $this->_type->setParam($k, $v);
            $this->_params[$k] = $v;

            return $this;
        }

        public function getParam($k, $default = null)
        {
            return coalesce($this->_params[$k], $default);
        }

        public function getParams()
        {
            return $this->_params;
        }

        public function getParamsAsJson()
        {
            $ret = $this->getParams();
            if (count($ret) == 0)
                $ret = '{}';
            else
                $ret = Zend_Json::encode($ret);

            return $ret;
        }

        public function getIndexableField($name)
        {
            return $this->getType()->getIndexableField($this->getName());
        }

        public function addError($error)
        {
            $this->_errors[] = $error;
            return $this;
        }

        public function hasError()
        {
            return count($this->_errors) > 0;
        }

        public function getErrors()
        {
            return $this->_errors;
        }

        public function clearErrors()
        {
            $this->_errors = array();
        }

        public function toArray()
        {
            $type = $this->getType();

            $ret = array(
                'type'      => $type->getType(),
                'typeTitle' => (string) $type,
                'name'      => $this->getName(),
                'title'     => $this->getTitle(),
                'required'  => $this->getRequired(),
                'help'      => $this->getHelp(),
                'protected' => $this->getProtected(),
                'noedit'    => $this->getNoEdit()
            );

            foreach ($this->getParams() as $k => $v) {
                $ret['option_' . $k] = $v;
            }

            return $ret;
        }


        /**
         * Data backup routines
         */

        public function xmlExport(DOMDocument $doc, DOMElement $root, $exportValue = false)
        {
            $type = $this->getType();

            $node = $doc->createElement('field');
            $node->setAttribute('xmlns', 'http://www.recite.com.au/schema/backup/module:forms/fieldlist');

            $n = $doc->createElement('type');
            $n->appendChild($doc->createCDATASection($type->getType()));
            $node->appendChild($n);

            $n = $doc->createElement('title');
            $n->appendChild($doc->createCDATASection($this->getTitle()));
            $node->appendChild($n);

            $n = $doc->createElement('name');
            $n->appendChild($doc->createCDATASection($this->getName()));
            $node->appendChild($n);

            foreach ($this->getParams() as $k => $v) {
                $param = $doc->createElement('param');

                $n = $doc->createElement('name');
                $n->appendChild($doc->createCDATASection($k));
                $param->appendChild($n);

                $n = $doc->createElement('value');
                $n->appendChild($doc->createCDATASection($v));
                $param->appendChild($n);

                $node->appendChild($param);
            }

            if ($exportValue) {
                $n = $doc->createElement('value');

                $value = $type->export();

                if ($value instanceof Profile_Field_ComplexValue) {
                    $value->exportToXmlNode($doc, $n);
                }
                else {
                    $n->appendChild($doc->createCDATASection($value));
                }

                $node->appendChild($n);
            }

            $root->appendChild($node);
        }

        public function xmlExportProfileValues(DOMDocument $doc, DOMElement $root, Profile $profile, $field = 'field')
        {
            $type = clone $this->getType();
            $type->set($profile->get($this->getName()));

            $node = $doc->createElement($field);

            $n = $doc->createElement('type');
            $n->appendChild($doc->createCDATASection($type->getType()));
            $node->appendChild($n);

            $n = $doc->createElement('title');
            $n->appendChild($doc->createCDATASection($this->getTitle()));
            $node->appendChild($n);

            $n = $doc->createElement('name');
            $n->appendChild($doc->createCDATASection($this->getName()));
            $node->appendChild($n);

            foreach ($this->getParams() as $k => $v) {
                $param = $doc->createElement('param');

                $n = $doc->createElement('name');
                $n->appendChild($doc->createCDATASection($k));
                $param->appendChild($n);

                $n = $doc->createElement('value');
                $n->appendChild($doc->createCDATASection($v));
                $param->appendChild($n);

                $node->appendChild($param);
            }

            $n = $doc->createElement('value');

            $value = $type->export();

            if ($value instanceof Profile_Field_ComplexValue) {
                $value->exportToXmlNode($doc, $n);
            }
            else {
                $n->appendChild($doc->createCDATASection($value));
            }

            $node->appendChild($n);

            $root->appendChild($node);
        }

        public static function FromXml($xml)
        {
            $ret = new self((string) $xml->type);
            $ret->setTitle((string) $xml->title);

            if (isset($xml->name)) {
                $name = (string) $xml->name;
                if (strlen($name) > 0) {
                    $ret->setName($name);
                }
            }

            if (isset($xml->value)) {
                $type = $ret->getType();

                if (isset($xml->value['type']) && $xml->value['type'] == 'complex') {
                    $type->import(
                        Profile_Field_ComplexValue::FromXml($xml->value)
                    );
                }
                else {
                    $type->import((string) $xml->value);
                }
            }

            foreach ($xml->param as $param) {
                $ret->setParam((string) $param->name, (string) $param->value);
            }

            return $ret;
        }

        public static function FromXmlForProfile($xml, Profile $profile)
        {
            $field = new self((string) $xml->type);
            $field->setTitle((string) $xml->title);

            foreach ($xml->param as $param) {
                $field->setParam((string) $param->name, (string) $param->value);
            }

            if (isset($xml->name)) {
                $name = (string) $xml->name;
                if (strlen($name) > 0) {
                    $field->setName($name);
                }
            }

            $type = $field->getType();

            if (isset($xml->value['type']) && $xml->value['type'] == 'complex') {
                $type->import(
                    Profile_Field_ComplexValue::FromXml($xml->value)
                );
            }
            else {
                $type->import((string) $xml->value);
            }

            $profile->set($field->getName(), $type->get());
        }
    }
?>