<?php
    /**
     * Text
     *
     * Various methods used to manipulate strings
     *
     * @author  Quentin Zervaas
     */
    class Text
    {
        /**
         * asciify
         *
         * Convert a string so it only uses ascii characters
         *
         * @param   string  $str    The string to convert
         * @return  string          A string of ascii characters
         */
        public static function asciify($str)
        {
            $unicode_array = array(
                "&#8211;" => "-",
                "&#8212;" => "-",
                ""       => "-",
                "&#8216;" => "'",
                ""       => "'",
                "&#8217;" => "'",
                ""       => "'",
                "&#8230;" => "...",
                ""       => "...",
                ""       => "\"",
                "&#8220;" => "\"",
                ""       => "\"",
                "&#8221;" => "\"",
                ''       => '"',
                ''       => '"'
            );
            $str = strtr($str, $unicode_array);
            return strtr($str, "",
                               "SOZsozYYuAAAAAAACEEEEIIIIDNOOOOOOUUUUYsaaaaaaaceeeeiiiionoooooouuuuyy");
        }

        /**
         * urlize
         *
         * Make a string URL friendly
         *
         * @param   string  $val        The string to convert
         * @param   string  $separators
         * @return  string              The urlized string
         */
        public static function urlize($val, $separators = '-')
        {
            $separator = $separators{0};

            $sepRegex = preg_quote($separators, '/');

            $val = strtolower(self::asciify($val));
            $val = str_replace('&', 'and', $val);
            $val = str_replace("'", '', $val);
            $val = preg_replace('/[^a-z0-9' . $sepRegex . ']+/', $separator, $val);
            $val = preg_replace('/[^a-z0-9' . $sepRegex . ']$/', '', $val);
            return $val;
        }


        /**
         * prettify
         *
         * Clean up a string so things like underscores are removed and words
         * are capitalized
         *
         * @todo    Detect acronyms or things like 'McDonald'
         * @param   string  $str    The string to prettify
         * @return  string          The prettified string
         */
        public static function prettify($str)
        {
            $str = preg_replace('/_+/', ' ', $str);
            return ucwords(strtolower($str));
        }


        /**
         * addSlashes
         *
         * Add slashes based on the magic_quotes_gpc setting
         */
        public static function addSlashes($str)
        {
            return (get_magic_quotes_gpc()) ? $str : addslashes($str);
        }


        /**
         * stripSlashes
         *
         * Strip slashes based on the magic_quotes_gpc setting
         */
        public static function stripSlashes($str)
        {
            return (get_magic_quotes_gpc()) ? stripslashes($str) : $str;
        }

        /**
         * toBoolean
         *
         * Convert a string such as "Yes" or "True" or something similar
         * to a boolean value
         *
         * @param   string  $val    The string to convert to a boolean
         * @return  bool            The resultant booleant value
         */
        public static function toBoolean($val)
        {
            if (is_bool($val))
                return $val;

            $val = trim($val);

            if (strlen($val) == 0)
                return false;

            if (preg_match('/^[0-9]+$/', $val)) {
                $val = (int) $val;
                return $val != 0;
            }

            $first = strtolower($val{0});

            return $first == 't' || $first == 'y';
        }


        /**
         * sanitize
         *
         * Basic sanitization function that trims and strips HTML tags. For
         * more advanced HTML filtering a different algorithm would be required
         * such as Zend_Filter
         *
         * @todo    Change this to use Zend_Filter, or remove this altogether
         * @todo    Change calls to this from form processors to use form processor sanitization instead
         * @see     Zend_Filter
         */
        public static function sanitize($str, $keepTags = false)
        {
            if ($keepTags) {
                return trim(self::stripSlashes($str));
            }

            return trim(strip_tags(self::stripSlashes($str)));
        }

        public static function Truncate($string, $length = 80, $etc = '...', $break_words = false, $middle = false)
        {
            if ($length == 0) {
                return '';
            }

            if (strlen($string) > $length) {
                $length -= min($length, strlen($etc));
                if (!$break_words && !$middle) {
                    $string = preg_replace('/\s+?(\S+)?$/', '', substr($string, 0, $length+1));
                }
                if(!$middle) {
                    return substr($string, 0, $length) . $etc;
                }
                else {
                    return substr($string, 0, $length/2) . $etc . substr($string, -$length/2);
                }
            }
            else {
                return $string;
            }
        }
    }
?>