<?php
    /**
     *  Copyright 2005 Zervaas Enterprises (www.zervaas.com.au)
     *
     *  Licensed under the Apache License, Version 2.0 (the "License");
     *  you may not use this file except in compliance with the License.
     *  You may obtain a copy of the License at
     *
     *      http://www.apache.org/licenses/LICENSE-2.0
     *
     *  Unless required by applicable law or agreed to in writing, software
     *  distributed under the License is distributed on an "AS IS" BASIS,
     *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
     *  See the License for the specific language governing permissions and
     *  limitations under the License.
     */

    /**
     * Wizard
     *
     * A class to manage multi-step forms or wizards. This involves managing
     * the various steps, storing its values and switching between each
     * step
     *
     * @author  Quentin Zervaas
     */
    abstract class Wizard implements FormProcessor_Interface
    {
        // whether or not all steps of the form are complete
        protected $_complete = false;

        // internal array to store the various steps
        protected $_steps = array();

        // the current step
        private $_currentStep = null;

        // options to use for the wizard
        public $options = array('redirectAfterPost' => true);

        // action that resets the container
        protected $_resetAction = '__reset';

        protected $containers;


        public function __construct()
        {
            $namespace = get_class($this);

            $this->containers = array('values' => new Zend_Session_Namespace($namespace),
                                      'errors' => new Zend_Session_Namespace($namespace . '_errors'),
                                      'status' => new Zend_Session_Namespace($namespace . '_status'));

            if (!isset($this->containers['status']->fresh))
                $this->containers['status']->fresh = true;
        }


        /**
         * process
         *
         * Processes the form for the specified step. If the processed step
         * is complete, then the wizard is set to use the next step. If this
         * is the initial call to process, then the wizard is set to use the
         * first step. Once the next step is determined, the prepare method
         * is called for the step. This has the method name prepare_[step name]()
         *
         * @todo    Need a way to jump between steps, e.g. from step 2 to 4 and validating all data
         * @param   string  $action     The step being processed. This should correspond
         *                              to a step created in addStep()
         * @param   Zend_Controller_Request_Abstract $request   The client request
         * @param   bool    $process    True if the step is being processed, false if being prepared
         */
        public function process($action, Zend_Controller_Request_Abstract $request, $process = true)
        {
            if ($request->getPost($this->_resetAction)) {
                $this->clearContainer();
                $this->setCurrentStep($this->getFirstIncompleteStep());
            }
            else if ($request->getPost('previous') && !$this->isFirstStep()) {
                $this->_clearErrors();

                $this->setCurrentStep($this->getPreviousStep($this->getExpectedStep()));
                $this->doRedirect();
            }
            else {
                $proceed = false;

                // check if the step to be processed is valid
                if (strlen($action) == 0)
                    $action = $this->getExpectedStep();

                if ($this->stepCanBeProcessed($action)) {
                    if ($this->getStepNumber($action) <= $this->getStepNumber($this->getExpectedStep()))
                        $proceed = true;
                    else
                        $proceed = false;
                }

                if ($proceed) {

                    if ($process) {
                        $this->containers['status']->fresh = false;

                        $this->_clearErrors();

                        // processing callback must exist and validate to proceed
                        $callback = 'process_' . $action;
                        $complete = $this->$callback($request);

                        // php < 5.2.1 session bug hack
                        $steps = $this->containers['status']->steps;
                        $steps[$action] = $complete;
                        $this->containers['status']->steps = $steps;

                        if ($complete)
                            $this->setCurrentStep($this->getFollowingStep($action)); // all ok, go to next step
                        else
                            $this->setCurrentStep($action); // error occurred, redo step

                        // final processing once complete
                        if ($this->isComplete())
                            $this->completeCallback();

                        $this->doRedirect();
                    }
                    else
                        $this->setCurrentStep($action);
                }
                else // when initally starting the wizard
                    $this->setCurrentStep($this->getFirstIncompleteStep());
            }

            // setup any required data for this step
            $callback = 'prepare_' . $this->getStepName();
            if (method_exists($this, $callback))
                $this->$callback();

        }


        /**
         * completeCallback
         *
         * Function to run once the final step has been processed and is valid.
         * This should be overwritten in child classes
         */
        abstract protected function completeCallback();


        public function doRedirect()
        {
            if ($this->options['redirectAfterPost']) {
                $redir = $_SERVER['REQUEST_URI'];
                $redir = preg_replace('/\?.*$/', '', $redir);

                header('Location: ' . $redir);
                exit;
            }
        }
        /**
         * isComplete
         *
         * Check if the form is complete. This can only be properly determined
         * after process() has been called.
         *
         * @return  bool    True if the form is complete and valid, false if not
         */
        public function isComplete()
        {
            return $this->_complete;
        }


        /**
         * setCurrentStep
         *
         * Sets the current step in the form. This should generally only be
         * called internally but you may have reason to change the current
         * step.
         *
         * @param   string  $step   The step to set as current
         */
        protected function setCurrentStep($step)
        {
            if (is_null($step) || !$this->stepExists($step)) {
                $this->_complete = true;
                $this->containers['status']->expected = null;
            }
            else {
                $this->_currentStep = $step;
                $this->containers['status']->expected = $step;
            }
        }


        protected function getExpectedStep()
        {
            $step = $this->containers['status']->expected;

            if ($this->stepExists($step))
                return $step;
            return null;
        }


        /**
         * stepExists
         *
         * Check if the given step exists
         *
         * @param   string  $stepname   The name of the step to check for
         * @return  bool                True if the step exists, false if not
         */
        public function stepExists($stepname)
        {
            return array_key_exists($stepname, $this->_steps);
        }


        /**
         * getStepName
         *
         * Get the name of the current step
         *
         * @return  string  The name of the current step
         */
        public function getStepName()
        {
            return $this->_currentStep;
        }


        /**
         * getStepNumber
         *
         * Gets the step number (from 1 to N where N is the number of steps
         * in the wizard) of the current step
         *
         * @param   string  $step   Optional. The step to get the number for. If null then uses current step
         * @return  int             The number of the step. 0 if something went wrong
         */
        public function getStepNumber($step = null)
        {
            $steps = array_keys($this->_steps);
            $numSteps = $this->getNumSteps();

            if (strlen($step) == 0)
                $step = $this->getStepName();

            $ret = 0;
            for ($n = 1; $n <= $numSteps && $ret == 0; $n++) {
                if ($step == $steps[$n-1])
                    $ret = $n;
            }
            return $ret;
        }


        public function stepCanBeProcessed($step)
        {
            $steps = array_keys($this->_steps);
            $numSteps = count($steps);

            for ($i = 0; $i < $numSteps; $i++) {
                $_step = $steps[$i];
                if ($_step == $step)
                    break;

                if (!$this->containers['status']->steps[$_step])
                    return false;
            }
            return true;
        }


        /**
         * getStepProperty
         *
         * Retrieve a property for a given step. At this stage, the only
         * property steps have is a title property.
         *
         * @param   string  $key        The key to get a property for
         * @param   mixed   $default    The value to return if the key isn't found
         * @return  mixed               The property value or the default value
         */
        public function getStepProperty($key, $default = null)
        {
            $step = $this->getStepName();
            if (isset($this->_steps[$step][$key]))
                return $this->_steps[$step][$key];
            return $default;
        }


        /**
         * getFirstStep
         *
         * Get the step name of the first step
         *
         * @return  string  The name of the first step, or null if no steps
         */
        public function getFirstStep()
        {
            $steps = array_keys($this->_steps);
            return count($steps) > 0 ? $steps[0] : null;
        }


        public function getFirstIncompleteStep()
        {
            $steps = array_keys($this->_steps);
            $numSteps = count($steps);

            for ($i = 0; $i < $numSteps; $i++) {
                $_step = $steps[$i];

                if (!isset($this->containers['status']->steps) || !$this->containers['status']->steps[$_step])
                    return $_step;
            }
            return null;
        }


        /**
         * getPreviousStep
         *
         * Gets the step name of the previous step. If the current
         * step is the first step, then null is returned
         *
         * @return  string  The name of the previous step, or null
         */
        public function getPreviousStep($step)
        {
            $ret = null;
            $steps = array_keys($this->_steps);

            $done = false;
            foreach ($steps as $s) {
                if ($s == $step) {
                    $done = true;
                    break;
                }
                $ret = $s;
            }

            return $ret;
        }


        /**
         * getFollowingStep
         *
         * Get the step name of the next step. If the current
         * step is the last step, returns null
         *
         * @return  string  The name of the next step, or null
         */
        public function getFollowingStep($step)
        {
            $ret = null;
            $steps = array_keys($this->_steps);

            $ready = false;
            foreach ($steps as $s) {
                if ($s == $step)
                    $ready = true;
                else if ($ready) {
                    $ret = $s;
                    break;
                }
            }

            return $ret;
        }


        /**
         * addStep
         *
         * Adds a step to the wizard
         *
         * @param   string  $stepname   The name of the step
         * @param   string  $title      The title of the current step
         */
        public function addStep($stepname, $title)
        {
            if (array_key_exists($stepname, $this->_steps)) {
                $this->addError('step', 'Step with name ' . $stepname . ' already exists');
                return;
            }

            $this->_steps[$stepname] = array('title' => $title);


            if (!isset($this->containers['status']->steps))
                $this->containers['status']->steps = array();

            if (!array_key_exists($stepname, $this->containers['status']->steps)) {
                // ugly hack for pre 5.2.1 with arrays in session
                $steps = $this->containers['status']->steps;
                $steps[$stepname] = false;
                $this->containers['status']->steps = $steps;
            }
        }

        public function getSteps()
        {
            return $this->_steps;
        }


        /**
         * isFirstStep
         *
         * Check if the current step is the first step
         *
         * @return  bool    True if the current step is the first step
         */
        public function isFirstStep()
        {
            $steps = array_keys($this->_steps);
            return count($steps) > 0 && $steps[0] == $this->getStepName();
        }


        /**
         * isLastStep
         *
         * Check if the current step is the last step
         *
         * @return  bool    True if the current step is the last step
         */
        public function isLastStep()
        {
            $steps = array_keys($this->_steps);
            return count($steps) > 0 && array_pop($steps) == $this->getStepName();
        }


        public function getNumSteps()
        {
            return count($this->_steps);
        }


        /**
         * set
         *
         * Sets a value in the container
         *
         * @param   string  $key    The key for the value to set
         * @param   mixed   $val    The value
         */
        public function set($key, $val, $force = true)
        {
            if ($force || !$this->hasValue($key))
                $this->containers['values']->$key = $val;
        }
        
        /** 
         * remove
         * 
         * unsets a value in the container
         * @param   string   $key   The key to be unset
         * @author  Dan Herbert - http://bugs.recite.com.au/view.php?id=2983
         */
        public function remove($key) {
            if ($this->hasValue($key)) {
                unset($this->containers['values']->$key);
            }
        }


        public function has($key)
        {
            return $this->hasValue($key);
        }


        /**
         * get
         *
         * Gets a value from the container
         *
         * @param   string  $key        The key for the value to get
         * @param   mixed   $default    The value to return if the key doesn't exist
         * @return  mixed               Either the key's value or the default value
         */
        public function get($key, $default = null)
        {
            return $this->hasValue($key) ? $this->containers['values']->$key : $default;
        }


        public function getValues()
        {
            $ret = array();

            foreach ($this->containers['values'] as $k => $v)
                $ret[$k] = $v;

            return $ret;
        }


        /**
         * hasValue
         *
         * Check if a value is set in the container
         *
         * @param   string  $key        The key to check for
         * @return  bool                True if value is set, false if not
         */
        public function hasValue($k)
        {
            if (strlen($k) == 0)
                return false;

            return isset($this->containers['values']->$k);
        }


        /**
         * clearContainer
         *
         * Removes all data from the container. This is primarily used
         * to reset the wizard data completely
         */
        public function clearContainer()
        {
            foreach ($this->containers as $name => $container) {
                $this->_clearContainer($name);
            }
        }

        protected function _clearContainer($name)
        {
            foreach ($this->containers[$name] as $k => $v)
                unset($this->containers[$name]->$k);
        }


        /**
         * addError
         *
         * Add an error
         *
         * @param   string  $key    An identifier for the error (e.g. the field name)
         * @param   string  $val    An error message
         */
        public function addError($key, $val)
        {
            $this->containers['errors']->$key = $val;
        }


        public function getError($key)
        {
            return $this->hasError($key) ? $this->containers['errors']->$key : null;
        }

        public function getErrors()
        {
            $ret = array();
            foreach ($this->containers['errors'] as $k => $v) {
                $ret[$k] = $v;
            }

            return $ret;
        }

        public function getNumErrors()
        {
            return count($this->container['errors']);
        }

        public function hasError($key = null)
        {
            if (strlen($key) > 0)
                return isset($this->containers['errors']->$key);

           foreach ($this->containers['errors'] as $error)
                return true;
           return false;
        }

        protected function _clearErrors()
        {
            $this->_clearContainer('errors');
        }


        /**
         * isFresh
         *
         * Determines whether or not the wizard form has been submitted at all
         * or if the user just freshly loaded it. This will also be true if they
         * complete the form and then return to it
         *
         * @return  bool    True if the form is fresh, false if it has been submitted
         */
        public function isFresh()
        {
            return (bool) $this->containers['status']->fresh;
        }
    }
?>
