#!/usr/local/bin/php -q
<?php
    /**
     * This script is used to restore files to the Recite data directory when they have been
     * deleted and the records still exist in the database. It requires a base Recite path
     * and the path where files exist. Any files in the database that are missing from Recite
     * are searched for in the specified path (including sub-directories). This script does
     * not take into account the path a file is in to help resolve its correct location.
     *
     * @author  Quentin Zervaas <quentin@recite.com.au>
     * @since   Recite CMS 2.1.10
     */

    function exitWithMessage($message, $code = 1)
    {
        $message = trim($message);

        if (strlen($message) > 0) {
            echo $message . "\n";
        }

        echo sprintf("Usage: %s /path/to/recite /path/to/files [client id] [--simulate]\n", $GLOBALS['argv'][0]);
        exit($code);
    }

    $cwd  = getcwd();
    $args = $GLOBALS['argv'];
    array_shift($args);

    $simulateKey = array_search('--simulate', $args);
    $simulate = $simulateKey !== false;

    if ($simulate) {
        unset($args[$simulateKey]);
        echo "SIMULATING ONLY\n";
    }

    $paths = array(
        'recite' => array('title' => 'Recite', 'path' => ''),
        'files'  => array('title' => 'Files', 'path' => '')
    );

    foreach ($paths as $k => $v) {
        $p = array_shift($args);

        if (strlen($p) == 0) {
            exitWithMessage(sprintf('%s path not specified', $v['title']));
        }

        if (substr($p, 0, 1) == '/') {
            $path = $p;
        }
        else {
            $path = $cwd . '/' . $p;
        }

        $fullpath = realpath($path);

        if (strlen($fullpath) == 0) {
            exitWithMessage('Path not found: ' . $p);
        }

        $paths[$k]['path'] = $fullpath;
    }

    $clientId = (int) array_shift($args);

    $includePath = realpath(sprintf('%s/application/include', $paths['recite']['path']));

    if (strlen($includePath) == 0) {
        exitWithMessage('Invalid Recite path specified');
    }

    ini_set('include_path', $includePath);
    require_once('Application/CommandLine/Bootstrap.php');

    $client = new Application_Client();
    $client->load($clientId);

    if (!$client->saved()) {
        exitWithMessage('Unknown client: ' . $clientId);
    }

    $missingFiles = array();

    $db = Application::GetDb();

    $query = sprintf('select client_id, file_id, original_filename from %s where client_id = %d', Module_Assets_Asset::$table, $client->getId());
    $result = $db->query($query);

    while ($row = $result->fetch()) {
        $clientId = (int) $row->client_id;
        $fileId   = (int) $row->file_id;
        $filename = $row->original_filename;

        $fullpath = sprintf('%s/%d',
            Application_Client_Data::GetNamespacedPathForClient($clientId, 'assets/files'),
            $fileId
        );

        if (file_exists($fullpath)) {
            continue;
        }

        if (!array_key_exists($filename, $missingFiles)) {
            $missingFiles[$filename] = array();
        }

        $missingFiles[$filename][] = array(
            'id'   => $fileId,
            'path' => $fullpath
        );
    }

    if (count($missingFiles) == 0) {
        echo sprintf("No missing files detected\n");
        exit(0);
    }

    $allFiles = array();

    $iterator = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($paths['files']['path']), RecursiveIteratorIterator::SELF_FIRST);

    foreach ($iterator as $fileObject) {
        $filename = $fileObject->getFilename();

        if (!array_key_exists($filename, $allFiles)) {
            $allFiles[$filename] = array();
        }

        $allFiles[$filename][] = $fileObject;
    }

    $multipleSrc = array();
    $multipleDst = array();
    $missing     = array();
    $restored    = array();

    foreach ($missingFiles as $filename => $files) {
        $skip = false;

        if (count($files) > 1) {
            $multipleDst[$filename] = $files;
            $skip = true;
        }

        if (!array_key_exists($filename, $allFiles)) {
            $missing[$filename] = $files;
            $skip = true;
        }

        $srcFiles = $allFiles[$filename];

        if (count($srcFiles) > 1) {
            $hash = '';

            foreach ($srcFiles as $srcFile) {
                if (strlen($hash) == 0) {
                    $hash = md5_file($srcFile->getPathname());
                }
                else if (md5_file($srcFile->getPathname()) != $hash) {
                    $multipleSrc[$filename] = $srcFiles;
                    $skip = true;
                    break;
                }
            }
        }

        if ($skip) {
            continue;
        }

        $cmd = sprintf(
            '%s %s %s',
            escapeshellcmd('cp'),
            escapeshellarg($srcFiles[0]->getPathname()),
            escapeshellarg($files[0]['path'])
        );

        echo $cmd . "\n";

        $retval = 0;

        if (!$simulate) {
            system($cmd, $retval);

            if ($retval != 0) {
                exitWithMessage("Error copying file", $retval);
            }
        }

        $restored[$filename] = $files[0];
    }

    echo "\nRESTORED:\n";

    if (count($restored) == 0) {
        echo "(none)\n";
    }
    else foreach ($restored as $filename => $file) {
        echo sprintf("ID %d - %s - %s\n", $file['id'], $filename, $file['path']);
    }

    if (count($missing) > 0) {
        echo "\nNOT FOUND IN SOURCE FILES:\n";

        foreach ($missing as $filename => $files) {
            foreach ($files as $file) {
                echo sprintf("%s - ID %d - belongs in %s\n", $filename, $file['id'], $file['path']);
            }
        }
    }

    if (count($multipleDst) > 0) {
        echo "\nMULTIPLE FILES IN RECITE DATABASE:\n";

        foreach ($multipleDst as $filename => $files) {
            foreach ($files as $file) {
                echo sprintf("%s - ID %d - %s\n", $filename, $file['id'], $file['path']);
            }
        }
    }

    if (count($multipleSrc) > 0) {
        echo "\nMULTIPLE FILES IN SOURCE FILES:\n";

        foreach ($multipleSrc as $filename => $files) {
            foreach ($files as $file) {
                echo sprintf("%s - %s\n", $filename, $file->getPathname());
            }
        }
    }
?>