<?php
    class Driver_forms_fields_html5number extends Profile_Field
    {
        public function __toString()
        {
            return 'HTML5 Number';
        }

        public function isCore()
        {
            return true;
        }
        
        public function isValid($val)
        {
            $ret = true;
            
            if(strlen($val) > 0){
                //test basic number format to start
                if(preg_match('/^-?\d+(\.\d+)?$/', $val)){
                    //remove scientic notation so we can look at decimal places
                    $step = rtrim(rtrim(sprintf("%.10F", $this->getParam('step', 0)), '0'), '.');
                    
                    //default step is 1
                    if(strlen($step) == 0 || $step == 0){
                        $step = 1;
                    }
                    
                    $stepDecimalPlaces = strlen(substr(strrchr($step, "."), 1));
                    $valDecimalPlaces = strlen(substr(strrchr($val, "."), 1));
                    
                    //handle integer steps
                    if($stepDecimalPlaces == 0){
                        $epsilon = 0.00000001;
                        
                        //integer step means value cannot have any decimal values
                        if($valDecimalPlaces > 0){
                            $ret = false;
                        }
                    }
                    //floating point step
                    else{
                        if($valDecimalPlaces > $stepDecimalPlaces){
                            $ret = false;
                        }
                        
                        $epsilon = 1 * pow(10, -1 * ($stepDecimalPlaces + 1));
                    }
                    
                    if($ret){
                        //floating point calculations are not exact, so use a small threshold instead of directly comparing
                        $normalized = round($val / $step) * $step;
                        if (abs($normalized - $val) > $epsilon){
                            $ret = false;
                        }
                    }
                    
                    if($ret && strlen($this->getParam('minimum_value')) > 0 && $this->getParam('minimum_value') > $val){
                        $ret = false;
                    }
                    
                    if($ret && strlen($this->getParam('maximum_value')) > 0 && $this->getParam('maximum_value') < $val){
                        $ret = false;
                    }
                }
                else{
                    $ret = false;
                }
            }
            return $ret;
        }

        public function set($val)
        {
            if(strlen($val) == 0){
                $this->_value = null;
            }
            else{
                //strip off any trailing zeros and the decimal place if possible
                if(strpos($val, '.') !== false){
                    $this->_value = rtrim(rtrim(FormProcessor::sanitize($val), '0'), '.');
                }
                else{
                    $this->_value = FormProcessor::sanitize($val);
                }
                
                if(!$this->isValid($this->_value)){
                    return false;
                }
            }

            return true;
        }

        public function getInputTemplate($name, $attrs = array())
        {
            if (!is_array($attrs)) {
                $attrs = array();
            }

            $attrs['type']  = 'number';
            $attrs['name']  = $name;
            
            if(strlen($this->getParam('placeholder_text')) > 0){
                $attrs['placeholder'] = $this->getParam('placeholder_text');
            }
            
            if(strlen($this->getParam('minimum_value')) > 0){
                $attrs['min'] = rtrim(rtrim(sprintf("%.10F", $this->getParam('minimum_value')), '0'), '.');
            }
            
            if(strlen($this->getParam('maximum_value')) > 0){
                $attrs['max'] = rtrim(rtrim(sprintf("%.10F", $this->getParam('maximum_value')), '0'), '.');
            }
            
            if(strlen($this->getParam('step')) > 0){
                $attrs['step'] = rtrim(rtrim(sprintf("%.10F", $this->getParam('step')), '0'), '.');
            }
            
            if(strlen($this->getParam('validation_regex')) > 0){
                $attrs['pattern'] = $this->getParam('validation_regex');
            }
            
            $attrs['value'] = $this->get();
            
            if (!array_key_exists('class', $attrs)) {
                $attrs['class'] = 'profile-number';
            }

            $elt = new HTML_Element('input', $attrs);

            return (string) $elt;
        }

        public function getOptions()
        {
            $list = new Module_Forms_FieldList();
            
            $placeholderText = new Module_Forms_FieldList_Field('string');
            $placeholderText->setName('placeholder_text');
            $placeholderText->setTitle('Placeholder Text');
            $placeholderText->setHelp('Placeholder text that displays in the element. This is cleared when the element gets focus.');
            $list->add($placeholderText);
            
            $step = new Module_Forms_FieldList_Field('number');
            $step->setName('step');
            $step->setTitle('Step');
            $step->setHelp("Specifies the granularity of the element's value e.g. 2 allows -2, 0 2, 4 etc. 0.1 allows single decimal values like 1.2 and -10.4. Defaults to 1");
            $list->add($step);
            
            $minimumValue = new Module_Forms_FieldList_Field('number');
            $minimumValue->setName('minimum_value');
            $minimumValue->setTitle('Minimum Value');
            $minimumValue->setHelp("Specifies the lowest allowed number");
            $list->add($minimumValue);
            
            $maximumValue = new Module_Forms_FieldList_Field('number');
            $maximumValue->setName('maximum_value');
            $maximumValue->setTitle('Maximum Value');
            $maximumValue->setHelp("Specifies the highest allowed number");
            $list->add($maximumValue);
            
            $validationRegex = new Module_Forms_FieldList_Field('string');
            $validationRegex->setName('validation_regex');
            $validationRegex->setTitle('Validation Regex');
            $validationRegex->setHelp('Validation regular expression. Used on the website in browsers that support it and in validation on the server. Delimiters and string start/end anchors are automatically added for use in the server side check, so leave them off here.');
            $list->add($validationRegex);

            return $list;
        }
    }
?>
